@@ -1,8 +1,9 @@
-/* radare - LGPL - Copyright 2010-2024 - nibble, pancake */
+/* radare - LGPL - Copyright 2010-2025 - nibble, pancake */
 
 #define R_LOG_ORIGIN "anal.diff"
 
 #include <r_anal.h>
+#include <r_core.h>
 
 R_API RAnalDiff *r_anal_diff_new(void) {
 	RAnalDiff *diff = R_NEW0 (RAnalDiff);
@@ -23,20 +24,16 @@ R_API void r_anal_diff_free(RAnalDiff *diff) {
 	}
 }
 
-R_API void r_anal_diff_setup(RAnal *anal, int doops, double thbb, double thfcn) {
+R_API void r_anal_diff_setup(RAnal *anal, bool doops, double thbb, double thfcn) {
 	R_RETURN_IF_FAIL (anal);
-	if (doops >= 0) {
-		anal->diff_ops = doops;
-	}
+	anal->diff_ops = doops;
 	anal->diff_thbb = (thbb >= 0)? thbb: R_ANAL_THRESHOLDBB;
 	anal->diff_thfcn = (thfcn >= 0)? thfcn: R_ANAL_THRESHOLDFCN;
 }
 
-R_API void r_anal_diff_setup_i(RAnal *anal, int doops, int thbb, int thfcn) {
+R_API void r_anal_diff_setup_i(RAnal *anal, bool doops, int thbb, int thfcn) {
 	R_RETURN_IF_FAIL (anal);
-	if (doops >= 0) {
-		anal->diff_ops = doops;
-	}
+	anal->diff_ops = doops;
 	anal->diff_thbb = (thbb >= 0)? ((double)thbb) / 100: R_ANAL_THRESHOLDBB;
 	anal->diff_thfcn = (thfcn >= 0)? ((double)thfcn) / 100: R_ANAL_THRESHOLDFCN;
 }
@@ -151,10 +148,10 @@ R_API bool r_anal_diff_bb(RAnal *anal, RAnalFunction *fcn, RAnalFunction *fcn2)
 		}
 		if (mbb && mbb2) {
 			if (!mbb->diff) {
-				mbb->diff = r_anal_diff_new();
+				mbb->diff = r_anal_diff_new ();
 			}
 			if (!mbb2->diff) {
-				mbb2->diff = r_anal_diff_new();
+				mbb2->diff = r_anal_diff_new ();
 			}
 			if (!mbb->diff || !mbb2->diff) {
 				return false;
@@ -195,6 +192,8 @@ R_API int r_anal_diff_fcn(RAnal *anal, RList *fcns, RList *fcns2) {
 		return anal->cur->diff_fcn (anal, fcns, fcns2);
 	}
 	if (fcns == fcns2 || (r_list_length (fcns) == 1 && 1 == r_list_length (fcns2))) {
+		RCore *core = anal->coreb.core;
+		RCons *cons = core->cons;
 		double threshold = 0.8; // XXX make this configurable by the user
 		// inplace diffing
 		r_list_foreach (fcns, iter, fcn) {
@@ -222,7 +221,7 @@ R_API int r_anal_diff_fcn(RAnal *anal, RList *fcns, RList *fcns2) {
 					fcn->diff->dist = fcn2->diff->dist = t;
 					if (fcn->diff->type == R_ANAL_DIFF_TYPE_MATCH) {
 						// R_LOG_INFO ("match %lf %s", t, fcn2->name);
-						r_cons_printf ("match %lf %s\n", t, fcn2->name);
+						r_cons_printf (cons, "match %lf %s\n", t, fcn2->name);
 					}
 					R_FREE (fcn->fingerprint);
 					R_FREE (fcn2->fingerprint);
@@ -364,12 +363,3 @@ R_API int r_anal_diff_fcn(RAnal *anal, RList *fcns, RList *fcns2) {
 	}
 	return true;
 }
-
-// R2_600 UNUSED
-R_API int r_anal_diff_eval(RAnal *anal) {
-	R_RETURN_VAL_IF_FAIL (anal, false);
-	if (anal->cur && anal->cur->diff_eval) {
-		return (anal->cur->diff_eval (anal));
-	}
-	return false;
-}
