@@ -1,9 +1,9 @@
-/* radare - LGPL - Copyright 2010-2024 - nibble, alvaro, pancake */
+/* radare - LGPL - Copyright 2010-2025 - nibble, alvaro, pancake */
 
 #define R_LOG_ORIGIN "fcn"
 
 #include <r_anal.h>
-#include <r_parse.h>
+#include <r_core.h>
 #include <r_vec.h>
 
 #define READ_AHEAD 1
@@ -19,9 +19,6 @@
 /* speedup analysis by removing some function overlapping checks */
 #define JAYRO_04 1
 
-// 16 KB is the maximum size for a basic block
-#define MAX_FLG_NAME_SIZE 64
-
 #define FIX_JMP_FWD 0
 #define D if (a->verbose)
 
@@ -301,7 +298,7 @@ static ut64 try_get_cmpval_from_parents(RAnal *anal, RAnalFunction *fcn, RAnalBl
 		if (tmp_bb->jump == my_bb->addr || tmp_bb->fail == my_bb->addr) {
 			if (tmp_bb->cmpreg == cmp_reg) {
 				if (tmp_bb->cond) {
-					if (tmp_bb->cond->type == R_ANAL_COND_HI || tmp_bb->cond->type == R_ANAL_COND_GT) {
+					if (tmp_bb->cond->type == R_ANAL_CONDTYPE_HI || tmp_bb->cond->type == R_ANAL_CONDTYPE_GT) {
 						return tmp_bb->cmpval + 1;
 					}
 				}
@@ -459,6 +456,7 @@ typedef struct {
 static bool fcn_takeover_block_recursive_followthrough_cb(RAnalBlock *block, void *user) {
 	BlockTakeoverCtx *ctx = user;
 	RAnalFunction *our_fcn = ctx->fcn;
+	RAnal *anal = our_fcn->anal;
 	r_anal_block_ref (block);
 	while (!r_list_empty (block->fcns)) {
 		RAnalFunction *other_fcn = r_list_first (block->fcns);
@@ -488,7 +486,7 @@ static bool fcn_takeover_block_recursive_followthrough_cb(RAnalBlock *block, voi
 				}
 				if (our_var) {
 					RAnalVarAccess *acc = r_anal_var_get_access_at (other_var, addr);
-					r_anal_var_set_access (our_var, acc->reg, addr, acc->type, acc->stackptr);
+					r_anal_var_set_access (anal, our_var, acc->reg, addr, acc->type, acc->stackptr);
 				}
 				r_anal_var_remove_access_at (other_var, addr);
 				if (r_vector_empty (&other_var->accesses)) {
@@ -516,7 +514,7 @@ static void fcn_takeover_block_recursive(RAnalFunction *fcn, RAnalBlock *start_b
 }
 
 static const char *retpoline_reg(RAnal *anal, ut64 addr) {
-	RFlagItem *flag = anal->flag_get (anal->flb.f, addr);
+	RFlagItem *flag = anal->flag_get (anal->flb.f, false, addr);
 	if (flag) {
 		const char *token = "x86_indirect_thunk_";
 		const char *thunk = strstr (flag->name, token);
@@ -622,23 +620,21 @@ static int fcn_recurse(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int
 	} delay = {
 		0
 	};
+	RCore *core = anal->coreb.core;
+	RCons *cons = core->cons;
 	const char *arch = anal->config? anal->config->arch: R_SYS_ARCH;
 	bool arch_destroys_dst = does_arch_destroys_dst (arch);
-#if R2_USE_NEW_ABI
 	const bool flagends = anal->opt.flagends;
-#else
-	const bool flagends = anal->coreb.cfggeti (anal->coreb.core, "anal.flagends");
-#endif
 	const bool is_arm = r_str_startswith (arch, "arm");
 	const bool is_mips = !is_arm && r_str_startswith (arch, "mips");
-	const bool is_v850 = is_arm ? false: (arch && (!strncmp (arch, "v850", 4) || !strncmp (anal->coreb.cfgGet (anal->coreb.core, "asm.cpu"), "v850", 4)));
+	const bool is_v850 = is_arm ? false: (arch && (!strncmp (arch, "v850", 4) || !strncmp (anal->coreb.cfgGet (core, "asm.cpu"), "v850", 4)));
 	const bool is_x86 = is_arm ? false: arch && !strncmp (arch, "x86", 3);
-	const bool is_amd64 = is_x86 ? fcn->cc && !strcmp (fcn->cc, "amd64") : false;
+	const bool is_amd64 = is_x86 ? fcn->callconv && !strcmp (fcn->callconv, "amd64") : false;
 	const bool is_dalvik = is_x86 ? false : arch && !strncmp (arch, "dalvik", 6);
 	const bool propagate_noreturn = anal->opt.propagate_noreturn;
 	ut64 v1 = UT64_MAX;
 
-	if (r_cons_is_breaked ()) {
+	if (r_cons_is_breaked (cons)) {
 		return R_ANAL_RET_END;
 	}
 	if (anal->sleep) {
@@ -735,8 +731,8 @@ static int fcn_recurse(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int
 		}
 		maxlen = 0;
 	}
-	const char *_bp_reg = anal->reg->name[R_REG_NAME_BP];
-	const char *_sp_reg = anal->reg->name[R_REG_NAME_SP];
+	const char *_bp_reg = r_reg_alias_getname (anal->reg, R_REG_ALIAS_BP);
+	const char *_sp_reg = r_reg_alias_getname (anal->reg, R_REG_ALIAS_SP);
 	const bool has_stack_regs = _bp_reg && _sp_reg;
 	if (has_stack_regs) {
 		free (bp_reg);
@@ -748,6 +744,13 @@ static int fcn_recurse(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int
 		variadic_reg = "rax";
 	}
 	bool has_variadic_reg = !!variadic_reg;
+	bool nopskip = anal->opt.nopskip;
+	if (nopskip) {
+		const bool isvm = r_anal_archinfo (anal, R_ARCH_INFO_ISVM) == R_ARCH_INFO_ISVM;
+		if (isvm) {
+			nopskip = false;
+		}
+	}
 
 	op = r_anal_op_new ();
 	const ut32 opflags = R_ARCH_OP_MASK_BASIC | R_ARCH_OP_MASK_ESIL | R_ARCH_OP_MASK_VAL | R_ARCH_OP_MASK_HINT;
@@ -759,7 +762,7 @@ static int fcn_recurse(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int
 			break;
 		}
 repeat:
-		if (r_cons_is_breaked ()) {
+		if (r_cons_is_breaked (cons)) {
 			break;
 		}
 		ut8 buf[32]; // 32 bytes is enough to hold any instruction.
@@ -795,7 +798,7 @@ repeat:
 		op_src = (src0 && src0->reg)? strdup (src0->reg): NULL;
 		src1 = r_vector_at (&op->srcs, 1);
 
-		if (anal->opt.nopskip && fcn->addr == at) {
+		if (nopskip && fcn->addr == at) {
 			const int codealign = r_anal_archinfo (anal, R_ARCH_INFO_CODE_ALIGN);
 			if (codealign > 1) {
 				if (at % codealign) {
@@ -850,7 +853,7 @@ noskip:
 			}
 		}
 		if (flagends && fcn->addr != at) {
-			RFlagItem *flag = anal->flag_get (anal->flb.f, at);
+			RFlagItem *flag = anal->flag_get (anal->flb.f, false, at);
 			if (flag) {
 				if (r_str_startswith (flag->name, "sym")) {
 					gotoBeach (R_ANAL_RET_END);
@@ -1108,7 +1111,7 @@ noskip:
 					// TODO: -1-
 					if (ready) {
 						ret = casetbl_addr == op->ptr
-							? try_walkthrough_jmptbl (anal, fcn, bb, depth, addr, case_shift, jmptbl_addr, op->ptr, 4, table_size, default_case, 4)
+							? r_anal_jmptbl_walk (anal, fcn, bb, depth, addr, case_shift, jmptbl_addr, op->ptr, 4, table_size, default_case, 4)
 							: try_walkthrough_casetbl (anal, fcn, bb, depth, addr, case_shift, jmptbl_addr, casetbl_addr, op->ptr, 4, table_size, default_case, 4);
 						if (ret) {
 							anal->lea_jmptbl_ip = addr;
@@ -1122,10 +1125,7 @@ noskip:
 			// R2R db/anal/arm db/esil/apple
 			//v1 = UT64_MAX; // reset v1 jmptable pointer value for mips only
 			// on stm8 this must be disabled.. but maybe we need a global option to disable icod refs
-			bool want_icods = true;
-#if R2_USE_NEW_ABI
-			// want_icods = anal->opt.icods;
-#endif
+			bool want_icods = anal->opt.icods;
 			{
 				const char *arch = R_UNWRAP3 (anal, config, arch);
 				if (r_str_startswith (arch, "stm8")) {
@@ -1225,7 +1225,7 @@ noskip:
 					}
 				}
 			}
-			if (r_cons_is_breaked ()) {
+			if (r_cons_is_breaked (cons)) {
 				gotoBeach (R_ANAL_RET_END);
 			}
 			if (anal->opt.jmpref) {
@@ -1241,7 +1241,7 @@ noskip:
 				bool must_eob = true;
 				RIOMap *map = anal->iob.map_get_at (anal->iob.io, addr);
 				if (map) {
-					must_eob = ( ! r_io_map_contain (map, op->jump) );
+					must_eob = ! r_io_map_contain (map, op->jump);
 				}
 				if (must_eob) {
 					op->jump = UT64_MAX;
@@ -1267,9 +1267,18 @@ noskip:
 				(void)anal->iob.read_at (anal->iob.io, op->jump, (ut8 *) buf, sizeof (buf));
 				if (r_anal_is_prelude (anal, op->jump, buf, sizeof (buf))) {
 					R_LOG_DEBUG ("tail call jump found at 0x%08"PFMT64x, op->addr);
+					// XXX using type-jump wont analyze the destination as a function
+					// calling fcn_recurse wont make it analyze it either
+#if 0
+					(void) r_anal_xrefs_set (anal, op->addr, op->jump, R_ANAL_REF_TYPE_CALL | R_ANAL_REF_TYPE_EXEC);
+#else
 					(void) r_anal_xrefs_set (anal, op->addr, op->jump, R_ANAL_REF_TYPE_JUMP | R_ANAL_REF_TYPE_EXEC);
-					fcn_recurse (anal, fcn, op->jump, anal->opt.bb_max_size, depth - 1);
-					op->jump = UT64_MAX;
+					// using type-jump wont analyze the destination as a function
+					// fcn_recurse (anal, fcn, op->jump, anal->opt.bb_max_size, depth - 1);
+					/// XXX RAnalFunction *fcn = r_anal_function_new (anal, op->jump);
+					RAnalFunction *nfcn = r_anal_create_function (anal, NULL, op->jump, 0, NULL);
+					r_anal_function (anal, nfcn? nfcn: fcn, op->jump, R_ANAL_REF_TYPE_CALL);
+#endif
 					gotoBeach (R_ANAL_RET_END);
 				}
 			}
@@ -1332,7 +1341,7 @@ noskip:
 					if (anal->cmpval != UT64_MAX && default_case != UT64_MAX && (op->reg || op->ireg)) {
 						// TODO -1
 						if (op->ireg) {
-							ret = try_walkthrough_jmptbl (anal, fcn, bb, depth, op->addr, 0, op->ptr, op->ptr, anal->config->bits >> 3, table_size, default_case, ret);
+							ret = r_anal_jmptbl_walk (anal, fcn, bb, depth, op->addr, 0, op->ptr, op->ptr, anal->config->bits >> 3, table_size, default_case, ret);
 						} else { // op->reg
 							ret = walkthrough_arm_jmptbl_style (anal, fcn, bb, depth, op->addr, op->ptr, anal->config->bits >> 3, table_size, default_case, ret);
 						}
@@ -1426,12 +1435,12 @@ noskip:
 				tablesize *= 4;
 				ut64 tblloc = jmptbl_ptr_addr;
 				int sz = 4;
-				ret = try_walkthrough_jmptbl (anal, fcn, bb, depth, op->addr, 0,
+				ret = r_anal_jmptbl_walk (anal, fcn, bb, depth, op->addr, 0,
 						tblloc, jmptbl_ptr_addr, sz, tablesize, default_case, ret);
 			} else if (is_v850 && anal->opt.jmptbl) {
 				int ptsz = (anal->cmpval && anal->cmpval != UT64_MAX)? anal->cmpval + 1: 4;
 				if ((int)anal->cmpval > 0) {
-					ret = try_walkthrough_jmptbl (anal, fcn, bb, depth, op->addr,
+					ret = r_anal_jmptbl_walk (anal, fcn, bb, depth, op->addr,
 							0, op->addr + 2, op->addr + 2, 2, ptsz, 0, ret);
 				}
 				gotoBeach (R_ANAL_RET_END);
@@ -1473,14 +1482,14 @@ noskip:
 						}
 						r_anal_op_free (prev_op);
 						if (!case_table) {
-							ret = try_walkthrough_jmptbl (anal, fcn, bb, depth, op->addr, case_shift, op->ptr, op->ptr, anal->config->bits >> 3, table_size, default_case, ret);
+							ret = r_anal_jmptbl_walk (anal, fcn, bb, depth, op->addr, case_shift, op->ptr, op->ptr, anal->config->bits >> 3, table_size, default_case, ret);
 						}
 					}
 				} else if (op->ptr != UT64_MAX && op->reg) { // direct jump
 					ut64 table_size, default_case;
 					st64 case_shift = 0;
 					if (try_get_jmptbl_info (anal, fcn, op->addr, bb, &table_size, &default_case, &case_shift)) {
-						ret = try_walkthrough_jmptbl (anal, fcn, bb, depth - 1, op->addr, case_shift, op->ptr, op->ptr, anal->config->bits >> 3, table_size, default_case, ret);
+						ret = r_anal_jmptbl_walk (anal, fcn, bb, depth - 1, op->addr, case_shift, op->ptr, op->ptr, anal->config->bits >> 3, table_size, default_case, ret);
 					}
 				} else if (movdisp != UT64_MAX) {
 					st64 case_shift = 0;
@@ -1505,7 +1514,7 @@ noskip:
 						table_size = anal->cmpval + 1;
 						default_case = -1;
 					}
-					ret = try_walkthrough_jmptbl (anal, fcn, bb, depth - 1, op->addr, case_shift, jmptbl_base + movdisp, jmptbl_base, movscale, table_size, default_case, ret);
+					ret = r_anal_jmptbl_walk (anal, fcn, bb, depth - 1, op->addr, case_shift, jmptbl_base + movdisp, jmptbl_base, movscale, table_size, default_case, ret);
 					anal->cmpval = UT64_MAX;
 #if 0
 				} else if (movdisp != UT64_MAX) {
@@ -1513,7 +1522,7 @@ noskip:
 					st64 case_shift;
 					if (try_get_jmptbl_info (anal, fcn, op->addr, bb, &table_size, &default_case, &case_shift)) {
 						op->ptr = movdisp;
-						ret = try_walkthrough_jmptbl (anal, fcn, bb, depth - 1, op->addr, case_shift, op->ptr, op->ptr, anal->config->bits >> 3, table_size, default_case, ret);
+						ret = r_anal_jmptbl_walk (anal, fcn, bb, depth - 1, op->addr, case_shift, op->ptr, op->ptr, anal->config->bits >> 3, table_size, default_case, ret);
 					}
 					movdisp = UT64_MAX;
 #endif
@@ -1559,7 +1568,7 @@ noskip:
 							count++;
 						}
 						// table_addr = 0x100004114;
-						ret = try_walkthrough_jmptbl (anal,
+						ret = r_anal_jmptbl_walk (anal,
 								fcn, bb, depth - 1,
 								op->addr - 12, 0,
 								table_addr,
@@ -1576,7 +1585,7 @@ noskip:
 						} else {
 							table_size += anal->cmpval;
 						}
-						ret = try_walkthrough_jmptbl (anal, fcn, bb, depth - 1, op->addr, 0, op->addr + op->size,
+						ret = r_anal_jmptbl_walk (anal, fcn, bb, depth - 1, op->addr, 0, op->addr + op->size,
 								op->addr + 4, 1, table_size, UT64_MAX, ret);
 						// skip inlined jumptable
 						idx += table_size;
@@ -1588,7 +1597,7 @@ noskip:
 						} else {
 							tablesize += anal->cmpval;
 						}
-						ret = try_walkthrough_jmptbl (anal, fcn, bb, depth - 1, op->addr, 0, op->addr + op->size,
+						ret = r_anal_jmptbl_walk (anal, fcn, bb, depth - 1, op->addr, 0, op->addr + op->size,
 								op->addr + 4, 2, tablesize, UT64_MAX, ret);
 						// skip inlined jumptable
 						idx += (tablesize * 2);
@@ -1820,7 +1829,7 @@ R_API void r_anal_del_jmprefs(RAnal *anal, RAnalFunction *fcn) {
 }
 
 /* Does NOT invalidate read-ahead cache. */
-R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int reftype) {
+R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, int reftype) {
 	R_RETURN_VAL_IF_FAIL (anal && fcn, 0);
 	RPVector *metas = r_meta_get_all_in (anal, addr, R_META_TYPE_ANY);
 	if (metas) {
@@ -1860,7 +1869,7 @@ R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len,
 		fcn->addr = addr;
 	}
 	fcn->maxstack = 0;
-	if (fcn->cc && !strcmp (fcn->cc, "ms")) {
+	if (fcn->callconv && !strcmp (fcn->callconv, "ms")) {
 		// Probably should put this on the cc sdb
 		const int shadow_store = 0x28; // First 4 args + retaddr
 		fcn->stack = fcn->maxstack = fcn->reg_save_area = shadow_store;
@@ -1876,7 +1885,7 @@ R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len,
 R_API int r_anal_function_del(RAnal *a, ut64 addr) {
 	RAnalFunction *fcn = r_anal_get_function_at (a, addr);
 	if (fcn) {
-		r_anal_function_delete (fcn);
+		r_anal_function_delete (a, fcn);
 		// r_anal_function_free (fcn);
 		return true;
 	}
@@ -1939,12 +1948,12 @@ R_API RAnalFunction *r_anal_get_function_byname(RAnal *a, const char *name) {
 R_API bool r_anal_function_add_bb(RAnal *a, RAnalFunction *fcn, ut64 addr, ut64 size, ut64 jump, ut64 fail, R_BORROW RAnalDiff *diff) {
 	if (size == 0) { // empty basic blocks allowed?
 		R_LOG_WARN ("empty basic block at 0x%08"PFMT64x" is not allowed. pending discussion", addr);
-		r_warn_if_reached ();
+		R_WARN_IF_REACHED ();
 		return false;
 	}
 	if (size > a->opt.bb_max_size) {
 		R_LOG_WARN ("can't allocate such big bb of %"PFMT64d" bytes at 0x%08"PFMT64x, (st64)size, addr);
-		r_warn_if_reached ();
+		R_WARN_IF_REACHED ();
 		return false;
 	}
 
@@ -2008,6 +2017,7 @@ R_API int r_anal_function_loops(RAnalFunction *fcn) {
 }
 
 R_API int r_anal_function_complexity(RAnalFunction *fcn) {
+	R_RETURN_VAL_IF_FAIL (fcn, 0);
 #if 0
 	* CC = E - N + 2P
 	* E = the number of edges of the graph.
@@ -2037,7 +2047,7 @@ R_API int r_anal_function_complexity(RAnalFunction *fcn) {
 		}
 	}
 
-	int result = E - N + (2 * P);
+	const int result = E - N + (2 * P);
 	if (result < 1 && (!anal || anal->verbose)) {
 		R_LOG_WARN ("CC = E(%d) - N(%d) + (2 * P(%d)) < 1 at 0x%08"PFMT64x, E, N, P, fcn->addr);
 	}
@@ -2045,75 +2055,6 @@ R_API int r_anal_function_complexity(RAnalFunction *fcn) {
 	return result;
 }
 
-// tfj and afsj call this function
-R_API char *r_anal_function_get_json(RAnalFunction *function) {
-	RAnal *a = function->anal;
-	PJ *pj = a->coreb.pjWithEncoding (a->coreb.core);
-	const char *realname = NULL, *import_substring = NULL;
-
-	RFlagItem *flag = a->flag_get (a->flb.f, function->addr);
-	// Can't access R_FLAGS_FS_IMPORTS, since it is defined in r_core.h
-	if (flag && flag->space && !strcmp (flag->space->name, "imports")) {
-		// Get substring after last dot
-		import_substring = r_str_rchr (function->name, NULL, '.');
-		if (import_substring) {
-			realname = import_substring + 1;
-		}
-	} else {
-		realname = function->name;
-	}
-
-	char *args = strdup ("");
-	char *sdb_ret = r_str_newf ("func.%s.ret", realname);
-	char *sdb_args = r_str_newf ("func.%s.args", realname);
-	// RList *args_list = r_list_newf ((RListFree) free);
-	unsigned int i;
-	const char *ret_type = sdb_const_get (a->sdb_types, sdb_ret, 0);
-	const char *argc_str = sdb_const_get (a->sdb_types, sdb_args, 0);
-
-	int argc = argc_str? atoi (argc_str): 0;
-
-	pj_o (pj);
-	pj_ks (pj, "name", function->name);
-	const bool no_return = r_anal_noreturn_at_addr (a, function->addr);
-	pj_kb (pj, "noreturn", no_return);
-	pj_ks (pj, "ret", r_str_get_fail (ret_type, "void"));
-	if (function->cc) {
-		pj_ks (pj, "cc", function->cc);
-	}
-	pj_kn (pj, "argc", argc);
-	pj_k (pj, "args");
-	pj_a (pj);
-	for (i = 0; i < argc; i++) {
-		char *sdb_arg_i = r_str_newf ("func.%s.arg.%d", realname, i);
-		char *arg_i = sdb_get (a->sdb_types, sdb_arg_i, 0);
-		if (!arg_i) {
-			continue;
-		}
-		pj_o (pj);
-		char *comma = strchr (arg_i, ',');
-		if (comma) {
-			*comma = 0;
-			pj_ks (pj, "name", comma + 1);
-			pj_ks (pj, "type", arg_i);
-			r_strf_var (regname, 32, "A%d", i);
-			const char *cc_arg = r_reg_get_name (a->reg, r_reg_get_name_idx (regname));
-			if (cc_arg) {
-				pj_ks (pj, "cc", cc_arg);
-			}
-		}
-		free (arg_i);
-		free (sdb_arg_i);
-		pj_end (pj);
-	}
-	pj_end (pj);
-	free (sdb_args);
-	free (sdb_ret);
-	free (args);
-	pj_end (pj);
-	return pj_drain (pj);
-}
-
 R_API bool r_anal_function_del_signature(RAnal *a, const char *name) {
 	Sdb *db = a->sdb_types;
 	const char *s = sdb_const_get (db, name, 0);
@@ -2143,7 +2084,7 @@ R_API char *r_anal_function_get_signature(RAnalFunction *function) {
 	RAnal *a = function->anal;
 	const char *realname = NULL, *import_substring = NULL;
 
-	RFlagItem *flag = a->flag_get (a->flb.f, function->addr);
+	RFlagItem *flag = a->flag_get (a->flb.f, false, function->addr);
 	// Can't access R_FLAGS_FS_IMPORTS, since it is defined in r_core.h
 	if (flag && flag->space && !strcmp (flag->space->name, "imports")) {
 		// Get substring after last dot
@@ -2175,12 +2116,17 @@ R_API char *r_anal_function_get_signature(RAnalFunction *function) {
 		// parse commas
 		int arg_i_len = strlen (arg_i);
 		for (j = 0; j < arg_i_len; j++) {
-			if (j > 0 && arg_i[j] == ',') {
-				if (arg_i[j - 1] == '*') {
-					// remove whitespace
-					memmove (arg_i + j, arg_i + j + 1, strlen (arg_i) - j);
+			if (arg_i[j] == ',') {
+				if (j> 0) {
+					if (arg_i[j - 1] == '*') {
+						// remove whitespace
+						memmove (arg_i + j, arg_i + j + 1, strlen (arg_i) - j);
+					} else {
+						arg_i[j] = ' ';
+					}
 				} else {
-					arg_i[j] = ' ';
+					// untyped arg. fex: `printf(...)`
+					memmove (arg_i, arg_i + 1, strlen (arg_i));
 				}
 			}
 		}
@@ -2320,7 +2266,7 @@ R_API ut32 r_anal_function_cost(RAnalFunction *fcn) {
 	return totalCycles;
 }
 
-R_API int r_anal_function_count_edges(const RAnalFunction *fcn, R_NULLABLE int *ebbs) {
+R_API int r_anal_function_count_edges(const RAnalFunction *fcn, int * R_NULLABLE ebbs) {
 	R_RETURN_VAL_IF_FAIL (fcn, 0);
 	RListIter *iter;
 	RAnalBlock *bb;
@@ -2360,13 +2306,16 @@ static bool can_affect_bp(RAnal *anal, RAnalOp* op) {
 	RAnalValue *src = r_vector_at (&op->srcs, 0);
 	const char *opdreg = dst? dst->reg: NULL;
 	const char *opsreg = src? src->reg: NULL;
-	const char *bp_name = anal->reg->name[R_REG_NAME_BP];
-	bool dst_is_bp = opdreg && !dst->memref && !strcmp (opdreg, bp_name);
-	bool src_is_bp = opsreg && !src->memref && !strcmp (opsreg, bp_name);
-	if (op->type == R_ANAL_OP_TYPE_XCHG) {
-		return src_is_bp || dst_is_bp;
+	const char *bpreg = r_reg_alias_getname (anal->reg, R_REG_ALIAS_BP);
+	if (bpreg) {
+		bool dst_is_bp = opdreg && !dst->memref && !strcmp (opdreg, bpreg);
+		bool src_is_bp = opsreg && !src->memref && !strcmp (opsreg, bpreg);
+		if (op->type == R_ANAL_OP_TYPE_XCHG) {
+			return src_is_bp || dst_is_bp;
+		}
+		return dst_is_bp;
 	}
-	return dst_is_bp;
+	return false;
 }
 
 /*
@@ -2380,9 +2329,12 @@ R_API void r_anal_function_check_bp_use(RAnalFunction *fcn) {
 	RAnalBlock *bb;
 	char *pos;
 	// XXX omg this is one of the most awful things ive seen lately
-	char str_to_find[40];
-	snprintf (str_to_find, sizeof (str_to_find),
-		"\"type\":\"reg\",\"value\":\"%s", anal->reg->name[R_REG_NAME_BP]);
+	char str_to_find[40] = {0};
+	const char *bpreg = r_reg_alias_getname (anal->reg, R_REG_ALIAS_BP);
+	if (bpreg) {
+		snprintf (str_to_find, sizeof (str_to_find),
+			"\"type\":\"reg\",\"value\":\"%s", bpreg);
+	}
 	r_list_foreach (fcn->bbs, iter, bb) {
 		RAnalOp op;
 		RAnalValue *src = NULL;
@@ -2403,8 +2355,8 @@ R_API void r_anal_function_check_bp_use(RAnalFunction *fcn) {
 			case R_ANAL_OP_TYPE_MOV:
 			case R_ANAL_OP_TYPE_LEA:
 				if (can_affect_bp (anal, &op)) {
-					const char *spreg = anal->reg->name[R_REG_NAME_SP];
-					if (src && src->reg && strcmp (src->reg, spreg)) {
+					const char *spreg = r_reg_alias_getname (anal->reg, R_REG_ALIAS_SP);
+					if (spreg && src && src->reg && strcmp (src->reg, spreg)) {
 						fcn->bp_frame = false;
 						r_anal_op_fini (&op);
 						free (buf);
@@ -2425,21 +2377,31 @@ R_API void r_anal_function_check_bp_use(RAnalFunction *fcn) {
 			case R_ANAL_OP_TYPE_SUB:
 			case R_ANAL_OP_TYPE_XOR:
 			case R_ANAL_OP_TYPE_SHL:
-// op.dst is not filled for these operations, so for now, check for bp as dst looks like this; in the future it may be just replaced with call to can_affect_bp
- 				pos = op.opex.ptr ? strstr (op.opex.ptr, str_to_find) : NULL;
-				if (pos && pos - op.opex.ptr < 60) {
-					fcn->bp_frame = false;
-					r_anal_op_fini (&op);
-					free (buf);
-					return;
+				// op.dst is not filled for these operations, so for now,
+				// check for bp as dst looks like this; in the future
+				// it may be just replaced with call to can_affect_bp
+				if (*str_to_find) {
+					pos = op.opex.ptr ? strstr (op.opex.ptr, str_to_find) : NULL;
+					if (pos && pos - op.opex.ptr < 60) {
+						fcn->bp_frame = false;
+						r_anal_op_fini (&op);
+						free (buf);
+						return;
+					}
+				} else {
+					R_LOG_WARN ("No string to find");
 				}
 				break;
 			case R_ANAL_OP_TYPE_XCHG:
-				if (op.opex.ptr && strstr (op.opex.ptr, str_to_find)) {
-					fcn->bp_frame = false;
-					r_anal_op_fini (&op);
-					free (buf);
-					return;
+				if (*str_to_find) {
+					if (op.opex.ptr && strstr (op.opex.ptr, str_to_find)) {
+						fcn->bp_frame = false;
+						r_anal_op_fini (&op);
+						free (buf);
+						return;
+					}
+				} else {
+					R_LOG_WARN ("No string to find");
 				}
 				break;
 			case R_ANAL_OP_TYPE_POP:
@@ -2509,7 +2471,8 @@ static void update_var_analysis(RAnalFunction *fcn, int align, ut64 from, ut64 t
 	}
 	for (cur_addr = from; cur_addr < to; cur_addr += opsz, len -= opsz) {
 		RAnalOp op;
-		int ret = r_anal_op (anal->coreb.core, &op, cur_addr, buf, len, R_ARCH_OP_MASK_ESIL | R_ARCH_OP_MASK_VAL);
+		// int ret = r_anal_op (anal->coreb.core, &op, cur_addr, buf, len, R_ARCH_OP_MASK_ESIL | R_ARCH_OP_MASK_VAL);
+		int ret = r_anal_op (anal, &op, cur_addr, buf, len, R_ARCH_OP_MASK_ESIL | R_ARCH_OP_MASK_VAL);
 		if (ret < 1 || op.size < 1) {
 			r_anal_op_fini (&op);
 			break;
