@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2014-2016 - jfrankowski, pancake */
+/* radare - LGPL - Copyright 2014-2024 - jfrankowski, pancake */
 /* credits to IDA for the flirt tech */
 /* original cpp code from Rheax <rheaxmascot@gmail.com> */
 /* thanks LemonBoy for the improved research on rheax original work */
@@ -384,7 +384,7 @@ static ut32 read_word(RFlirt *f) {
 }
 
 static ut16 read_max_2_bytes(RFlirt *f) {
-	ut16 r = read_byte (f);
+	const ut16 r = read_byte (f);
 	return (r & 0x80) ? ((r & 0x7f) << 8) + read_byte (f) : r;
 }
 
@@ -582,7 +582,7 @@ static bool module_match_buffer(RAnal *anal, const RFlirtModule *module, ut8 *b,
 							r_anal_function_add_block (next_module_function, block);
 						}
 						next_module_function->ninstr += fcn->ninstr;
-						r_anal_function_delete (fcn);
+						r_anal_function_delete (anal, fcn);
 					}
 				}
 				r_anal_function_resize (next_module_function, flirt_fcn_size);
@@ -814,6 +814,7 @@ static ut8 read_module_public_functions(RFlirt *f, RFlirtModule *module, ut8 *fl
 	ut8 current_byte;
 
 	module->public_functions = r_list_new ();
+	RFlirtFunction *function = NULL;
 
 	do {
 		if (f->version >= 9) {   // seems like version 9 introduced some larger offsets
@@ -827,7 +828,7 @@ static ut8 read_module_public_functions(RFlirt *f, RFlirtModule *module, ut8 *fl
 				goto beach;
 			}
 		}
-		RFlirtFunction *function = R_NEW0 (RFlirtFunction);
+		function = R_NEW0 (RFlirtFunction);
 		function->offset = offset;
 
 		current_byte = read_byte (f);
@@ -849,7 +850,6 @@ static ut8 read_module_public_functions(RFlirt *f, RFlirtModule *module, ut8 *fl
 #endif
 			current_byte = read_byte (f);
 			if (f->buf_eof || f->buf_err) {
-				free (function);
 				goto beach;
 			}
 		}
@@ -858,7 +858,6 @@ static ut8 read_module_public_functions(RFlirt *f, RFlirtModule *module, ut8 *fl
 			function->name[i] = current_byte;
 			current_byte = read_byte (f);
 			if (f->buf_eof || f->buf_err) {
-				free (function);
 				goto beach;
 			}
 		}
@@ -879,6 +878,7 @@ static ut8 read_module_public_functions(RFlirt *f, RFlirtModule *module, ut8 *fl
 	return true;
 
 beach:
+	free (function);
 	return false;
 }
 
@@ -1254,6 +1254,10 @@ static RFlirtNode *flirt_parse(RFlirt *f) {
 
 	r_buf_seek (f->b, 0, R_BUF_SET);
 	header = parse_header (f);
+	if (!header) {
+		R_LOG_ERROR ("Cannot parse flirt header");
+		goto beach;
+	}
 	if (version >= 6) {
 		idasig_v6_v7_t *v6_v7 = parse_v6_v7_header (f);
 		if (!v6_v7) {
@@ -1349,6 +1353,7 @@ beach:
 
 // if buf is a flirt signature, returns signature version, or zero
 R_API int r_sign_is_flirt(RBuffer *buf) {
+	R_RETURN_VAL_IF_FAIL (buf, 0);
 	int ret = 0;
 	idasig_v5_t *header = R_NEW0 (idasig_v5_t);
 	if (!header) {
@@ -1387,7 +1392,7 @@ static inline void flirt_free(RFlirt *f) {
 
 // dump a flirt signature content on screen
 R_API void r_sign_flirt_dump(const RAnal *anal, const char *flirt_file) {
-	RBuffer *flirt_buf = r_buf_new_slurp (flirt_file);
+	RBuffer *flirt_buf = r_buf_new_from_file (flirt_file);
 	if (!flirt_buf) {
 		R_LOG_ERROR ("Can't slurp %s", flirt_file);
 		return;
@@ -1405,7 +1410,7 @@ R_API void r_sign_flirt_dump(const RAnal *anal, const char *flirt_file) {
 
 // parses a flirt signature file and scan the currently opened file with it
 R_API void r_sign_flirt_scan(RAnal *anal, const char *flirt_file) {
-	RBuffer *flirt_buf = r_buf_new_slurp (flirt_file);
+	RBuffer *flirt_buf = r_buf_new_from_file (flirt_file);
 	if (!flirt_buf) {
 		R_LOG_ERROR ("Can't slurp %s", flirt_file);
 		return;
