@@ -1,9 +1,8 @@
-/* radare - LGPL - Copyright 2008-2023 - nibble, pancake, thestr4ng3r */
+/* radare - LGPL - Copyright 2008-2025 - nibble, pancake, thestr4ng3r */
 
-#include <r_anal.h>
 #include <r_core.h>
 
-static bool item_matches_filter(RAnalMetaItem *item, RAnalMetaType type, R_NULLABLE const RSpace *space) {
+static bool item_matches_filter(RAnalMetaItem *item, RAnalMetaType type, const RSpace * R_NULLABLE space) {
 	return (type == R_META_TYPE_ANY || item->type == type) && (!space || item->space == space);
 }
 
@@ -22,7 +21,7 @@ static bool find_node_cb(RIntervalNode *node, void *user) {
 	return true;
 }
 
-static RIntervalNode *find_node_at(RAnal *anal, RAnalMetaType type, R_NULLABLE const RSpace *space, ut64 addr) {
+static RIntervalNode *find_node_at(RAnal *anal, RAnalMetaType type, const RSpace * R_NULLABLE space, ut64 addr) {
 	FindCtx ctx = {
 		.type = type,
 		.space = space,
@@ -32,7 +31,7 @@ static RIntervalNode *find_node_at(RAnal *anal, RAnalMetaType type, R_NULLABLE c
 	return ctx.node;
 }
 
-static RIntervalNode *find_node_in(RAnal *anal, RAnalMetaType type, R_NULLABLE const RSpace *space, ut64 addr) {
+static RIntervalNode *find_node_in(RAnal *anal, RAnalMetaType type, const RSpace * R_NULLABLE space, ut64 addr) {
 	FindCtx ctx = {
 		.type = type,
 		.space = space,
@@ -57,7 +56,7 @@ static bool collect_nodes_cb(RIntervalNode *node, void *user) {
 	return true;
 }
 
-static RPVector *collect_nodes_at(RAnal *anal, RAnalMetaType type, R_NULLABLE const RSpace *space, ut64 addr) {
+static RPVector *collect_nodes_at(RAnal *anal, RAnalMetaType type, const RSpace * R_NULLABLE space, ut64 addr) {
 	CollectCtx ctx = {
 		.type = type,
 		.space = space,
@@ -70,7 +69,7 @@ static RPVector *collect_nodes_at(RAnal *anal, RAnalMetaType type, R_NULLABLE co
 	return ctx.result;
 }
 
-static RPVector *collect_nodes_in(RAnal *anal, RAnalMetaType type, R_NULLABLE const RSpace *space, ut64 addr) {
+static RPVector *collect_nodes_in(RAnal *anal, RAnalMetaType type, const RSpace * R_NULLABLE space, ut64 addr) {
 	CollectCtx ctx = {
 		.type = type,
 		.space = space,
@@ -83,7 +82,7 @@ static RPVector *collect_nodes_in(RAnal *anal, RAnalMetaType type, R_NULLABLE co
 	return ctx.result;
 }
 
-static RPVector *collect_nodes_intersect(RAnal *anal, RAnalMetaType type, R_NULLABLE const RSpace *space, ut64 start, ut64 end) {
+static RPVector *collect_nodes_intersect(RAnal *anal, RAnalMetaType type, const RSpace * R_NULLABLE space, ut64 start, ut64 end) {
 	CollectCtx ctx = {
 		.type = type,
 		.space = space,
@@ -96,7 +95,7 @@ static RPVector *collect_nodes_intersect(RAnal *anal, RAnalMetaType type, R_NULL
 	return ctx.result;
 }
 
-static bool meta_set(RAnal *a, RAnalMetaType type, int subtype, ut64 from, ut64 to, const char *str) {
+static bool meta_set(RAnal *a, RAnalMetaType type, int subtype, ut64 from, ut64 to, const char * R_NULLABLE str) {
 	if (to < from) {
 		return false;
 	}
@@ -118,7 +117,7 @@ static bool meta_set(RAnal *a, RAnalMetaType type, int subtype, ut64 from, ut64
 	} else {
 		item->str = NULL;
 	}
-	R_DIRTY (a);
+	R_DIRTY_SET (a);
 	if (!node) {
 		r_interval_tree_insert (&a->meta, from, to, item);
 	} else if (node->end != to) {
@@ -128,16 +127,18 @@ static bool meta_set(RAnal *a, RAnalMetaType type, int subtype, ut64 from, ut64
 }
 
 R_API bool r_meta_set_string(RAnal *a, RAnalMetaType type, ut64 addr, const char *s) {
+	R_RETURN_VAL_IF_FAIL (a && s, false);
 	return meta_set (a, type, 0, addr, addr, s);
 }
 
 R_API const char *r_meta_get_string(RAnal *a, RAnalMetaType type, ut64 addr) {
+	R_RETURN_VAL_IF_FAIL (a, NULL);
 	RIntervalNode *node = find_node_at (a, type, r_spaces_current (&a->meta_spaces), addr);
-	if (!node) {
-		return NULL;
+	if (node) {
+		RAnalMetaItem *item = node->data;
+		return item->str;
 	}
-	RAnalMetaItem *item = node->data;
-	return item->str;
+	return NULL;
 }
 
 static void del(RAnal *a, RAnalMetaType type, const RSpace *space, ut64 addr, ut64 size) {
@@ -173,10 +174,12 @@ static void del(RAnal *a, RAnalMetaType type, const RSpace *space, ut64 addr, ut
 }
 
 R_API void r_meta_del(RAnal *a, RAnalMetaType type, ut64 addr, ut64 size) {
+	R_RETURN_IF_FAIL (a);
 	del (a, type, r_spaces_current (&a->meta_spaces), addr, size);
 }
 
-R_API bool r_meta_set(RAnal *a, RAnalMetaType type, ut64 addr, ut64 size, const char *str) {
+R_API bool r_meta_set(RAnal *a, RAnalMetaType type, ut64 addr, ut64 size, const char * R_NULLABLE str) {
+	R_RETURN_VAL_IF_FAIL (a, false);
 	return r_meta_set_with_subtype (a, type, 0, addr, size, str);
 }
 
@@ -189,7 +192,8 @@ R_API bool r_meta_set_with_subtype(RAnal *m, RAnalMetaType type, int subtype, ut
 	return meta_set (m, type, subtype, addr, end, str);
 }
 
-R_API RAnalMetaItem *r_meta_get_at(RAnal *a, ut64 addr, RAnalMetaType type, R_OUT R_NULLABLE ut64 *size) {
+R_API RAnalMetaItem *r_meta_get_at(RAnal *a, ut64 addr, RAnalMetaType type, R_OUT ut64 * R_NULLABLE size) {
+	R_RETURN_VAL_IF_FAIL (a, NULL);
 	RIntervalNode *node = find_node_at (a, type, r_spaces_current (&a->meta_spaces), addr);
 	if (node && size) {
 		*size = r_meta_item_size (node->start, node->end);
@@ -198,14 +202,17 @@ R_API RAnalMetaItem *r_meta_get_at(RAnal *a, ut64 addr, RAnalMetaType type, R_OU
 }
 
 R_API RIntervalNode *r_meta_get_in(RAnal *a, ut64 addr, RAnalMetaType type) {
+	R_RETURN_VAL_IF_FAIL (a, NULL);
 	return find_node_in (a, type, r_spaces_current (&a->meta_spaces), addr);
 }
 
 R_API RPVector/*<RIntervalNode<RMetaItem> *>*/ *r_meta_get_all_at(RAnal *a, ut64 at) {
+	R_RETURN_VAL_IF_FAIL (a, NULL);
 	return collect_nodes_at (a, R_META_TYPE_ANY, r_spaces_current (&a->meta_spaces), at);
 }
 
 R_API RPVector *r_meta_get_all_in(RAnal *a, ut64 at, RAnalMetaType type) {
+	R_RETURN_VAL_IF_FAIL (a, NULL);
 	return collect_nodes_in (a, type, r_spaces_current (&a->meta_spaces), at);
 }
 
@@ -236,7 +243,7 @@ R_API const char *r_meta_type_tostring(int type) {
 	return "# unknown meta # ";
 }
 
-R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int rad, PJ *pj, bool show_full) {
+R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int rad, PJ *pj, RTable *t, bool show_full) {
 	R_RETURN_IF_FAIL (!(rad == 'j' && !pj)); // rad == 'j' => pj
 	char *pstr, *base64_str;
 	RCore *core = a->coreb.core;
@@ -297,9 +304,9 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int r
 				ut8 r = 0, g = 0, b = 0, A = 0;
 				const char *esc = strchr (d->str, '\x1b');
 				if (esc) {
-					r_cons_rgb_parse (esc, &r, &g, &b, &A);
+					r_str_html_rgbparse (esc, &r, &g, &b, &A);
 					char *rgb_str = r_cons_rgb_tostring (r, g, b);
-					base64_str = r_base64_encode_dyn (rgb_str, -1);
+					base64_str = r_base64_encode_dyn ((const ut8*)rgb_str, -1);
 					if (d->type == 's' && base64_str) {
 						pj_s (pj, base64_str);
 						free (base64_str);
@@ -312,7 +319,7 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int r
 				}
 			} else {
 				pj_k (pj, "name");
-				if (d->type == 's' && (base64_str = r_base64_encode_dyn (d->str, -1))) {
+				if (d->type == 's' && (base64_str = r_base64_encode_dyn ((const ut8*)d->str, -1))) {
 					pj_s (pj, base64_str);
 				} else {
 					pj_s (pj, str);
@@ -456,7 +463,7 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int r
 				{
 					ut8 r = 0, g = 0, b = 0, A = 0;
 					const char *esc = strchr (d->str, '\x1b');
-					r_cons_rgb_parse (esc, &r, &g, &b, &A);
+					r_str_html_rgbparse (esc, &r, &g, &b, &A);
 					a->cb_printf ("%s rgb:%02x%02x%02x @ 0x%08" PFMT64x "\n",
 						r_meta_type_tostring (d->type), r, g, b, start);
 					// TODO: d->size
@@ -483,24 +490,28 @@ R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int r
 	}
 }
 
-R_API void r_meta_print_list_at(RAnal *a, ut64 addr, int rad, const char *tq) {
+R_API void r_meta_print_list_at(RAnal *a, ut64 addr, int rad, const char *tq, RTable *t) {
+	R_RETURN_IF_FAIL (a);
 	RPVector *nodes = collect_nodes_at (a, R_META_TYPE_ANY, r_spaces_current (&a->meta_spaces), addr);
-	if (!nodes) {
-		return;
+	if (nodes) {
+		void **it;
+		r_pvector_foreach (nodes, it) {
+			RIntervalNode *node = *it;
+			size_t ns = r_meta_node_size (node);
+			r_meta_print (a, node->data, node->start, ns, rad, NULL, t, true);
+		}
+		r_pvector_free (nodes);
 	}
-	void **it;
-	r_pvector_foreach (nodes, it) {
-		RIntervalNode *node = *it;
-		r_meta_print (a, node->data, node->start, r_meta_node_size (node), rad, NULL, true);
-	}
-	r_pvector_free (nodes);
 }
 
-static void print_meta_list(RAnal *a, int type, int rad, ut64 addr, const char *tq) {
+static void print_meta_list(RAnal *a, int type, int rad, ut64 addr, const char *tq, RTable *t) {
+	RCore *core = a->coreb.core;
+	RCons *cons = core->cons;
 	PJ *pj = NULL;
-	RTable *t = NULL;
 	if (rad == ',') {
-		t = r_table_new ("meta");
+		if (!t) {
+			t = r_table_new ("meta");
+		}
 		RTableColumnType *s = r_table_type ("string");
 		RTableColumnType *n = r_table_type ("number");
 		r_table_add_column (t, n, "addr", 0);
@@ -541,10 +552,9 @@ static void print_meta_list(RAnal *a, int type, int rad, ut64 addr, const char *
 				r_meta_node_size (node),
 				type, name);
 		} else {
-			r_meta_print (a, item, node->start, r_meta_node_size (node), rad, pj, true);
+			r_meta_print (a, item, node->start, r_meta_node_size (node), rad, pj, t, true);
 		}
 	}
-
 beach:
 	if (t && tq) {
 		if (!r_table_query (t, tq)) {
@@ -556,25 +566,27 @@ beach:
 	if (!tq || !strstr (tq, "?")) {
 		if (t) {
 			char *s = r_table_tostring (t);
-			r_cons_printf ("%s\n", s);
+			r_cons_print (cons, s);
 			free (s);
 		} else if (pj) {
 			pj_end (pj);
-			r_cons_printf ("%s\n", pj_string (pj));
+			r_cons_printf (cons, "%s\n", pj_string (pj));
 		}
 	}
 	pj_free (pj);
 }
 
-R_API void r_meta_print_list_all(RAnal *a, int type, int rad, const char *tq) {
-	print_meta_list (a, type, rad, UT64_MAX, tq);
+// TODO: return char*
+R_API void r_meta_print_list_all(RAnal *a, int type, int rad, const char *tq, RTable *t) {
+	print_meta_list (a, type, rad, UT64_MAX, tq, t);
 }
 
-R_API void r_meta_print_list_in_function(RAnal *a, int type, int rad, ut64 addr, const char *tq) {
-	print_meta_list (a, type, rad, addr, tq);
+R_API void r_meta_print_list_in_function(RAnal *a, int type, int rad, ut64 addr, const char *tq, RTable *t) {
+	print_meta_list (a, type, rad, addr, tq, t);
 }
 
 R_API void r_meta_rebase(RAnal *anal, ut64 diff) {
+	R_RETURN_IF_FAIL (anal);
 	if (!diff) {
 		return;
 	}
@@ -623,6 +635,7 @@ R_API ut64 r_meta_get_size(RAnal *a, RAnalMetaType type) {
 }
 
 R_API int r_meta_space_count_for(RAnal *a, const RSpace *space) {
+	R_RETURN_VAL_IF_FAIL (a && space, 0);
 	int r = 0;
 	RIntervalTreeIter it;
 	RAnalMetaItem *item;
