@@ -300,12 +300,13 @@ static const char *type_tostring(RTypeInfoType type) {
 	case R_TYPEINFO_TYPE_VMI_CLASS:
 		return VMI_CLASS_TYPE_INFO_NAME;
 	default:
-		r_return_val_if_reached (CLASS_TYPE_INFO_NAME);
+		R_RETURN_VAL_IF_REACHED (CLASS_TYPE_INFO_NAME);
 	}
 	return NULL;
 }
 static void rtti_itanium_print_class_type_info(class_type_info *cti, const char *prefix) {
-	r_cons_printf ("%sType Info at 0x%08" PFMT64x ":\n"
+	RCons *cons = r_cons_singleton ();
+	r_cons_printf (cons, "%sType Info at 0x%08" PFMT64x ":\n"
 			"%s  Type Info type: %s\n"
 			"%s  Belongs to class vtable: 0x%08" PFMT64x "\n"
 			"%s  Reference to RTTI's type class: 0x%08" PFMT64x "\n"
@@ -337,12 +338,14 @@ static void rtti_itanium_print_class_type_info_json(class_type_info *cti) {
 	pj_kb (pj, "name_unique", cti->name_unique);
 	pj_end (pj);
 
-	r_cons_print (pj_string (pj));
+	RCons *cons = r_cons_singleton ();
+	r_cons_print (cons, pj_string (pj));
 	pj_free (pj);
 }
 
 static void rtti_itanium_print_vmi_class_type_info(vmi_class_type_info *vmi_cti, const char *prefix) {
-	r_cons_printf ("%sType Info at 0x%08" PFMT64x ":\n"
+	RCons *cons = r_cons_singleton ();
+	r_cons_printf (cons, "%sType Info at 0x%08" PFMT64x ":\n"
 			"%s  Type Info type: %s\n"
 			"%s  Belongs to class vtable: 0x%08" PFMT64x "\n"
 			"%s  Reference to RTTI's type class: 0x%08" PFMT64x "\n"
@@ -364,7 +367,7 @@ static void rtti_itanium_print_vmi_class_type_info(vmi_class_type_info *vmi_cti,
 
 	int i;
 	for (i = 0; i < vmi_cti->vmi_base_count; i++) {
-		r_cons_printf ("%s    Base class type descriptor address: 0x%08" PFMT64x "\n"
+		r_cons_printf (cons, "%s    Base class type descriptor address: 0x%08" PFMT64x "\n"
 			       "%s    Base class flags: 0x%" PFMT64x
 			       "\n",
 			prefix, vmi_cti->vmi_bases[i].base_class_addr,
@@ -399,12 +402,14 @@ static void rtti_itanium_print_vmi_class_type_info_json(vmi_class_type_info *vmi
 	pj_end (pj);
 	pj_end (pj);
 
-	r_cons_print (pj_string (pj));
+	RCons *cons = r_cons_singleton ();
+	r_cons_print (cons, pj_string (pj));
 	pj_free (pj);
 }
 
 static void rtti_itanium_print_si_class_type_info(si_class_type_info *si_cti, const char *prefix) {
-	r_cons_printf ("%sType Info at 0x%08" PFMT64x ":\n"
+	RCons *cons = r_cons_singleton ();
+	r_cons_printf (cons, "%sType Info at 0x%08" PFMT64x ":\n"
 			"%s  Type Info type: %s\n"
 			"%s  Belongs to class vtable: 0x%08" PFMT64x "\n"
 			"%s  Reference to RTTI's type class: 0x%08" PFMT64x "\n"
@@ -439,13 +444,14 @@ static void rtti_itanium_print_si_class_type_info_json(si_class_type_info *si_ct
 	pj_kn (pj, "ref_to_parent_type", si_cti->base_class_addr);
 	pj_end (pj);
 
-	r_cons_print (pj_string (pj));
+	RCons *cons = r_cons_singleton ();
+	r_cons_print (cons, pj_string (pj));
 	pj_free (pj);
 }
 
 static RTypeInfoType rtti_itanium_type_info_type_from_flag(RVTableContext *context, ut64 atAddress) {
 	RCore *core = context->anal->coreb.core;
-	r_return_val_if_fail (core, R_TYPEINFO_TYPE_CLASS);
+	R_RETURN_VAL_IF_FAIL (core, R_TYPEINFO_TYPE_CLASS);
 
 	// get the reloc flags
 	const RList *flags = context->anal->flb.get_list (core->flags, atAddress);
@@ -668,14 +674,14 @@ static class_type_info *rtti_itanium_type_info_new(RVTableContext *context, ut64
 	case R_TYPEINFO_TYPE_CLASS:
 		return rtti_itanium_class_type_info_new (context, rtti_addr, vtable_addr);
 	default:
-		r_return_val_if_reached (NULL);
+		R_RETURN_VAL_IF_REACHED (NULL);
 	}
 	return false;
 }
 
 static void rtti_itanium_type_info_free(void *info) {
 	class_type_info *cti = info;
-	if (!cti) {
+	if (R_UNLIKELY (!cti)) {
 		return;
 	}
 
@@ -690,7 +696,7 @@ static void rtti_itanium_type_info_free(void *info) {
 		rtti_itanium_class_type_info_free (cti);
 		return;
 	default:
-		r_return_if_reached ();
+		R_RETURN_IF_REACHED ();
 	}
 }
 
@@ -733,26 +739,21 @@ R_API bool r_anal_rtti_itanium_print_at_vtable(RVTableContext *context, ut64 add
 		return true;
 	default:
 		rtti_itanium_class_type_info_free (cti);
-		r_return_val_if_reached (false);
+		R_RETURN_VAL_IF_REACHED (false);
 	}
 	return false;
 }
 
 R_API char *r_anal_rtti_itanium_demangle_class_name(RVTableContext *context, const char *name) {
-	if (!name || !*name) {
+	if (R_STR_ISEMPTY (name)) {
 		return NULL;
 	}
-
-	char *result = NULL;
-
-	if (name[0] != '_') {
-		char *to_demangle = r_str_newf ("_Z%s", name);
-		result = context->anal->binb.demangle (NULL, "cxx", to_demangle, 0, false);
-		free (to_demangle);
-	} else {
-		result = context->anal->binb.demangle (NULL, "cxx", name, 0, false);
+	if (name[0] == '_') {
+		return context->anal->binb.demangle (NULL, "cxx", name, 0, false);
 	}
-
+	char *to_demangle = r_str_newf ("_Z%s", name);
+	char *result = context->anal->binb.demangle (NULL, "cxx", to_demangle, 0, false);
+	free (to_demangle);
 	return result;
 }
 
