@@ -5,7 +5,7 @@
 static void _ac_free(RArchConfig *cfg) {
 	if (cfg) {
 		free (cfg->decoder);
-		free (cfg->arch);
+//		free (cfg->arch);
 		free (cfg->abi);
 		free (cfg->cpu);
 		free (cfg->os);
@@ -19,13 +19,14 @@ R_API void r_arch_config_free(RArchConfig *r) {
 	}
 }
 
-R_API void r_arch_config_use(RArchConfig *config, R_NULLABLE const char *arch) {
+R_API void r_arch_config_use(RArchConfig *config, const char * R_NULLABLE arch) {
 	R_RETURN_IF_FAIL (config);
 	if (arch && !strcmp (arch, "null")) {
 		return;
 	}
-	// free (config->arch)
-	config->arch = R_STR_ISNOTEMPTY (arch) ? strdup (arch) : NULL;
+	if (arch) {
+		r_str_ncpy (config->arch, arch, sizeof (config->arch));
+	}
 }
 
 R_API bool r_arch_config_iseq(RArchConfig *a, RArchConfig *b) {
@@ -33,7 +34,7 @@ R_API bool r_arch_config_iseq(RArchConfig *a, RArchConfig *b) {
 	return false;
 }
 
-R_API void r_arch_config_set_cpu(RArchConfig *config, R_NULLABLE const char *cpu) {
+R_API void r_arch_config_set_cpu(RArchConfig *config, const char * R_NULLABLE cpu) {
 	R_RETURN_IF_FAIL (config);
 	// R_LOG_DEBUG ("RArch.CPU (%s)", cpu);
 	free (config->cpu);
@@ -73,12 +74,10 @@ R_API bool r_arch_config_set_syntax(RArchConfig *config, int syntax) {
 R_API RArchConfig *r_arch_config_clone(RArchConfig *c) {
 	R_RETURN_VAL_IF_FAIL (c, NULL);
 	RArchConfig *ac = R_NEW0 (RArchConfig);
-	if (R_LIKELY (ac)) {
-		ac->arch = R_STR_DUP (c->arch);
-		ac->abi = R_STR_DUP (c->abi);
-		ac->cpu = R_STR_DUP (c->cpu);
-		ac->os = R_STR_DUP (c->os);
-	}
+	r_str_ncpy (ac->arch, c->arch, sizeof (c->arch));
+	ac->abi = c->abi? strdup (c->abi): NULL;
+	ac->cpu = c->cpu? strdup (c->cpu): NULL;
+	ac->os = c->os? strdup (c->os): NULL;
 	return ac;
 }
 
@@ -87,14 +86,8 @@ R_API RArchConfig *r_arch_config_new(void) {
 	if (!ac) {
 		return NULL;
 	}
-	ac->arch = strdup (R_SYS_ARCH);
-#if R_SYS_BITS == R_SYS_BITS_32
-	ac->bits = 32;
-#elif R_SYS_BITS == R_SYS_BITS_64
-	ac->bits = 64;
-#else
-	ac->bits = 64;
-#endif
+	r_str_ncpy (ac->arch, R_SYS_ARCH, sizeof (ac->arch));
+	ac->bits = R_SYS_BITS_CHECK (R_SYS_BITS, 64)? 64: 32;
 	ac->bitshift = 0;
 	ac->syntax = R_ARCH_SYNTAX_INTEL;
 	r_ref_init (ac, &_ac_free);
