@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2012-2024 - pancake, condret */
+/* radare - LGPL - Copyright 2012-2025 - pancake, condret */
 
 #include <r_arch.h>
 #include "./gbdis.c"
@@ -57,7 +57,7 @@ static inline void gb_anal_esil_ccall(RAnalOp *op, const ut8 data) {
 		cond = 'C';
 		break;
 	}
-	if (op->cond == R_ANAL_COND_EQ) {
+	if (op->cond == R_ANAL_CONDTYPE_EQ) {
 		r_strbuf_setf (&op->esil, "%c,?{,2,sp,-=,pc,sp,=[2],%"PFMT64d",pc,:=,}", cond, (op->jump & 0xffff));
 	} else {
 		r_strbuf_setf (&op->esil, "%c,!,?{,2,sp,-=,pc,sp,=[2],%" PFMT64d ",pc,:=,}", cond, (op->jump & 0xffff));
@@ -75,7 +75,7 @@ static inline void gb_anal_esil_cret(RAnalOp *op, const ut8 data) {
 	} else {
 		cond = 'Z';
 	}
-	if (op->cond == R_ANAL_COND_EQ) {
+	if (op->cond == R_ANAL_CONDTYPE_EQ) {
 		r_strbuf_setf (&op->esil, "%c,?{,sp,[2],pc,:=,2,sp,+=,}", cond);
 	} else {
 		r_strbuf_setf (&op->esil, "%c,!,?{,sp,[2],pc,:=,2,sp,+=,}", cond);
@@ -94,7 +94,7 @@ static inline void gb_anal_esil_cjmp(RAnalOp *op, const ut8 data) {
 		default:
 			cond = 'C';
 	}
-	if (op->cond == R_ANAL_COND_EQ) {
+	if (op->cond == R_ANAL_CONDTYPE_EQ) {
 		r_strbuf_setf (&op->esil, "%c,?{,0x%"PFMT64x",pc,:=,}", cond, (op->jump & 0xffff));
 	} else {
 		r_strbuf_setf (&op->esil, "%c,!,?{,0x%"PFMT64x",pc,:=,}", cond, (op->jump & 0xffff));
@@ -271,9 +271,9 @@ static inline void gb_anal_cond(RAnalOp *op, const ut8 data) {
 	src = r_vector_push (&op->srcs, NULL);
 	src->imm = 1;
 	if (data & 0x8) {
-		op->cond = R_ANAL_COND_EQ;
+		op->cond = R_ANAL_CONDTYPE_EQ;
 	} else {
-		op->cond = R_ANAL_COND_NE;
+		op->cond = R_ANAL_CONDTYPE_NE;
 	}
 	switch (data) {
 	case 0x20:
@@ -685,9 +685,7 @@ static inline void gb_anal_cb_srl(RAnalOp *op, const ut8 data) {
 }
 
 static bool gb_custom_daa(REsil *esil) {
-	if (!esil || !esil->anal || !esil->anal->reg) {
-		return false;
-	}
+	R_RETURN_VAL_IF_FAIL (esil, false);
 	char *v = r_esil_pop (esil);
 	ut64 n;
 	if (!v || !r_esil_get_parm (esil, v, &n)) {
@@ -695,11 +693,17 @@ static bool gb_custom_daa(REsil *esil) {
 	}
 	R_FREE (v);
 	ut8 val = (ut8)n;
-	r_esil_reg_read (esil, "H", &n, NULL);
+	if (R_UNLIKELY (!r_esil_reg_read (esil, "H", &n, NULL))) {
+		return false;
+	}
 	const ut8 H = (ut8)n;
-	r_esil_reg_read (esil, "C", &n, NULL);
+	if (R_UNLIKELY (!r_esil_reg_read (esil, "C", &n, NULL))) {
+		return false;
+	}
 	const ut8 C = (ut8)n;
-	r_esil_reg_read (esil, "N", &n, NULL);
+	if (R_UNLIKELY (!r_esil_reg_read (esil, "N", &n, NULL))) {
+		return false;
+	}
 	if (n) {
 		if (C) {
 			val = (val - 0x60) & 0xff;
@@ -707,18 +711,18 @@ static bool gb_custom_daa(REsil *esil) {
 		if (H) {
 			val = (val - 0x06) & 0xff;
 		}
-	} else {
-		if (C || (val > 0x99)) {
-			val = (val + 0x60) & 0xff;
-		}
-		if (H || ((val & 0x0f) > 0x09)) {
-			val += 0x06;
-		};
+		return r_esil_pushnum (esil, val);
+	}
+	if (C || (val > 0x99)) {
+		val = (val + 0x60) & 0xff;
+	}
+	if (H || ((val & 0x0f) > 0x09)) {
+		val += 0x06;
 	}
 	return r_esil_pushnum (esil, val);
 }
 
-static bool decode(RArchSession *as, RAnalOp *op, RAnalOpMask mask) {
+static bool decode(RArchSession *as, RAnalOp *op, RArchDecodeMask mask) {
 	const ut64 addr = op->addr;
 	const ut8 *data = op->bytes;
 	const int len = op->size;
@@ -1504,37 +1508,15 @@ static char* regs(RArchSession *as) {
 
 static int esil_gb_init(REsil *esil) {
 	R_RETURN_VAL_IF_FAIL (esil, false);
-
-	// XXX esil-init shouldnt touch the registers or write into memory or antyhing like dat
-	GBUser *user = R_NEW0 (GBUser);
 	REsilOp *op = r_esil_get_op (esil, "}{");
-	r_esil_set_op (esil, "halt", op->code, 0, 0, R_ESIL_OP_TYPE_CUSTOM);
-	r_esil_set_op (esil, "stop", op->code, 0, 0, R_ESIL_OP_TYPE_CUSTOM);
-	r_esil_set_op (esil, "daa", gb_custom_daa, 1, 1, R_ESIL_OP_TYPE_MATH | R_ESIL_OP_TYPE_CUSTOM);
-	if (user) {
-		if (esil->anal) {
-			esil->anal->iob.read_at (esil->anal->iob.io, 0x147, &user->mbc_id, 1);
-			esil->anal->iob.read_at (esil->anal->iob.io, 0x148, &user->romsz_id, 1);
-			esil->anal->iob.read_at (esil->anal->iob.io, 0x149, &user->ramsz_id, 1);
-			if (esil->anal->reg) { //initial values
-				r_reg_setv (esil->anal->reg, "mpc", 0x100);
-				r_reg_setv (esil->anal->reg, "sp", 0xfffe);
-				r_reg_setv (esil->anal->reg, "af", 0x01b0);
-				r_reg_setv (esil->anal->reg, "bc", 0x0013);
-				r_reg_setv (esil->anal->reg, "de", 0x00d8);
-				r_reg_setv (esil->anal->reg, "hl", 0x014d);
-				r_reg_setv (esil->anal->reg, "ime", true);
-			}
-		}
-		esil->cb.user = user;
-	}
+	r_esil_set_op (esil, "halt", op->code, 0, 0, R_ESIL_OP_TYPE_CUSTOM, NULL);
+	r_esil_set_op (esil, "stop", op->code, 0, 0, R_ESIL_OP_TYPE_CUSTOM, NULL);
+	r_esil_set_op (esil, "daa", gb_custom_daa, 1, 1, R_ESIL_OP_TYPE_MATH | R_ESIL_OP_TYPE_CUSTOM, NULL);
 	return true;
 }
 
 static int esil_gb_fini(REsil *esil) {
 	R_RETURN_VAL_IF_FAIL (esil, false);
-
-	R_FREE (esil->cb.user);
 	return true;
 }
 
@@ -1574,7 +1556,7 @@ const RArchPlugin r_arch_plugin_gb = {
 	.meta = {
 		.name = "gb",
 		.author = "condret",
-		.desc = "Gameboy CPU code analysis plugin",
+		.desc = "Gameboy CPU (modified Z80)",
 		.license = "LGPL-3.0-only",
 	},
 	.arch = "z80",
