@@ -353,7 +353,7 @@ static bool wasm_encode(RArchSession *s, RAnalOp *op, RArchEncodeMask mask) {
 }
 
 // analyzes the wasm opcode.
-static bool wasm_decode(RArchSession *s, RAnalOp *op, RAnalOpMask mask) {
+static bool wasm_decode(RArchSession *s, RAnalOp *op, RArchDecodeMask mask) {
 	R_RETURN_VAL_IF_FAIL (s && op, false);
 	WasmOp wop = {{0}};
 	const bool txt = mask & R_ARCH_OP_MASK_DISASM;
@@ -630,7 +630,7 @@ static void _kv_free(HtUPKv *kv) {
 }
 
 static bool cache_new(RArchSession *s) {
-	R_RETURN_VAL_IF_FAIL (!s->user, false);
+	R_RETURN_VAL_IF_FAIL (s, false);
 	s->user = ht_up_new (NULL, _kv_free, NULL);
 	return s->user? true: false;
 }
@@ -645,14 +645,14 @@ const RArchPlugin r_arch_plugin_wasm = {
 	.meta = {
 		.name = "wasm",
 		.author = "pancake,xvilka,condret",
-		.desc = "WebAssembly analysis plugin",
+		.desc = "WebAssembly bytecode",
 		.license = "LGPL-3.0-only",
 	},
 	.arch = "wasm",
-	.bits = R_SYS_BITS_PACK1 (64),
+	.bits = R_SYS_BITS_PACK2 (32,64),
 	.regs = wasm_regs,
-	.decode = &wasm_decode,
-	.encode = &wasm_encode,
+	.decode = wasm_decode,
+	.encode = wasm_encode,
 	.init = cache_new,
 	.fini = cache_clean,
 };
