@@ -36,7 +36,7 @@ static bool check_swift(RBinSymbol *sym) {
 
 static bool check_golang(RBinSymbol *sym) {
 	const char *sym_name = r_bin_name_tostring (sym->name);
-	return r_str_startswith (sym_name, "go.");
+	return r_str_startswith (sym_name, "go:");
 }
 
 static inline bool is_cxx_symbol(const char *name) {
@@ -276,7 +276,7 @@ R_API int r_bin_load_languages(RBinFile *bf) {
 }
 
 // if its ipi no need to be prefixed with r_
-R_IPI int r_bin_lang_type(R_NULLABLE RBinFile *bf, R_NULLABLE const char *def, R_NULLABLE const char *sym) {
+R_IPI int r_bin_lang_type(RBinFile * R_NULLABLE bf, const char * R_NULLABLE def, const char * R_NULLABLE sym) {
 	int type = R_BIN_LANG_NONE;
 	if (sym) {
 		if (r_str_startswith (sym, "__")) {
