@@ -43,6 +43,9 @@ static void object_delete_items(RBinObject *o) {
 	r_list_free (o->strings);
 	ht_up_free (o->strings_db);
 
+	if (!RVecRBinImport_empty (&o->imports_vec)) {
+		RVecRBinImport_fini (&o->imports_vec);
+	}
 	if (!RVecRBinSymbol_empty (&o->symbols_vec)) {
 		RVecRBinSymbol_fini (&o->symbols_vec);
 		if (o->symbols) {
@@ -209,23 +212,17 @@ R_IPI RBinObject *r_bin_object_new(RBinFile *bf, RBinPlugin *plugin, ut64 basead
 	bo->plugin = plugin;
 	bo->loadaddr = loadaddr != UT64_MAX ? loadaddr : 0;
 	RVecRBinSymbol_init (&bo->symbols_vec);
-	bo->pool = r_strpool_new (0);
+	bo->pool = r_strpool_new ();
 	bf->bo = bo;
 
 	if (plugin && plugin->load) {
 		if (!plugin->load (bf, bf->buf, loadaddr)) {
 			R_LOG_DEBUG ("load failed for %s plugin", plugin->meta.name);
-			sdb_free (bo->kv);
-			free (bo);
-			bf->bo = NULL;
-			return NULL;
+			goto fail;
 		}
 	} else {
 		R_LOG_WARN ("Plugin %s should implement load method", plugin->meta.name);
-		sdb_free (bo->kv);
-		free (bo);
-		bf->bo = NULL;
-		return NULL;
+		goto fail;
 	}
 
 	// XXX - object size can't be set here and needs to be set where where
@@ -260,6 +257,23 @@ R_IPI RBinObject *r_bin_object_new(RBinFile *bf, RBinPlugin *plugin, ut64 basead
 		bf->sdb->refs++;
 	}
 	return bo;
+
+fail:
+	r_strpool_free (bo->pool);
+	if (!RVecRBinSymbol_empty (&bo->symbols_vec)) {
+		RVecRBinSymbol_fini (&bo->symbols_vec);
+		if (bo->symbols) {
+			bo->symbols->free = NULL;
+		}
+	}
+	ht_pp_free (bo->methods_ht);
+	ht_pp_free (bo->classes_ht);
+	r_list_free (bo->classes);
+	sdb_free (bo->kv);
+	ht_up_free (bo->strings_db);
+	free (bo);
+	bf->bo = NULL;
+	return NULL;
 }
 
 static bool filter_classes(RBinFile *bf, RList *list) {
@@ -337,10 +351,15 @@ R_API int r_bin_object_set_items(RBinFile *bf, RBinObject *bo) {
 	bool isSwift = false;
 	RBin *bin = bf->rbin;
 	RBinPlugin *p = bo->plugin;
-	int minlen = (bf->rbin->minstrlen > 0) ? bf->rbin->minstrlen : p->minstrlen;
+	int minlen = (bf->rbin->options.minstrlen > 0) ? bf->rbin->options.minstrlen : p->minstrlen;
 	bf->bo = bo;
 
-	bo->info = p->info? p->info (bf): NULL;
+	if (p->info) {
+		r_bin_info_free (bo->info);
+		bo->info = p->info (bf);
+	} else {
+		bo->info = NULL;
+	}
 	if (bo->info && bo->info->type) {
 		if (strstr (bo->info->type, "CORE")) {
 			if (p->regstate) {
@@ -406,7 +425,12 @@ R_API int r_bin_object_set_items(RBinFile *bf, RBinObject *bo) {
 			}
 		}
 	}
-	bo->info = p->info? p->info (bf): NULL;
+	if (p->info) {
+		r_bin_info_free (bo->info);
+		bo->info = p->info (bf);
+	} else {
+		bo->info = NULL;
+	}
 	if (p->libs) {
 		bo->libs = p->libs (bf);
 	}
@@ -422,12 +446,12 @@ R_API int r_bin_object_set_items(RBinFile *bf, RBinObject *bo) {
 	}
 	if (bin->filter_rules & (R_BIN_REQ_RELOCS | R_BIN_REQ_IMPORTS)) {
 		if (p->relocs) {
-			const RList *l = p->relocs (bf); // XXX this is an internal list (should be a vector), and shouldnt be freed by the caller
+			RList *l = (RList *)p->relocs (bf); // XXX this is an internal list (should be a vector), and shouldnt be freed by the caller
 			if (l) {
 				REBASE_PADDR (bo, l, RBinReloc);
 				bo->relocs = list2rbtree ((RList*)l);
-				// l->free = NULL;
-				// r_list_free (l);
+				l->free = NULL; // may leak or crash
+				r_list_free (l);
 			}
 		}
 	}
@@ -435,7 +459,7 @@ R_API int r_bin_object_set_items(RBinFile *bf, RBinObject *bo) {
 		bo->strings = p->strings
 			? p->strings (bf)
 			: r_bin_file_get_strings (bf, minlen, 0, bf->rawstr);
-		if (bin->debase64) {
+		if (bin->options.debase64) {
 			r_bin_object_filter_strings (bo);
 		}
 		REBASE_PADDR (bo, bo->strings, RBinString);
@@ -578,11 +602,11 @@ R_IPI void r_bin_object_filter_strings(RBinObject *bo) {
 	RBinString *ptr;
 	RListIter *iter;
 	r_list_foreach (strings, iter, ptr) {
-		char *dec = (char *)r_base64_decode_dyn (ptr->string, -1);
+		char *dec = (char *)r_base64_decode_dyn ((const char *)ptr->string, -1, NULL);
 		if (dec) {
 			char *s = ptr->string;
 			for (;;) {
-				char *dec2 = (char *)r_base64_decode_dyn (s, -1);
+				char *dec2 = (char *)r_base64_decode_dyn ((const char *)s, -1, NULL);
 				if (!dec2) {
 					break;
 				}
