@@ -1,9 +1,8 @@
-/* radare - LGPL - Copyright 2016 - Oscar Salvador */
+/* radare - LGPL - Copyright 2016-2022 - Oscar Salvador */
 
 #ifndef BFLT_H
 #define BFLT_H
 
-#include <r_types.h>
 #include <r_util.h>
 #include <r_lib.h>
 #include <r_bin.h>
@@ -16,8 +15,8 @@
 #define FLAT_FLAG_GZDATA	0x8	/* only data/relocs are compressed (for XIP) */
 #define FLAT_FLAG_KTRACE	0x10	/* output useful kernel trace for debugging */
 
-struct bflt_hdr {
-	char magic[4];
+typedef struct bflt_hdr {
+	ut8 magic[4];
 	ut32 rev;
 	ut32 entry;
 	ut32 data_start;
@@ -29,30 +28,29 @@ struct bflt_hdr {
 	ut32 flags;
 	ut32 build_date;
 	ut32 filler[5];
-};
+} RBinBfltHeader;
 
-//typedef reloc_struct_t
-
-struct reloc_struct_t {
+typedef struct reloc_struct_t {
 	ut32 addr_to_patch;
 	ut32 data_offset;
-};
+} RBinBfltReloc;
 
-struct r_bin_bflt_obj {
-	struct bflt_hdr *hdr;
-	struct reloc_struct_t *reloc_table;
-	struct reloc_struct_t *got_table;
+typedef struct r_bin_bflt_obj {
+	RBinBfltHeader *hdr;
+	RBinBfltReloc *reloc_table;
+	RBinBfltReloc *got_table;
+	RList *relocs_list;
 	RBuffer *b;
 	ut8 endian;
 	size_t size;
-	uint32_t n_got;
-};
+	ut32 n_got;
+} RBinBfltObj;
 
-#define BFLT_HDR_SIZE		sizeof (struct bflt_hdr)
-#define VALID_GOT_ENTRY(x)	(x != 0xFFFFFFFF)
+#define BFLT_HDR_SIZE sizeof (RBinBfltHeader)
+#define VALID_GOT_ENTRY(x) (x != UT32_MAX)
 
-RBinAddr *r_bflt_get_entry(struct r_bin_bflt_obj *bin);
-struct r_bin_bflt_obj *r_bin_bflt_new_buf(RBuffer *buf);
-void r_bin_bflt_free (struct r_bin_bflt_obj *obj);
+R_IPI RBinAddr *r_bflt_get_entry(RBinBfltObj *bin);
+R_IPI RBinBfltObj *r_bin_bflt_new_buf(RBuffer *buf);
+R_IPI void r_bin_bflt_free(RBinBfltObj *obj);
 
 #endif
