@@ -1,7 +1,4 @@
-/* radare2 - LGPL - Copyright 2020 - abcSup */
-
-#include <r_types.h>
-#include <r_util.h>
+/* radare2 - LGPL - Copyright 2020-2024 - abcSup */
 
 #include "dmp64.h"
 
@@ -9,12 +6,12 @@ static int r_bin_dmp64_init_memory_runs(struct r_bin_dmp64_obj_t *obj) {
 	int i, j;
 	dmp64_p_memory_desc *mem_desc = &obj->header->PhysicalMemoryBlockBuffer;
 	if (!memcmp (mem_desc, DMP_UNUSED_MAGIC, 4)) {
-		eprintf ("Warning: Invalid PhysicalMemoryDescriptor\n");
+		R_LOG_WARN ("Invalid PhysicalMemoryDescriptor");
 		return false;
 	}
 	ut64 num_runs = mem_desc->NumberOfRuns;
 	if (num_runs * sizeof (dmp_p_memory_run) >= r_offsetof (dmp64_header, ContextRecord)) {
-		eprintf ("Warning: Invalid PhysicalMemoryDescriptor\n");
+		R_LOG_WARN ("Invalid PhysicalMemoryDescriptor");
 		return false;
 	}
 	obj->pages = r_list_newf (free);
@@ -24,7 +21,7 @@ static int r_bin_dmp64_init_memory_runs(struct r_bin_dmp64_obj_t *obj) {
 	dmp_p_memory_run *runs = calloc (num_runs, sizeof (dmp_p_memory_run));
 	ut64 num_runs_offset = r_offsetof (dmp64_header, PhysicalMemoryBlockBuffer) + r_offsetof (dmp64_p_memory_desc, NumberOfRuns);
 	if (r_buf_read_at (obj->b, num_runs_offset, (ut8*)runs, num_runs * sizeof (dmp_p_memory_run)) < 0) {
-		eprintf ("Warning: read memory runs\n");
+		R_LOG_WARN ("read memory runs");
 		free (runs);
 		return false;
 	};
@@ -39,14 +36,14 @@ static int r_bin_dmp64_init_memory_runs(struct r_bin_dmp64_obj_t *obj) {
 				free (runs);
 				return false;
 			}
-			page->start = (run->BasePage + j) * DMP_PAGE_SIZE ;
+			page->start = (run->BasePage + j) * DMP_PAGE_SIZE;
 			page->file_offset = base + num_page * DMP_PAGE_SIZE;
 			r_list_append (obj->pages, page);
 			num_page++;
 		}
 	}
 	if (mem_desc->NumberOfPages != num_page) {
-		eprintf ("Warning: Number of Pages not matches\n");
+		R_LOG_WARN ("Number of Pages not matches");
 	}
 
 	free (runs);
@@ -54,14 +51,40 @@ static int r_bin_dmp64_init_memory_runs(struct r_bin_dmp64_obj_t *obj) {
 }
 
 static int r_bin_dmp64_init_header(struct r_bin_dmp64_obj_t *obj) {
-	if (!(obj->header = R_NEW0 (dmp64_header))) {
+	ut8 buf[sizeof (dmp64_header)];
+	if (r_buf_read_at (obj->b, 0, buf, sizeof (buf)) < 0) {
+		R_LOG_WARN ("read header");
+		return false;
+	}
+	obj->header = R_NEW0 (dmp64_header);
+	if (!obj->header) {
 		r_sys_perror ("R_NEW0 (header)");
 		return false;
 	}
-	if (r_buf_read_at (obj->b, 0, (ut8*)obj->header, sizeof (dmp64_header)) < 0) {
-		eprintf ("Warning: read header\n");
-		return false;
-	}
+	memcpy (obj->header, buf, sizeof (buf));
+#define DMPREAD(x) obj->header->x = r_read_le32 (buf + r_offsetof (dmp64_header, x))
+#define DMPREAD_64(x) obj->header->x = r_read_le64 (buf + r_offsetof (dmp64_header, x))
+	DMPREAD (MajorVersion);
+	DMPREAD (MinorVersion);
+	DMPREAD_64 (DirectoryTableBase);
+	DMPREAD_64 (PfnDataBase);
+	DMPREAD_64 (PsLoadedModuleList);
+	DMPREAD_64 (PsActiveProcessHead);
+	DMPREAD (MachineImageType);
+	DMPREAD (NumberProcessors);
+	DMPREAD (BugCheckCode);
+	DMPREAD_64 (KdDebuggerDataBlock);
+	DMPREAD (DumpType);
+	DMPREAD (MiniDumpFields);
+	DMPREAD (SecondaryDataState);
+	DMPREAD (ProductType);
+	DMPREAD (SuiteMask);
+	DMPREAD (WriterStatus);
+	DMPREAD_64 (RequiredDumpSpace);
+	DMPREAD_64 (SystemUpTime);
+	DMPREAD_64 (SystemTime);
+#undef DMPREAD_64
+#undef DMPREAD
 	obj->dtb = obj->header->DirectoryTableBase;
 
 	return true;
@@ -77,17 +100,20 @@ static int r_bin_dmp64_init_bmp_pages(struct r_bin_dmp64_obj_t *obj) {
 		return false;
 	}
 	ut64 paddr_base = obj->bmp_header->FirstPage;
-	ut64 num_pages = obj->bmp_header->Pages;
+	int num_pages = obj->bmp_header->Pages;
+	if (num_pages < 1) {
+		return false;
+	}
 	RBitmap *bitmap = r_bitmap_new (num_pages);
 	r_bitmap_set_bytes (bitmap, obj->bitmap, num_pages / 8);
 
 	ut64 num_bitset = 0;
-	for(i = 0; i < num_pages; i++) {
+	for (i = 0; i < num_pages; i++) {
 		if (!r_bitmap_test(bitmap, i)) {
 			continue;
 		}
 		dmp_page_desc *page = R_NEW0 (dmp_page_desc);
-		if (!page) {
+		if (R_UNLIKELY (!page)) {
 			return false;
 		}
 		page->start = i * DMP_PAGE_SIZE;
@@ -96,7 +122,7 @@ static int r_bin_dmp64_init_bmp_pages(struct r_bin_dmp64_obj_t *obj) {
 		num_bitset++;
 	}
 	if (obj->bmp_header->TotalPresentPages != num_bitset) {
-		eprintf ("Warning: TotalPresentPages not matched\n");
+		R_LOG_WARN ("TotalPresentPages not matched");
 		return false;
 	}
 
@@ -109,18 +135,26 @@ static int r_bin_dmp64_init_bmp_header(struct r_bin_dmp64_obj_t *obj) {
 		r_sys_perror ("R_NEW0 (dmp_bmp_header)");
 		return false;
 	}
-	if (r_buf_read_at (obj->b, sizeof (dmp64_header), (ut8*)obj->bmp_header, offsetof (dmp_bmp_header, Bitmap)) < 0) {
-		eprintf ("Warning: read bmp_header\n");
+	if (r_buf_read_at (obj->b, sizeof (dmp64_header), (ut8*)obj->bmp_header,
+			r_offsetof (dmp_bmp_header, Bitmap)) < 0) {
+		R_LOG_WARN ("read bmp_header");
 		return false;
 	}
+	obj->bmp_header->FirstPage = r_read_le64 (&obj->bmp_header->FirstPage);
+	obj->bmp_header->TotalPresentPages = r_read_le64 (&obj->bmp_header->TotalPresentPages);
+	obj->bmp_header->Pages = r_read_le64 (&obj->bmp_header->Pages);
 	if (!!memcmp (obj->bmp_header, DMP_BMP_MAGIC, 8)) {
-		eprintf ("Warning: Invalid Bitmap Magic\n");
+		R_LOG_WARN ("Invalid Bitmap Magic");
+		return false;
+	}
+	int bitmapsize = obj->bmp_header->Pages / 8;
+	if (bitmapsize < 1) {
+		R_LOG_WARN ("Invalid Bitmap Size");
 		return false;
 	}
-	ut64 bitmapsize = obj->bmp_header->Pages / 8;
 	obj->bitmap = calloc (1, bitmapsize);
 	if (r_buf_read_at (obj->b, sizeof (dmp64_header) + offsetof (dmp_bmp_header, Bitmap), obj->bitmap, bitmapsize) < 0) {
-		eprintf ("Warning: read bitmap\n");
+		R_LOG_WARN ("read bitmap");
 		return false;
 	}
 
@@ -129,7 +163,7 @@ static int r_bin_dmp64_init_bmp_header(struct r_bin_dmp64_obj_t *obj) {
 
 static int r_bin_dmp64_init(struct r_bin_dmp64_obj_t *obj) {
 	if (!r_bin_dmp64_init_header (obj)) {
-		eprintf ("Warning: Invalid Kernel Dump x64 Format\n");
+		R_LOG_WARN ("Invalid Kernel Dump x64 Format");
 		return false;
 	}
 	switch (obj->header->DumpType) {
@@ -148,11 +182,10 @@ static int r_bin_dmp64_init(struct r_bin_dmp64_obj_t *obj) {
 	return true;
 }
 
-void r_bin_dmp64_free(struct r_bin_dmp64_obj_t *obj) {
+R_IPI void r_bin_dmp64_free(struct r_bin_dmp64_obj_t *obj) {
 	if (!obj) {
 		return;
 	}
-
 	r_buf_free (obj->b);
 	obj->b = NULL;
 	free (obj->header);
@@ -163,7 +196,7 @@ void r_bin_dmp64_free(struct r_bin_dmp64_obj_t *obj) {
 	free (obj);
 }
 
-struct r_bin_dmp64_obj_t *r_bin_dmp64_new_buf(RBuffer* buf) {
+R_IPI struct r_bin_dmp64_obj_t *r_bin_dmp64_new_buf(RBuffer* buf) {
 	struct r_bin_dmp64_obj_t *obj = R_NEW0 (struct r_bin_dmp64_obj_t);
 	if (!obj) {
 		return NULL;
