@@ -5,26 +5,27 @@
 
 #include "mdmp_pe.h"
 
-static void PE_(add_tls_callbacks)(struct PE_(r_bin_pe_obj_t) * bin, RList *list) {
+static void PE_(add_tls_callbacks)(struct PE_(r_bin_pe_obj_t) *bin, RList *list) {
+	r_strf_buffer (64);
 	char *key;
 	int count = 0;
 	PE_DWord haddr, paddr, vaddr;
 	RBinAddr *ptr = NULL;
 
 	do {
-		key = sdb_fmt ("pe.tls_callback%d_paddr", count);
+		key = r_strf ("pe.tls_callback%d_paddr", count);
 		paddr = sdb_num_get (bin->kv, key, 0);
 		if (!paddr) {
 			break;
 		}
 
-		key = sdb_fmt ("pe.tls_callback%d_vaddr", count);
+		key = r_strf ("pe.tls_callback%d_vaddr", count);
 		vaddr = sdb_num_get (bin->kv, key, 0);
 		if (!vaddr) {
 			break;
 		}
 
-		key = sdb_fmt ("pe.tls_callback%d_haddr", count);
+		key = r_strf ("pe.tls_callback%d_haddr", count);
 		haddr = sdb_num_get (bin->kv, key, 0);
 		if (!haddr) {
 			break;
@@ -74,11 +75,11 @@ RList *PE_(r_bin_mdmp_pe_get_entrypoint) (struct PE_(r_bin_mdmp_pe_bin) * pe_bin
 	return ret;
 }
 
-static void filter_import(ut8 *n) {
-	int I;
-	for (I = 0; n[I]; I++) {
-		if (n[I] < 30 || n[I] >= 0x7f) {
-			n[I] = 0;
+static inline void filter_import(ut8 *n) {
+	int i;
+	for (i = 0; n[i]; i++) {
+		if (n[i] < 30 || n[i] >= 0x7f) {
+			n[i] = 0;
 			break;
 		}
 	}
@@ -109,7 +110,7 @@ RList *PE_(r_bin_mdmp_pe_get_imports) (struct PE_(r_bin_mdmp_pe_bin) * pe_bin) {
 			break;
 		}
 		filter_import (imports[i].name);
-		ptr->name = strdup ((const char *)imports[i].name);
+		ptr->name = r_bin_name_new ((const char *)imports[i].name);
 		ptr->libname = *imports[i].libname ? strdup ((const char *)imports[i].libname) : NULL;
 		ptr->bind = "NONE";
 		ptr->type = R_BIN_TYPE_FUNC_STR;
@@ -229,7 +230,7 @@ RList *PE_(r_bin_mdmp_pe_get_symbols) (RBin *rbin, struct PE_(r_bin_mdmp_pe_bin)
 			if (offset > pe_bin->vaddr) {
 				offset -= pe_bin->vaddr;
 			}
-			ptr->name = strdup ((char *)symbols[i].name);
+			ptr->name = r_bin_name_new ((const char *)symbols[i].name);
 			ptr->libname = *symbols[i].libname ? strdup ((char *)symbols[i].libname) : NULL;
 			ptr->forwarder = r_str_constpool_get (&rbin->constpool, (char *)symbols[i].forwarder);
 			ptr->bind = R_BIN_BIND_GLOBAL_STR;
@@ -253,7 +254,7 @@ RList *PE_(r_bin_mdmp_pe_get_symbols) (RBin *rbin, struct PE_(r_bin_mdmp_pe_bin)
 			if (offset > pe_bin->vaddr) {
 				offset -= pe_bin->vaddr;
 			}
-			ptr->name = strdup ((const char *)imports[i].name);
+			ptr->name = r_bin_name_new ((const char *)imports[i].name);
 			ptr->libname = *imports[i].libname ? strdup ((const char *)imports[i].libname) : NULL;
 			ptr->is_imported = true;
 			ptr->bind = "NONE";
