@@ -1,6 +1,6 @@
 /* radare - LGPL - 2015 - maijin */
 
-//CPU_memory_map: http://wiki.nesdev.com/w/index.php/CPU_memory_map
+//CPU_memory_map: https://wiki.nesdev.com/w/index.php/CPU_memory_map
 
 #ifndef _NES_H
 #define _NES_H
@@ -35,7 +35,10 @@
 #define ROM_START_ADDRESS                   0x8000
 #define ROM_SIZE                            0x8000
 
-#define NMI_VECTOR_START_ADDRESS            0xFFFA
+#define ROM_MIRROR_ADDRESS                  0xC000
+#define ROM_MIRROR_SIZE 0x8000
+
+#define NMI_VECTOR_START_ADDRESS 0xFFFA
 #define RESET_VECTOR_START_ADDRESS          0xFFFC
 #define IRQ_VECTOR_START_ADDRESS            0xFFFE
 
