@@ -1,7 +1,8 @@
-/* radare - MIT - 2024 - pancake */
+/* radare - MIT - 2024-2025 - pancake */
 
 // the info loaded here is never updated maybe we should have a way to refresh it
 #include <r_bin.h>
+#include <r_core.h>
 #include <r_io.h>
 
 static ut64 baddr(RBinFile *bf) {
@@ -24,7 +25,9 @@ static char *iocmd(RBinFile *bf, const char *s) {
 	}
 	char *res = r_io_system (io, s);
 	if (!res) {
-		const char *buffer = r_cons_get_buffer ();
+		RCore *core = io->coreb.core;
+		RCons *cons = core->cons;
+		const char *buffer = r_cons_get_buffer (cons, NULL);
 		if (buffer != NULL) {
 			res = strdup (buffer);
 		}
@@ -33,13 +36,8 @@ static char *iocmd(RBinFile *bf, const char *s) {
 }
 
 static RBinInfo *info(RBinFile *bf) {
-	char *res = iocmd (bf, "i");
-	eprintf ("-->(i) %s\n", res);
-	free (res);
-	RBinInfo *ret = NULL;
-	if (!(ret = R_NEW0 (RBinInfo))) {
-		return NULL;
-	}
+	free (iocmd (bf, "i"));
+	RBinInfo *ret = R_NEW0 (RBinInfo);
 	ret->file = strdup (bf->file);
 	ret->type = strdup ("IO");
 	ret->machine = strdup ("IO");
@@ -68,9 +66,7 @@ static void addsym(RList *ret, const char *name, ut64 addr) {
 #endif
 
 static RList *symbols(RBinFile *bf) {
-	char *res = iocmd (bf, "is");
-	eprintf ("-->(is) %s\n", res);
-	free (res);
+	free (iocmd (bf, "is"));
 	RList *ret = r_list_newf (free);
 #if 0
 	addsym (ret, "rom_start", r_read_be32 (&hdr.RomStart));
@@ -79,9 +75,7 @@ static RList *symbols(RBinFile *bf) {
 }
 
 static RList *sections(RBinFile *bf) {
-	char *res = iocmd (bf, "iS");
-	eprintf ("-->(iS) %s\n", res);
-	free (res);
+	free (iocmd (bf, "iS"));
 	RList *ret = r_list_new ();
 #if 0
 	RBinSection *ptr;
@@ -127,11 +121,13 @@ static RList *sections(RBinFile *bf) {
 static RList *entries(RBinFile *bf) {
 	RList *ret = r_list_newf (free);
 	char *res = iocmd (bf, "ie");
-	ut64 entry0 = r_num_get (NULL, res);
-	free (res);
-	RBinAddr *ptr = R_NEW0 (RBinAddr);
-	ptr->paddr = ptr->vaddr = entry0;
-	r_list_append (ret, ptr);
+	if (res) {
+		ut64 entry0 = r_num_get (NULL, res);
+		free (res);
+		RBinAddr *ptr = R_NEW0 (RBinAddr);
+		ptr->paddr = ptr->vaddr = entry0;
+		r_list_append (ret, ptr);
+	}
 	return ret;
 }
 
@@ -139,7 +135,7 @@ RBinPlugin r_bin_plugin_io = {
 	.meta = {
 		.name = "io",
 		.author = "pancake",
-		.desc = "bin plugin using the io interface",
+		.desc = "Use IO plugins for RBin",
 		.license = "MIT",
 	},
 	.load = &load,
