@@ -436,7 +436,7 @@ static void header(RBinFile *bf) {
 RBinPlugin r_bin_plugin_pe = {
 	.meta = {
 		.name = "pe",
-		.desc = "PE bin plugin",
+		.desc = "Portable Executable for 32bit",
 		.author = "pancake",
 		.license = "LGPL-3.0-only",
 	},
@@ -455,7 +455,7 @@ RBinPlugin r_bin_plugin_pe = {
 	.header = &header,
 	.fields = &fields,
 	.libs = &libs,
-	.relocs = &relocs,
+	.relocs = relocs,
 	.minstrlen = 4,
 	.create = &create,
 	.get_vaddr = &get_vaddr,
