@@ -126,8 +126,9 @@ static RList *symbols(RBinFile *arch) {
 RBinPlugin r_bin_plugin_pyc = {
 	.meta = {
 		.name = "pyc",
-		.desc = "Python byte-compiled file plugin",
-		.license = "LGPL3",
+		.author = "c0riolis,x0urc3",
+		.desc = "Python byte-compiled",
+		.license = "LGPL-3.0-only",
 	},
 	.info = &info,
 	.load = &load,
