@@ -105,7 +105,7 @@ static RList *sections(RBinFile *bf) {
 		/* All TE files have _TEXT_RE section, which is 16-bit, because of
 		 * CPU start in this mode */
 		if (!strncmp (ptr->name, "_TEXT_RE", 8)) {
-			ptr->bits = R_SYS_BITS_16;
+			ptr->bits = R_SYS_BITS_PACK (16);
 		}
 		r_list_append (ret, ptr);
 	}
@@ -150,7 +150,7 @@ RBinPlugin r_bin_plugin_te = {
 	.meta = {
 		.name = "te",
 		.author = "xvilka",
-		.desc = "Terse Executable format",
+		.desc = "Terse Executable (based on PE/COFF for PE32/+)",
 		.license = "LGPL-3.0-only",
 	},
 	.get_sdb = &get_sdb,
