@@ -9,6 +9,34 @@
 #define W(y) r_cons_canvas_write (c, y)
 #define G(x, y) r_cons_canvas_gotoxy (c, x, y)
 
+static const char* r_cons_get_rune(const ut8 ch) {
+	/* Fast lookup table for runes mapped by RUNECODE_* constants.
+	 * The table is indexed by (ch - RUNECODE_MIN) and covers the
+	 * continuous range [RUNECODE_MIN, RUNECODE_MAX).
+	 */
+	static const char *const rune_table[] = {
+		/* 0xc8 */ RUNE_LINE_VERT,
+		/* 0xc9 */ RUNE_LINE_CROSS,
+		/* 0xca */ RUNE_CORNER_BR,
+		/* 0xcb */ RUNE_CORNER_BL,
+		/* 0xcc */ RUNE_ARROW_RIGHT,
+		/* 0xcd */ RUNE_ARROW_LEFT,
+		/* 0xce */ RUNE_LINE_HORIZ,
+		/* 0xcf */ RUNE_CORNER_TL,
+		/* 0xd0 */ RUNE_CORNER_TR,
+		/* 0xd1 */ RUNE_LINE_UP,
+		/* 0xd2 */ RUNE_CURVE_CORNER_TL,
+		/* 0xd3 */ RUNE_CURVE_CORNER_TR,
+		/* 0xd4 */ RUNE_CURVE_CORNER_BR,
+		/* 0xd5 */ RUNE_CURVE_CORNER_BL,
+	};
+
+	if (ch < RUNECODE_MIN || ch >= RUNECODE_MAX) {
+		return NULL;
+	}
+	return rune_table[ch - RUNECODE_MIN];
+}
+
 static inline bool __isAnsiSequence(const char *s) {
 	return s && s[0] == 033 && s[1] == '[';
 }
@@ -360,8 +388,9 @@ R_API void r_cons_canvas_write(RConsCanvas *c, const char *_s) {
 }
 
 R_API void r_cons_canvas_write_at(RConsCanvas *c, const char *s, int x, int y) {
-	r_cons_canvas_gotoxy (c, x, y);
-	r_cons_canvas_write (c, s);
+	if (r_cons_canvas_gotoxy (c, x, y)) {
+		r_cons_canvas_write (c, s);
+	}
 }
 
 R_API void r_cons_canvas_background(RConsCanvas *c, const char *color) {
@@ -530,7 +559,7 @@ R_API void r_cons_canvas_circle(RConsCanvas *c, int x, int y, int w, int h, cons
 	}
 }
 
-R_API void r_cons_canvas_box(RConsCanvas *c, int x, int y, int w, int h, const char *color) {
+R_API void r_cons_canvas_box(RConsCanvas *c, int x, int y, int w, int h, const char * R_NULLABLE color) {
 	// NOTE: As long as utf and curvy flags are tied to the canvas, we need to
 	// regenerate the canvas to get such changes now. before the kons refactoring
 	// this changed when cconfig settings were modified. not sure if its worth.
@@ -539,7 +568,11 @@ R_API void r_cons_canvas_box(RConsCanvas *c, int x, int y, int w, int h, const c
 	const char *hline = useutf? RUNECODESTR_LINE_HORIZ : "-";
 	const char *vtmp = useutf? RUNECODESTR_LINE_VERT : "|";
 	RStrBuf *vline = r_strbuf_new (NULL);
-	r_strbuf_appendf (vline, Color_RESET"%s%s", color, vtmp);
+	if (color) {
+		r_strbuf_appendf (vline, Color_RESET"%s%s", color, vtmp);
+	} else {
+		r_strbuf_appendf (vline, Color_RESET"%s", vtmp);
+	}
 	const char *tl_corner = useutf ? (usecrv ? RUNECODESTR_CURVE_CORNER_TL : RUNECODESTR_CORNER_TL) : ".";
 	const char *tr_corner = useutf ? (usecrv ? RUNECODESTR_CURVE_CORNER_TR : RUNECODESTR_CORNER_TR) : ".";
 	const char *bl_corner = useutf ? (usecrv ? RUNECODESTR_CURVE_CORNER_BL : RUNECODESTR_CORNER_BL) : "`";
