@@ -1073,9 +1073,10 @@ static bool w32_xterm_get_size(RCons *cons) {
 }
 #endif
 
-// XXX: if this function returns <0 in rows or cols expect MAYHEM
+// This function will never return rows or cols lower than 0
 R_API int r_cons_get_size(RCons *cons, int * R_NULLABLE rows) {
 	R_RETURN_VAL_IF_FAIL (cons, 0);
+	bool pick_defaults = false;
 #if R2__WINDOWS__
 	CONSOLE_SCREEN_BUFFER_INFO csbi;
 	bool ret = GetConsoleScreenBufferInfo (GetStdHandle (STD_OUTPUT_HANDLE), &csbi);
@@ -1087,15 +1088,10 @@ R_API int r_cons_get_size(RCons *cons, int * R_NULLABLE rows) {
 			ret = w32_xterm_get_size (cons);
 		}
 		if (!ret || (cons->columns == -1 && cons->rows == 0)) {
-			// Stdout is probably redirected so we set default values
-			cons->columns = 80;
-			cons->rows = 23;
+			pick_defaults = true;
 		}
 	}
-#elif EMSCRIPTEN || __wasi__
-	cons->columns = 80;
-	cons->rows = 23;
-#elif R2__UNIX__
+#elif R2__UNIX__ && !__wasi__
 	struct winsize win = {0};
 	if (isatty (0) && !ioctl (0, TIOCGWINSZ, &win)) {
 		if ((!win.ws_col) || (!win.ws_row)) {
@@ -1110,44 +1106,38 @@ R_API int r_cons_get_size(RCons *cons, int * R_NULLABLE rows) {
 			if (fd != -1) {
 				int ret = ioctl (fd, TIOCGWINSZ, &win);
 				if (ret || !win.ws_col || !win.ws_row) {
-					win.ws_col = 80;
-					win.ws_row = 23;
+					pick_defaults = true;
 				}
 				close (fd);
 			}
 		}
-		cons->columns = win.ws_col;
-		cons->rows = win.ws_row;
+		if (!pick_defaults) {
+			cons->columns = win.ws_col;
+			cons->rows = win.ws_row;
+		}
 	} else {
-		cons->columns = 80;
-		cons->rows = 23;
-	}
-#else
-	char *str = r_sys_getenv ("COLUMNS");
-	if (str) {
-		cons->columns = atoi (str);
-		cons->rows = 23; // XXX. windows must get console size
-		free (str);
-	} else {
-		cons->columns = 80;
-		cons->rows = 23;
+		pick_defaults = true;
 	}
 #endif
+	if (pick_defaults || cons->columns < 1 || cons->rows < 1) {
+		char *cols = r_sys_getenv ("COLUMNS");
+		cons->columns = cols? atoi (cols): 80;
+		free (cols);
+		char *rows = r_sys_getenv ("ROWS");
+		cons->rows = rows? atoi (rows): 23;
+		free (rows);
+	}
 #if SIMULATE_ADB_SHELL
 	cons->rows = 0;
 	cons->columns = 0;
-#endif
-#if SIMULATE_MAYHEM
-	// expect tons of crashes
-	cons->rows = -1;
-	cons->columns = -1;
-#endif
+#else
 	if (cons->rows < 0) {
 		cons->rows = 0;
 	}
 	if (cons->columns < 0) {
 		cons->columns = 0;
 	}
+#endif
 	if (cons->force_columns) {
 		cons->columns = cons->force_columns;
 	}
@@ -1160,12 +1150,16 @@ R_API int r_cons_get_size(RCons *cons, int * R_NULLABLE rows) {
 	if (cons->fix_rows) {
 		cons->rows += cons->fix_rows;
 	}
+	if (cons->rows < 0) {
+		cons->rows = 0;
+	}
 	if (rows) {
 		*rows = cons->rows;
 	}
 	cons->rows = R_MAX (0, cons->rows);
 	return R_MAX (0, cons->columns);
 }
+
 #if 0
 Enable/Disable scrolling in terminal:
 FMI: cd libr/cons/t ; make ti ; ./ti
@@ -1199,14 +1193,6 @@ R_API bool r_cons_set_cup(bool enable) {
 	return true;
 }
 
-#if 0
-// same as r_cons_lastline(), but len will be the number of
-// utf-8 characters excluding ansi escape sequences as opposed to just bytes
-R_API char *r_cons_lastline_utf8_ansi_len(int *len) {
-	return r_cons_lastline_utf8_ansi_len (I, len);
-}
-#endif
-
 /* swap color from foreground to background, returned value must be freed */
 R_API char *r_cons_swap_ground(const char *col) {
 	if (!col) {
@@ -1248,23 +1234,6 @@ R_API void r_cons_bind(RCons *cons, RConsBind *bind) {
 	bind->is_breaked = r_cons_is_breaked;
 }
 
-R_API const char* r_cons_get_rune(const ut8 ch) {
-	switch (ch) {
-	case RUNECODE_LINE_HORIZ: return RUNE_LINE_HORIZ;
-	case RUNECODE_LINE_VERT:  return RUNE_LINE_VERT;
-	case RUNECODE_LINE_CROSS: return RUNE_LINE_CROSS;
-	case RUNECODE_CORNER_TL:  return RUNE_CORNER_TL;
-	case RUNECODE_CORNER_TR:  return RUNE_CORNER_TR;
-	case RUNECODE_CORNER_BR:  return RUNE_CORNER_BR;
-	case RUNECODE_CORNER_BL:  return RUNE_CORNER_BL;
-	case RUNECODE_CURVE_CORNER_TL:  return RUNE_CURVE_CORNER_TL;
-	case RUNECODE_CURVE_CORNER_TR:  return RUNE_CURVE_CORNER_TR;
-	case RUNECODE_CURVE_CORNER_BR:  return RUNE_CURVE_CORNER_BR;
-	case RUNECODE_CURVE_CORNER_BL:  return RUNE_CURVE_CORNER_BL;
-	}
-	return NULL;
-}
-
 #if WITH_STATIC_THEMES
 #include "d_themes.inc.c"
 
@@ -1611,7 +1580,6 @@ R_API void r_cons_context_free(RConsContext * R_NULLABLE ctx) {
 	}
 }
 
-
 R_API RConsContext *r_cons_context_clone(RConsContext *ctx) {
 	RConsContext *c = r_mem_dup (ctx, sizeof (RConsContext));
 	if (ctx->buffer) {
@@ -2167,3 +2135,18 @@ R_API int r_cons_printf(RCons *cons, const char *format, ...) {
 	return 0;
 }
 
+#if R2_USE_NEW_ABI
+R_API void r_cons_break(RCons *cons) {
+	if (!cons) {
+		if (!I) {
+			return;
+		}
+		cons = I;
+	}
+	r_cons_context_break (cons->context);
+#if R2__UNIX__ && !__wasi__
+	/* Trigger a SIGINT so threads or blocking syscalls can be interrupted. */
+	raise (SIGINT);
+#endif
+}
+#endif
