@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2024 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2025 - pancake, nibble */
 
 #include <r_cons.h>
 #include <r_util/r_print.h>
@@ -11,25 +11,27 @@
 #define I(x) r_cons_singleton ()->x
 
 static char *strchr_ns(char *s, const char ch) {
-	char *p = strchr (s, ch);
-	if (p && p > s) {
-		char *prev = p - 1;
-		if (*prev == '\\') {
-			memmove (prev, p, strlen (p) + 1);
-			return strchr_ns (p, ch);
+	if (!s) {
+		return NULL;
+	}
+	char *p;
+	while ((p = strchr (s, ch)) != NULL) {
+		if (p > s && *(p - 1) == '\\') {
+			memmove (p - 1, p, strlen (p) + 1);
+			s = p;
+		} else {
+			return p;
 		}
 	}
-	return p;
+	return NULL;
 }
 
-#if R2_USE_NEW_ABI
 static void r_cons_grep_word_free(RConsGrepWord *gw) {
 	if (gw) {
 		free (gw->str);
 		free (gw);
 	}
 }
-#endif
 
 static RCoreHelpMessage help_detail_tilde = {
 	"Usage: [command]~[modifier][word,word][endmodifier][[column]][:line]\n"
@@ -82,56 +84,55 @@ static RCoreHelpMessage help_detail_tilde = {
 	NULL
 };
 
-R_API void r_cons_grep_help(void) {
-	r_cons_cmd_help (help_detail_tilde, true);
+static void grep_word_free(RConsGrepWord *gw) {
+	if (gw) {
+		free (gw->str);
+		free (gw);
+	}
 }
 
-// R2_600 remove this limit
-#define R_CONS_GREP_BUFSIZE 4096
+R_API void r_cons_grep_help(RCons *cons) {
+	r_cons_cmd_help (cons, help_detail_tilde, true);
+}
 
-R_API void r_cons_grep_expression(const char *str) {
-	char buf[R_CONS_GREP_BUFSIZE];
-	char *ptrs[R_CONS_GREP_COUNT];
-	int wlen, is_range, num_is_parsed, fail = false;
-	char *optr, *ptr2, *ptr3, *end_ptr = NULL, last;
-	ut64 range_begin, range_end;
-	size_t ptrs_length = 1;
-	char *ptr = buf;
-	ptrs[0] = ptr;
-
-	if (R_STR_ISEMPTY (str)) {
+R_API void r_cons_grep_expression(RCons *cons, const char *str) {
+	if (R_STR_ISEMPTY(str)) {
 		return;
 	}
-	RCons *cons = r_cons_singleton ();
 	RConsContext *ctx = cons->context;
 	RConsGrep *grep = &ctx->grep;
 
-	int len = strlen (str) - 1;
-	if (len < 0) {
-		len = 0;
-	}
-	if (len > R_CONS_GREP_BUFSIZE - 1) {
-		R_LOG_ERROR ("r_cons_grep: too long!");
-		return;
-	}
-	if (len > 0 && str[len] == '?') {
+	size_t str_len = strlen (str);
+	size_t buf_len = str_len;
+	bool has_counter = str_len > 0 && str[str_len - 1] == '?';
+
+	if (has_counter) {
 		grep->counter = 1;
-		r_str_ncpy (buf, str, R_MIN (len, sizeof (buf) - 1));
-		buf[len] = 0;
-		len--;
-	} else {
-		r_str_ncpy (buf, str, sizeof (buf) - 1);
+		buf_len--;
 	}
 
-	// TODO: replace with r_str_split_by ("~");
-	while ((ptrs[ptrs_length] = (strchr (ptr, '~')))) {
+	char *buf = malloc (buf_len + 1);
+	if (!buf) {
+		R_LOG_ERROR ("r_cons_grep: cannot allocate buffer");
+		return;
+	}
+
+	memcpy (buf, str, buf_len);
+	buf[buf_len] = '\0';
+
+	char *ptrs[R_CONS_GREP_COUNT];
+	size_t ptrs_length = 1;
+	ptrs[0] = buf;
+	char *ptr = buf;
+
+	while ((ptrs[ptrs_length] = strchr (ptr, '~'))) {
 		*(ptrs[ptrs_length]) = '\0';
 		ptrs[ptrs_length]++;
 		ptr = ptrs[ptrs_length];
 		ptrs_length++;
 		if (ptrs_length >= R_CONS_GREP_COUNT) {
 			R_LOG_ERROR ("too many nested greps");
-			return;
+			goto cleanup;
 		}
 	}
 
@@ -139,21 +140,21 @@ R_API void r_cons_grep_expression(const char *str) {
 	bool first = true;
 	ctx->sorted_column = 0;
 	size_t i;
+	if (!grep->strings) {
+		grep->strings = r_list_newf ((RListFree)grep_word_free);
+	}
 	for (i = 0; i < ptrs_length; i++) {
-#if R2_USE_NEW_ABI
 		bool gw_begin = false;
 		bool gw_neg = false;
 		bool gw_end = false;
-#endif
 		ptr = ptrs[i];
-		end_ptr = ptr2 = ptr3 = NULL;
+		char *end_ptr = NULL, *ptr2 = NULL, *ptr3 = NULL;
 		while (*ptr) {
 			switch (*ptr) {
 			case ':':
 				if (ptr[1] == ')') { // ":)"
 					if (ptr[2] == ')') { // ":))"
-						// R2_600 - add grep->colorcode option
-						grep->sort = -123;
+						grep->colorcode = true;
 						ptr++;
 					}
 					grep->code = true;
@@ -171,7 +172,7 @@ R_API void r_cons_grep_expression(const char *str) {
 					} else {
 						grep->less = 1;
 					}
-					return;
+					goto cleanup;
 				}
 				ptr++;
 				break;
@@ -206,7 +207,7 @@ R_API void r_cons_grep_expression(const char *str) {
 					} else {
 						free (jsonPath);
 					}
-					return;
+					goto cleanup;
 				}
 				ptr++;
 				break;
@@ -225,7 +226,7 @@ R_API void r_cons_grep_expression(const char *str) {
 				} else {
 					grep->sort_invert = false;
 				}
-				while (isdigit (*ptr)) {
+				while (*ptr && isdigit (*ptr)) {
 					ptr++;
 				}
 				if (*ptr == ':') {
@@ -254,7 +255,6 @@ R_API void r_cons_grep_expression(const char *str) {
 					goto while_end;
 				}
 				break;
-#if R2_USE_NEW_ABI
 			case '^':
 				ptr++;
 				gw_begin = true;
@@ -263,16 +263,6 @@ R_API void r_cons_grep_expression(const char *str) {
 				ptr++;
 				gw_neg = true;
 				break;
-#else
-			case '^':
-				ptr++;
-				grep->begin[grep->nstrings] = true;
-				break;
-			case '!':
-				ptr++;
-				grep->neg[grep->nstrings] = true;
-				break;
-#endif
 			case '?':
 				ptr++;
 				grep->counter = 1;
@@ -280,12 +270,11 @@ R_API void r_cons_grep_expression(const char *str) {
 					grep->charCounter = true;
 					ptr++;
 				} else if (!strcmp (ptr, "ea")) {
-					// "?ea"
 					grep->ascart = true;
 				} else if (*ptr == '?') {
 					cons->context->filter = true;
-					r_cons_grep_help ();
-					return;
+					r_cons_grep_help (cons);
+					goto cleanup;
 				}
 				break;
 			default:
@@ -293,22 +282,21 @@ R_API void r_cons_grep_expression(const char *str) {
 			}
 			first = false;
 		}
-	while_end:
+while_end:
 		ptr2 = strchr (ptr, '[');
 		ptr3 = strchr (ptr, ']');
-		is_range = 0;
-		num_is_parsed = 0;
-		fail = false;
-		range_begin = range_end = -1;
+		int is_range = 0, num_is_parsed = 0;
+		bool fail = false;
+		ut64 range_begin = -1, range_end = -1;
 
 		if (ptr2 && ptr3) {
 			end_ptr = ptr2;
-			last = ptr3[1];
+			char last = ptr3[1];
 			ptr3[1] = '\0';
 			ptr2++;
 			for (; ptr2 <= ptr3; ptr2++) {
 				if (fail) {
-					ZERO_FILL (grep->tokens);
+					ZERO_FILL(grep->tokens);
 					grep->tokens_used = 0;
 					break;
 				}
@@ -328,7 +316,6 @@ R_API void r_cons_grep_expression(const char *str) {
 						grep->tokens[range_begin] = 1;
 						grep->tokens_used = 1;
 					}
-					// case of [n-]
 					if (*ptr2 == ']' && is_range && !num_is_parsed) {
 						num_is_parsed = true;
 						range_end = -1;
@@ -341,9 +328,8 @@ R_API void r_cons_grep_expression(const char *str) {
 					if (!num_is_parsed) {
 						if (is_range) {
 							range_end = r_num_get (cons->num, ptr2);
-							// check for bad value, if range_end == 0, we check if ptr2 == '0'
-							if (range_end == 0 && *ptr != '0') {
-								range_end = -1; // this allow [n- ]
+							if (range_end == 0 && *ptr2 != '0') {
+								range_end = -1;
 							}
 						} else {
 							range_begin = range_end = r_num_get (cons->num, ptr2);
@@ -356,53 +342,50 @@ R_API void r_cons_grep_expression(const char *str) {
 		}
 
 		ptr2 = strchr_ns (ptr, ':'); // line number
-		grep->range_line = 2; // there is not :
+		grep->range_line = 2; // there is no :
 		if (ptr2 && ptr2[1] != ':' && ptr2[1] && (isdigit (ptr2[1]) || ptr2[1] == '-' || ptr2[1] == '.')) {
-			end_ptr = end_ptr? R_MIN (end_ptr, ptr2): ptr2;
+			end_ptr = end_ptr ? R_MIN (end_ptr, ptr2) : ptr2;
 			char *p, *token = ptr2 + 1;
-			p = strstr (token, "..");
+			p = strstr(token, "..");
 			if (!p) {
 				grep->line = r_num_get (cons->num, ptr2 + 1);
 				grep->range_line = 0;
 			} else {
 				*p = '\0';
 				grep->range_line = 1;
-				grep->f_line = *token? r_num_get (cons->num, token): 0;
-				grep->l_line = p[2]? r_num_get (cons->num, p + 2): 0;
+				grep->f_line = *token ? r_num_get (cons->num, token) : 0;
+				grep->l_line = p[2] ? r_num_get (cons->num, p + 2) : 0;
 			}
 		}
 		if (end_ptr) {
 			*end_ptr = '\0';
 		}
 
-		len = strlen (ptr) - 1;
-		if (len > 1 && ptr[len] == '$' && ptr[len - 1] != '\\') {
-#if R2_USE_NEW_ABI
+		int len = strlen (ptr);
+		if (len > 1 && ptr[len - 1] == '$' && ptr[len - 2] != '\\') {
 			gw_end = true;
-#else
-			grep->end[i] = true;
-#endif
-			ptr[len] = '\0';
+			ptr[len - 1] = '\0';
 		}
 
 		if (*ptr) {
 			if (!grep->str) {
-				grep->str = (char *)strdup (ptr);
+				grep->str = strdup (ptr);
 			} else {
-				grep->str = r_str_append (grep->str, ",");
-				grep->str = r_str_append (grep->str, ptr);
+				char *s = r_str_newf (",%s", ptr);
+				grep->str = r_str_append (grep->str, s);
+				free (s);
 			}
+			char *optr;
 			do {
 				optr = ptr;
 				ptr = strchr (ptr, ','); // grep keywords
 				if (ptr) {
 					*ptr++ = '\0';
 				}
-				wlen = strlen (optr);
+				int wlen = strlen (optr);
 				if (!wlen) {
 					continue;
 				}
-#if R2_USE_NEW_ABI
 				RConsGrepWord *gw = R_NEW0 (RConsGrepWord);
 				gw->str = strdup (optr);
 				gw->begin = gw_begin;
@@ -410,41 +393,21 @@ R_API void r_cons_grep_expression(const char *str) {
 				gw->end = gw_end;
 				gw_end = false;
 				r_list_append (grep->strings, gw);
-#else
-				if (wlen >= R_CONS_GREP_WORD_SIZE - 1) {
-					R_LOG_ERROR ("grep string too long");
-					continue;
-				}
-				grep->nstrings++;
-				if (grep->nstrings > R_CONS_GREP_WORDS - 1) {
-					R_LOG_ERROR ("too many grep strings");
-					break;
-				}
-				r_str_ncpy (grep->strings[grep->nstrings - 1],
-					optr, R_CONS_GREP_WORD_SIZE);
-#endif
 			} while (ptr);
 		}
 	}
-#if R2_USE_NEW_ABI
-	// XXX this is a hack
+
 	if (!grep->str) {
 		RConsGrepWord *gw = R_NEW0 (RConsGrepWord);
 		gw->str = strdup ("");
 		grep->str = strdup ("");
 		r_list_append (grep->strings, gw);
 	}
-#else
-	if (!grep->str) {
-		grep->str = strdup ("");
-		grep->nstrings++;
-		grep->strings[0][0] = 0;
-	}
-#endif
+cleanup:
+	free (buf);
 }
 
-// Finds and returns next intgrep expression,
-// unescapes escaped twiddles
+// Finds and returns next intgrep expression, unescapes escaped twiddles
 static char *find_next_intgrep(char *cmd, const char *quotes) {
 	do {
 		char *p = (char *)r_str_firstbut (cmd, '~', quotes);
@@ -500,12 +463,12 @@ static char *preprocess_filter_expr(char *cmd, const char *quotes) {
 	return r_str_append (ns, p1 + 1);
 }
 
-R_API void r_cons_grep_parsecmd(char *cmd, const char *quotestr) {
+R_API void r_cons_grep_parsecmd(RCons *cons, char *cmd, const char *quotestr) {
 	R_RETURN_IF_FAIL (cmd && quotestr);
 	char *ptr = preprocess_filter_expr (cmd, quotestr);
 	if (ptr) {
 		r_str_trim (cmd);
-		r_cons_grep_expression (ptr);
+		r_cons_grep_expression (cons, ptr);
 		free (ptr);
 	}
 }
@@ -525,9 +488,17 @@ static int cmp(const void *a, const void *b) {
 	const char *ca = r_str_trim_head_ro (a);
 	const char *cb = r_str_trim_head_ro (b);
 	if (!a || !b) {
-		return (int) (size_t) ((char *) a - (char *) b);
+		ptrdiff_t diff = (char*)a - (char*)b;
+		if (diff > INT_MAX) {
+			return INT_MAX;
+		}
+		if (diff < INT_MIN) {
+			return INT_MIN;
+		}
+		return (int)diff;
 	}
-	RConsContext *ctx = r_cons_context ();
+	RCons *cons = r_cons_singleton ();
+	RConsContext *ctx = cons->context;
 	if (ctx->sorted_column > 0) {
 		da = strdup (ca);
 		db = strdup (cb);
@@ -613,144 +584,20 @@ static inline ut64 cmpstrings(const void *a) {
 	return r_str_hash64 (a);
 }
 
-static char *colorword(char *res, const char *k, const char *color) {
-	char *tv = r_str_newf ("~~[%s]~~", k);
-	r_str_case (tv, true);
-	char *nv = r_str_newf ("%s%s"Color_RESET, color, k);
-	res = r_str_replace_all (res, k, tv);
-	res = r_str_replace_all (res, tv, nv);
-	free (nv);
-	free (tv);
-	return res;
-}
+#include <r_core.h>
 
-static void colorcode(void) {
-	// TODO : dupped from libr/util/print_code.c r_print_code_tocolor
-	RCons *cons = r_cons_singleton ();
-	int i;
+
+static void colorcode(RCons *cons) {
 	char *res = r_str_ndup (cons->context->buffer, cons->context->buffer_len);
-	if (res) {
-		bool linecomment = false;
-		bool comment = false;
-		bool string = false;
-		RStrBuf *sb = r_strbuf_new ("");
-		for (i = 0; res[i]; i++) {
-			const char ch = res[i];
-			const char ch2 = res[i + 1];
-			if (linecomment) {
-				if (ch == '\n') {
-					r_strbuf_append (sb, Color_RESET);
-					r_strbuf_append_n (sb, &ch, 1);
-					linecomment = false;
-				} else {
-					r_strbuf_append_n (sb, &ch, 1);
-				}
-			} else if (comment) {
-				if (ch == '*' && res[i + 1] == '/') {
-					r_strbuf_append_n (sb, &ch, 1);
-					r_strbuf_append_n (sb, &ch2, 1);
-					r_strbuf_append (sb, Color_RESET);
-					comment = false;
-					i++;
-				} else {
-					r_strbuf_append_n (sb, &ch, 1);
-				}
-			} else if (string) {
-				if (ch == '\\') {
-					if (res[i + 1]) {
-						r_strbuf_append_n (sb, &ch, 1);
-						r_strbuf_append_n (sb, &ch2, 1);
-						i++;
-					} else {
-						r_strbuf_append_n (sb, &ch, 1);
-					}
-				} else if (ch == '"') {
-					r_strbuf_append_n (sb, &ch, 1);
-					r_strbuf_append (sb, Color_RESET);
-					string = false;
-				} else {
-					r_strbuf_append_n (sb, &ch, 1);
-				}
-			} else {
-				if (i == 0 && ch == '#') {
-					r_strbuf_append (sb, Color_BLUE);
-					r_strbuf_append_n (sb, &ch, 1);
-					linecomment = true;
-				} else if (ch == '\n') {
-					if (ch2 == '#') {
-						r_strbuf_append_n (sb, &ch, 1);
-						r_strbuf_append (sb, Color_BLUE);
-						r_strbuf_append_n (sb, &ch2, 1);
-						linecomment = true;
-						i++;
-						i++;
-					} else {
-						r_strbuf_append_n (sb, &ch, 1);
-					}
-				} else if (ch == '/') {
-					if (ch2 == '*') {
-						comment = true;
-						r_strbuf_append (sb, Color_BLUE);
-						r_strbuf_append_n (sb, &ch, 1);
-					} else if (ch2 == '/') {
-						linecomment = true;
-						r_strbuf_append (sb, Color_BLUE);
-						r_strbuf_append_n (sb, &ch, 1);
-					} else {
-						r_strbuf_append_n (sb, &ch, 1);
-					}
-				} else if (ch == '"') {
-					string = true;
-					r_strbuf_append (sb, Color_RED);
-					r_strbuf_append_n (sb, &ch, 1);
-				} else {
-					r_strbuf_append_n (sb, &ch, 1);
-				}
-			}
-		}
-		free (res);
-		res = r_strbuf_drain (sb);
-		// ugly temporary hack
-#if 0
-		res = colorword (res, "if ", Color_RED);
-		res = colorword (res, " else ", Color_RED);
-#endif
-		res = colorword (res, "for ", Color_RED);
-		res = colorword (res, "while ", Color_RED);
-		res = colorword (res, "switch ", Color_RED);
-		res = colorword (res, "static ", Color_RED);
-		res = colorword (res, "inline ", Color_RED);
-		// res = colorword (res, " -> ", Color_RED);
-		res = colorword (res, "return", Color_RED);
-		res = colorword (res, "string ", Color_RED);
-		res = colorword (res, "number ", Color_RED);
-
-		res = colorword (res, "void ", Color_GREEN);
-		res = colorword (res, "bool ", Color_GREEN);
-		res = colorword (res, "ut64 ", Color_GREEN);
-		res = colorword (res, "uint32_t", Color_GREEN);
-		res = colorword (res, "uint64_t", Color_GREEN);
-		res = colorword (res, "int32_t", Color_GREEN);
-		res = colorword (res, "int64_t", Color_GREEN);
-		res = colorword (res, "int8_t", Color_GREEN);
-		res = colorword (res, "uint8_t", Color_GREEN);
-		res = colorword (res, "int ", Color_GREEN);
-		res = colorword (res, "char ", Color_GREEN);
-		res = colorword (res, "const ", Color_GREEN);
-#if 0
-		res = colorword (res, "{", Color_YELLOW);
-		res = colorword (res, "}", Color_YELLOW);
-#endif
-		// bring back the colorized buffer
-		cons->context->buffer_len = strlen (res);
-		cons->context->buffer_sz = cons->context->buffer_len;
-		free (cons->context->buffer);
-		cons->context->buffer = res;
-	}
+	RConsCodeColors codepal = r_cons_codecolors (cons);
+	char *cres = r_print_code_tocolor (res, &codepal);
+	cons->context->buffer_len = strlen (cres);
+	cons->context->buffer_sz = cons->context->buffer_len;
+	free (cons->context->buffer);
+	cons->context->buffer = cres;
 }
 
-R_API void r_cons_grepbuf(void) {
-	RCons *cons = r_cons_singleton ();
+R_API void r_cons_grepbuf(RCons *cons) {
 	const char *buf = cons->context->buffer;
 	size_t len = cons->context->buffer_len;
 	RConsGrep *grep = &cons->context->grep;
@@ -762,8 +609,8 @@ R_API void r_cons_grepbuf(void) {
 		R_FREE (cons->context->buffer);
 		return;
 	}
-	if (grep->sort == -123) { // R2_600 - colorcode
-		colorcode ();
+	if (grep->colorcode) {
+		colorcode (cons);
 		grep->sort = 0;
 		grep->code = false;
 		return;
@@ -772,6 +619,9 @@ R_API void r_cons_grepbuf(void) {
 		char *sbuf = r_str_ndup (cons->context->buffer, cons->context->buffer_len);
 		if (sbuf) {
 			char *res = r_str_tokenize_json (sbuf);
+			if (!res) {
+				return;
+			}
 			char *nres = r_print_json_indent (res, I(context->color_mode), "  ", NULL);
 			free (res);
 			res = r_str_newf ("%s\n", nres);
@@ -838,8 +688,8 @@ R_API void r_cons_grepbuf(void) {
 		R_FREE (cons->context->buffer);
 		cons->context->buffer_len = 0;
 		cons->context->buffer_sz = 0;
-		r_cons_print (s);
-		buf = cons->context->buffer;
+		r_cons_print (cons, s);
+		in = buf = cons->context->buffer;
 		len = cons->context->buffer_len;
 		r_json_free (node);
 		free (a);
@@ -864,7 +714,7 @@ R_API void r_cons_grepbuf(void) {
 				cons->context->buffer_len = strlen (u);
 				cons->context->buffer_sz = cons->context->buffer_len + 1;
 				grep->json = false;
-				r_cons_newline ();
+				r_cons_newline (cons);
 			}
 			R_FREE (grep->json_path);
 		} else {
@@ -895,41 +745,23 @@ R_API void r_cons_grepbuf(void) {
 			cons->context->grep_color = true;
 			// R2R db/cmd/cmd_iz
 			R_FREE (grep->str);
-#if R2_USE_NEW_ABI
 			cons->context->grep_color = false;
 			RConsGrepWord *gw = r_list_pop_head (grep->strings);
 			r_cons_grep_word_free (gw);
-#else
-			if (grep->nstrings > 0) {
-				cons->context->grep_color = false;
-				// shift them all!!
-				int i;
-				for (i = 0; i < grep->nstrings; i++) {
-					r_str_cpy (grep->strings[i], grep->strings[i + 1]);
-				}
-				grep->nstrings--;
-			}
-#endif
 			if (grep->hud) {
 				grep->hud = false;
-				r_cons_hud_string (cons->context->buffer);
+				r_cons_hud_string (cons, cons->context->buffer);
 				return;
 			}
 			if (grep->less) {
 				grep->less = 0;
-				r_cons_less_str (cons->context->buffer, NULL);
+				r_cons_less_str (cons, cons->context->buffer, NULL);
 				return;
 			}
 		}
-#if R2_USE_NEW_ABI
 		if (r_list_empty (grep->strings)) {
 			return;
 		}
-#else
-		if (grep->nstrings < 1) {
-			return;
-		}
-#endif
 		buf = cons->context->buffer;
 		len = cons->context->buffer_len;
 		grep->range_line = 1;
@@ -940,19 +772,19 @@ R_API void r_cons_grepbuf(void) {
 		int less = grep->less;
 		grep->less = 0;
 		if (less == 3) {
-			char *res = r_cons_hud_line_string (buf);
+			char *res = r_cons_hud_line_string (cons, buf);
 			if (res) {
-				r_cons_println (res);
+				r_cons_println (cons, res);
 				free (res);
 			}
 		} else if (less == 2) {
-			char *res = r_cons_hud_string (buf);
+			char *res = r_cons_hud_string (cons, buf);
 			if (res) {
-				r_cons_println (res);
+				r_cons_println (cons, res);
 				free (res);
 			}
 		} else {
-			r_cons_less_str (buf, NULL);
+			r_cons_less_str (cons, buf, NULL);
 			cons->context->buffer_len = 0;
 			cons->context->buffer_sz = 0;
 			R_FREE (cons->context->buffer);
@@ -1011,7 +843,7 @@ continuation:
 			if (tl < 0) {
 				ret = -1;
 			} else {
-				ret = r_cons_grep_line (tline, tl);
+				ret = r_cons_grep_line (cons, tline, tl);
 				if (!grep->range_line) {
 					if (grep->line == cons->lines) {
 						show = true;
@@ -1034,7 +866,6 @@ continuation:
 				if (show) {
 					char *str = r_str_ndup (tline, ret);
 					if (cons->context->grep_highlight) {
-#if R2_USE_NEW_ABI
 						RListIter *iter;
 						RConsGrepWord *gw;
 						r_list_foreach (grep->strings, iter, gw) {
@@ -1048,20 +879,6 @@ continuation:
 							}
 							free (newstr);
 						}
-#else
-						int i;
-						for (i = 0; i < grep->nstrings; i++) {
-							char *newstr = r_str_newf (Color_INVERT"%s"Color_RESET, grep->strings[i]);
-							if (str && newstr) {
-								if (grep->icase) {
-									str = r_str_replace_icase (str, grep->strings[i], newstr, 1, 1);
-								} else {
-									str = r_str_replace (str, grep->strings[i], newstr, 1);
-								}
-							}
-							free (newstr);
-						}
-#endif
 					}
 					if (str) {
 						r_strbuf_append (ob, str);
@@ -1085,9 +902,12 @@ continuation:
 		}
 	}
 
-	const int ob_len = r_strbuf_length (ob);
+	int ob_len = r_strbuf_length (ob);
 	cons->context->buffer_len = ob_len;
-	if (grep->counter) {
+
+	// count before uniq
+	// XXX dupe from the code below
+	if (grep->counter && !grep->sort_uniq) {
 		int cnt = grep->charCounter? strlen (cons->context->buffer): cons->lines;
 		free (cons->context->buffer);
 		char *cntstr = r_str_newf ("%d\n", cnt);
@@ -1095,11 +915,12 @@ continuation:
 		cons->context->buffer = cntstr;
 		cons->context->buffer_len = cntstr_len;
 		cons->context->buffer_sz = cntstr_len + 1;
-		cons->num->value = cons->lines;
+		if (cons->num) {
+			cons->num->value = cons->lines;
+		}
 		r_strbuf_free (ob);
 		return;
 	}
-
 	if (ob_len >= cons->context->buffer_sz) {
 		cons->context->buffer_sz = ob_len + 1;
 		cons->context->buffer = r_strbuf_drain (ob);
@@ -1108,7 +929,7 @@ continuation:
 		cons->context->buffer[ob_len] = 0;
 		r_strbuf_free (ob);
 	}
-
+	ob = NULL;
 	if (grep->sort != -1 || grep->sort_invert) {
 #define INSERT_LINES(list) \
 		if (list) { \
@@ -1120,43 +941,54 @@ continuation:
 					memcpy (ptr + slen, "\n", 2); \
 					ptr += slen + 1; \
 				} \
-				nl++; \
 			} \
 		}
 		RListIter *iter;
-		int nl = 0;
 		char *ptr = cons->context->buffer;
 		char *str;
 		RConsContext *ctx = cons->context;
 		ctx->sorted_column = grep->sort;
 
-		if (grep->sort != -1) {
-			r_list_sort (ctx->sorted_lines, cmp);
-		}
-		if (grep->sort_invert) {
-			r_list_reverse (ctx->sorted_lines);
-		}
-		if (grep->sort_uniq) {
-			r_list_uniq_inplace (ctx->sorted_lines, cmpstrings);
+		if (ctx->sorted_lines) {
+			if (grep->sort != -1) {
+				r_list_sort (ctx->sorted_lines, cmp);
+			}
+			if (grep->sort_invert) {
+				r_list_reverse (ctx->sorted_lines);
+			}
+			if (grep->sort_uniq) {
+				r_list_uniq_inplace (ctx->sorted_lines, cmpstrings);
+				r_list_free (ctx->unsorted_lines);
+				ctx->unsorted_lines = NULL;
+			}
+			const int nl = r_list_length (ctx->sorted_lines);
+			cons->context->buffer_len = 0;
+			INSERT_LINES (ctx->unsorted_lines);
+			INSERT_LINES (ctx->sorted_lines);
+			cons->context->buffer_len = (ptr - cons->context->buffer);
+			cons->lines = nl;
+			r_list_free (ctx->sorted_lines);
+			ctx->sorted_lines = NULL;
 			r_list_free (ctx->unsorted_lines);
 			ctx->unsorted_lines = NULL;
-			nl = 0;
 		}
-		cons->context->buffer_len = 0;
-		INSERT_LINES (ctx->unsorted_lines);
-		INSERT_LINES (ctx->sorted_lines);
-		cons->context->buffer_len = (ptr - cons->context->buffer);
-		cons->lines = nl;
-		r_list_free (ctx->sorted_lines);
-		ctx->sorted_lines = NULL;
-		r_list_free (ctx->unsorted_lines);
-		ctx->unsorted_lines = NULL;
+	}
+	// count after uniq
+	if (grep->counter && grep->sort_uniq) {
+		int cnt = grep->charCounter? strlen (cons->context->buffer): cons->lines;
+		free (cons->context->buffer);
+		char *cntstr = r_str_newf ("%d\n", cnt);
+		size_t cntstr_len = cntstr? strlen (cntstr): 0;
+		cons->context->buffer = cntstr;
+		cons->context->buffer_len = cntstr_len;
+		cons->context->buffer_sz = cntstr_len + 1;
+		cons->num->value = cons->lines;
+		r_strbuf_free (ob);
 	}
 }
 
-R_API int r_cons_grep_line(char *buf, int len) {
+R_API int r_cons_grep_line(RCons *cons, char *buf, int len) {
 	R_RETURN_VAL_IF_FAIL (buf && len >= 0, 0);
-	RCons *cons = r_cons_singleton ();
 	RConsGrep *grep = &cons->context->grep;
 	const char *delims = " |,;=\t";
 	char *tok = NULL;
@@ -1176,18 +1008,13 @@ R_API int r_cons_grep_line(char *buf, int len) {
 		return 0;
 	}
 	memcpy (in, buf, len);
-#if R2_USE_NEW_ABI
 	const bool have_strings = !r_list_empty (grep->strings);
-#else
-	const bool have_strings = grep->nstrings > 0;
-#endif
 
 	if (have_strings) {
 		bool all_hits = true;
 		if (grep->icase) {
 			r_str_case (in, false);
 		}
-#if R2_USE_NEW_ABI
 		RListIter *iter;
 		RConsGrepWord *gw;
 		r_list_foreach (grep->strings, iter, gw) {
@@ -1216,34 +1043,6 @@ R_API int r_cons_grep_line(char *buf, int len) {
 				break;
 			}
 		}
-#else
-		for (i = 0; i < grep->nstrings; i++) {
-			char *str = grep->strings[i];
-			if (grep->icase) {
-				r_str_case (str, false);
-			}
-			const char *p = r_strstr_ansi (in, grep->strings[i]);
-			if (!p) {
-				hit = grep->neg[i];
-				all_hits &= hit;
-				continue;
-			}
-			hit = grep->begin[i]
-				? grep->neg[i]
-					? p != in
-					: p == in
-				: !grep->neg[i];
-
-			// TODO: optimize without strlen without breaking t/feat_grep (grep end)
-			if (grep->end[i] && (strlen (grep->strings[i]) != strlen (p))) {
-				hit = false;
-			}
-			all_hits &= hit;
-			if (!grep->amp) {
-				break;
-			}
-		}
-#endif
 		if (grep->amp) {
 			hit = all_hits;
 		}
@@ -1251,7 +1050,7 @@ R_API int r_cons_grep_line(char *buf, int len) {
 		hit = true;
 	}
 
-	RConsContext *ctx = r_cons_context ();
+	RConsContext *ctx = cons->context;
 	if (hit) {
 		if (!grep->range_line) {
 			if (grep->line == cons->lines) {
@@ -1330,8 +1129,10 @@ R_API int r_cons_grep_line(char *buf, int len) {
 	return len;
 }
 
-R_API void r_cons_grep(const char *grep) {
+#if 0
+R_API void r_cons_grep(RCons * R_NONNULL cons, const char *grep) {
 	R_RETURN_IF_FAIL (grep);
-	r_cons_grep_expression (grep);
-	r_cons_grepbuf ();
+	r_cons_grep_expression (cons, grep);
+	r_cons_grepbuf (cons);
 }
+#endif
