@@ -1,28 +1,29 @@
-/* radare2 - LGPL - Copyright 2014-2019 - pancake, Judge_Dredd */
+/* radare2 - LGPL - Copyright 2014-2025 - pancake */
 
 #include <r_cons.h>
 #include <r_regex.h>
 #include <r_util.h>
-#include "pager_private.h"
+#include "private.h"
 
-R_API int r_cons_less_str(const char *str, const char *exitkeys) {
-	r_return_val_if_fail (str && *str, 0);
-	if (!r_cons_is_interactive ()) {
-		eprintf ("Internal less requires scr.interactive=true.\n");
+static const char *r_cons_less_help = \
+	" u/space  - page up/down (same as ^F / ^B)\n"
+	" jk       - line down/up\n"
+	" gG       - begin/end buffer\n"
+	" /        - search in buffer\n"
+	" _        - enter the hud mode\n"
+	" n/p      - next/prev search result\n"
+	" q        - quit\n"
+	" ?        - show this help\n"
+	"\n";
+
+R_API int r_cons_less_str(RCons * R_NONNULL cons, const char * R_NONNULL str, const char * R_NULLABLE exitkeys) {
+	R_RETURN_VAL_IF_FAIL (R_STR_ISNOTEMPTY (str), 0);
+	if (!r_cons_is_interactive (cons)) {
+		R_LOG_ERROR ("Internal less requires scr.interactive=true");
 		return 0;
 	}
 
-	static int in_help = false;
-	static const char *r_cons_less_help = \
-		" u/space  - page up/down\n"
-		" jk       - line down/up\n"
-		" gG       - begin/end buffer\n"
-		" /        - search in buffer\n"
-		" _        - enter the hud mode\n"
-		" n/p      - next/prev search result\n"
-		" q        - quit\n"
-		" ?        - show this help\n"
-		"\n";
+	bool in_help = false;
 	int lines_count = 0;
 	RRegex *rx = NULL;
 	int w, h, ch, to, ui = 1, from = 0, i;
@@ -54,12 +55,12 @@ R_API int r_cons_less_str(const char *str, const char *exitkeys) {
 	for (i = 0; i < lines_count; i++) {
 		mla[i] = r_list_new ();
 	}
-	r_cons_set_raw (true);
-	r_cons_show_cursor (false);
-	r_cons_reset ();
+	r_cons_set_raw (cons, true);
+	r_cons_show_cursor (cons, false);
+	r_cons_reset (cons);
 	h = 0;
 	while (ui) {
-		w = r_cons_get_size (&h);
+		w = r_cons_get_size (cons, &h);
 		to = R_MIN (lines_count, from + h);
 		if (from + 3 > lines_count) {
 			from = lines_count - 3;
@@ -67,8 +68,8 @@ R_API int r_cons_less_str(const char *str, const char *exitkeys) {
 		if (from < 0) {
 			from = 0;
 		}
-		pager_printpage (p, lines, mla, from, to, w);
-		ch = r_cons_readchar ();
+		pager_printpage (cons, p, lines, mla, from, to, w);
+		ch = r_cons_readchar (cons);
 		if (exitkeys && strchr (exitkeys, ch)) {
 			for (i = 0; i < lines_count; i++) {
 				r_list_free (mla[i]);
@@ -79,45 +80,47 @@ R_API int r_cons_less_str(const char *str, const char *exitkeys) {
 			free (lines);
 			return ch;
 		}
-		ch = r_cons_arrow_to_hjkl (ch);
+		ch = r_cons_arrow_to_hjkl (cons, ch);
 		switch (ch) {
 		case '_':
-			r_cons_hud_string (ostr);
+			r_cons_hud_string (cons, ostr);
 			break;
 		case '?':
 			if (!in_help) {
 				in_help = true;
-				(void)r_cons_less_str (r_cons_less_help, NULL);
+				(void)r_cons_less_str (cons, r_cons_less_help, NULL);
 				in_help = false;
 			}
 			break;
+		case 104: // ^B
 		case 'u':
 			from -= h;
 			if (from < 0) {
 				from = 0;
 			}
 			break;
+		case 108: // ^F
 		case ' ': from += h; break;
 		case 'g': from = 0; break;
-		case 'G': from = lines_count-h; break;
+		case 'G': from = lines_count - h; break;
 		case -1: // EOF
 		case '\x03': // ^C
 		case 'q': ui = 0; break;
 		case '\r':
 		case '\n':
 		case 'j': from++; break;
-		case 'J': from+=h; break;
+		case 'J': from += h; break;
 		case 'k':
 			if (from > 0) {
 				from--;
 			}
 			break;
-		case 'K': from = (from>=h)? from-h: 0;
+		case 'K': from = (from >= h)? from - h: 0;
 			break;
 		case '/': 	/* search */
-			r_cons_reset_colors ();
-			r_line_set_prompt ("/");
-			sreg = r_line_readline ();
+			r_cons_reset_colors (cons);
+			r_line_set_prompt (cons->line, "/");
+			sreg = r_line_readline (cons);
 			from = R_MIN (lines_count - 1, from);
 			/* repeat last search if empty string is provided */
 			if (sreg[0]) { /* prepare for a new search */
@@ -136,6 +139,7 @@ R_API int r_cons_less_str(const char *str, const char *exitkeys) {
 			if (pager_all_matches (p, rx, mla, lines, lines_count)) {
 				from = pager_next_match (from, mla, lines_count);
 			}
+			r_cons_set_raw (cons, true);
 			break;
 		case 'n': 	/* next match */
 			/* search already performed */
@@ -158,21 +162,13 @@ R_API int r_cons_less_str(const char *str, const char *exitkeys) {
 	r_regex_free (rx);
 	free (lines);
 	free (p);
-	r_cons_reset_colors ();
-	r_cons_set_raw (false);
-	r_cons_show_cursor (true);
+	r_cons_reset_colors (cons);
+	r_cons_set_raw (cons, false);
+	r_cons_show_cursor (cons, true);
 	free (ostr);
 	return 0;
 }
 
-R_API void r_cons_less(void) {
-	(void)r_cons_less_str (r_cons_context ()->buffer, NULL);
+R_API void r_cons_less(RCons *cons) {
+	(void)r_cons_less_str (cons, cons->context->buffer, NULL);
 }
-
-#if 0
-main (int argc, char **argv) {
-	char *s = r_file_slurp (argv[1], NULL);
-	r_cons_new ();
-	r_cons_less (s);
-}
-#endif
