@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2009-2024 - pancake */
+/* radare2 - LGPL - Copyright 2009-2025 - pancake */
 
 #if R_INCLUDE_BEGIN
 
@@ -26,7 +26,6 @@ static RCoreHelpMessage help_msg_e = {
 #endif
 	"e.", "a=b", "same as 'e a=b' but without using a space",
 	"e,", "[table-query]", "show the output in table format",
-	"e/", "asm", "filter configuration variables by name",
 	"e:", "k=v:k=v:k=v", "comma or colon separated k[=v]",
 	"e-", "", "reset config vars",
 	"e*", "", "dump config vars in r commands",
@@ -34,6 +33,8 @@ static RCoreHelpMessage help_msg_e = {
 	"ec", "[?] [k] [color]", "set color for given key (prompt, offset, ...)",
 	"ee", " [var]", "open cfg.editor to change the value of var",
 	"ed", "", "open editor to change the ~/.radare2rc",
+	"ed*", "", "show contents of your ~/.radare2c",
+	"ed+", "", "add or set an eval config line into your ~/.radare2c",
 	"ed-", "[!]", "delete ~/.radare2c (Use ed-! to delete without prompting)",
 	"ej", "", "list config vars in JSON",
 	"eJ", "", "list config vars in verbose JSON",
@@ -83,23 +84,10 @@ static RCoreHelpMessage help_msg_eco = {
 	NULL
 };
 
-static bool load_theme(RCore *core, const char *path) {
-	if (!r_file_exists (path)) {
-		return false;
-	}
-	core->cmdfilter = "ec ";
-	bool res = r_core_cmd_file (core, path);
-	if (res) {
-		r_cons_pal_update_event ();
-	}
-	core->cmdfilter = NULL;
-	return res;
-}
-
 static void cmd_eval_table(RCore *core, const char *input) {
 	const char fmt = *input;
 	const char *q = input;
-	RTable *t = r_core_table (core, "eval");
+	RTable *t = r_core_table_new (core, "eval");
 	RTableColumnType *typeString = r_table_type ("string");
 	RTableColumnType *typeBoolean = r_table_type ("bool");
 	r_table_add_column (t, typeBoolean, "ro", 0);
@@ -119,7 +107,7 @@ static void cmd_eval_table(RCore *core, const char *input) {
 		char *s = (fmt == 'j')
 			? r_table_tojson (t)
 			: r_table_tostring (t);
-		r_cons_printf ("%s\n", s);
+		r_cons_printf (core->cons, "%s\n", s);
 		free (s);
 	}
 	r_table_free (t);
@@ -135,7 +123,7 @@ static bool nextpal_item(RCore *core, PJ *pj, int mode, const char *file) {
 		pj_s (pj, fn);
 		break;
 	case 'l': // list
-		r_cons_println (fn);
+		r_cons_println (core->cons, fn);
 		break;
 	case 'p': // previous
 		// TODO: move logic here
@@ -164,78 +152,70 @@ static bool nextpal_item(RCore *core, PJ *pj, int mode, const char *file) {
 	return true;
 }
 
-static bool cmd_load_theme(RCore *core, const char *_arg) {
-	bool failed = false;
-	char *path;
-	if (R_STR_ISEMPTY (_arg)) {
-		return false;
+static char *get_theme_path(RCore *core, const char *theme_name) {
+	// check home directory
+	char *home = r_xdg_datadir ("cons");
+	char *theme_path = r_file_new (home, theme_name, NULL);
+	if (r_file_exists (theme_path)) {
+		// TODO read this one
+		return theme_path;
 	}
+	free (theme_path);
+	// check system directory
+	const char *r2pfx = r_sys_prefix (NULL);
+	theme_path = r_file_new (r2pfx, R2_THEMES, theme_name, NULL);
+	if (r_file_exists (theme_path)) {
+		return theme_path;
+	}
+	free (theme_path);
+	return NULL;
+}
+
+static char *get_theme_script(RCore *core, const char *theme_name) {
+	if (!strcmp (theme_name, "default")) {
+		// reserved name
+		return NULL;
+	}
+	char *theme_path = get_theme_path (core, theme_name);
+	if (theme_path) {
+		char *theme_script = r_file_slurp (theme_path, NULL);
+		free (theme_path);
+		return theme_script;
+	}
+#if WITH_STATIC_THEMES
+	const RConsTheme *theme = r_cons_themes ();
+	while (theme && theme->name) {
+		if (!strcmp (theme->name, theme_name)) {
+			return strdup (theme->script);
+		}
+		theme++;
+	}
+#endif
+	return NULL;
+}
+
+static bool cmd_load_theme(RCore *core, const char *_arg) {
 	if (!strcmp (_arg, "default")) {
 		if (_arg != core->theme) {
 			free (core->theme);
 			core->theme = strdup (_arg);
 		}
-		r_cons_pal_init (core->cons->context);
+		r_cons_pal_init (core->cons);
 		return true;
 	}
-	char *arg = strdup (_arg);
-
-	// system themes directory
-	char *home = r_xdg_datadir ("cons");
-
-	// system themes directory
-	char *tmp = r_str_newf (R_JOIN_2_PATHS (R2_THEMES, "%s"), arg);
-	path = tmp ? r_str_r2_prefix (tmp) : NULL;
-	free (tmp);
-
-	if (load_theme (core, home)) {
-		free (core->theme);
-		core->theme = strdup (arg);
-		free (core->themepath);
-		core->themepath = home;
-		home = NULL;
+	bool ret = false;
+	char *theme_script = get_theme_script (core, _arg);
+	if (R_STR_ISNOTEMPTY (theme_script)) {
+		core->cmdfilter = "ec ";
+		r_core_cmd_lines (core, theme_script);
+		r_cons_pal_reload (core->cons);
+		core->cmdfilter = NULL;
+		ret = true; // maybe the script fails?
 	} else {
-		if (load_theme (core, path)) {
-			free (core->theme);
-			core->theme = strdup (arg);
-			free (core->themepath);
-			core->themepath = path;
-			path = NULL;
-		} else {
-			if (load_theme (core, arg)) {
-				free (core->theme);
-				core->theme = strdup (arg);
-				free (core->themepath);
-				core->themepath = arg;
-				arg = NULL;
-			} else {
-				failed = true;
-			}
-		}
+		R_LOG_ERROR ("Cannot open '%s' colors theme", _arg);
 	}
-	if (failed) {
-#if WITH_STATIC_THEMES
-		const RConsTheme *theme = r_cons_themes ();
-		while (theme && theme->name) {
-			if (!strcmp (theme->name, arg)) {
-				r_core_cmd0 (core, theme->script);
-				R_FREE (arg);
-				failed = false;
-				break;
-			}
-			theme++;
-		}
-		if (failed) {
-			R_LOG_ERROR ("cannot open '%s' colorscheme", arg);
-		}
-#else
-		R_LOG_ERROR ("cannot open '%s' colorscheme", arg);
-#endif
-	}
-	free (home);
-	free (path);
-	free (arg);
-	return !failed;
+	free (theme_script);
+	return ret;
 }
 
 static void list_themes_in_path(RList *list, const char *path) {
@@ -276,7 +256,7 @@ R_API RList *r_core_list_themes(RCore *core) {
 }
 
 static void nextpal(RCore *core, int mode) {
-// TODO: use r_core_list_themes() here instead of rewalking all the time
+	// TODO: use r_core_list_themes() here instead of rewalking all the time
 	RList *files = NULL;
 	RListIter *iter;
 	const char *fn;
@@ -378,7 +358,7 @@ done:
 	files = NULL;
 	if (mode == 'j') {
 		pj_end (pj);
-		r_cons_println (pj_string (pj));
+		r_cons_println (core->cons, pj_string (pj));
 		pj_free (pj);
 	}
 }
@@ -388,14 +368,14 @@ R_API void r_core_echo(RCore *core, const char *input) {
 		char *buf = strdup (input);
 		r_base64_decode ((ut8*)buf, input + 3, -1);
 		if (*buf) {
-			r_cons_echo (buf);
+			r_cons_echo (core->cons, buf);
 		}
 		free (buf);
 	} else {
 		char *p = strchr (input, ' ');
 		if (p) {
-			r_cons_print (p + 1);
-			r_cons_newline ();
+			r_cons_print (core->cons, p + 1);
+			r_cons_newline (core->cons);
 		}
 	}
 }
@@ -412,29 +392,345 @@ static bool is_static_theme(const char *th) {
 	return false;
 }
 
+static bool cmd_ec(RCore *core, const char *input) {
+	switch (input[1]) {
+	case 'd': // "ecd"
+		r_cons_pal_init (core->cons);
+		break;
+	case '?':
+		r_core_cmd_help (core, help_msg_ec);
+		break;
+	case 'o': // "eco"
+		switch (input[2]) {
+		case 'j': // "ecoj"
+			if (input[3]) {
+				r_core_return_invalid_command (core, "ecoj", input[3]);
+			} else {
+				nextpal (core, 'j');
+			}
+			break;
+		case '*': // "eco*"
+			{
+				const char *theme_name = core->theme;
+				if (input[3]) {
+					theme_name = r_str_trim_head_ro (input + 3);
+				}
+				char *theme_script = get_theme_script (core, theme_name);
+				if (R_STR_ISNOTEMPTY (theme_script)) {
+					r_cons_printf (core->cons, "%s\n", theme_script);
+				} else {
+					R_LOG_ERROR ("Cannot find theme '%s'", theme_name);
+				}
+				free (theme_script);
+			}
+			break;
+		case '!':
+			free (r_core_editor (core, core->themepath, NULL));
+			cmd_load_theme (core, core->theme); // reload
+			break;
+		case ' ':
+			cmd_load_theme (core, r_str_trim_head_ro (input + 3));
+			break;
+		case 'o':
+			cmd_load_theme (core, core->theme);
+			break;
+		case 'c':
+		case '.':
+			r_cons_printf (core->cons, "%s\n", core->theme);
+			break;
+		case '?':
+			r_core_cmd_help (core, help_msg_eco);
+			break;
+		default:
+			{
+			RList *themes_list = r_core_list_themes (core);
+			RListIter *th_iter;
+			const char *th;
+			const RConsTheme *themes = r_cons_themes ();
+			const RConsTheme *theme = themes;
+			while (theme && theme->name) {
+				const char *th = theme->name;
+				if (input[2] == 'q') {
+					r_cons_printf (core->cons, "%s\n", th);
+				} else if (core->theme && !strcmp (core->theme, th)) {
+					r_cons_printf (core->cons, "- %s\n", th);
+				} else {
+					r_cons_printf (core->cons, "  %s\n", th);
+				}
+				theme++;
+			}
+			r_list_foreach (themes_list, th_iter, th) {
+				if (is_static_theme (th)) {
+					continue;
+				}
+				if (input[2] == 'q') {
+					r_cons_printf (core->cons, "%s\n", th);
+				} else if (core->theme && !strcmp (core->theme, th)) {
+					r_cons_printf (core->cons, "- %s\n", th);
+				} else {
+					r_cons_printf (core->cons, "  %s\n", th);
+				}
+			}
+			r_list_free (themes_list);
+			}
+			break;
+		}
+		break;
+	case 's': // "ecs"
+		r_cons_pal_show (core->cons);
+		break;
+	case '*': // "ec*"
+		r_cons_pal_list (core->cons, 1, NULL);
+		break;
+	case 'h': // echo
+		if (input[2] == 'o') {
+			r_core_echo (core, input + 3);
+		} else {
+			r_cons_pal_list (core->cons, 'h', NULL);
+		}
+		break;
+	case 'j': // "ecj"
+		r_cons_pal_list (core->cons, 'j', NULL);
+		break;
+	case 'c': // "ecc"
+		if (input[2]) {
+			r_cons_pal_list (core->cons, 'c', input + 2);
+		} else {
+			r_cons_pal_list (core->cons, 'c', r_config_get (core->config, "scr.css.prefix"));
+		}
+		break;
+	case '\0': // "ec"
+		r_cons_pal_list (core->cons, 0, NULL);
+		break;
+	case 'r': // "ecr"
+		r_cons_pal_random (core->cons);
+		break;
+	case 'n': // "ecn"
+		nextpal (core, 'n');
+		break;
+	case 'p': // "ecp"
+		nextpal (core, 'p');
+		break;
+	case 'H': { // "ecH"
+			  char *color_code = NULL;
+			  char *word = NULL;
+			  int argc = 0;
+			  int delta = (input[2])? 3: 2;
+			  char** argv = r_str_argv (r_str_trim_head_ro (input + delta), &argc);
+			  switch (input[2]) {
+			  case '?':
+				  r_core_cmd_help (core, help_msg_ecH);
+				  r_str_argv_free (argv);
+				  return false;
+			  case '-': // ecH-
+				  if (input[3] == '*') {
+					  r_meta_del (core->anal, R_META_TYPE_HIGHLIGHT, 0, UT64_MAX);
+				  } else {
+					  r_meta_del (core->anal, R_META_TYPE_HIGHLIGHT, core->addr, 1);
+					  // r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->addr, "");
+				  }
+				  r_str_argv_free (argv);
+				  return false;
+			  case '.':
+				  r_meta_print_list_in_function (core->anal, R_META_TYPE_HIGHLIGHT, 0, core->addr, NULL, NULL);
+				  r_str_argv_free (argv);
+				  return false;
+			  case '\0':
+				  r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 0, NULL, NULL);
+				  r_str_argv_free (argv);
+				  return false;
+			  case 'j':
+				  r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 'j', NULL, NULL);
+				  r_str_argv_free (argv);
+				  return false;
+			  case '*':
+				  r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, '*', NULL, NULL);
+				  r_str_argv_free (argv);
+				  return false;
+			  case ' ':
+			  case 'i': // "ecHi"
+				  if (argc) {
+					  char *dup = r_str_newf ("bgonly %s", argv[0]);
+					  color_code = r_cons_pal_parse (core->cons, dup, NULL);
+					  free (dup);
+					  if (!color_code) {
+						  R_LOG_ERROR ("Unknown color %s", argv[0]);
+						  r_str_argv_free (argv);
+						  return true;
+					  }
+				  }
+				  break;
+			  case 'w': // "ecHw"
+				  if (!argc) {
+					  r_core_cmd_help_match (core, help_msg_ecH, "ecHw");
+					  r_str_argv_free (argv);
+					  return true;
+				  }
+				  word = strdup (argv[0]);
+				  if (argc > 1) {
+					  char *dup = r_str_newf ("bgonly %s", argv[1]);
+					  color_code = r_cons_pal_parse (core->cons, dup, NULL);
+					  R_FREE (dup);
+					  if (!color_code) {
+						  R_LOG_ERROR ("Unknown color %s", argv[1]);
+						  r_str_argv_free (argv);
+						  free (word);
+						  return true;
+					  }
+				  }
+				  break;
+			  default:
+				  R_LOG_INFO ("See ecH?");
+				  r_str_argv_free (argv);
+				  return true;
+			  }
+			  r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->addr, "");
+			  const char *str = r_meta_get_string (core->anal, R_META_TYPE_HIGHLIGHT, core->addr);
+			  char *dup = r_str_newf ("%s \"%s%s\"", r_str_get (str), r_str_get (word),
+					  color_code ? color_code : core->cons->context->pal.wordhl);
+			  r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->addr, dup);
+			  r_str_argv_free (argv);
+			  free (color_code);
+			  R_FREE (word);
+			  R_FREE (dup);
+		  }
+		  break;
+	case ' ':
+		  {
+			 char *p = strdup (input + 2);
+			 char *q = strchr (p, '=');
+			 if (!q) {
+				 q = strchr (p, ' ');
+			 }
+			 if (q) {
+				 // Set color
+				 *q++ = 0;
+				 if (r_cons_pal_set (core->cons, p, q)) {
+					 r_cons_pal_reload (core->cons);
+				 }
+			 } else {
+				 char color[32] = {0};
+				 RColor rcolor = r_cons_pal_get (core->cons, p);
+				 r_cons_rgb_str (core->cons, color, sizeof (color), &rcolor);
+				 if (*color) {
+					 eprintf ("(%s)(%sCOLOR"Color_RESET")\n", p, color);
+				 } else {
+					 R_LOG_ERROR ("Invalid palette color '%s'", p);
+				 }
+			 }
+			 free (p);
+		 }
+		break;
+	default:
+		r_core_return_invalid_command (core, "ec", input[1]);
+		break;
+	}
+	return true;
+}
+
+static void r2rc_set(RCore *core, const char * R_NULLABLE k, const char * R_NULLABLE v) {
+	char *rcfile = r_file_home (".radare2rc");
+	char *rcdata = r_file_slurp (rcfile, NULL);
+	if (k) {
+		char *line;
+		RListIter *iter;
+		RList *lines = r_str_split_list (rcdata, "\n", 0);
+		RStrBuf *sb = r_strbuf_new ("");
+		bool found = false;
+		char *kk = r_str_newf ("e %s", k);
+		r_list_foreach (lines, iter, line) {
+			const char *oline = line;
+			if (*oline == '\'') {
+				oline++;
+			}
+			if (r_str_startswith (oline, kk)) {
+				if (v) {
+					if (!found && *v) {
+						r_strbuf_appendf (sb, "'e %s=%s\n", k, v);
+					}
+				} else {
+					r_cons_println (core->cons, line);
+				}
+				found = true;
+			} else {
+				r_strbuf_appendf (sb, "%s\n", line);
+			}
+		}
+		free (kk);
+		if (v) {
+			if (!found && *v) {
+				r_strbuf_appendf (sb, "'e %s=%s\n", k, v);
+			}
+			char *out = r_strbuf_drain (sb);
+			r_list_free (lines);
+			r_str_trim (out);
+			r_file_dump (rcfile, (const ut8*)out, -1, false);
+			free (out);
+		} else {
+			r_strbuf_free (sb);
+		}
+	} else {
+		r_cons_println (core->cons, rcdata);
+	}
+	free (rcdata);
+	free (rcfile);
+}
+
+static void cmd_eplus(RCore *core, const char *input) {
+	char *s = r_str_trim_dup (input);
+	char *eq = strchr (s, '=');
+	if (*s) {
+		if (eq) {
+			r_str_trim (s);
+			*eq++ = 0;
+			r_str_trim (eq);
+			const char *k = s;
+			const char *v = eq;
+			r2rc_set (core, k, v);
+		} else {
+			r2rc_set (core, s, NULL);
+		}
+	} else {
+		r2rc_set (core, NULL, NULL);
+	}
+	free (s);
+}
+
+static void core_config_list(RCore *core, const char *str, int rad) {
+	char *res = r_config_list (core->config, str, rad);
+	r_cons_print (core->cons, res);
+	free (res);
+}
+
+static void core_config_eval(RCore *core, const char *input, bool uhm) {
+	char *res = r_config_eval (core->config, input, uhm, NULL);
+	r_cons_print (core->cons, res);
+	free (res);
+}
+
 static int cmd_eval(void *data, const char *input) {
 	RCore *core = (RCore *)data;
+	RCons *cons = core->cons;
 	switch (input[0]) {
 	case '\0': // "e"
-		r_config_list (core->config, NULL, 0);
+		core_config_list (core, NULL, 0);
 		break;
 	case '?': // "e?"
-	default:
 		switch (input[1]) {
 		case '\0': r_core_cmd_help (core, help_msg_e); break;
-		case '?': r_config_list (core->config, input + 2, 2); break;
-		default: r_config_list (core->config, input + 1, 3); break;
+		case '?': core_config_list (core, input + 2, 2); break;
+		default: core_config_list (core, input + 1, 3); break;
 		}
 		break;
 	case 't': // "et"
 		if (input[1] == 'a') {
-			r_cons_printf ("%s\n", (r_num_rand (10) % 2)? "wen": "son");
+			r_cons_printf (cons, "%s\n", (r_num_rand (10) % 2)? "wen": "son");
 		} else if (input[1] == ' ' && input[2]) {
 			RConfigNode *node = r_config_node_get (core->config, input+2);
 			if (node) {
 				const char *type = r_config_node_type (node);
 				if (type && *type) {
-					r_cons_println (type);
+					r_cons_println (cons, type);
 				}
 			}
 		} else {
@@ -445,7 +741,8 @@ static int cmd_eval(void *data, const char *input) {
 		if (strchr (input, '?')) {
 			r_core_cmd_help_contains (core, help_msg_e, "en");
 			break;
-		} else if (!strcmp (input + 1, "vj")) {
+		}
+		if (!strcmp (input + 1, "vj")) {
 			char **e = r_sys_get_environ ();
 			PJ *pj = r_core_pj_new (core);
 			pj_o (pj);
@@ -463,13 +760,13 @@ static int cmd_eval(void *data, const char *input) {
 			}
 			pj_end (pj);
 			char *s = pj_drain (pj);
-			r_cons_println (s);
+			r_cons_println (cons, s);
 			free (s);
 		} else if (!strcmp (input + 1, "v*")) {
 			char **e = r_sys_get_environ ();
 			if (e != NULL) {
 				while (*e) {
-					r_cons_printf ("%%%s\n", *e);
+					r_cons_printf (cons, "%%%s\n", *e);
 					e++;
 				}
 			}
@@ -480,13 +777,13 @@ static int cmd_eval(void *data, const char *input) {
 			}
 			char *p = r_sys_getenv (var);
 			if (p) {
-				r_cons_println (p);
+				r_cons_println (cons, p);
 				free (p);
-			} else {
+			} else if (!var || !*var) {
 				char **e = r_sys_get_environ ();
 				if (e != NULL) {
 					while (*e) {
-						r_cons_println (*e);
+						r_cons_println (cons, *e);
 						e++;
 					}
 				}
@@ -515,218 +812,35 @@ static int cmd_eval(void *data, const char *input) {
 		// XXX we need headers for the cmd_xxx files.
 		return cmd_quit (data, "");
 	case 'J': // "eJ"
-		r_config_list (core->config, NULL, 'J');
+		core_config_list (core, NULL, 'J');
 		break;
 	case 'j': // "ej"
-		r_config_list (core->config, NULL, 'j');
+		core_config_list (core, NULL, 'j');
 		break;
 	case 'v': // verbose
-		r_config_list (core->config, r_str_trim_head_ro (input + 1), 'v');
+		core_config_list (core, r_str_trim_head_ro (input + 1), 'v');
 		break;
 	case 'q': // quiet list of eval keys
-		r_config_list (core->config, NULL, 'q');
+		core_config_list (core, NULL, 'q');
 		break;
 	case 'c': // "ec"
-		switch (input[1]) {
-		case 'd': // "ecd"
-			r_cons_pal_init (core->cons->context);
-			break;
-		case '?':
-			r_core_cmd_help (core, help_msg_ec);
-			break;
-		case 'o': // "eco"
-			if (input[2] == 'j') {
-				nextpal (core, 'j');
-			} else if (input[2] == '*') {
-				r_core_cmd_callf (core, "cat %s", core->themepath);
-			} else if (input[2] == '!') {
-				char *res = r_core_editor (core, core->themepath, NULL);
-				free (res);
-				cmd_load_theme (core, core->theme); // reload
-			} else if (input[2] == ' ') {
-				cmd_load_theme (core, input + 3);
-			} else if (input[2] == 'o') {
-				cmd_load_theme (core, core->theme);
-			} else if (input[2] == 'c' || input[2] == '.') {
-				r_cons_printf ("%s\n", core->theme);
-			} else if (input[2] == '?') {
-				r_core_cmd_help (core, help_msg_eco);
-			} else {
-				RList *themes_list = r_core_list_themes (core);
-				RListIter *th_iter;
-				const char *th;
-				const RConsTheme *themes = r_cons_themes ();
-				const RConsTheme *theme = themes;
-				while (theme && theme->name) {
-					const char *th = theme->name;
-					if (input[2] == 'q') {
-						r_cons_printf ("%s\n", th);
-					} else if (core->theme && !strcmp (core->theme, th)) {
-						r_cons_printf ("- %s\n", th);
-					} else {
-						r_cons_printf ("  %s\n", th);
-					}
-					theme++;
-				}
-				r_list_foreach (themes_list, th_iter, th) {
-					if (is_static_theme (th)) {
-						continue;
-					}
-					if (input[2] == 'q') {
-						r_cons_printf ("%s\n", th);
-					} else if (core->theme && !strcmp (core->theme, th)) {
-						r_cons_printf ("- %s\n", th);
-					} else {
-						r_cons_printf ("  %s\n", th);
-					}
-				}
-				r_list_free (themes_list);
-			}
-			break;
-		case 's': // "ecs"
-			r_cons_pal_show ();
-			break;
-		case '*': // "ec*"
-			r_cons_pal_list (1, NULL);
-			break;
-		case 'h': // echo
-			if (input[2] == 'o') {
-				r_core_echo (core, input + 3);
-			} else {
-				r_cons_pal_list ('h', NULL);
-			}
-			break;
-		case 'j': // "ecj"
-			r_cons_pal_list ('j', NULL);
-			break;
-		case 'c': // "ecc"
-			r_cons_pal_list ('c', input + 2);
-			break;
-		case '\0': // "ec"
-			r_cons_pal_list (0, NULL);
-			break;
-		case 'r': // "ecr"
-			r_cons_pal_random ();
-			break;
-		case 'n': // "ecn"
-			nextpal (core, 'n');
-			break;
-		case 'p': // "ecp"
-			nextpal (core, 'p');
-			break;
-		case 'H': { // "ecH"
-			char *color_code = NULL;
-			char *word = NULL;
-			int argc = 0;
-			int delta = (input[2])? 3: 2;
-			char** argv = r_str_argv (r_str_trim_head_ro (input + delta), &argc);
-			switch (input[2]) {
-			case '?':
-				r_core_cmd_help (core, help_msg_ecH);
-				r_str_argv_free (argv);
-				return false;
-			case '-': // ecH-
-				if (input[3] == '*') {
-					r_meta_del (core->anal, R_META_TYPE_HIGHLIGHT, 0, UT64_MAX);
-				} else {
-					r_meta_del (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, 1);
-					// r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, "");
-				}
-				r_str_argv_free (argv);
-				return false;
-			case '.':
-				r_meta_print_list_in_function (core->anal, R_META_TYPE_HIGHLIGHT, 0, core->offset, NULL);
-				r_str_argv_free (argv);
-				return false;
-			case '\0':
-				r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 0, NULL);
-				r_str_argv_free (argv);
-				return false;
-			case 'j':
-				r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, 'j', NULL);
-				r_str_argv_free (argv);
-				return false;
-			case '*':
-				r_meta_print_list_all (core->anal, R_META_TYPE_HIGHLIGHT, '*', NULL);
-				r_str_argv_free (argv);
-				return false;
-			case ' ':
-			case 'i': // "ecHi"
-				if (argc) {
-					char *dup = r_str_newf ("bgonly %s", argv[0]);
-					color_code = r_cons_pal_parse (dup, NULL);
-					R_FREE (dup);
-					if (!color_code) {
-						R_LOG_ERROR ("Unknown color %s", argv[0]);
-						r_str_argv_free (argv);
-						return true;
-					}
-				}
-				break;
-			case 'w': // "ecHw"
-				if (!argc) {
-					r_core_cmd_help_match (core, help_msg_ecH, "ecHw");
-					r_str_argv_free (argv);
-					return true;
-				}
-				word = strdup (argv[0]);
-				if (argc > 1) {
-					char *dup = r_str_newf ("bgonly %s", argv[1]);
-					color_code = r_cons_pal_parse (dup, NULL);
-					R_FREE (dup);
-					if (!color_code) {
-						R_LOG_ERROR ("Unknown color %s", argv[1]);
-						r_str_argv_free (argv);
-						free (word);
-						return true;
-					}
-				}
-				break;
-			default:
-				R_LOG_INFO ("See ecH?");
-				r_str_argv_free (argv);
-				return true;
-			}
-			r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, "");
-			const char *str = r_meta_get_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset);
-			char *dup = r_str_newf ("%s \"%s%s\"", r_str_get (str), r_str_get (word),
-				color_code ? color_code : r_cons_singleton ()->context->pal.wordhl);
-			r_meta_set_string (core->anal, R_META_TYPE_HIGHLIGHT, core->offset, dup);
-			r_str_argv_free (argv);
-			R_FREE (word);
-			R_FREE (dup);
-			break;
-			  }
-		default: {
-				 char *p = strdup (input + 2);
-				 char *q = strchr (p, '=');
-				 if (!q) {
-					 q = strchr (p, ' ');
-				 }
-				 if (q) {
-					 // Set color
-					 *q++ = 0;
-					 if (r_cons_pal_set (p, q)) {
-						 r_cons_pal_update_event ();
-					 }
-				 } else {
-					 char color[32];
-					 RColor rcolor = r_cons_pal_get (p);
-					 r_cons_rgb_str (color, sizeof (color), &rcolor);
-					 eprintf ("(%s)(%sCOLOR"Color_RESET")\n", p, color);
-				 }
-				 free (p);
-			 }
-		}
-		break;
+		return cmd_ec (core, input);
 	case 'd': // "ed"
 		if (input[1] == '?') {
 			r_core_cmd_help_contains (core, help_msg_e, "ed");
+		} else if (input[1] == '*') {
+			char *file = r_file_home (".radare2rc");
+			char *data = r_file_slurp (file, NULL);
+			r_cons_println (cons, data);
+			free (data);
+			free (file);
+		} else if (input[1] == '+') {
+			cmd_eplus (core, input + 2);
 		} else if (input[1] == '-') { // "ed-"
 			const bool prompt = (input[2] != '!');
 			char *file = r_file_home (".radare2rc");
 			if (file) {
-				const bool rmfile = !prompt || r_cons_yesno ('n', "Do you want to delete ~/.radare2? (Y/n)");
+				const bool rmfile = !prompt || r_cons_yesno (cons, 'n', "Do you want to delete ~/.radare2? (Y/n)");
 				if (rmfile) {
 					r_file_rm (file);
 				}
@@ -734,11 +848,11 @@ static int cmd_eval(void *data, const char *input) {
 			}
 		} else {
 			char *file = r_file_home (".radare2rc");
-			if (r_cons_is_interactive ()) {
+			if (r_cons_is_interactive (cons)) {
 				r_file_touch (file);
-				char *res = r_cons_editor (file, NULL);
+				char *res = r_cons_editor (cons, file, NULL);
 				if (res) {
-					if (r_cons_yesno ('y', "Reload? (Y/n)")) {
+					if (r_cons_yesno (cons, 'y', "Reload? (Y/n)")) {
 						r_core_run_script (core, file);
 					}
 				}
@@ -748,6 +862,9 @@ static int cmd_eval(void *data, const char *input) {
 			free (file);
 		}
 		break;
+	case '+': // "e+"
+		cmd_eplus (core, input + 1);
+		break;
 	case 'e': // "ee"
 		if (input[1] == ' ') {
 			char *p;
@@ -774,14 +891,14 @@ static int cmd_eval(void *data, const char *input) {
 		}
 		break;
 	case 's': // "es"
-		r_config_list (core->config, (input[1])? input + 1: NULL, 's');
+		core_config_list (core, (input[1])? input + 1: NULL, 's');
 		break;
 	case '-': // "e-"
 		r_core_config_init (core);
 		//eprintf ("BUG: 'e-' command locks the eval hashtable. patches are welcome :)\n");
 		break;
 	case '*': // "e*"
-		r_config_list (core->config, NULL, 1);
+		core_config_list (core, NULL, 1);
 		break;
 	case 'r': // "er"
 		if (input[1]) {
@@ -794,24 +911,31 @@ static int cmd_eval(void *data, const char *input) {
 		}
 		break;
 	case ':': // "e:"
-		r_config_eval (core->config, input + 1, true);
+		core_config_eval (core, input + 1, true);
 		break;
 	case ',': // "e,"
 		cmd_eval_table (core, input + 1);
 		break;
 	case '.': // "e "
 	case ' ': // "e "
-		if (r_str_endswith (input, ".") && !r_str_endswith (input, "..")) {
-			r_config_list (core->config, input + 1, 0);
-		} else if (r_str_endswith (input, ".?")) {
-			char *w = r_str_ndup (input, strlen (input) - 1);
-			r_config_list (core->config, w, 2);
-			free (w);
+		if (strchr (input, '=')) {
+			core_config_eval (core, r_str_trim_head_ro (input + 1), false);
 		} else {
-			// XXX we cant do "e cmd.gprompt=dr=", because the '=' is a token, and quotes dont affect him
-			r_config_eval (core->config, r_str_trim_head_ro (input + 1), false);
+			if (r_str_endswith (input, ".") && !r_str_endswith (input, "..")) {
+				core_config_list (core, input + 1, 0);
+			} else if (r_str_endswith (input, ".?")) {
+				char *w = r_str_ndup (input, strlen (input) - 1);
+				core_config_list (core, w, 2);
+				free (w);
+			} else {
+				// XXX we cant do "e cmd.gprompt=dr=", because the '=' is a token, and quotes dont affect him
+				core_config_eval (core, r_str_trim_head_ro (input + 1), false);
+			}
 		}
 		break;
+	default:
+		r_core_return_invalid_command (core, "e", *input);
+		break;
 	}
 	return 0;
 }
