@@ -159,7 +159,7 @@ static int cmd_project(void *data, const char *input) {
 			r_core_return_code (core, success? 0: 1);
 		} else {
 			if (R_STR_ISNOTEMPTY (str)) {
-				r_cons_println (str);
+				r_cons_println (core->cons, str);
 			}
 		}
 		break;
@@ -350,9 +350,9 @@ static int cmd_project(void *data, const char *input) {
 					char *str = r_core_project_notes_file (core, fileproject);
 					if (str) {
 						char *data = r_file_slurp (str, &len);
-						char *res = r_base64_encode_dyn (data, (int)len);
+						char *res = r_base64_encode_dyn ((const ut8*)data, (int)len);
 						if (res) {
-							r_cons_println (res);
+							r_cons_println (core->cons, res);
 							free (res);
 						}
 						free (data);
@@ -360,7 +360,7 @@ static int cmd_project(void *data, const char *input) {
 					}
 				} else if (input[2] == ' ') {
 					/* set base64 string */
-					ut8 *data = r_base64_decode_dyn (input + 3, -1);
+					ut8 *data = r_base64_decode_dyn (input + 3, -1, NULL);
 					if (data) {
 						char *str = r_core_project_notes_file (core, fileproject);
 						if (str) {
@@ -381,7 +381,7 @@ static int cmd_project(void *data, const char *input) {
 				char *str = r_core_project_notes_file (core, fileproject);
 				char *data = r_file_slurp (str, NULL);
 				if (data) {
-					r_cons_println (data);
+					r_cons_println (core->cons, data);
 					free (data);
 				}
 				free (str);
@@ -394,20 +394,20 @@ static int cmd_project(void *data, const char *input) {
 		if (R_STR_ISNOTEMPTY (file)) {
 			char *prj_name = r_core_project_name (core, file);
 			if (!R_STR_ISEMPTY (prj_name)) {
-				r_cons_println (prj_name);
+				r_cons_println (core->cons, prj_name);
 				free (prj_name);
 			}
 		} else if (r_project_is_loaded (core->prj)) {
 			if (R_STR_ISNOTEMPTY (core->prj->name)) {
-				r_cons_println (core->prj->name);
+				r_cons_println (core->cons, core->prj->name);
 			}
 			if (R_STR_ISNOTEMPTY (core->prj->path)) {
-				r_cons_println (core->prj->path);
+				r_cons_println (core->cons, core->prj->path);
 			}
 		}
 		break;
 	case '.': // "P."
-		r_cons_printf ("%s\n", fileproject);
+		r_cons_printf (core->cons, "%s\n", fileproject);
 		break;
 	case 'l':
 		r_core_project_list (core, input[1]);
