@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2024 - pancake */
+/* radare - LGPL - Copyright 2009-2025 - pancake */
 
 #if R_INCLUDE_BEGIN
 
@@ -8,7 +8,7 @@ static RCoreHelpMessage help_msg_s = {
 	"Usage: s", "", " # Help for the seek commands. See ?$? to see all variables",
 	"s", "", "print current address",
 	"s", " addr", "seek to address",
-	"s.", "[?]hexoff", "seek honoring a base from core->offset",
+	"s.", "[?]hexoff", "seek honoring a base from core->addr",
 	"s:", "pad", "print current address with N padded zeros (defaults to 8)",
 	"s-", "", "undo seek",
 	"s-*", "", "reset undo seek history",
@@ -109,16 +109,16 @@ static void printPadded(RCore *core, int pad) {
 		pad = 8;
 	}
 	char *fmt = r_str_newf ("0x%%0%d" PFMT64x, pad);
-	char *off = r_str_newf (fmt, core->offset);
-	r_cons_printf ("%s\n", off);
+	char *off = r_str_newf (fmt, core->addr);
+	r_cons_printf (core->cons, "%s\n", off);
 	free (off);
 	free (fmt);
 }
 
 static void __get_current_line(RCore *core) {
 	if (core->print->lines_cache_sz > 0) {
-		int curr = r_util_lines_getline (core->print->lines_cache, core->print->lines_cache_sz, core->offset);
-		r_cons_printf ("%d\n", curr);
+		int curr = r_util_lines_getline (core->print->lines_cache, core->print->lines_cache_sz, core->addr);
+		r_cons_printf (core->cons, "%d\n", curr);
 	}
 }
 
@@ -131,7 +131,7 @@ static void __seek_line_absolute(RCore *core, int numline) {
 }
 
 static void __seek_line_relative(RCore *core, int numlines) {
-	int curr = r_util_lines_getline (core->print->lines_cache, core->print->lines_cache_sz, core->offset);
+	int curr = r_util_lines_getline (core->print->lines_cache, core->print->lines_cache_sz, core->addr);
 	if (numlines > 0 && curr + numlines >= core->print->lines_cache_sz - 1) {
 		R_LOG_ERROR ("Line must be < %d", core->print->lines_cache_sz - 1);
 	} else if (numlines < 0 && curr + numlines < 1) {
@@ -153,9 +153,9 @@ R_API int r_core_lines_currline(RCore *core) {  // make priv8 again
 
 	while (imin <= imax) {
 		imid = imin + ((imax - imin) / 2);
-		if (core->print->lines_cache[imid] == core->offset) {
+		if (core->print->lines_cache[imid] == core->addr) {
 			return imid;
-		} else if (core->print->lines_cache[imid] < core->offset) {
+		} else if (core->print->lines_cache[imid] < core->addr) {
 			imin = imid + 1;
 		} else {
 			imax = imid - 1;
@@ -171,11 +171,9 @@ R_API int r_core_lines_initcache(RCore *core, ut64 start_addr, ut64 end_addr) {
 		return -1;
 	}
 
+	ut64 *lines_cache = R_NEWS0 (ut64, bsz);
 	free (core->print->lines_cache);
-	core->print->lines_cache = R_NEWS0 (ut64, bsz);
-	if (!core->print->lines_cache) {
-		return -1;
-	}
+	core->print->lines_cache = lines_cache;
 
 	ut64 baddr = r_config_get_i (core->config, "bin.baddr");
 
@@ -185,9 +183,9 @@ R_API int r_core_lines_initcache(RCore *core, ut64 start_addr, ut64 end_addr) {
 	if (!buf) {
 		return -1;
 	}
-	r_cons_break_push (NULL, NULL);
+	r_cons_break_push (core->cons, NULL, NULL);
 	while (off < end_addr) {
-		if (r_cons_is_breaked ()) {
+		if (r_cons_is_breaked (core->cons)) {
 			break;
 		}
 		r_io_read_at (core->io, off, (ut8 *) buf, bsz);
@@ -215,7 +213,7 @@ R_API int r_core_lines_initcache(RCore *core, ut64 start_addr, ut64 end_addr) {
 		off += bsz;
 	}
 	free (buf);
-	r_cons_break_pop ();
+	r_cons_break_pop (core->cons);
 	return line_count;
 }
 
@@ -223,7 +221,7 @@ static void seek_to_register(RCore *core, const char *input, bool is_silent) {
 	if (r_config_get_b (core->config, "cfg.debug")) {
 		ut64 off = r_debug_reg_get (core->dbg, input);
 		if (!is_silent) {
-			r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+			r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 		}
 		r_core_seek (core, off, true);
 	} else {
@@ -232,7 +230,7 @@ static void seek_to_register(RCore *core, const char *input, bool is_silent) {
 		ut64 off = r_debug_reg_get (core->dbg, input);
 		core->dbg->reg = orig;
 		if (!is_silent) {
-			r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+			r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 		}
 		r_core_seek (core, off, true);
 	}
@@ -257,7 +255,7 @@ static int cmd_sort(void *data, const char *input) { // "sort"
 		} else {
 			char *res = r_syscmd_sort (arg);
 			if (res) {
-				r_cons_print (res);
+				r_cons_print (core->cons, res);
 				free (res);
 			}
 		}
@@ -269,24 +267,24 @@ static int cmd_sort(void *data, const char *input) { // "sort"
 static int cmd_seek_opcode_backward(RCore *core, int numinstr) {
 	int i, val = 0;
 	// N previous instructions
-	ut64 addr = core->offset;
+	ut64 addr = core->addr;
 	int ret = 0;
-	if (r_core_prevop_addr (core, core->offset, numinstr, &addr)) {
-		ret = core->offset - addr;
+	if (r_core_prevop_addr (core, core->addr, numinstr, &addr)) {
+		ret = core->addr - addr;
 	} else {
 #if 0
 		// core_asm_bwdis_len is buggy as hell we should kill it. seems like prevop_addr
 		// works as expected, because is the one used from visual
 		ret = r_core_asm_bwdis_len (core, &instr_len, &addr, numinstr);
 #endif
-		addr = core->offset;
+		addr = core->addr;
 		const int mininstrsize = r_anal_archinfo (core->anal, R_ARCH_INFO_MINOP_SIZE);
 		for (i = 0; i < numinstr; i++) {
 			ut64 prev_addr = r_core_prevop_addr_force (core, addr, 1);
 			if (prev_addr == UT64_MAX) {
 				prev_addr = addr - mininstrsize;
 			}
-			if (prev_addr == UT64_MAX || prev_addr >= core->offset) {
+			if (prev_addr == UT64_MAX || prev_addr >= core->addr) {
 				break;
 			}
 			RAnalOp op = {0};
@@ -310,7 +308,7 @@ static int cmd_seek_opcode_forward(RCore *core, int n) {
 	int i, ret, val = 0;
 	for (val = i = 0; i < n; i++) {
 		RAnalOp op;
-		ret = r_anal_op (core->anal, &op, core->offset, core->block,
+		ret = r_anal_op (core->anal, &op, core->addr, core->block,
 			core->blocksize, R_ARCH_OP_MASK_BASIC);
 		if (ret < 1) {
 			ret = 1;
@@ -339,10 +337,10 @@ static void cmd_seek_opcode(RCore *core, const char *input) {
 static int cmd_seek(void *data, const char *input) {
 	RCore *core = (RCore *) data;
 	char *cmd, *p;
-	ut64 off = core->offset;
+	ut64 off = core->addr;
 
 	if (!*input) {
-		r_cons_printf ("0x%"PFMT64x "\n", core->offset);
+		r_cons_printf (core->cons, "0x%"PFMT64x "\n", core->addr);
 		return 0;
 	}
 	char *ptr;
@@ -350,7 +348,7 @@ static int cmd_seek(void *data, const char *input) {
 		char *dup = strdup (input);
 		dup[ptr - input] = '\x00';
 		off = r_num_math (core->num, dup + 1);
-		core->offset = off;
+		core->addr = off;
 		free (dup);
 	}
 	const char *inputnum = strchr (input, ' ');
@@ -398,50 +396,50 @@ static int cmd_seek(void *data, const char *input) {
 	case 'd': // "sd"
 		{
 			st64 delta;
-			ut64 at = core->offset;
+			ut64 at = core->addr;
 			char *ro = r_core_get_reloff (core, RELOFF_TO_FLAG, at, &delta);
 			if (ro) {
-				r_cons_printf ("flag %s+0x%"PFMT64x"\n", ro, delta);
+				r_cons_printf (core->cons, "flag %s+0x%"PFMT64x"\n", ro, delta);
 				free (ro);
 			}
 			ro = r_core_get_reloff (core, RELOFF_TO_FUNC, at, &delta);
 			if (ro) {
-				r_cons_printf ("func %s+0x%"PFMT64x"\n", ro, delta);
+				r_cons_printf (core->cons, "func %s+0x%"PFMT64x"\n", ro, delta);
 				free (ro);
 			}
 			ro = r_core_get_reloff (core, RELOFF_TO_MAPS, at, &delta);
 			if (ro) {
-				r_cons_printf ("maps %s+0x%"PFMT64x"\n", ro, delta);
+				r_cons_printf (core->cons, "maps %s+0x%"PFMT64x"\n", ro, delta);
 				free (ro);
 			}
 			ro = r_core_get_reloff (core, RELOFF_TO_FILE, at, &delta);
 			if (ro) {
-				r_cons_printf ("file %s+0x%"PFMT64x"\n", ro, delta);
+				r_cons_printf (core->cons, "file %s+0x%"PFMT64x"\n", ro, delta);
 				free (ro);
 			}
 			ro = r_core_get_reloff (core, RELOFF_TO_FMAP, at, &delta);
 			if (ro) {
-				r_cons_printf ("fmap %s+0x%"PFMT64x"\n", ro, delta);
+				r_cons_printf (core->cons, "fmap %s+0x%"PFMT64x"\n", ro, delta);
 				free (ro);
 			}
 			ro = r_core_get_reloff (core, RELOFF_TO_LIBS, at, &delta);
 			if (ro) {
-				r_cons_printf ("libs %s+0x%"PFMT64x"\n", ro, delta);
+				r_cons_printf (core->cons, "libs %s+0x%"PFMT64x"\n", ro, delta);
 				free (ro);
 			}
 			ro = r_core_get_reloff (core, RELOFF_TO_SYMB, at, &delta);
 			if (ro) {
-				r_cons_printf ("symb %s+0x%"PFMT64x"\n", ro, delta);
+				r_cons_printf (core->cons, "symb %s+0x%"PFMT64x"\n", ro, delta);
 				free (ro);
 			}
 			ro = r_core_get_reloff (core, RELOFF_TO_SECT, at, &delta);
 			if (ro) {
-				r_cons_printf ("sect %s+0x%"PFMT64x"\n", ro, delta);
+				r_cons_printf (core->cons, "sect %s+0x%"PFMT64x"\n", ro, delta);
 				free (ro);
 			}
 			ro = r_core_get_reloff (core, RELOFF_TO_DMAP, at, &delta);
 			if (ro) {
-				r_cons_printf ("dmap %s+0x%"PFMT64x"\n", ro, delta);
+				r_cons_printf (core->cons, "dmap %s+0x%"PFMT64x"\n", ro, delta);
 				free (ro);
 			}
 		}
@@ -456,7 +454,7 @@ static int cmd_seek(void *data, const char *input) {
 			r_interval_tree_foreach (&core->anal->meta, it, meta) {
 				if (meta->type == R_META_TYPE_COMMENT && !strcmp (meta->str, input + 2)) {
 					if (!silent) {
-						r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+						r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 					}
 					r_core_seek (core, off, true);
 					r_core_block_read (core);
@@ -486,13 +484,13 @@ static int cmd_seek(void *data, const char *input) {
 		const char *trimin = r_str_trim_head_ro (input);
 		ut64 addr = r_num_math (core->num, trimin);
 		if (core->num->nc.errors) { // TODO expose an api for this char *r_num_failed();
-			if (r_cons_singleton ()->context->is_interactive) {
+			if (core->cons->context->is_interactive) {
 				R_LOG_ERROR ("Cannot seek to unknown address '%s'", trimin);
 			}
 			break;
 		}
 		if (!silent) {
-			r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+			r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 		}
 		r_core_seek (core, addr, true);
 		r_core_block_read (core);
@@ -525,7 +523,7 @@ static int cmd_seek(void *data, const char *input) {
 		case 'r':
 		case '/':
 		case 'x':
-			r_config_set_i (core->config, "search.from", core->offset + 1);
+			r_config_set_i (core->config, "search.from", core->addr + 1);
 			r_config_set_i (core->config, "search.maxhits", 1);
 			r_core_cmdf (core, "s+1; %s; s-1; s %s%d_0; f-%s%d_0",
 				input, pfx, kwidx, pfx, kwidx);
@@ -548,11 +546,11 @@ static int cmd_seek(void *data, const char *input) {
 			for (input++; *input == '.'; input++) {
 				;
 			}
-			r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+			r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 			r_core_seek_base (core, input);
 		} else {
 			// just re-read the current block
-			r_core_seek (core, core->offset, true);
+			r_core_seek (core, core->addr, true);
 			r_core_block_read (core);
 		}
 		break;
@@ -571,9 +569,9 @@ static int cmd_seek(void *data, const char *input) {
 
 					RFlagItem *f = r_flag_get_at (core->flags, undo->off, true);
 					if (f) {
-						if (f->offset != undo->off) {
+						if (f->addr != undo->off) {
 							name = r_str_newf ("%s+%d", f->name,
-									(int)(undo->off- f->offset));
+									(int)(undo->off - f->addr));
 						} else {
 							name = strdup (f->name);
 						}
@@ -611,7 +609,7 @@ static int cmd_seek(void *data, const char *input) {
 			}
 			pj_end (pj);
 			char *s = pj_drain (pj);
-			r_cons_printf ("%s\n", s);
+			r_cons_printf (core->cons, "%s\n", s);
 			free (s);
 			r_list_free (addrs);
 			r_list_free (names);
@@ -636,26 +634,26 @@ static int cmd_seek(void *data, const char *input) {
 
 					RFlagItem *f = r_flag_get_at (core->flags, undo->off, true);
 					if (f) {
-						if (f->offset != undo->off) {
+						if (f->addr != undo->off) {
 							name = r_str_newf ("%s + %d\n", f->name,
-									(int)(undo->off - f->offset));
+									(int)(undo->off - f->addr));
 						} else {
 							name = strdup (f->name);
 						}
 					}
 					if (mode) {
-						r_cons_printf ("0x%"PFMT64x" %s\n", undo->off, r_str_get (name));
+						r_cons_printf (core->cons, "0x%"PFMT64x" %s\n", undo->off, r_str_get (name));
 					} else {
 						if (!name) {
 							name = r_str_newf ("0x%"PFMT64x, undo->off);
 						}
-						r_cons_printf ("%s%s", name, iter->n? " > ":"");
+						r_cons_printf (core->cons, "%s%s", name, iter->n? " > ":"");
 					}
 					free (name);
 				}
 				r_list_free (list);
 				if (!mode) {
-					r_cons_newline ();
+					r_cons_newline (core->cons);
 				}
 			}
 		}
@@ -672,7 +670,7 @@ static int cmd_seek(void *data, const char *input) {
 			}
 			// int delta = (input[1] == '+')? core->blocksize: off;
 			if (!silent) {
-				r_io_sundo_push (core->io, core->offset,
+				r_io_sundo_push (core->io, core->addr,
 					r_print_get_cursor (core->print));
 			}
 			r_core_seek_delta (core, delta);
@@ -692,7 +690,7 @@ static int cmd_seek(void *data, const char *input) {
 			break;
 		case 0: // "s-"
 			{
-				RIOUndos *undo = r_io_sundo (core->io, core->offset);
+				RIOUndos *undo = r_io_sundo (core->io, core->addr);
 				if (undo) {
 					r_core_seek (core, undo->off, false);
 					r_core_block_read (core);
@@ -711,7 +709,7 @@ static int cmd_seek(void *data, const char *input) {
 					}
 				}
 				if (!silent) {
-					r_io_sundo_push (core->io, core->offset,
+					r_io_sundo_push (core->io, core->addr,
 							r_print_get_cursor (core->print));
 				}
 				r_core_seek_delta (core, delta);
@@ -727,7 +725,7 @@ static int cmd_seek(void *data, const char *input) {
 			__core_cmd_search_backward_prelude (core, true, true);
 		} else {
 			if (!silent) {
-				r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+				r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 			}
 			const char *nkey = (input[1] == ' ')
 				? input + 2
@@ -742,7 +740,7 @@ static int cmd_seek(void *data, const char *input) {
 			__core_cmd_search_backward_prelude (core, true, false);
 		} else {
 			if (!silent) {
-				r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+				r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 			}
 			const char *nkey = (input[1] == ' ')
 				? input + 2
@@ -766,7 +764,7 @@ static int cmd_seek(void *data, const char *input) {
 				free (cmd);
 			}
 			if (!silent) {
-				r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+				r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 			}
 			r_core_seek_align (core, off, 0);
 		} else {
@@ -776,10 +774,10 @@ static int cmd_seek(void *data, const char *input) {
 	case 'b': // "sb"
 		if (input[1] == 0 || input[1] == ' ') {
 			if (off == 0) {
-				off = core->offset;
+				off = core->addr;
 			}
 			if (!silent) {
-				r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+				r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 			}
 			r_core_anal_bb_seek (core, off);
 		} else {
@@ -790,7 +788,7 @@ static int cmd_seek(void *data, const char *input) {
 		RAnalFunction *fcn;
 		switch (input[1]) {
 		case '\0': // "sf"
-			fcn = r_anal_get_fcn_in (core->anal, core->offset, 0);
+			fcn = r_anal_get_fcn_in (core->anal, core->addr, 0);
 			if (fcn) {
 				r_core_seek (core, r_anal_function_max_addr (fcn), true);
 			}
@@ -802,7 +800,7 @@ static int cmd_seek(void *data, const char *input) {
 			}
 			break;
 		case '.': // "sf."
-			fcn = r_anal_get_fcn_in (core->anal, core->offset, 0);
+			fcn = r_anal_get_fcn_in (core->anal, core->addr, 0);
 			if (fcn) {
 				r_core_seek (core, fcn->addr, true);
 			}
@@ -848,7 +846,7 @@ static int cmd_seek(void *data, const char *input) {
 		if (input[1] == '?') {
 			r_core_cmd_help_contains (core, help_msg_s, "sg");
 		} else {
-			RIOMap *map  = r_io_map_get_at (core->io, core->offset);
+			RIOMap *map  = r_io_map_get_at (core->io, core->addr);
 			if (map) {
 				r_core_seek (core, r_io_map_begin (map), true);
 			} else {
@@ -863,7 +861,7 @@ static int cmd_seek(void *data, const char *input) {
 			if (!core->io->desc) {
 				break;
 			}
-			RIOMap *map = r_io_map_get_at (core->io, core->offset);
+			RIOMap *map = r_io_map_get_at (core->io, core->addr);
 			// XXX: this +2 is a hack. must fix gap between sections
 			if (map) {
 				r_core_seek (core, r_io_map_end (map) + 2, true);
@@ -883,9 +881,9 @@ static int cmd_seek(void *data, const char *input) {
 			} else {
 				if (r_config_get_b (core->config, "scr.interactive")) {
 					// open shell
-					r_line_set_prompt ("sh> ");
+					r_line_set_prompt (core->cons->line, "sh> ");
 					for (;;) {
-						const char *line = r_line_readline ();
+						const char *line = r_line_readline (core->cons);
 						if (!line || !strcmp (line, "exit")) {
 							break;
 						}
@@ -908,7 +906,7 @@ static int cmd_seek(void *data, const char *input) {
 				const char *arg = strchr (input, ' ');
 				if (arg) {
 					arg++;
-					void *bed = r_cons_sleep_begin ();
+					void *bed = r_cons_sleep_begin (core->cons);
 					if (strchr (arg, '.')) {
 						double d = 0;
 						sscanf (arg, "%lf", &d);
@@ -916,7 +914,7 @@ static int cmd_seek(void *data, const char *input) {
 					} else {
 						r_sys_sleep (atoi (arg));
 					}
-					r_cons_sleep_end (bed);
+					r_cons_sleep_end (core->cons, bed);
 				} else {
 					r_core_cmd_help_match (core, help_msg_sl, "sleep");
 				}
@@ -948,7 +946,7 @@ static int cmd_seek(void *data, const char *input) {
 			if (!core->print->lines_cache) {
 				__init_seek_line (core);
 			}
-			r_cons_printf ("%d\n", core->print->lines_cache_sz - 1);
+			r_cons_printf (core->cons, "%d\n", core->print->lines_cache_sz - 1);
 			break;
 		case '?': // "sl?"
 			r_core_cmd_help (core, help_msg_sl);
@@ -967,7 +965,7 @@ static int cmd_seek(void *data, const char *input) {
 			ut64 n = r_num_math (core->num, input);
 			if (n) {
 				if (!silent) {
-					r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+					r_io_sundo_push (core->io, core->addr, r_print_get_cursor (core->print));
 				}
 				r_core_seek (core, n, true);
 				r_core_block_read (core);
