@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2024 - pancake, oddcoder, Anton Kochkov, Jody Frankowski */
+/* radare - LGPL - Copyright 2009-2025 - pancake, oddcoder, Anton Kochkov, Jody Frankowski */
 
 #if R_INCLUDE_BEGIN
 
@@ -58,7 +58,7 @@ static RCoreHelpMessage help_msg_tf = {
 	"Usage: tf[...]", "", "",
 	"tf", "", "list all function definitions loaded",
 	"tf", " <name>", "show function signature",
-	"tfc", " <name>", "show function signature in C syntax",
+	"tfc", " [name]", "list all/given function signatures in C output format with newlines",
 	"tfcj", " <name>", "same as above but in JSON",
 	"tfj", "", "list all function definitions in JSON",
 	"tfj", " <name>", "show function signature in JSON",
@@ -202,21 +202,21 @@ static void cmd_afcl(RCore *core, const char *input) {
 			pj_end (pj);
 		} else if (mode == 'l') {
 			char *sig = r_anal_cc_get (core->anal, cc);
-			r_cons_println (sig);
+			r_cons_println (core->cons, sig);
 			free (sig);
 		} else if (mode == '*') {
 			char *ccexpr = r_anal_cc_get (core->anal, cc);
-			r_cons_printf ("tcc %s\n", ccexpr);
+			r_cons_printf (core->cons, "tcc %s\n", ccexpr);
 			free (ccexpr);
 		} else {
-			r_cons_println (cc);
+			r_cons_println (core->cons, cc);
 		}
 	}
 	r_list_free (list);
 	if (pj) {
 		pj_end (pj);
 		char *j = pj_drain (pj);
-		r_cons_println (j);
+		r_cons_println (core->cons, j);
 		free (j);
 	}
 }
@@ -225,7 +225,7 @@ static void cmd_afck(RCore *core, const char *c) {
 	const char *s = "anal/cc/*";
 	char *out = sdb_querys (core->sdb, NULL, 0, s);
 	if (out) {
-		r_cons_print (out);
+		r_cons_print (core->cons, out);
 	}
 	free (out);
 }
@@ -274,7 +274,7 @@ static void cmd_tcc(RCore *core, const char *input) {
 			const char *ccname = r_str_trim_head_ro (input + 1);
 			char *cc = r_anal_cc_get (core->anal, ccname);
 			if (cc) {
-				r_cons_printf ("%s\n", cc);
+				r_cons_printf (core->cons, "%s\n", cc);
 				free (cc);
 			}
 		}
@@ -299,14 +299,14 @@ static void showFormat(RCore *core, const char *name, int mode) {
 				pj_ks (pj, "name", name);
 				pj_ks (pj, "format", fmt);
 				pj_end (pj);
-				r_cons_printf ("%s", pj_string (pj));
+				r_cons_printf (core->cons, "%s", pj_string (pj));
 				pj_free (pj);
 			} else {
 				if (R_STR_ISNOTEMPTY (fmt)) {
 					if (mode) {
-						r_cons_printf ("pf.%s %s\n", name, fmt);
+						r_cons_printf (core->cons, "pf.%s %s\n", name, fmt);
 					} else {
-						r_cons_printf ("pf %s\n", fmt);
+						r_cons_printf (core->cons, "pf %s\n", fmt);
 					}
 				} else {
 					// This happens when the type hasnt been fully removed
@@ -359,7 +359,7 @@ static int cmd_tac(void *data, const char *_input) { // "tac"
 				RListIter *iter;
 				char *line;
 				r_list_foreach_prev (lines, iter, line) {
-					r_cons_printf ("%s\n", line);
+					r_cons_printf (core->cons, "%s\n", line);
 				}
 				r_list_free (lines);
 				free (filedata);
@@ -408,7 +408,7 @@ static int cmd_tail(void *data, const char *_input) { // "tail"
 		} else {
 			char *res = r_syscmd_tail (arg, lines);
 			if (res) {
-				r_cons_print (res);
+				r_cons_print (core->cons, res);
 				free (res);
 			}
 		}
@@ -498,13 +498,13 @@ static bool stdifstruct(void *user, const char *k, const char *v) {
 }
 
 /*!
- * \brief print the data types details in JSON format
- * \param TDB pointer to the sdb for types
- * \param filter a callback function for the filtering
- * \return 1 if success, 0 if failure
- */
-static int print_struct_union_list_json(Sdb *TDB, SdbForeachCallback filter) {
-	PJ *pj = pj_new ();
+* \brief print the data types details in JSON format
+* \param TDB pointer to the sdb for types
+* \param filter a callback function for the filtering
+* \return 1 if success, 0 if failure
+*/
+static int print_struct_union_list_json(RCore *core, Sdb *TDB, SdbForeachCallback filter) {
+	PJ *pj = r_core_pj_new (core);
 	if (!pj) {
 		return 0;
 	}
@@ -524,14 +524,14 @@ static int print_struct_union_list_json(Sdb *TDB, SdbForeachCallback filter) {
 	}
 	pj_end (pj); // ]
 
-	r_cons_println (pj_string (pj));
+	r_cons_println (core->cons, pj_string (pj));
 	pj_free (pj);
 	ls_free (l);
 	return 1;
 }
 
 // Rename to char *RAnal.type_to_c() {}
-static void print_struct_union_in_c_format(Sdb *TDB, SdbForeachCallback filter, const char *arg, bool multiline) {
+static void print_struct_union_in_c_format(RCore *core, Sdb *TDB, SdbForeachCallback filter, const char *arg, bool multiline) {
 	char *name = NULL;
 	SdbKv *kv;
 	SdbListIter *iter;
@@ -539,6 +539,8 @@ static void print_struct_union_in_c_format(Sdb *TDB, SdbForeachCallback filter,
 	const char *space = "";
 	bool match = false;
 
+	RStrBuf *sb = r_strbuf_new ("");
+
 	ls_foreach (l, iter, kv) {
 		if (name && !strcmp (sdbkv_value (kv), name)) {
 			continue;
@@ -553,7 +555,7 @@ static void print_struct_union_in_c_format(Sdb *TDB, SdbForeachCallback filter,
 				continue;
 			}
 		}
-		r_cons_printf ("%s %s {%s", sdbkv_value (kv), name, multiline? "\n": "");
+		r_strbuf_appendf (sb, "%s %s {%s", sdbkv_value (kv), name, multiline? "\n": "");
 		char *p, *var = r_str_newf ("%s.%s", sdbkv_value (kv), name);
 		for (n = 0; (p = sdb_array_get (TDB, var, n, NULL)); n++) {
 			char *var2 = r_str_newf ("%s.%s", var, p);
@@ -561,23 +563,23 @@ static void print_struct_union_in_c_format(Sdb *TDB, SdbForeachCallback filter,
 				char *val = sdb_array_get (TDB, var2, 0, NULL);
 				if (val) {
 					char *arr = sdb_array_get (TDB, var2, 2, NULL);
-					int arrnum = atoi (arr);
+					int arrnum = arr? atoi (arr): 0;
 					free (arr);
 					if (multiline) {
-						r_cons_printf ("  %s", val);
+						r_strbuf_appendf (sb, "  %s", val);
 						if (p && p[0] != '\0') {
-							r_cons_printf ("%s%s", strstr (val, " *")? "": " ", p);
+							r_strbuf_appendf (sb, "%s%s", strstr (val, " *")? "": " ", p);
 							if (arrnum) {
-								r_cons_printf ("[%d]", arrnum);
+								r_strbuf_appendf (sb, "[%d]", arrnum);
 							}
 						}
-						r_cons_println (";");
+						r_strbuf_append (sb, ";\n");
 					} else {
-						r_cons_printf ("%s%s %s", space, val, p);
+						r_strbuf_appendf (sb, "%s%s %s", space, val, p);
 						if (arrnum) {
-							r_cons_printf ("[%d]", arrnum);
+							r_strbuf_appendf (sb, "[%d]", arrnum);
 						}
-						r_cons_print (";");
+						r_strbuf_append (sb, ";");
 						space = " ";
 					}
 					free (val);
@@ -587,17 +589,20 @@ static void print_struct_union_in_c_format(Sdb *TDB, SdbForeachCallback filter,
 			free (p);
 		}
 		free (var);
-		r_cons_println ("};");
+		r_strbuf_append (sb, "};\n");
 		space = "";
 		if (match) {
 			break;
 		}
 	}
+	char *s = r_strbuf_drain (sb);
+	r_cons_print (core->cons, s);
+	free (s);
 	free (name);
 	ls_free (l);
 }
 
-static void print_enum_in_c_format(Sdb *TDB, const char *arg, bool multiline) {
+static void print_enum_in_c_format(RCore *core, Sdb *TDB, const char *arg, bool multiline) {
 	char *name = NULL;
 	SdbKv *kv;
 	SdbListIter *iter;
@@ -616,7 +621,7 @@ static void print_enum_in_c_format(Sdb *TDB, const char *arg, bool multiline) {
 						continue;
 					}
 				}
-				r_cons_printf ("%s %s {%s", sdbkv_value (kv), name, multiline? "\n": "");
+				r_cons_printf (core->cons, "%s %s {%s", sdbkv_value (kv), name, multiline? "\n": "");
 				{
 					RList *list = r_type_get_enum (TDB, name);
 					if (list && !r_list_empty (list)) {
@@ -624,13 +629,13 @@ static void print_enum_in_c_format(Sdb *TDB, const char *arg, bool multiline) {
 						RTypeEnum *member;
 						separator = multiline? "\t": "";
 						r_list_foreach (list, iter, member) {
-							r_cons_printf ("%s%s = %" PFMT64u, separator, member->name, r_num_math (NULL, member->val));
+							r_cons_printf (core->cons, "%s%s = %" PFMT64u, separator, member->name, r_num_math (NULL, member->val));
 							separator = multiline? ",\n\t": ", ";
 						}
 					}
 					r_list_free (list);
 				}
-				r_cons_println (multiline? "\n};": "};");
+				r_cons_println (core->cons, multiline? "\n};": "};");
 				if (match) {
 					break;
 				}
@@ -642,7 +647,8 @@ static void print_enum_in_c_format(Sdb *TDB, const char *arg, bool multiline) {
 }
 
 static bool printkey_cb(void *user, const char *k, const char *v) {
-	r_cons_println (k);
+	RCore *core = (RCore *)user;
+	r_cons_println (core->cons, k);
 	return true;
 }
 
@@ -662,16 +668,16 @@ static void printFunctionTypeC(RCore *core, const char *input) {
 		return;
 	}
 
-	r_cons_printf ("%s %s (", ret, name);
+	r_cons_printf (core->cons, "%s %s (", ret, name);
 	for (i = 0; i < args; i++) {
 		char *type = sdb_get (TDB, r_strf ("func.%s.arg.%d", name, i), 0);
 		char *name = strchr (type, ',');
 		if (name) {
 			*name++ = 0;
 		}
-		r_cons_printf ("%s%s %s", (i == 0)? "": ", ", type, name);
+		r_cons_printf (core->cons, "%s%s %s", (i == 0)? "": ", ", type, name);
 	}
-	r_cons_printf (");\n");
+	r_cons_printf (core->cons, ");\n");
 	free (res);
 }
 
@@ -713,12 +719,10 @@ static void printFunctionType(RCore *core, const char *input) {
 	pj_end (pj);
 	pj_end (pj);
 	char *s = pj_drain (pj);
-	if (R_STR_ISEMPTY (s)) {
-		r_cons_printf ("{}");
-	} else {
-		r_cons_printf ("%s,", s);
+	if (s) {
+		r_cons_printf (core->cons, "%s", s);
+		free (s);
 	}
-	free (s);
 	free (res);
 }
 
@@ -748,18 +752,21 @@ static bool stdiflink(void *p, const char *k, const char *v) {
 }
 
 static bool print_link_cb(void *p, const char *k, const char *v) {
-	r_cons_printf ("0x%s = %s\n", k + strlen ("link."), v);
+	RCore *core = (RCore *)p;
+	r_cons_printf (core->cons, "0x%s = %s\n", k + strlen ("link."), v);
 	return true;
 }
 
 //TODO PJ
 static bool print_link_json_cb(void *p, const char *k, const char *v) {
-	r_cons_printf ("{\"0x%s\":\"%s\"}", k + strlen ("link."), v);
+	RCore *core = (RCore *)p;
+	r_cons_printf (core->cons, "{\"0x%s\":\"%s\"}", k + strlen ("link."), v);
 	return true;
 }
 
 static bool print_link_r_cb(void *p, const char *k, const char *v) {
-	r_cons_printf ("tl %s = 0x%s\n", v, k + strlen ("link."));
+	RCore *core = (RCore *)p;
+	r_cons_printf (core->cons, "tl %s = 0x%s\n", v, k + strlen ("link."));
 	return true;
 }
 
@@ -770,7 +777,7 @@ static bool print_link_readable_cb(void *p, const char *k, const char *v) {
 		R_LOG_ERROR ("Can't find type %s", v);
 		return 1;
 	}
-	r_cons_printf ("(%s)\n", v);
+	r_cons_printf (core->cons, "(%s)\n", v);
 	r_core_cmdf (core, "pf %s @ 0x%s", fmt, k + strlen ("link."));
 	return true;
 }
@@ -783,9 +790,9 @@ static bool print_link_readable_json_cb(void *p, const char *k, const char *v) {
 		R_LOG_ERROR ("Can't find type %s", v);
 		return true;
 	}
-	r_cons_printf ("{\"%s\":", v);
+	r_cons_printf (core->cons, "{\"%s\":", v);
 	r_core_cmdf (core, "pfj %s @ 0x%s", fmt, k + strlen ("link."));
-	r_cons_printf ("}");
+	r_cons_printf (core->cons, "}");
 	return true;
 }
 
@@ -794,7 +801,8 @@ static bool stdiftype(void *p, const char *k, const char *v) {
 }
 
 static bool print_typelist_r_cb(void *p, const char *k, const char *v) {
-	r_cons_printf ("'tk %s=%s\n", k, v);
+	RCore *core = (RCore *)p;
+	r_cons_printf (core->cons, "'tk %s=%s\n", k, v);
 	return true;
 }
 
@@ -810,21 +818,19 @@ static bool print_typelist_json_cb(void *p, const char *k, const char *v) {
 	char *sizecmd = r_str_newf ("type.%s.size", k);
 	char *size_s = sdb_get (sdb, sizecmd, NULL);
 	char *formatcmd = r_str_newf ("type.%s", k);
+	pj_ks (pj, "type", k);
+	pj_ki (pj, "size", size_s ? atoi (size_s) : -1);
 	char *format_s = sdb_get (sdb, formatcmd, NULL);
-	if (size_s && format_s) {
+	if (format_s) {
 		r_str_trim (format_s);
-		pj_ks (pj, "type", k);
-		pj_ki (pj, "size", size_s ? atoi (size_s) : -1);
 		pj_ks (pj, "format", format_s);
-		pj_end (pj);
-		r_cons_printf ("%s,", pj_string (pj));
-	} else {
-		R_LOG_DEBUG ("Internal sdb inconsistency for %s", sizecmd);
+		free (format_s);
 	}
+	pj_end (pj);
+	r_cons_printf (core->cons, "%s", pj_string (pj));
 	pj_free (pj);
 	free (size_s);
 	free (sizecmd);
-	free (format_s);
 	free (formatcmd);
 	return true;
 }
@@ -835,8 +841,9 @@ static void print_keys(Sdb *TDB, RCore *core, SdbForeachCallback filter, SdbFore
 	SdbKv *kv;
 
 	if (json) {
-		r_cons_print ("{\"types\":[");
+		r_cons_print (core->cons, "{\"types\":[");
 	}
+	bool first = true;
 	ls_foreach (l, it, kv) {
 		const char *k = sdbkv_key (kv);
 		const char *v = sdbkv_value (kv);
@@ -844,11 +851,17 @@ static void print_keys(Sdb *TDB, RCore *core, SdbForeachCallback filter, SdbFore
 			continue;
 		}
 		if (v) {
+			if (json) {
+				if (!first) {
+					r_cons_print (core->cons, ",");
+				}
+				first = false;
+			}
 			printfn_cb (core, k, v);
 		}
 	}
 	if (json) {
-		r_cons_println ("{}]}\n");
+		r_cons_println (core->cons, "]}\n");
 	}
 	ls_free (l);
 }
@@ -936,6 +949,9 @@ beach:
 }
 
 R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
+	if (!fcn) {
+		return;
+	}
 	RAnalBlock *bb;
 	RListIter *it;
 	RAnalOp aop = {0};
@@ -949,13 +965,8 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 	int iotrap = r_config_get_i (core->config, "esil.iotrap");
 	int stacksize = r_config_get_i (core->config, "esil.stack.depth");
 	unsigned int addrsize = r_config_get_i (core->config, "esil.addr.size");
-	const char *pc_name = r_reg_get_name (core->anal->reg, R_REG_NAME_PC);
-	const char *sp_name = r_reg_get_name (core->anal->reg, R_REG_NAME_SP);
-	RRegItem *pc = r_reg_get (core->anal->reg, pc_name, -1);
+	RRegItem *pc = r_reg_get (core->anal->reg, "PC", -1);
 
-	if (!fcn) {
-		return;
-	}
 	if (!(esil = r_esil_new (stacksize, iotrap, addrsize))) {
 		return;
 	}
@@ -972,11 +983,11 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 	}
 	r_reg_arena_push (core->anal->reg);
 	r_debug_reg_sync (core->dbg, R_REG_TYPE_ALL, true);
-	ut64 spval = r_reg_getv (esil->anal->reg, sp_name);
+	ut64 spval = r_reg_getv (esil->anal->reg, "SP");
 	if (spval) {
 		// reset stack pointer to initial value
-		RRegItem *sp = r_reg_get (esil->anal->reg, sp_name, -1);
-		ut64 curpc = r_reg_getv (esil->anal->reg, pc_name);
+		RRegItem *sp = r_reg_get (esil->anal->reg, "SP", -1);
+		ut64 curpc = r_reg_getv (esil->anal->reg, "PC");
 		int stacksz = r_core_get_stacksz (core, fcn->addr, curpc);
 		if (stacksz > 0) {
 			r_reg_arena_zero (esil->anal->reg); // clear prev reg values
@@ -989,8 +1000,8 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 	}
 	r_config_set_b (core->config, "io.cache", true);
 	r_config_set_i (core->config, "dbg.follow", 0);
-	ut64 oldoff = core->offset;
-	r_cons_break_push (NULL, NULL);
+	ut64 oldoff = core->addr;
+	r_cons_break_push (core->cons, NULL, NULL);
 	// TODO: The algorithm can be more accurate if blocks are followed by their jmp/fail, not just by address
 	r_list_sort (fcn->bbs, bb_cmpaddr);
 	r_list_foreach (fcn->bbs, it, bb) {
@@ -998,7 +1009,7 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 		ut64 to = bb->addr + bb->size;
 		r_reg_set_value (esil->anal->reg, pc, at);
 		for (i = 0; at < to; i++) {
-			if (r_cons_is_breaked ()) {
+			if (r_cons_is_breaked (core->cons)) {
 				goto beach;
 			}
 			if (at < bb->addr) {
@@ -1058,8 +1069,8 @@ R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn) {
 				// var int local_e0h --> var struct foo
 				if (strcmp (var->name , vlink) && !resolved) {
 					resolved = true;
-					r_anal_var_set_type (var, varpfx);
-					r_anal_var_rename (var, vlink, false);
+					r_anal_var_set_type (core->anal, var, varpfx);
+					r_anal_var_rename (core->anal, var, vlink);
 				}
 			} else if (slink) {
 				set_offset_hint (core, &aop, slink, src_addr, at - ret, src_imm);
@@ -1089,43 +1100,54 @@ beach:
 	r_esil_free (esil);
 	r_reg_arena_pop (core->anal->reg);
 	r_core_cmd0 (core, ".ar*");
-	r_cons_break_pop ();
+	r_cons_break_pop (core->cons);
 	free (buf);
 }
 
-static void test_flag(RCore *core, bool res) {
+static void test_flag(RCore *core, bool res, bool verbose) {
 	r_core_return_value (core, res? 0: 1);
+	if (verbose) {
+		r_cons_println (core->cons, res? "found": "not found");
+	}
 }
 
 static int cmd_test(RCore *core, const char *input) {
+	bool verbose = false;
 	int type = 'f';
 	if (*input == '-') {
+		if (input[1] == 'v') {
+			input++;
+			verbose = true;
+		}
 		type = input[1];
 	}
 	const char *arg = strchr (input, ' ');
 	if (arg) {
 		arg = r_str_trim_head_ro (arg + 1);
 	} else {
-		R_LOG_ERROR ("Missing file argument. Use 'test -fdx [file]'");
+		R_LOG_ERROR ("Missing file argument. Use 'test -[v]fdx [file]'");
 		return 1;
 	}
+	char *filePath = r_file_abspath_rel (NULL, arg);
 	switch (type) {
 	case 'f': // "test -f"
-		test_flag (core, r_file_exists (arg));
+		test_flag (core, r_file_exists (filePath), verbose);
 		break;
 	case 'x': // "test -x"
-		test_flag (core, r_file_is_executable (arg));
+		test_flag (core, r_file_is_executable (filePath), verbose);
 		break;
 	case 's': // "test -s"
-		test_flag (core, r_file_size (arg) > 0);
+		test_flag (core, r_file_size (filePath) > 0, verbose);
 		break;
 	case 'd': // "test -d"
-		test_flag (core, r_file_is_directory (arg));
+		test_flag (core, r_file_is_directory (filePath), verbose);
 		break;
 	default:
 		R_LOG_ERROR ("Unknown flag for test. Use -f, -x or -d");
+		free (filePath);
 		return 1;
 	}
+	free (filePath);
 	return 0;
 }
 
@@ -1161,16 +1183,16 @@ static int cmd_type(void *data, const char *input) {
 		case 'j': // "tuj"
 			if (input[2]) {
 				showFormat (core, r_str_trim_head_ro (input + 2), 'j');
-				r_cons_newline ();
+				r_cons_newline (core->cons);
 			} else {
-				print_struct_union_list_json (TDB, stdifunion);
+				print_struct_union_list_json (core, TDB, stdifunion);
 			}
 			break;
 		case 'c':
-			print_struct_union_in_c_format (TDB, stdifunion, r_str_trim_head_ro (input + 2), true);
+			print_struct_union_in_c_format (core, TDB, stdifunion, r_str_trim_head_ro (input + 2), true);
 			break;
 		case 'd':
-			print_struct_union_in_c_format (TDB, stdifunion, r_str_trim_head_ro (input + 2), false);
+			print_struct_union_in_c_format (core, TDB, stdifunion, r_str_trim_head_ro (input + 2), false);
 			break;
 		case ' ':
 			showFormat (core, r_str_trim_head_ro (input + 1), 0);
@@ -1181,6 +1203,17 @@ static int cmd_type(void *data, const char *input) {
 		}
 		break;
 	}
+	case 'i': // "ti"
+		if (r_str_startswith (input, "ime")) {
+			if (input[3] == ' ') {
+				r_core_cmdf (core, "?t %s", r_str_trim_head_ro (input + 4));
+			} else {
+				R_LOG_INFO ("Usage: time [command] # alias for the `?t` command");
+			}
+		} else {
+			r_core_return_invalid_command (core, "ti", input[1]);
+		}
+		break;
 	case 'k': // "tk"
 		if (input[1] == '?') {
 			r_core_cmd_help_match (core, help_msg_t, "tk");
@@ -1189,7 +1222,7 @@ static int cmd_type(void *data, const char *input) {
 				? sdb_querys (TDB, NULL, -1, input + 2)
 				: sdb_querys (TDB, NULL, -1, "*");
 			if (res) {
-				r_cons_print (res);
+				r_cons_print (core->cons, res);
 				free (res);
 			}
 		}
@@ -1267,7 +1300,7 @@ static int cmd_type(void *data, const char *input) {
 			break;
 		case 's': // "tss"
 			if (input[2] == ' ') {
-				r_cons_printf ("%" PFMT64u "\n", (r_type_get_bitsize (TDB, input + 3) / 8));
+				r_cons_printf (core->cons, "%" PFMT64u "\n", (r_type_get_bitsize (TDB, input + 3) / 8));
 			} else {
 				r_core_cmd_help (core, help_msg_ts);
 			}
@@ -1276,18 +1309,18 @@ static int cmd_type(void *data, const char *input) {
 			print_keys (TDB, core, stdifstruct, printkey_cb, false);
 			break;
 		case 'c': // "tsc"
-			print_struct_union_in_c_format (TDB, stdifstruct, r_str_trim_head_ro (input + 2), true);
+			print_struct_union_in_c_format (core, TDB, stdifstruct, r_str_trim_head_ro (input + 2), true);
 			break;
 		case 'd': // "tsd"
-			print_struct_union_in_c_format (TDB, stdifstruct, r_str_trim_head_ro (input + 2), false);
+			print_struct_union_in_c_format (core, TDB, stdifstruct, r_str_trim_head_ro (input + 2), false);
 			break;
 		case 'j': // "tsj"
 			// TODO: current output is a bit poor, will be good to improve
 			if (input[2]) {
 				showFormat (core, r_str_trim_head_ro (input + 2), 'j');
-				r_cons_newline ();
+				r_cons_newline (core->cons);
 			} else {
-				print_struct_union_list_json (TDB, stdifstruct);
+				print_struct_union_list_json (core, TDB, stdifstruct);
 			}
 			break;
 		} // end of switch (input[1])
@@ -1344,7 +1377,7 @@ static int cmd_type(void *data, const char *input) {
 					}
 				}
 				pj_end (pj);
-				r_cons_printf ("%s\n", pj_string (pj));
+				r_cons_printf (core->cons, "%s\n", pj_string (pj));
 				pj_free (pj);
 				free (name);
 				ls_free (l);
@@ -1370,7 +1403,7 @@ static int cmd_type(void *data, const char *input) {
 						pj_end (pj);
 						pj_end (pj);
 					}
-					r_cons_printf ("%s\n", pj_string (pj));
+					r_cons_printf (core->cons, "%s\n", pj_string (pj));
 					pj_free (pj);
 					r_list_free (list);
 				}
@@ -1387,14 +1420,14 @@ static int cmd_type(void *data, const char *input) {
 			if (input[2] == '?') {
 				r_core_cmd_help_match (core, help_msg_te, "tec");
 			} else {
-				print_enum_in_c_format (TDB, r_str_trim_head_ro (input + 2), true);
+				print_enum_in_c_format (core, TDB, r_str_trim_head_ro (input + 2), true);
 			}
 			break;
 		case 'd': // "ted"
 			if (input[2] == '?') {
 				r_core_cmd_help_match (core, help_msg_te, "ted");
 			} else {
-				print_enum_in_c_format (TDB, r_str_trim_head_ro (input + 2), false);
+				print_enum_in_c_format (core, TDB, r_str_trim_head_ro (input + 2), false);
 			}
 			break;
 		case ' ': // "te "
@@ -1405,7 +1438,7 @@ static int cmd_type(void *data, const char *input) {
 				RListIter *iter;
 				RTypeEnum *member;
 				r_list_foreach (list, iter, member) {
-					r_cons_printf ("%s = %s\n", member->name, member->val);
+					r_cons_printf (core->cons, "%s = %s\n", member->name, member->val);
 				}
 				r_list_free (list);
 			}
@@ -1420,7 +1453,7 @@ static int cmd_type(void *data, const char *input) {
 					if (!name || strcmp (sdbkv_value (kv), name)) {
 						free (name);
 						name = strdup (sdbkv_key (kv));
-						r_cons_println (name);
+						r_cons_println (core->cons, name);
 					}
 				}
 			}
@@ -1435,7 +1468,7 @@ static int cmd_type(void *data, const char *input) {
 		} // end of switch (input[1])
 		free (name);
 		if (res) {
-			r_cons_println (res);
+			r_cons_println (core->cons, res);
 		} else if (member_name) {
 			R_LOG_ERROR ("Invalid enum member");
 		}
@@ -1486,7 +1519,7 @@ static int cmd_type(void *data, const char *input) {
 						char *errmsg = NULL;
 						char *out = r_anal_cparse (core->anal, tmp, &errmsg);
 						if (out) {
-							// r_cons_print (out);
+							// r_cons_print (core->cons, out);
 							r_anal_save_parsed_type (core->anal, out);
 							free (out);
 						}
@@ -1500,7 +1533,7 @@ static int cmd_type(void *data, const char *input) {
 					char *errmsg = NULL;
 					char *out = r_anal_cparse_file (core->anal, filename, dir, &errmsg);
 					if (out) {
-						//r_cons_print (out);
+						// r_cons_print (core->cons, out);
 						r_anal_save_parsed_type (core->anal, out);
 						free (out);
 					}
@@ -1583,7 +1616,7 @@ static int cmd_type(void *data, const char *input) {
 		case '.': // "tx." type xrefs
 		case 'f': // "txf" type xrefs
 			{
-			ut64 addr = core->offset;
+			ut64 addr = core->addr;
 			if (input[2] == ' ') {
 				addr = r_num_math (core->num, input + 2);
 			}
@@ -1591,7 +1624,7 @@ static int cmd_type(void *data, const char *input) {
 			if (fcn) {
 				RList *uniq = r_anal_types_from_fcn (core->anal, fcn);
 				r_list_foreach (uniq , iter , type) {
-					r_cons_println (type);
+					r_cons_println (core->cons, type);
 				}
 				r_list_free (uniq);
 			} else {
@@ -1603,10 +1636,10 @@ static int cmd_type(void *data, const char *input) {
 			r_list_foreach (core->anal->fcns, iter, fcn) {
 				RList *uniq = r_anal_types_from_fcn (core->anal, fcn);
 				if (r_list_length (uniq)) {
-					r_cons_printf ("%s: ", fcn->name);
+					r_cons_printf (core->cons, "%s: ", fcn->name);
 				}
 				r_list_foreach (uniq , iter2, type) {
-					r_cons_printf ("%s%s", type, iter2->n ? ",":"\n");
+					r_cons_printf (core->cons, "%s%s", type, iter2->n ? ",":"\n");
 				}
 			}
 			break;
@@ -1615,13 +1648,13 @@ static int cmd_type(void *data, const char *input) {
 				r_list_foreach (core->anal->fcns, iter, fcn) {
 					RList *uniq = r_anal_types_from_fcn (core->anal, fcn);
 					if (r_list_length (uniq)) {
-						r_cons_printf ("agn %s\n", fcn->name);
+						r_cons_printf (core->cons, "agn %s\n", fcn->name);
 					}
 					r_list_foreach (uniq , iter2, type) {
 						char *myType = strdup (type);
 						r_str_replace_ch (myType, ' ', '_', true);
-						r_cons_printf ("agn %s\n", myType);
-						r_cons_printf ("age %s %s\n", myType, fcn->name);
+						r_cons_printf (core->cons, "agn %s\n", myType);
+						r_cons_printf (core->cons, "age %s %s\n", myType, fcn->name);
 						free (myType);
 					}
 				}
@@ -1640,7 +1673,7 @@ static int cmd_type(void *data, const char *input) {
 				}
 				r_list_sort (uniqList, (RListComparator)strcmp);
 				r_list_foreach (uniqList, iter, type) {
-					r_cons_printf ("%s\n", type);
+					r_cons_printf (core->cons, "%s\n", type);
 				}
 				r_list_free (uniqList);
 			}
@@ -1652,7 +1685,7 @@ static int cmd_type(void *data, const char *input) {
 				RList *uniq = r_anal_types_from_fcn (core->anal, fcn);
 				r_list_foreach (uniq , iter2, type2) {
 					if (!strcmp (type2, type)) {
-						r_cons_printf ("%s\n", fcn->name);
+						r_cons_printf (core->cons, "%s\n", fcn->name);
 						break;
 					}
 				}
@@ -1691,7 +1724,7 @@ static int cmd_type(void *data, const char *input) {
 		case ' ': {
 			char *type = strdup (input + 2);
 			char *ptr = strchr (type, '=');
-			ut64 addr = core->offset;
+			ut64 addr = core->addr;
 
 			if (ptr) {
 				*ptr++ = 0;
@@ -1708,7 +1741,7 @@ static int cmd_type(void *data, const char *input) {
 			char *tmp = sdb_get (TDB, type, 0);
 			if (R_STR_ISNOTEMPTY (tmp)) {
 				r_type_set_link (TDB, type, addr);
-				RList *fcns = r_anal_get_functions_in (core->anal, core->offset);
+				RList *fcns = r_anal_get_functions_in (core->anal, core->addr);
 				if (r_list_length (fcns) > 1) {
 					R_LOG_ERROR ("Multiple functions found in here");
 				} else if (r_list_length (fcns) == 1) {
@@ -1778,7 +1811,7 @@ static int cmd_type(void *data, const char *input) {
 			const char *type_name = r_str_trim_head_ro (input + 2);
 			char *fmt = r_type_format (TDB, type_name);
 			if (fmt && *fmt) {
-				ut64 val = core->offset;
+				ut64 val = core->addr;
 				r_core_cmdf (core, "pf %s @v:0x%08" PFMT64x, fmt, val);
 			} else {
 				r_core_cmd_help_match (core, help_msg_tp, "tpv");
@@ -1809,10 +1842,10 @@ static int cmd_type(void *data, const char *input) {
 				if (input[1] == 'x' && arg) { // "tpx"
 					r_core_cmdf (core, "pf %s @x:%s", fmt, arg);
 				} else {
-					ut64 addr = arg ? r_num_math (core->num, arg): core->offset;
+					ut64 addr = arg ? r_num_math (core->num, arg): core->addr;
 					ut64 original_addr = addr;
 					if (!addr && arg) {
-						RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->offset, -1);
+						RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->addr, -1);
 						if (fcn) {
 							RAnalVar *var = r_anal_function_get_var_byname (fcn, arg);
 							if (var) {
@@ -1871,12 +1904,24 @@ static int cmd_type(void *data, const char *input) {
 		case 'c': // "tfc"
 			if (input[2] == ' ') {
 				printFunctionTypeC (core, input + 3);
+			} else {
+				// No argument: print all function signatures in C syntax
+				SdbList *l = sdb_foreach_list_filter (TDB, stdiffunc, true);
+				SdbListIter *it;
+				SdbKv *kv;
+				ls_foreach (l, it, kv) {
+					const char *fname = sdbkv_key (kv);
+					if (R_STR_ISNOTEMPTY (fname)) {
+						printFunctionTypeC (core, fname);
+					}
+				}
+				ls_free (l);
 			}
 			break;
 		case 'j': // "tfj"
 			if (input[2] == ' ') {
 				printFunctionType (core, input + 2);
-				r_cons_newline ();
+				r_cons_newline (core->cons);
 			} else {
 				print_keys (TDB, core, stdiffunc, printfunc_json_cb, true);
 			}
@@ -1886,7 +1931,7 @@ static int cmd_type(void *data, const char *input) {
 			char *res = sdb_querys (TDB, NULL, -1, k);
 			free (k);
 			if (res) {
-				r_cons_printf ("%s", res);
+				r_cons_printf (core->cons, "%s", res);
 				free (res);
 			}
 			break;
@@ -1913,7 +1958,7 @@ static int cmd_type(void *data, const char *input) {
 						free (name);
 						name = strdup (sdbkv_key (kv));
 						if (!input[1]) {
-							r_cons_println (name);
+							r_cons_println (core->cons, name);
 						} else {
 							char *q = r_str_newf ("typedef.%s", name);
 							const char *res = sdb_const_get (TDB, q, 0);
@@ -1930,14 +1975,14 @@ static int cmd_type(void *data, const char *input) {
 				pj_end (pj);
 			}
 			if (pj) {
-				r_cons_printf ("%s\n", pj_string (pj));
+				r_cons_printf (core->cons, "%s\n", pj_string (pj));
 				pj_free (pj);
 			}
 			free (name);
 			ls_free (l);
 			break;
 		}
-		if (input[1] == 'c') {
+		if (input[1] == 'c') { // "ttc"
 			char *name = NULL;
 			SdbKv *kv;
 			SdbListIter *iter;
@@ -1960,7 +2005,7 @@ static int cmd_type(void *data, const char *input) {
 						const char *res = sdb_const_get (TDB, q, 0);
 						free (q);
 						if (res) {
-							r_cons_printf ("%s %s %s;\n", sdbkv_value (kv), res, name);
+							r_cons_printf (core->cons, "%s %s %s;\n", sdbkv_value (kv), res, name);
 						}
 						if (match) {
 							break;
@@ -1983,7 +2028,7 @@ static int cmd_type(void *data, const char *input) {
 			char *q = r_str_newf ("typedef.%s", s);
 			const char *res = sdb_const_get (TDB, q, 0);
 			if (res) {
-				r_cons_println (res);
+				r_cons_println (core->cons, res);
 			}
 			free (q);
 		} else {
