@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2024 - pancake, nibble, Dennis Goodlett */
+/* radare - LGPL - Copyright 2009-2025 - pancake, nibble, Dennis Goodlett */
 
 #if R_INCLUDE_BEGIN
 
@@ -147,7 +147,7 @@ static inline bool za_add(RCore *core, const char *input) {
 	char *stype = r_str_tok_r (NULL, " ", &save_ptr);
 	char *sig = r_str_tok_r (NULL, "", &save_ptr);
 
-	if (!stype || !sig || stype[1] != '\0') {
+	if (!name || !stype || !sig || stype[1] != '\0') {
 		R_LOG_ERROR ("Invalid input");
 		free (args);
 		return false;
@@ -173,6 +173,29 @@ static inline bool za_add(RCore *core, const char *input) {
 	case R_SIGN_NAME:
 		ret = r_sign_add_name (core->anal, name, sig);
 		break;
+	case R_SIGN_RAWNAME:
+	case R_SIGN_DEMANGLED:
+		{
+			RSignItem *it = item_new_named (core->anal, name);
+			if (it) {
+				char *dec = (char *)r_base64_decode_dyn (sig, -1, NULL);
+				if (dec) {
+					if (t == R_SIGN_RAWNAME) {
+						it->rawname = dec;
+					} else {
+						it->demangled = dec;
+					}
+					ret = r_sign_add_item (core->anal, it);
+					if (!ret) {
+						free (dec);
+						r_sign_item_free (it);
+					}
+				} else {
+					r_sign_item_free (it);
+				}
+			}
+			break;
+		}
 	case R_SIGN_TYPES:
 		ret = r_sign_add_types (core->anal, name, sig);
 		break;
@@ -208,8 +231,8 @@ static inline bool za_add(RCore *core, const char *input) {
 	return ret;
 }
 
-static void cmd_za_detailed_help(void) {
-	r_cons_printf ("Adding Zignatures (examples and documentation)\n\n"
+static void cmd_za_detailed_help(RCore *core) {
+	r_cons_printf (core->cons, "Adding Zignatures (examples and documentation)\n\n"
 			"Zignature types:\n"
 			"  a: bytes pattern, r2 creates mask from analysis\n"
 			"  b: bytes pattern\n"
@@ -243,7 +266,7 @@ static void cmd_za_detailed_help(void) {
 static int cmd_za(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 	if (r_str_endswith (input, "??")) {
-		cmd_za_detailed_help ();
+		cmd_za_detailed_help (core);
 		return 0;
 	}
 	if (*input && input[1] == '?') {
@@ -269,7 +292,7 @@ static int cmd_za(void *data, const char *input) {
 			if (n > 0) {
 				fcni = r_anal_get_function_byname (core->anal, r_str_word_get0 (args, 0));
 			} else {
-				fcni = r_anal_get_function_at (core->anal, core->offset);
+				fcni = r_anal_get_function_at (core->anal, core->addr);
 			}
 			if (fcni) {
 				r_sign_add_func (core->anal, fcni, zigname);
@@ -285,9 +308,9 @@ static int cmd_za(void *data, const char *input) {
 		if (input[1] == '?') {
 			r_core_cmd_help_contains (core, help_msg_za, "zac");
 		} else {
-			r_cons_break_push (NULL, NULL);
+			r_cons_break_push (core->cons, NULL, NULL);
 			r_sign_resolve_collisions (core->anal);
-			r_cons_break_pop ();
+			r_cons_break_pop (core->cons);
 		}
 		break;
 	case 'F': // "zaF"
@@ -462,11 +485,22 @@ struct ctxSearchCB {
 
 static void apply_name(RCore *core, RAnalFunction *fcn, RSignItem *it, bool rad) {
 	R_RETURN_IF_FAIL (core && fcn && it && it->name);
-	const char *name = it->realname? it->realname: it->name;
+	// Prefer demangled or mangled depending on setting; keep the alternate as realname
+	const bool prefer_mangled = r_config_get_b (core->config, "zign.mangled");
+	const char *best = NULL;
+	const char *alt = NULL;
+	if (prefer_mangled) {
+		best = it->rawname? it->rawname: it->name;
+		alt = it->demangled? it->demangled: it->realname;
+	} else {
+		best = it->demangled? it->demangled: it->name;
+		alt = it->rawname? it->rawname: it->realname;
+	}
+	const char *name = best? best: (it->realname? it->realname: it->name);
 	if (rad) {
 		char *tmp = r_name_filter_dup (name);
 		if (tmp) {
-			r_cons_printf ("'@0x%08"PFMT64x"'afn %s\n", fcn->addr, tmp);
+			r_cons_printf (core->cons, "'@0x%08"PFMT64x"'afn %s\n", fcn->addr, tmp);
 			free (tmp);
 		}
 		return;
@@ -476,6 +510,13 @@ static void apply_name(RCore *core, RAnalFunction *fcn, RSignItem *it, bool rad)
 		r_flag_rename (core->flags, flag, name);
 	}
 	r_anal_function_rename (fcn, name);
+	// set alternate spelling as realname for the function flag if available
+	if (R_STR_ISNOTEMPTY (alt)) {
+		RFlagItem *nf = r_flag_get (core->flags, fcn->name);
+		if (nf) {
+			r_flag_item_set_realname (core->flags, nf, alt);
+		}
+	}
 	if (core->anal->cb.on_fcn_rename) {
 		core->anal->cb.on_fcn_rename (core->anal, core->anal->user, fcn, name);
 	}
@@ -488,7 +529,7 @@ static void apply_flag(RCore *core, RSignItem *it, ut64 addr, int size, int coun
 		if (rad) {
 			char *tmp = r_name_filter_dup (name);
 			if (tmp) {
-				r_cons_printf ("f %s %d @ 0x%08" PFMT64x "\n", tmp, size, addr);
+				r_cons_printf (core->cons, "f %s %d @ 0x%08" PFMT64x "\n", tmp, size, addr);
 				free (tmp);
 			}
 		} else {
@@ -569,7 +610,7 @@ static int fcnMatchCB(RSignItem *it, RAnalFunction *fcn, RSignType *types, void
 			ctx->next_count++;
 			break;
 		default:
-			r_warn_if_reached ();
+			R_WARN_IF_REACHED ();
 			break;
 		}
 		if (prefix) {
@@ -612,9 +653,9 @@ static bool searchRange(RCore *core, ut64 from, ut64 to, bool rad, struct ctxSea
 	ss->search->align = r_config_get_i (core->config, "search.align");
 	r_sign_search_init (core->anal, ss, minsz, searchBytesHitCB, ctx);
 
-	r_cons_break_push (NULL, NULL);
+	r_cons_break_push (core->cons, NULL, NULL);
 	for (at = from; at < to; at += core->blocksize) {
-		if (r_cons_is_breaked ()) {
+		if (r_cons_is_breaked (core->cons)) {
 			retval = false;
 			break;
 		}
@@ -630,7 +671,7 @@ static bool searchRange(RCore *core, ut64 from, ut64 to, bool rad, struct ctxSea
 			break;
 		}
 	}
-	r_cons_break_pop ();
+	r_cons_break_pop (core->cons);
 	free (buf);
 	r_sign_search_free (ss);
 
@@ -712,7 +753,7 @@ static void print_ctx_hits(struct ctxSearchCB *ctx) {
 static bool search(RCore *core, bool rad, bool only_func) {
 	const char *zign_prefix = r_config_get (core->config, "zign.prefix");
 	if (rad) {
-		r_cons_printf ("fs+%s\n", zign_prefix);
+		r_cons_printf (core->cons, "fs+%s\n", zign_prefix);
 	} else {
 		if (!r_flag_space_push (core->flags, zign_prefix)) {
 			R_LOG_ERROR ("cannot create flagspace");
@@ -758,7 +799,7 @@ static bool search(RCore *core, bool rad, bool only_func) {
 	}
 
 	if (rad) {
-		r_cons_printf ("fs-\n");
+		r_cons_printf (core->cons, "fs-\n");
 	} else {
 		if (!r_flag_space_pop (core->flags)) {
 			R_LOG_ERROR ("cannot restore flagspace");
@@ -784,21 +825,21 @@ static void print_possible_matches(RList *list, bool json, RCore *core) {
 			pj_end (pj);
 		}
 		pj_end (pj);
-		r_cons_printf ("%s\n", pj_string (pj));
+		r_cons_printf (core->cons, "%s\n", pj_string (pj));
 		pj_free (pj);
 	} else {
 		r_list_foreach (list, itr, row) {
 			// total score
 			if (row->bscore > 0.0 && row->gscore > 0.0) {
-				r_cons_printf ("%02.5lf  ", row->score);
+				r_cons_printf (core->cons, "%02.5lf  ", row->score);
 			}
 			if (row->bscore > 0.0) {
-				r_cons_printf ("%02.5lf B  ", row->bscore);
+				r_cons_printf (core->cons, "%02.5lf B  ", row->bscore);
 			}
 			if (row->gscore > 0.0) {
-				r_cons_printf ("%02.5lf G  ", row->gscore);
+				r_cons_printf (core->cons, "%02.5lf G  ", row->gscore);
 			}
-			r_cons_printf (" %s\n", row->item->name);
+			r_cons_printf (core->cons, " %s\n", row->item->name);
 		}
 	}
 }
@@ -908,9 +949,9 @@ static bool bestmatch_sig(RCore *core, const char *input, bool json) {
 		}
 	}
 
-	RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->offset, 0);
+	RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->addr, 0);
 	if (!fcn) {
-		R_LOG_ERROR ("No function at 0x%08" PFMT64x, core->offset);
+		R_LOG_ERROR ("No function at 0x%08" PFMT64x, core->addr);
 		return false;
 	}
 
@@ -934,14 +975,14 @@ static bool bestmatch_sig(RCore *core, const char *input, bool json) {
 	double th = get_zb_threshold (core);
 	bool found = false;
 	if (item->graph || item->bytes) {
-		r_cons_break_push (NULL, NULL);
+		r_cons_break_push (core->cons, NULL, NULL);
 		RList *list = r_sign_find_closest_sig (core->anal, item, count, th);
 		if (list) {
 			found = true;
 			print_possible_matches (list, json, core);
 			r_list_free (list);
 		}
-		r_cons_break_pop ();
+		r_cons_break_pop (core->cons);
 	} else {
 		R_LOG_WARN ("no signatures types available for testing");
 	}
@@ -1003,7 +1044,7 @@ static bool _sig_bytediff_cb(RLevBuf *va, RLevBuf *vb, ut32 ia, ut32 ib) {
 	free (x.land); \
 	memset (&x, 0, sizeof (x));
 
-static void print_zig_diff(RCore *c, RSignBytes *ab, RSignBytes *bb, RLevOp *ops) {
+static void print_zig_diff(RCore *core, RSignBytes *ab, RSignBytes *bb, RLevOp *ops) {
 	struct lines {
 		char *mask, *bytes, *land;
 	} al, bl;
@@ -1012,7 +1053,7 @@ static void print_zig_diff(RCore *c, RSignBytes *ab, RSignBytes *bb, RLevOp *ops
 
 	char *colsub, *coladd, *coldel;
 	colsub = coladd = coldel = NULL;
-	if (r_config_get_b (c->config, "scr.color")) {
+	if (r_config_get_b (core->config, "scr.color")) {
 		coldel = "\x1b[1;31m";
 		coladd = "\x1b[1;32m";
 		colsub = "\x1b[1;33m";
@@ -1047,7 +1088,7 @@ static void print_zig_diff(RCore *c, RSignBytes *ab, RSignBytes *bb, RLevOp *ops
 			printb = true;
 			break;
 		default:
-			r_warn_if_reached ();
+			R_WARN_IF_REACHED ();
 			freelines (al);
 			freelines (bl);
 			return;
@@ -1061,21 +1102,21 @@ static void print_zig_diff(RCore *c, RSignBytes *ab, RSignBytes *bb, RLevOp *ops
 
 		if (i % 16 == 15 || ops[i + 1] == LEVEND) {
 			if (i > 16) {
-				r_cons_printf ("\n");
+				r_cons_printf (core->cons, "\n");
 			}
-			r_cons_printf ("Fnc cmp   0x%04x %s\n", iastart, al.land);
+			r_cons_printf (core->cons, "Fnc cmp   0x%04x %s\n", iastart, al.land);
 			if (printb) {
-				r_cons_printf ("Sig cmp   0x%04x %s\n", ibstart, bl.land);
+				r_cons_printf (core->cons, "Sig cmp   0x%04x %s\n", ibstart, bl.land);
 			}
-			r_cons_printf ("Fnc Mask  0x%04x %s\n", iastart, al.mask);
+			r_cons_printf (core->cons, "Fnc Mask  0x%04x %s\n", iastart, al.mask);
 			if (printb) {
-				r_cons_printf ("Sig Mask  0x%04x %s\n", ibstart, bl.mask);
+				r_cons_printf (core->cons, "Sig Mask  0x%04x %s\n", ibstart, bl.mask);
 			}
-			r_cons_printf ("Fnc Bytes 0x%04x %s\n", iastart, al.bytes);
+			r_cons_printf (core->cons, "Fnc Bytes 0x%04x %s\n", iastart, al.bytes);
 			if (printb) {
-				r_cons_printf ("Sig Bytes 0x%04x %s\n", ibstart, bl.bytes);
+				r_cons_printf (core->cons, "Sig Bytes 0x%04x %s\n", ibstart, bl.bytes);
 			} else {
-				r_cons_printf ("== Signature was same ==\n");
+				r_cons_printf (core->cons, "== Signature was same ==\n");
 			}
 			freelines (al);
 			freelines (bl);
@@ -1093,9 +1134,9 @@ static bool cmd_zd(void *data, const char *input) {
 	R_RETURN_VAL_IF_FAIL (data && input, false);
 	RCore *core = (RCore *)data;
 
-	RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->offset, 0);
+	RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->addr, 0);
 	if (!fcn) {
-		R_LOG_ERROR ("No function at 0x%08" PFMT64x, core->offset);
+		R_LOG_ERROR ("No function at 0x%08" PFMT64x, core->addr);
 		return false;
 	}
 
@@ -1228,7 +1269,7 @@ static int cmd_zdot(void *data, const char *input) {
 
 	const char *zign_prefix = r_config_get (core->config, "zign.prefix");
 	if (ctx.rad) {
-		r_cons_printf ("fs+%s\n", zign_prefix);
+		r_cons_printf (core->cons, "fs+%s\n", zign_prefix);
 	} else {
 		if (!r_flag_space_push (core->flags, zign_prefix)) {
 			R_LOG_ERROR ("cannot create flagspace");
@@ -1237,17 +1278,17 @@ static int cmd_zdot(void *data, const char *input) {
 	}
 
 	R_LOG_INFO ("searching function metrics");
-	RAnalFunction *fcn = r_anal_get_function_at (core->anal, core->offset);
+	RAnalFunction *fcn = r_anal_get_function_at (core->anal, core->addr);
 	if (fcn) {
-		r_cons_break_push (NULL, NULL);
+		r_cons_break_push (core->cons, NULL, NULL);
 		r_sign_fcn_match_metrics (&sm, fcn);
-		r_cons_break_pop ();
+		r_cons_break_pop (core->cons);
 	} else {
-		R_LOG_ERROR ("No function at 0x%08" PFMT64x, core->offset);
+		R_LOG_ERROR ("No function at 0x%08" PFMT64x, core->addr);
 	}
 
 	if (ctx.rad) {
-		r_cons_printf ("fs-\n");
+		r_cons_printf (core->cons, "fs-\n");
 	} else {
 		if (!r_flag_space_pop (core->flags)) {
 			R_LOG_ERROR ("cannot restore flagspace");
@@ -1298,7 +1339,11 @@ static int cmd_zi(void *data, const char *input) {
 	case 0:
 	case ' ':
 		r_flag_space_push (core->flags, R_FLAGS_FS_SIGNS);
-		r_flag_list (core->flags, *input, input[0] ? input + 1: "");
+		{
+			char *s = r_flag_list (core->flags, *input, input[0] ? input + 1: "");
+			r_cons_print (core->cons, s);
+			free (s);
+		}
 		r_flag_space_pop (core->flags);
 		break;
 	default:
@@ -1334,7 +1379,7 @@ static bool listCB(RSignItem *it, void *user) {
 }
 
 static int csvZignatures(RCore *core, const char *arg) {
-	RTable *t = r_table_new ("pxr");
+	RTable *t = r_core_table_new (core, "pxr");
 	RTableColumnType *n = r_table_type ("number");
 	RTableColumnType *s = r_table_type ("string");
 	r_table_add_column (t, n, "addr", 0);
@@ -1346,7 +1391,7 @@ static int csvZignatures(RCore *core, const char *arg) {
 	r_sign_foreach (core->anal, listCB, &ctx);
 	if (r_table_query (t, arg)) {
 		char *ts = r_table_tostring (t);
-		r_cons_printf ("%s", ts); // \n?
+		r_cons_printf (core->cons, "%s", ts); // \n?
 		free (ts);
 	}
 	r_table_free (t);
@@ -1371,10 +1416,10 @@ static int cmd_zign(void *data, const char *input) {
 					naddr = UT64_MAX;
 				}
 			}
-			ut64 oaddr = core->offset;
-			core->offset = naddr; // XXX R2_600 - this is a hack because we cant break the abi
+			ut64 oaddr = core->addr;
+			core->addr = naddr; // XXX R2_600 - this is a hack because we cant break the abi
 			r_sign_list (core->anal, *input);
-			core->offset = oaddr;
+			core->addr = oaddr;
 		}
 		break;
 	case ',': // "z,"
