@@ -197,8 +197,9 @@ static bool r_debug_native_continue_syscall(RDebug *dbg, int pid, int num) {
 #if !R2__WINDOWS__ && !__APPLE__ && !R2__BSD__
 /* Callback to trigger SIGINT signal */
 static void interrupt_process(RDebug *dbg) {
+	RCore *core = dbg->coreb.core;
 	r_debug_kill (dbg, dbg->pid, dbg->tid, SIGINT);
-	r_cons_break_pop ();
+	r_cons_break_pop (core->cons);
 }
 #endif
 
@@ -225,9 +226,10 @@ static bool r_debug_native_continue(RDebug *dbg, int pid, int tid, int sig) {
 	if (sig == -1) {
 		   sig = dbg->reason.signum;
 	}
+	RCore *core = dbg->coreb.core;
 	/* SIGINT handler for attached processes: dbg.consbreak (disabled by default) */
 	if (dbg->consbreak) {
-		r_cons_break_push ((RConsBreak)interrupt_process, dbg);
+		r_cons_break_push (core->cons, (RConsBreak)interrupt_process, dbg);
 	}
 	if (dbg->continue_all_threads && dbg->n_threads && dbg->threads) {
 		RDebugPid *th;
@@ -264,6 +266,7 @@ static RDebugInfo* r_debug_native_info(RDebug *dbg, const char *arg) {
 
 #if R2__WINDOWS__
 static bool tracelib(RDebug *dbg, const char *mode, PLIB_ITEM item) {
+	RCore *core = dbg->coreb.core;
 	const char *needle = NULL;
 	int tmp = 0;
 	if (mode) {
@@ -272,9 +275,9 @@ static bool tracelib(RDebug *dbg, const char *mode, PLIB_ITEM item) {
 		case 'u': needle = dbg->glob_unlibs; break;
 		}
 	}
-	r_cons_printf ("(%d) %sing library at 0x%p (%s) %s\n", item->pid, mode,
+	r_cons_printf (core->cons, "(%d) %sing library at 0x%p (%s) %s\n", item->pid, mode,
 		item->BaseOfDll, item->Path, item->Name);
-	r_cons_flush ();
+	r_cons_flush (core->cons);
 	if (needle && strlen (needle)) {
 		tmp = r_str_glob (item->Name, needle);
 	}
@@ -295,6 +298,8 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 	int orig_tid = dbg->tid;
 	bool restore_thread = false;
 	RW32Dw *wrap = dbg->user;
+	RCore *core = dbg->coreb.core;
+	RCons *cons = core->cons;
 
 	if (pid == -1) {
 		R_LOG_ERROR ("r_debug_native_wait called with pid -1");
@@ -311,16 +316,16 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 
 			/* Check if autoload PDB is set, and load PDB information if yes */
 			RCore *core = dbg->coreb.core;
-			bool autoload_pdb = dbg->coreb.cfggeti (core, "pdb.autoload");
+			bool autoload_pdb = dbg->coreb.cfgGetI (core, "pdb.autoload");
 			if (autoload_pdb) {
 				PLIB_ITEM lib = r->lib;
 #if 0
 				dbg->coreb.cmdf (core, "\"o \\\"%s\\\" 0x%p\"", lib->Path, lib->BaseOfDll);
-				char *o_res = dbg->coreb.cmdstrf (core, "o~+%s", lib->Name);
+				char *o_res = dbg->coreb.cmdStrF (core, "o~+%s", lib->Name);
 				int fd = atoi (o_res);
 				free (o_res);
 				if (fd) {
-					char *pdb_file = dbg->coreb.cmdstr (core, "i~dbg_file");
+					char *pdb_file = dbg->coreb.cmdStr (core, "i~dbg_file");
 					if (pdb_file && (r_str_trim (pdb_file), *pdb_file)) {
 						if (!r_file_exists (pdb_file + 9)) {
 #else
@@ -347,7 +352,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 			}
 			r_debug_info_free (r);
 		} else {
-			r_cons_flush ();
+			r_cons_flush (core->cons);
 			R_LOG_WARN ("Loading unknown library");
 		}
 		restore_thread = true;
@@ -359,16 +364,16 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 			}
 			r_debug_info_free (r);
 		} else {
-			r_cons_printf ("Unloading unknown library.\n");
-			r_cons_flush ();
+			r_cons_printf (core->cons, "Unloading unknown library.\n");
+			r_cons_flush (core->cons);
 		}
 		restore_thread = true;
 	} else if (reason == R_DEBUG_REASON_NEW_TID) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
 		if (r && r->thread) {
 			PTHREAD_ITEM item = r->thread;
-			r_cons_printf ("(%d) Created thread %d (start @ %p) (teb @ %p)\n", item->pid, item->tid, item->lpStartAddress, item->lpThreadLocalBase);
-			r_cons_flush ();
+			r_cons_printf (cons, "(%d) Created thread %d (start @ %p) (teb @ %p)\n", item->pid, item->tid, item->lpStartAddress, item->lpThreadLocalBase);
+			r_cons_flush (cons);
 
 			r_debug_info_free (r);
 		}
@@ -377,8 +382,8 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
 		if (r && r->thread) {
 			PTHREAD_ITEM item = r->thread;
-			r_cons_printf ("(%d) Finished thread %d Exit code %lu\n", (ut32)item->pid, (ut32)item->tid, item->dwExitCode);
-			r_cons_flush ();
+			r_cons_printf (cons, "(%d) Finished thread %d Exit code %lu\n", (ut32)item->pid, (ut32)item->tid, item->dwExitCode);
+			r_cons_flush (cons);
 
 			r_debug_info_free (r);
 		}
@@ -389,8 +394,8 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
 		if (r && r->thread) {
 			PTHREAD_ITEM item = r->thread;
-			r_cons_printf ("(%d) Finished process with exit code %lu\n", dbg->main_pid, item->dwExitCode);
-			r_cons_flush ();
+			r_cons_printf (cons, "(%d) Finished process with exit code %lu\n", dbg->main_pid, item->dwExitCode);
+			r_cons_flush (cons);
 			r_debug_info_free (r);
 		}
 		dbg->pid = -1;
@@ -399,8 +404,8 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 		RDebugInfo *r = r_debug_native_info (dbg, "");
 		if (r && r->thread) {
 			PTHREAD_ITEM item = r->thread;
-			r_cons_printf ("(%d) Created DebugBreak thread %d (start @ %p)\n", item->pid, item->tid, item->lpStartAddress);
-			r_cons_flush ();
+			r_cons_printf (cons, "(%d) Created DebugBreak thread %d (start @ %p)\n", item->pid, item->tid, item->lpStartAddress);
+			r_cons_flush (cons);
 
 			r_debug_info_free (r);
 		}
@@ -461,11 +466,12 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 	if (pid < 0) {
 		return R_DEBUG_REASON_ERROR;
 	}
-	r_cons_break_push (NULL, NULL);
+	RCore *core = dbg->coreb.core;
+	r_cons_break_push (core->cons, NULL, NULL);
 	do {
 		reason = xnu_wait (dbg, pid);
 		if (reason == R_DEBUG_REASON_MACH_RCV_INTERRUPTED) {
-			if (r_cons_is_breaked ()) {
+			if (r_cons_is_breaked (core->cons)) {
 				// Perhaps check the inferior is still alive,
 				// otherwise xnu_stop will fail.
 				reason = xnu_stop (dbg, pid)
@@ -478,7 +484,7 @@ static RDebugReasonType r_debug_native_wait(RDebug *dbg, int pid) {
 		}
 		break;
 	} while (true);
-	r_cons_break_pop ();
+	r_cons_break_pop (core->cons);
 #else
 	int status = -1;
 	// XXX: this is blocking, ^C will be ignored
@@ -645,18 +651,13 @@ static int bsd_reg_read(RDebug *dbg, int type, ut8* buf, int size) {
 	}
 	switch (type) {
 	case R_REG_TYPE_DRX:
-#if __i386__ || __x86_64__
-#if __KFBSD__
-	{
-		// TODO
-		struct dbreg dbr;
-		ret = ptrace (PT_GETDBREGS, pid, (caddr_t)&dbr, sizeof (dbr));
-		if (ret != 0) {
-			return false;
+#if __KFBSD__ && (__i386__ || __x86_64__)
+		{
+			struct dbreg dbr;
+			if (ptrace (PT_GETDBREGS, pid, (caddr_t)&dbr, sizeof (dbr)) != 0) {
+				return false;
+			}
 		}
-		// XXX: maybe the register map is not correct, must review
-	}
-#endif
 #endif
 		return true;
 		break;
@@ -667,11 +668,11 @@ static int bsd_reg_read(RDebug *dbg, int type, ut8* buf, int size) {
 	case R_REG_TYPE_VEC128: // XMM
 	case R_REG_TYPE_VEC256: // YMM
 	case R_REG_TYPE_VEC512: // ZMM
-#if __KFBSD__
+#if __KFBSD__ && (__i386__ || __x86_64__)
 		struct ptrace_xstate_info info;
 		ret = ptrace (PT_GETXSTATE_INFO, pid, (caddr_t)&info, sizeof (info));
 		if (info.xsave_len != 0) {
-		ret = ptrace (PT_GETXSTATE, pid, (caddr_t)buf, info.xsave_len);
+			ret = ptrace (PT_GETXSTATE, pid, (caddr_t)buf, info.xsave_len);
 		}
 #endif
 		break;
@@ -939,7 +940,7 @@ err_linux_map_alloc:
 	return map;
 }
 
-static int linux_map_dealloc(RDebug *dbg, ut64 addr, int size) {
+static bool linux_map_dealloc(RDebug *dbg, ut64 addr, int size) {
 	RBuffer *buf = NULL;
 	char code[1024];
 	ut64 ret = 0;
@@ -977,7 +978,7 @@ static int linux_map_dealloc(RDebug *dbg, ut64 addr, int size) {
 		r_reg_arena_pop (dbg->reg);
 	}
 err_linux_map_dealloc:
-	return (int)ret;
+	return ret == 0;
 }
 #endif
 
@@ -996,7 +997,7 @@ static RDebugMap* r_debug_native_map_alloc(RDebug *dbg, ut64 addr, int size, boo
 #endif
 }
 
-static int r_debug_native_map_dealloc(RDebug *dbg, ut64 addr, int size) {
+static bool r_debug_native_map_dealloc(RDebug *dbg, ut64 addr, int size) {
 #if __APPLE__
 	return xnu_map_dealloc (dbg, addr, size);
 #elif R2__WINDOWS__
@@ -1185,7 +1186,6 @@ static RList *r_debug_native_map_get(RDebug *dbg) {
 }
 
 static RList *r_debug_native_modules_get(RDebug *dbg) {
-R_LOG_INFO ("modules.get");
 	char *lastname = NULL;
 	RDebugMap *map;
 	RListIter *iter, *iter2;
@@ -1324,7 +1324,7 @@ static void set_drx_regs(RDebug *dbg, drxt *regs, size_t num_regs) {
 }
 #endif
 
-static int r_debug_native_drx(RDebug *dbg, int n, ut64 addr, int sz, int rwx, int g, int api_type) {
+static bool r_debug_native_drx(RDebug *dbg, int n, ut64 addr, int sz, int rwx, int g, int api_type) {
 #if __i386__ || __x86_64__
 	int retval = false;
 #if NUM_DRX_REGISTERS > 0
@@ -1580,7 +1580,7 @@ static RList *r_debug_desc_native_list(int pid) {
 #endif
 }
 
-static int r_debug_native_map_protect(RDebug *dbg, ut64 addr, int size, int perms) {
+static bool r_debug_native_map_protect(RDebug *dbg, ut64 addr, int size, int perms) {
 #if R2__WINDOWS__
 	return w32_map_protect (dbg, addr, size, perms);
 #elif __APPLE__
@@ -1588,9 +1588,7 @@ static int r_debug_native_map_protect(RDebug *dbg, ut64 addr, int size, int perm
 #elif __linux__
 	RBuffer *buf = NULL;
 	char code[1024];
-	int num;
-
-	num = r_syscall_get_num (dbg->anal->syscall, "mprotect");
+	const int num = r_syscall_get_num (dbg->anal->syscall, "mprotect");
 	snprintf (code, sizeof (code),
 		"sc@syscall(%d);\n"
 		"main@global(0) { sc(%p,%d,%d);\n"
@@ -1619,12 +1617,8 @@ static int r_debug_native_map_protect(RDebug *dbg, ut64 addr, int size, int perm
 		r_reg_arena_pop (dbg->reg);
 		return true;
 	}
-
-	return false;
-#else
-	// mprotect not implemented for this platform
-	return false;
 #endif
+	return false;
 }
 
 static int r_debug_desc_native_open(const char *path) {
@@ -1658,23 +1652,23 @@ RDebugPlugin r_debug_plugin_native = {
 		.desc = "native debug plugin",
 	},
 #if __i386__
-	.bits = R_SYS_BITS_32,
+	.bits = R_SYS_BITS_PACK (32),
 	.arch = "x86",
 	.canstep = true,
 #elif __s390x__ || __s390__
-	.bits = R_SYS_BITS_64,
+	.bits = R_SYS_BITS_PACK (64),
 	.arch = "s390",
 	.canstep = true,
 #elif __riscv || __riscv__ || __riscv64__
-	.bits = R_SYS_BITS_64,
+	.bits = R_SYS_BITS_PACK (64),
 	.arch = "riscv",
 	.canstep = false,
 #elif __x86_64__
-	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
+	.bits = R_SYS_BITS_PACK2 (32, 64),
 	.arch = "x86",
 	.canstep = true, // XXX it's 1 on some platforms...
 #elif __aarch64__ || __arm64__ || __arm64e__
-	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
+	.bits = R_SYS_BITS_PACK2 (32, 64),
 	.arch = "arm",
 #if __APPLE__
 	.canstep = true,
@@ -1682,22 +1676,22 @@ RDebugPlugin r_debug_plugin_native = {
 	.canstep = false,
 #endif
 #elif __arm__
-	.bits = R_SYS_BITS_16 | R_SYS_BITS_32 | R_SYS_BITS_64,
+	.bits = R_SYS_BITS_PACK3 (16, 32, 64),
 	.arch = "arm",
 	.canstep = false,
 #elif __mips__
-	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
+	.bits = R_SYS_BITS_PACK2 (32, 64),
 	.arch = "mips",
 	.canstep = false,
 #elif __loongarch
-	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
+	.bits = R_SYS_BITS_PACK2 (32, 64),
 	.arch = "loongarch",
 	.canstep = false,
 #elif __powerpc__
 # if __powerpc64__
-	.bits = R_SYS_BITS_32 | R_SYS_BITS_64,
+	.bits = R_SYS_BITS_PACK2 (32, 64),
 # else
-	.bits = R_SYS_BITS_32,
+	.bits = R_SYS_BITS_PACK (32),
 #endif
 	.arch = "ppc",
 	.canstep = true,
