@@ -1,6 +1,7 @@
-/* radare - LGPL - Copyright 2020 - GustavoLCR */
+/* radare - LGPL - Copyright 2020-2025 - GustavoLCR */
 
 #include <r_debug.h>
+#include <r_core.h>
 #include <DbgEng.h>
 
 #ifndef CONTEXT_ARM
@@ -116,7 +117,7 @@ static RDebugReasonType exception_to_reason(DWORD ExceptionCode) {
 	}
 }
 
-static int windbg_stop(RDebug *dbg) {
+static bool windbg_stop(RDebug *dbg) {
 	DbgEngContext *idbg = dbg->user;
 	R_RETURN_VAL_IF_FAIL (idbg && idbg->initialized, 0);
 	return SUCCEEDED (ITHISCALL (dbgCtrl, SetInterrupt, DEBUG_INTERRUPT_ACTIVE));
@@ -135,20 +136,21 @@ static void break_action(void *user) {
 
 static RDebugReasonType windbg_wait(RDebug *dbg, int pid) {
 	DbgEngContext *idbg = dbg->user;
+	RCore *core = dbg->coreb.core;
 	R_RETURN_VAL_IF_FAIL (idbg && idbg->initialized, 0);
 	ULONG Type, ProcessId, ThreadId;
-	r_cons_break_push (break_action, dbg);
+	r_cons_break_push (core->cons, break_action, dbg);
 	const ULONG timeout = __is_target_kernel (idbg) ? INFINITE : TIMEOUT;
 	HRESULT hr;
 	while ((hr = ITHISCALL (dbgCtrl, WaitForEvent, DEBUG_WAIT_DEFAULT, timeout)) == S_FALSE) {
 		if (do_break) {
 			ITHISCALL (dbgCtrl, SetExecutionStatus, DEBUG_STATUS_BREAK);
 			do_break = false;
-			r_cons_break_pop ();
+			r_cons_break_pop (core->cons);
 			return R_DEBUG_REASON_USERSUSP;
 		}
 	}
-	r_cons_break_pop ();
+	r_cons_break_pop (core->cons);
 	if (FAILED (hr)) {
 		return R_DEBUG_REASON_DEAD;
 	}
@@ -256,7 +258,7 @@ static char *windbg_reg_profile(RDebug *dbg) {
 	DbgEngContext *idbg = dbg->user;
 	ULONG type;
 	if (!idbg || !idbg->initialized || FAILED (ITHISCALL (dbgCtrl, GetActualProcessorType, &type))) {
-		if (dbg->bits & R_SYS_BITS_64) {
+		if (R_SYS_BITS_CHECK (dbg->bits, 64)) {
 #include "native/reg/windows-x64.h"
 		} else {
 #include "native/reg/windows-x86.h"
@@ -613,7 +615,7 @@ RDebugPlugin r_debug_plugin_windbg = {
 		.author = "pancake",
 		.desc = "comunicate with a windbg",
 	},
-	.bits = R_SYS_BITS_64,
+	.bits = R_SYS_BITS_PACK (64),
 	.arch = "x86,x64,arm,arm64",
 	.canstep = 1,
 	.init_debugger = windbg_init,
