@@ -1,16 +1,14 @@
-/* radare - LGPL - Copyright 2009-2021 - pancake */
+/* radare - LGPL - Copyright 2009-2025 - pancake */
 
 #include <r_userconf.h>
 
 #if DEBUGGER
 #include <r_debug.h>
+#include <r_core.h>
 #include <r_asm.h>
-#include <r_reg.h>
 #include <r_lib.h>
 #include <r_anal.h>
-#include <signal.h>
 #include <sys/uio.h>
-#include <errno.h>
 #include "linux_debug.h"
 #include "../procfs.h"
 
@@ -32,15 +30,24 @@ char *linux_reg_profile (RDebug *dbg) {
 #elif __riscv
 #	include "reg/linux-riscv64.h"
 #elif __arm64__ || __aarch64__
-#	include "reg/linux-arm64.h"
+	const bool is64 = R_SYS_BITS_CHECK (dbg->bits, 64);
+	if (is64) {
+#		include "reg/linux-arm64.h"
+	} else {
+#		include "reg/linux-arm.h"
+	}
 #elif __mips__
-	if ((dbg->bits & R_SYS_BITS_32) && (dbg->bp->endian == 1)) {
+	const bool is32 = R_SYS_BITS_CHECK (dbg->bits, 32);
+	if (is32 && (dbg->bp->endian == 1)) {
 #		include "reg/linux-mips.h"
 	} else {
 #		include "reg/linux-mips64.h"
 	}
+#elif __loongarch__
+#		include "reg/linux-loongarch64.h"
 #elif (__i386__ || __x86_64__)
-	if (dbg->bits & R_SYS_BITS_32) {
+	const bool is32 = R_SYS_BITS_CHECK (dbg->bits, 32);
+	if (is32) {
 #if __x86_64__
 #		include "reg/linux-x64-32.h"
 #else
@@ -50,13 +57,19 @@ char *linux_reg_profile (RDebug *dbg) {
 #		include "reg/linux-x64.h"
 	}
 #elif __powerpc__
-	if (dbg->bits & R_SYS_BITS_32) {
-#		include "reg/linux-ppc.h"
-	} else {
+	const bool is64 = R_SYS_BITS_CHECK (dbg->bits, 64);
+	if (is64) {
 #		include "reg/linux-ppc64.h"
+	} else {
+#		include "reg/linux-ppc.h"
 	}
 #elif __s390x__
-#	include "reg/linux-s390x.h"
+	const bool is64 = R_SYS_BITS_CHECK (dbg->bits, 64);
+	if (is64) {
+#		include "reg/linux-zarch.h"
+	} else {
+#		include "reg/linux-s390x.h"
+	}
 #else
 #	error "Unsupported Linux CPU"
 	return NULL;
@@ -75,7 +88,8 @@ static void linux_dbg_wait_break(RDebug *dbg);
 static RDebugReasonType linux_handle_new_task(RDebug *dbg, int tid);
 
 int linux_handle_signals(RDebug *dbg, int tid) {
-	siginfo_t siginfo = { 0 };
+	RCore *core = dbg->coreb.core;
+	siginfo_t siginfo = {0};
 	int ret = r_debug_ptrace (dbg, PTRACE_GETSIGINFO, tid, 0, (r_ptrace_data_t)&siginfo);
 	if (ret == -1) {
 		/* ESRCH means the process already went away :-/ */
@@ -119,8 +133,8 @@ int linux_handle_signals(RDebug *dbg, int tid) {
 					if (p) {
 						if (r_str_startswith (p, "dbg.libs")) {
 							const char *name = strstr (b->data, "sym.imp.dlopen")
-								? r_reg_get_name (dbg->reg, R_REG_NAME_A0)
-								: r_reg_get_name (dbg->reg, R_REG_NAME_A1);
+								? r_reg_alias_getname (dbg->reg, R_REG_ALIAS_A0)
+								: r_reg_alias_getname (dbg->reg, R_REG_ALIAS_A1);
 							b->data = r_str_appendf (b->data, ";ps@r:%s", name);
 							dbg->reason.type = R_DEBUG_REASON_NEW_LIB;
 							break;
@@ -147,10 +161,10 @@ int linux_handle_signals(RDebug *dbg, int tid) {
 		default:
 			break;
 		}
-		if (dbg->reason.signum != SIGTRAP && (dbg->reason.signum != SIGINT || !r_cons_is_breaked ())) {
-			eprintf ("[+] SIGNAL %d errno=%d addr=0x%08"PFMT64x
-				" code=%d si_pid=%d ret=%d\n",
-				siginfo.si_signo, siginfo.si_errno,
+		if (dbg->reason.signum != SIGTRAP && (dbg->reason.signum != SIGINT || !r_cons_is_breaked (core->cons))) {
+			const char *name = r_signal_tostring (dbg->reason.signum);
+			eprintf ("[+] SIGNAL %d (aka %s) errno=%d addr=0x%08"PFMT64x " code=%d si_pid=%d ret=%d\n",
+				siginfo.si_signo, name, siginfo.si_errno,
 				(ut64) (size_t)siginfo.si_addr, siginfo.si_code, siginfo.si_pid, ret);
 		}
 		return true;
@@ -166,8 +180,6 @@ int linux_handle_signals(RDebug *dbg, int tid) {
 // Used to remove breakpoints before detaching from a fork, without it the child
 // will die upon hitting a breakpoint while not being traced
 static void linux_remove_fork_bps(RDebug *dbg) {
-	RListIter *iter;
-	RBreakpointItem *b;
 	int prev_pid = dbg->pid;
 	int prev_tid = dbg->tid;
 
@@ -175,14 +187,16 @@ static void linux_remove_fork_bps(RDebug *dbg) {
 	dbg->pid = dbg->forked_pid;
 	dbg->tid = dbg->forked_pid;
 	r_debug_select (dbg, dbg->forked_pid, dbg->forked_pid);
-
+#if __i386__ || __x86_64__
+	RListIter *iter;
+	RBreakpointItem *b;
 	// Unset all hw breakpoints in the child process
 	r_debug_reg_sync (dbg, R_REG_TYPE_DRX, false);
 	r_list_foreach (dbg->bp->bps, iter, b) {
 		r_debug_drx_unset (dbg, r_bp_get_index_at (dbg->bp, b->addr));
 	}
 	r_debug_reg_sync (dbg, R_REG_TYPE_DRX, true);
-
+#endif
 	// Unset software breakpoints in the child process
 	r_debug_bp_update (dbg);
 	r_bp_restore (dbg->bp, false);
@@ -234,7 +248,7 @@ RDebugReasonType linux_ptrace_event (RDebug *dbg, int ptid, int status, bool dow
 			// The new child has a pending SIGSTOP.  We can't affect it until it
 			// hits the SIGSTOP, but we're already attached.  */
 			if (waitpid ((int)data, &status, 0) == -1) {
-				perror ("waitpid");
+				r_sys_perror ("waitpid");
 			}
 		}
 
@@ -256,7 +270,7 @@ RDebugReasonType linux_ptrace_event (RDebug *dbg, int ptid, int status, bool dow
 			// The new child has a pending SIGSTOP. We can't affect it until it
 			// hits the SIGSTOP, but we're already attached.  */
 			if (waitpid (dbg->forked_pid, &status, 0) == -1) {
-				perror ("waitpid");
+				r_sys_perror ("waitpid");
 			}
 		}
 		eprintf ("(%d) Created process %d\n", ptid, (int)data);
@@ -265,7 +279,7 @@ RDebugReasonType linux_ptrace_event (RDebug *dbg, int ptid, int status, bool dow
 			// breakpoints are inherited from the parent
 			linux_remove_fork_bps (dbg);
 			if (r_debug_ptrace (dbg, PTRACE_DETACH, dbg->forked_pid, NULL, (r_ptrace_data_t)(size_t)NULL) == -1) {
-				perror ("PTRACE_DETACH");
+				r_sys_perror ("PTRACE_DETACH");
 			}
 		}
 		return R_DEBUG_REASON_NEW_PID;
@@ -309,7 +323,7 @@ static RDebugReasonType linux_handle_new_task(RDebug *dbg, int tid) {
 				continue;
 			}
 			// Retrieve the signal without consuming it with PTRACE_GETSIGINFO
-			siginfo_t siginfo = { 0 };
+			siginfo_t siginfo = {0};
 			ret = r_debug_ptrace (dbg, PTRACE_GETSIGINFO, th->pid, 0, (r_ptrace_data_t)(size_t)&siginfo);
 			// Skip if PTRACE_GETSIGINFO fails when the thread is running.
 			if (ret == -1) {
@@ -341,9 +355,8 @@ int linux_step(RDebug *dbg) {
 	int ret = false;
 	int pid = dbg->tid;
 	ret = r_debug_ptrace (dbg, PTRACE_SINGLESTEP, pid, 0, 0);
-	//XXX(jjd): why?? //linux_handle_signals (dbg);
 	if (ret == -1) {
-		perror ("native-singlestep");
+		r_sys_perror ("native-singlestep");
 		ret = false;
 	} else {
 		ret = true;
@@ -367,13 +380,14 @@ bool linux_set_options(RDebug *dbg, int pid) {
 	}
 	/* SIGTRAP | 0x80 on signal handler .. not supported on all archs */
 	traceflags |= PTRACE_O_TRACESYSGOOD;
+	RCore *core = dbg->coreb.core;
 
 	// PTRACE_SETOPTIONS can fail because of the asynchronous nature of ptrace
 	// If the target is traced, the loop will always end with success
 	while (r_debug_ptrace (dbg, PTRACE_SETOPTIONS, pid, 0, (r_ptrace_data_t)(size_t)traceflags) == -1) {
-		void *bed = r_cons_sleep_begin ();
+		void *bed = r_cons_sleep_begin (core->cons);
 		usleep (1000);
-		r_cons_sleep_end (bed);
+		r_cons_sleep_end (core->cons, bed);
 	}
 	return true;
 }
@@ -386,7 +400,7 @@ static void linux_detach_all(RDebug *dbg) {
 		r_list_foreach (th_list, it, th) {
 			if (th->pid != dbg->main_pid) {
 				if (r_debug_ptrace (dbg, PTRACE_DETACH, th->pid, NULL, (r_ptrace_data_t)(size_t)NULL) == -1) {
-					perror ("PTRACE_DETACH");
+					r_sys_perror ("PTRACE_DETACH");
 				}
 			}
 		}
@@ -394,7 +408,7 @@ static void linux_detach_all(RDebug *dbg) {
 
 	// Detaching from main proc
 	if (r_debug_ptrace (dbg, PTRACE_DETACH, dbg->main_pid, NULL, (r_ptrace_data_t)(size_t)NULL) == -1) {
-		perror ("PTRACE_DETACH");
+		r_sys_perror ("PTRACE_DETACH");
 	}
 }
 
@@ -459,14 +473,14 @@ static void linux_dbg_wait_break_main(RDebug *dbg) {
 	// in another process group.
 	if (dpgid != tpgid) {
 		if (!linux_kill_thread (dbg->pid, SIGINT)) {
-			eprintf ("Could not interrupt pid (%d)\n", dbg->pid);
+			R_LOG_ERROR ("Could not interrupt pid (%d)", dbg->pid);
 		}
 	}
 }
 
 static void linux_dbg_wait_break(RDebug *dbg) {
 	if (!linux_kill_thread (dbg->pid, SIGINT)) {
-		eprintf ("Could not interrupt pid (%d)\n", dbg->pid);
+		R_LOG_ERROR ("Could not interrupt pid (%d)", dbg->pid);
 	}
 }
 
@@ -484,26 +498,28 @@ RDebugReasonType linux_dbg_wait(RDebug *dbg, int pid) {
 		// In the main context, SIGINT is propagated to the debuggee if it is
 		// in the same process group. Otherwise, the task is running in
 		// background and SIGINT will not be propagated to the debuggee.
-		if (r_cons_context_is_main ()) {
-			r_cons_break_push ((RConsBreak)linux_dbg_wait_break_main, dbg);
+		RCore *core = dbg->coreb.core;
+		const bool is_main = r_cons_context_is_main (core->cons, core->cons->context);
+		if (is_main) {
+			r_cons_break_push (core->cons, (RConsBreak)linux_dbg_wait_break_main, dbg);
 		} else {
-			r_cons_break_push ((RConsBreak)linux_dbg_wait_break, dbg);
+			r_cons_break_push (core->cons, (RConsBreak)linux_dbg_wait_break, dbg);
 		}
-		void *bed = r_cons_sleep_begin ();
+		void *bed = r_cons_sleep_begin (core->cons);
 		if (dbg->continue_all_threads) {
 			ret = waitpid (-1, &status, flags);
 		} else {
 			ret = waitpid (pid, &status, flags);
 		}
-		r_cons_sleep_end (bed);
-		r_cons_break_pop ();
+		r_cons_sleep_end (core->cons, bed);
+		r_cons_break_pop (core->cons);
 
 		if (ret < 0) {
 			// Continue when interrupted by user;
 			if (errno == EINTR) {
 				continue;
 			}
-			perror ("waitpid");
+			r_sys_perror ("waitpid");
 			break;
 		} else if (ret == 0) {
 			// Unset WNOHANG to call next waitpid in blocking mode.
@@ -601,7 +617,7 @@ static bool linux_kill_thread(int tid, int signo) {
 	int ret = syscall (__NR_tkill, tid, signo);
 
 	if (ret == -1) {
-		perror ("tkill");
+		r_sys_perror ("tkill");
 		return false;
 	}
 
@@ -610,7 +626,7 @@ static bool linux_kill_thread(int tid, int signo) {
 
 static bool linux_stop_thread(RDebug *dbg, int tid) {
 	int status, ret;
-	siginfo_t siginfo = { 0 };
+	siginfo_t siginfo = {0};
 
 	// Return if the thread is already stopped
 	ret = r_debug_ptrace (dbg, PTRACE_GETSIGINFO, tid, 0,
@@ -621,7 +637,7 @@ static bool linux_stop_thread(RDebug *dbg, int tid) {
 
 	if (linux_kill_thread (tid, SIGSTOP)) {
 		if ((ret = waitpid (tid, &status, 0)) == -1) {
-			perror ("waitpid");
+			r_sys_perror ("waitpid");
 		}
 		return ret == tid;
 	}
@@ -645,7 +661,7 @@ bool linux_stop_threads(RDebug *dbg, int except) {
 }
 
 static bool linux_attach_single_pid(RDebug *dbg, int pid) {
-	siginfo_t sig = { 0 };
+	siginfo_t sig = {0};
 
 	if (pid < 0) {
 		return false;
@@ -658,20 +674,20 @@ static bool linux_attach_single_pid(RDebug *dbg, int pid) {
 	if (r_debug_ptrace (dbg, PTRACE_GETSIGINFO, pid, NULL,
 		(r_ptrace_data_t)&sig) == -1) {
 		if (r_debug_ptrace (dbg, PTRACE_ATTACH, pid, NULL, NULL) == -1) {
-			perror ("ptrace (PT_ATTACH)");
+			r_sys_perror ("ptrace (PT_ATTACH)");
 			return false;
 		}
 
 		// Make sure SIGSTOP is delivered and wait for it since we can't affect the pid
 		// until it hits SIGSTOP.
 		if (!linux_stop_thread (dbg, pid)) {
-			eprintf ("Could not stop pid (%d)\n", pid);
+			R_LOG_ERROR ("Could not stop pid (%d)", pid);
 			return false;
 		}
 	}
 
 	if (!linux_set_options (dbg, pid)) {
-		eprintf("failed set_options on %d\n", pid);
+		R_LOG_ERROR("failed set_options on %d", pid);
 		return false;
 	}
 	dbg->pid = pid;
@@ -680,7 +696,7 @@ static bool linux_attach_single_pid(RDebug *dbg, int pid) {
 
 static RList *get_pid_thread_list(RDebug *dbg, int main_pid) {
 	RList *list = r_list_new ();
-	if (list) {
+	if (R_LIKELY (list)) {
 		list = linux_thread_list (dbg, main_pid, list);
 		dbg->main_pid = main_pid;
 	}
@@ -849,12 +865,18 @@ RList *linux_thread_list(RDebug *dbg, int pid, RList *list) {
 	char *ptr, buf[PATH_MAX];
 	RDebugPid *pid_info = NULL;
 	ut64 pc = 0;
-	int prev_tid = dbg->tid;
-
-	if (!pid) {
+	if (pid < 1) {
 		r_list_free (list);
 		return NULL;
 	}
+	if (dbg->tid < 1) {
+		dbg->tid = pid;
+		dbg->pid = pid;
+	}
+	int prev_pid = dbg->pid;
+	int prev_tid = dbg->tid;
+	dbg->pid = pid;
+	dbg->tid = pid;
 
 	list->free = (RListFree)&r_debug_pid_free;
 	/* if this process has a task directory, use that */
@@ -863,7 +885,8 @@ RList *linux_thread_list(RDebug *dbg, int pid, RList *list) {
 		struct dirent *de;
 		DIR *dh = opendir (buf);
 		// Update the process' memory maps to set correct paths
-		dbg->corebind.syncDebugMaps (dbg->corebind.core);
+		dbg->pid = pid;
+		dbg->coreb.syncDebugMaps (dbg->coreb.core);
 		while ((de = readdir (dh))) {
 			if (!strcmp (de->d_name, ".") || !strcmp (de->d_name, "..")) {
 				continue;
@@ -908,7 +931,7 @@ RList *linux_thread_list(RDebug *dbg, int pid, RList *list) {
 		closedir (dh);
 		// Return to the original thread
 		linux_attach_single_pid (dbg, prev_tid);
-		dbg->pid = pid;
+		dbg->pid = prev_pid;
 		dbg->tid = prev_tid;
 		r_debug_reg_sync (dbg, R_REG_TYPE_GPR, false);
 	} else {
@@ -917,7 +940,7 @@ RList *linux_thread_list(RDebug *dbg, int pid, RList *list) {
 #define MAXPID 99999
 		/* otherwise, brute force the pids */
 		for (i = pid; i < MAXPID; i++) { // XXX
-			if (procfs_pid_slurp (i, "status", buf, sizeof(buf)) == -1) {
+			if (procfs_pid_slurp (i, "status", buf, sizeof (buf)) == -1) {
 				continue;
 			}
 			int uid = 0;
@@ -935,9 +958,9 @@ RList *linux_thread_list(RDebug *dbg, int pid, RList *list) {
 					continue;
 				}
 
-				if (procfs_pid_slurp (i, "comm", buf, sizeof(buf)) == -1) {
+				if (procfs_pid_slurp (i, "comm", buf, sizeof (buf)) == -1) {
 					/* fall back to auto-id */
-					snprintf (buf, sizeof(buf), "thread_%d", thid++);
+					snprintf (buf, sizeof (buf), "thread_%d", thid++);
 				}
 				r_list_append (list, r_debug_pid_new (buf, i, uid, 's', 0));
 			}
@@ -946,65 +969,73 @@ RList *linux_thread_list(RDebug *dbg, int pid, RList *list) {
 	return list;
 }
 
-#define PRINT_FPU(fpregs) \
-	r_cons_printf ("cwd = 0x%04x  ; control   ", (fpregs).cwd);\
-	r_cons_printf ("swd = 0x%04x  ; status\n", (fpregs).swd);\
-	r_cons_printf ("ftw = 0x%04x              ", (fpregs).ftw);\
-	r_cons_printf ("fop = 0x%04x\n", (fpregs).fop);\
-	r_cons_printf ("rip = 0x%016"PFMT64x"  ", (ut64)(fpregs).rip);\
-	r_cons_printf ("rdp = 0x%016"PFMT64x"\n", (ut64)(fpregs).rdp);\
-	r_cons_printf ("mxcsr = 0x%08x        ", (fpregs).mxcsr);\
-	r_cons_printf ("mxcr_mask = 0x%08x\n", (fpregs).mxcr_mask)\
+#define PRINT_FPU(cons, fpregs) \
+	r_cons_printf (cons, "cwd = 0x%04x  ; control   ", (fpregs).cwd);\
+	r_cons_printf (cons, "swd = 0x%04x  ; status\n", (fpregs).swd);\
+	r_cons_printf (cons, "ftw = 0x%04x              ", (fpregs).ftw);\
+	r_cons_printf (cons, "fop = 0x%04x\n", (fpregs).fop);\
+	r_cons_printf (cons, "rip = 0x%016"PFMT64x"  ", (ut64)(fpregs).rip);\
+	r_cons_printf (cons, "rdp = 0x%016"PFMT64x"\n", (ut64)(fpregs).rdp);\
+	r_cons_printf (cons, "mxcsr = 0x%08x        ", (fpregs).mxcsr);\
+	r_cons_printf (cons, "mxcr_mask = 0x%08x\n", (fpregs).mxcr_mask)\
 
-#define PRINT_FPU_NOXMM(fpregs) \
-	r_cons_printf ("cwd = 0x%04lx  ; control   ", (fpregs).cwd);\
-	r_cons_printf ("swd = 0x%04lx  ; status\n", (fpregs).swd);\
-	r_cons_printf ("twd = 0x%04lx              ", (fpregs).twd);\
-	r_cons_printf ("fip = 0x%04lx          \n", (fpregs).fip);\
-	r_cons_printf ("fcs = 0x%04lx              ", (fpregs).fcs);\
-	r_cons_printf ("foo = 0x%04lx          \n", (fpregs).foo);\
-	r_cons_printf ("fos = 0x%04lx              ", (fpregs).fos)
+#define PRINT_FPU_NOXMM(cons, fpregs) \
+	r_cons_printf (cons, "cwd = 0x%04lx  ; control   ", (fpregs).cwd);\
+	r_cons_printf (cons, "swd = 0x%04lx  ; status\n", (fpregs).swd);\
+	r_cons_printf (cons, "twd = 0x%04lx              ", (fpregs).twd);\
+	r_cons_printf (cons, "fip = 0x%04lx          \n", (fpregs).fip);\
+	r_cons_printf (cons, "fcs = 0x%04lx              ", (fpregs).fcs);\
+	r_cons_printf (cons, "foo = 0x%04lx          \n", (fpregs).foo);\
+	r_cons_printf (cons, "fos = 0x%04lx              ", (fpregs).fos)
 
-static void print_fpu(void *f){
+static void print_fpu(RCons *cons, void *f) {
+	if (!f) {
+		R_LOG_WARN ("getfpregs not implemented");
+		return;
+	}
 #if __x86_64__
 	int i,j;
 	struct user_fpregs_struct fpregs = *(struct user_fpregs_struct *)f;
 #if __ANDROID__
-	PRINT_FPU (fpregs);
+	PRINT_FPU (cons, fpregs);
 	for (i = 0;i < 8; i++) {
 		ut64 *b = (ut64 *)&fpregs.st_space[i*4];
 		ut32 *c = (ut32*)&fpregs.st_space;
 		float *f = (float *)&fpregs.st_space;
 		c = c + (i * 4);
 		f = f + (i * 4);
-		r_cons_printf ("st%d =%0.3lg (0x%016"PFMT64x") | %0.3f (%08x) | "\
+		r_cons_printf (cons, "st%d =%0.3lg (0x%016"PFMT64x") | %0.3f (%08x) | "\
 			"%0.3f (%08x) \n", i,
 			(double)*((double*)&fpregs.st_space[i*4]), *b, (float) f[0],
 			c[0], (float) f[1], c[1]);
 	}
 #else
-	r_cons_printf ("---- x86-64 ----\n");
-	PRINT_FPU (fpregs);
-	r_cons_printf ("size = 0x%08x\n", (ut32)sizeof (fpregs));
+	r_cons_printf (cons, "---- x86-64 ----\n");
+	PRINT_FPU (cons, fpregs);
+	r_cons_printf (cons, "size = 0x%08x\n", (ut32)sizeof (fpregs));
 	for (i = 0; i < 16; i++) {
 		ut32 *a = (ut32 *)&fpregs.xmm_space;
 		a = a + (i * 4);
-		r_cons_printf ("xmm%d = %08x %08x %08x %08x   ", i, (int)a[0], (int)a[1],
+		r_cons_printf (cons, "xmm%d = %08x %08x %08x %08x   ", i, (int)a[0], (int)a[1],
 					   (int)a[2], (int)a[3] );
 		if (i < 8) {
 			ut64 *st_u64 = (ut64*)&fpregs.st_space[i * 4];
 			ut8 *st_u8 = (ut8 *)&fpregs.st_space[i * 4];
 			long double *st_ld = (long double *)&fpregs.st_space[i * 4];
-			r_cons_printf ("mm%d = 0x%016" PFMT64x " | st%d = ", i, *st_u64, i);
+			r_cons_printf (cons, "mm%d = 0x%016" PFMT64x " | st%d = ", i, *st_u64, i);
 			// print as hex TBYTE - always little endian
 			for (j = 9; j >= 0; j--) {
-				r_cons_printf ("%02x", st_u8[j]);
+				r_cons_printf (cons, "%02x", st_u8[j]);
 			}
 			// Using %Lf and %Le even though we do not show the extra precision to avoid another cast
 			// %f with (double)*st_ld would also work
-			r_cons_printf (" %Le %Lf\n", *st_ld, *st_ld);
+#if R2_NO_LONG_DOUBLE
+			r_cons_printf (cons, " %e %f\n", (double)(*st_ld), (double)(*st_ld));
+#else
+			r_cons_printf (cons, " %Le %Lf\n", *st_ld, *st_ld);
+#endif
 		} else {
-			r_cons_printf ("\n");
+			r_cons_printf (cons, "\n");
 		}
 	}
 #endif // __ANDROID__
@@ -1012,17 +1043,17 @@ static void print_fpu(void *f){
 	int i;
 #if __ANDROID__
 	struct user_fpxregs_struct fpxregs = *(struct user_fpxregs_struct*)f;
-	r_cons_printf ("---- x86-32 ----\n");
-	r_cons_printf ("cwd = 0x%04x  ; control   ", fpxregs.cwd);
-	r_cons_printf ("swd = 0x%04x  ; status\n", fpxregs.swd);
-	r_cons_printf ("twd = 0x%04x ", fpxregs.twd);
-	r_cons_printf ("fop = 0x%04x\n", fpxregs.fop);
-	r_cons_printf ("fip = 0x%08x\n", (ut32)fpxregs.fip);
-	r_cons_printf ("fcs = 0x%08x\n", (ut32)fpxregs.fcs);
-	r_cons_printf ("foo = 0x%08x\n", (ut32)fpxregs.foo);
-	r_cons_printf ("fos = 0x%08x\n", (ut32)fpxregs.fos);
-	r_cons_printf ("mxcsr = 0x%08x\n", (ut32)fpxregs.mxcsr);
-	for(i = 0; i < 8; i++) {
+	r_cons_printf (cons, "---- x86-32 ----\n");
+	r_cons_printf (cons, "cwd = 0x%04x  ; control   ", fpxregs.cwd);
+	r_cons_printf (cons, "swd = 0x%04x  ; status\n", fpxregs.swd);
+	r_cons_printf (cons, "twd = 0x%04x ", fpxregs.twd);
+	r_cons_printf (cons, "fop = 0x%04x\n", fpxregs.fop);
+	r_cons_printf (cons, "fip = 0x%08x\n", (ut32)fpxregs.fip);
+	r_cons_printf (cons, "fcs = 0x%08x\n", (ut32)fpxregs.fcs);
+	r_cons_printf (cons, "foo = 0x%08x\n", (ut32)fpxregs.foo);
+	r_cons_printf (cons, "fos = 0x%08x\n", (ut32)fpxregs.fos);
+	r_cons_printf (cons, "mxcsr = 0x%08x\n", (ut32)fpxregs.mxcsr);
+	for (i = 0; i < 8; i++) {
 		ut32 *a = (ut32*)(&fpxregs.xmm_space);
 		ut64 *b = (ut64 *)(&fpxregs.st_space[i * 4]);
 		ut32 *c = (ut32*)&fpxregs.st_space;
@@ -1030,25 +1061,25 @@ static void print_fpu(void *f){
 		a = a + (i * 4);
 		c = c + (i * 4);
 		f = f + (i * 4);
-		r_cons_printf ("xmm%d = %08x %08x %08x %08x   ", i, (int)a[0],
+		r_cons_printf (cons, "xmm%d = %08x %08x %08x %08x   ", i, (int)a[0],
 			(int)a[1], (int)a[2], (int)a[3] );
-		r_cons_printf ("st%d = %0.3lg (0x%016"PFMT64x") | %0.3f (0x%08x) | "\
+		r_cons_printf (cons, "st%d = %0.3lg (0x%016"PFMT64x") | %0.3f (0x%08x) | "\
 			"%0.3f (0x%08x)\n", i,
 			(double)*((double*)(&fpxregs.st_space[i*4])), b[0],
 			f[0], c[0], f[1], c[1]);
 	}
 #else
 	struct user_fpregs_struct fpregs = *(struct user_fpregs_struct *)f;
-	r_cons_printf ("---- x86-32-noxmm ----\n");
-	PRINT_FPU_NOXMM (fpregs);
-	for(i = 0; i < 8; i++) {
+	r_cons_printf (cons, "---- x86-32-noxmm ----\n");
+	PRINT_FPU_NOXMM (cons, fpregs);
+	for (i = 0; i < 8; i++) {
 		ut64 *b = (ut64 *)(&fpregs.st_space[i*4]);
 		double *d = (double*)b;
 		ut32 *c = (ut32*)&fpregs.st_space;
 		float *f = (float *)&fpregs.st_space;
 		c = c + (i * 4);
 		f = f + (i * 4);
-		r_cons_printf ("st%d = %0.3lg (0x%016"PFMT64x") | %0.3f (0x%08x) | "\
+		r_cons_printf (cons, "st%d = %0.3lg (0x%016"PFMT64x") | %0.3f (0x%08x) | "\
 			"%0.3f (0x%08x)\n", i, d[0], b[0], f[0], c[0], f[1], c[1]);
 	}
 #endif
@@ -1057,13 +1088,15 @@ static void print_fpu(void *f){
 #endif
 }
 
-int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
+bool linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 	bool showfpu = false;
+	RCore *core = dbg->coreb.core;
+	RCons *cons = core->cons;
 	int pid = dbg->tid;
 	if (pid == -1) {
 		if (dbg->pid == -1) {
-			eprintf ("linux_reg_read: Invalid pid %d\n", pid);
-			return 0;
+			R_LOG_ERROR ("Invalid pid %d", pid);
+			return false;
 		}
 		pid = dbg->pid;
 	}
@@ -1087,8 +1120,8 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 			}
 			long ret = r_debug_ptrace (dbg, PTRACE_PEEKUSER, pid,
 					(void *)r_offsetof (struct user, u_debugreg[i]), 0);
-			if ((i+1) * sizeof (ret) > size) {
-				eprintf ("linux_reg_get: Buffer too small %d\n", size);
+			if ((i + 1) * sizeof (ret) > size) {
+				R_LOG_ERROR ("Buffer of %d is too small for ptrace.peekuser", size);
 				break;
 			}
 			memcpy (buf + (i * sizeof (ret)), &ret, sizeof (ret));
@@ -1103,12 +1136,10 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 		return true;
 		break;
 	case R_REG_TYPE_FPU:
-	case R_REG_TYPE_MMX:
-	case R_REG_TYPE_XMM:
-#if __POWERPC__
-		return false;
-#elif __x86_64__ || __i386__
+	case R_REG_TYPE_VEC64: // MMX
+	case R_REG_TYPE_VEC128: // XMM
 		{
+#if __x86_64__ || __i386__
 		struct user_fpregs_struct fpregs;
 		if (type == R_REG_TYPE_FPU) {
 #if __x86_64__
@@ -1118,7 +1149,7 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 				return false;
 			}
 			if (showfpu) {
-				print_fpu ((void *)&fpregs);
+				print_fpu (cons, (void *)&fpregs);
 			}
 			size = R_MIN (sizeof (fpregs), size);
 			memcpy (buf, &fpregs, size);
@@ -1129,7 +1160,7 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 			ret = r_debug_ptrace (dbg, PTRACE_GETFPXREGS, pid, NULL, &fpxregs);
 			if (ret == 0) {
 				if (showfpu) {
-					print_fpu ((void *)&fpxregs);
+					print_fpu (cons, (void *)&fpxregs);
 				}
 				size = R_MIN (sizeof (fpxregs), size);
 				memcpy (buf, &fpxregs, size);
@@ -1137,7 +1168,7 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 			} else {
 				ret = r_debug_ptrace (dbg, PTRACE_GETFPREGS, pid, NULL, &fpregs);
 				if (showfpu) {
-					print_fpu ((void *)&fpregs);
+					print_fpu (cons, (void *)&fpregs);
 				}
 				if (ret != 0) {
 					r_sys_perror ("PTRACE_GETFPREGS");
@@ -1150,7 +1181,7 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 #else
 			ret = r_debug_ptrace (dbg, PTRACE_GETFPREGS, pid, NULL, &fpregs);
 			if (showfpu) {
-				print_fpu ((void *)&fpregs);
+				print_fpu (cons, (void *)&fpregs);
 			}
 			if (ret != 0) {
 				r_sys_perror ("PTRACE_GETFPREGS");
@@ -1162,54 +1193,47 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 #endif // !__ANDROID__
 #endif // __i386__
 		}
+		return size;
+#elif (__arm64__ || __aarch64__) && defined(PTRACE_GETREGSET)
+		// ARM64 FPU register reading using ptrace GETREGSET
+		struct iovec iov;
+		ut8 fpu_regs[512]; // ARM64 FPU register space (32 * 16 bytes for v0-v31)
+		memset (fpu_regs, 0, sizeof (fpu_regs));
+		iov.iov_base = fpu_regs;
+		iov.iov_len = sizeof (fpu_regs);
+
+		ret = r_debug_ptrace (dbg, PTRACE_GETREGSET, pid, (void*)(size_t)NT_PRFPREG, &iov);
+		if (ret != 0) {
+			r_sys_perror ("PTRACE_GETREGSET NT_PRFPREG");
+			return false;
 		}
+
+		if (showfpu) {
+			print_fpu (cons, (void *)fpu_regs);
+		}
+
+		size = R_MIN (iov.iov_len, size);
+		memcpy (buf, fpu_regs, size);
+		return size;
 #else
+		if (showfpu) {
+			print_fpu (cons, NULL);
+		}
 	#warning getfpregs not implemented for this platform
 #endif
-		break;
-	case R_REG_TYPE_SEG:
-	case R_REG_TYPE_FLG:
-	case R_REG_TYPE_GPR:
-		{
-			R_DEBUG_REG_T regs;
-			memset (&regs, 0, sizeof (regs));
-			memset (buf, 0, size);
-#if (__arm64__ || __aarch64__ || __s390x__) && defined(PTRACE_GETREGSET)
-			struct iovec io = {
-				.iov_base = &regs,
-				.iov_len = sizeof (regs)
-			};
-			ret = r_debug_ptrace (dbg, PTRACE_GETREGSET, pid, (size_t)1, &io);
-			// ret = ptrace (PTRACE_GETREGSET, pid, (void*)(size_t)(NT_PRSTATUS), NULL); // &io);
-#elif __BSD__ && (__POWERPC__ || __sparc__)
-			ret = r_debug_ptrace (dbg, PTRACE_GETREGS, pid, &regs, NULL);
-#else
-			/* linux -{arm/mips/riscv/x86/x86_64} */
-			ret = r_debug_ptrace (dbg, PTRACE_GETREGS, pid, NULL, &regs);
-#endif
-			/*
-			 * if perror here says 'no such process' and the
-			 * process exists still.. is because there's a missing call
-			 * to 'wait'. and the process is not yet available to accept
-			 * more ptrace queries.
-			 */
-			if (ret != 0) {
-				r_sys_perror ("PTRACE_GETREGS");
-				return false;
-			}
-			size = R_MIN (sizeof (regs), size);
-			memcpy (buf, &regs, size);
-			return size;
 		}
 		break;
-	case R_REG_TYPE_YMM:
+	case R_REG_TYPE_VEC512: // ZMM
+		R_LOG_DEBUG ("zmm registers not supported yet");
+		break;
+	case R_REG_TYPE_VEC256: // YMM
 		{
 #if HAVE_YMM && __x86_64__ && defined(PTRACE_GETREGSET)
 		ut32 ymm_space[128];	// full ymm registers
 		struct _xstate xstate;
 		struct iovec iov = {};
 		iov.iov_base = &xstate;
-		iov.iov_len = sizeof(struct _xstate);
+		iov.iov_len = sizeof (struct _xstate);
 		ret = r_debug_ptrace (dbg, PTRACE_GETREGSET, pid, (void*)NT_X86_XSTATE, &iov);
 		if (errno == ENODEV) {
 			// ignore ENODEV, it just means this CPU or kernel doesn't support XSTATE
@@ -1224,7 +1248,7 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 		// stitch together xstate.fpstate._xmm and xstate.ymmh assuming LE
 		int ri,rj;
 		for (ri = 0; ri < 16; ri++)	{
-			for (rj=0; rj < 4; rj++)	{
+			for (rj = 0; rj < 4; rj++)	{
 #ifdef __ANDROID__
 				ymm_space[ri*8+rj] = ((struct _libc_fpstate*) &xstate.fpstate)->_xmm[ri].element[rj];
 #else
@@ -1242,11 +1266,53 @@ int linux_reg_read(RDebug *dbg, int type, ut8 *buf, int size) {
 		return false;
 		}
 		break;
+	case R_REG_TYPE_SEG:
+	case R_REG_TYPE_FLG:
+	case R_REG_TYPE_GPR:
+		{
+			R_DEBUG_REG_T regs;
+			memset (&regs, 0, sizeof (regs));
+			memset (buf, 0, size);
+#if (__arm64__ || __aarch64__ || __s390x__) && defined(PTRACE_GETREGSET)
+			struct iovec io = {
+				.iov_base = &regs,
+				.iov_len = sizeof (regs)
+			};
+			ret = r_debug_ptrace (dbg, PTRACE_GETREGSET, pid, (void*)(size_t)1, &io);
+			// ret = ptrace (PTRACE_GETREGSET, pid, (void*)(size_t)(NT_PRSTATUS), NULL); // &io);
+#elif R2__BSD__ && (__POWERPC__ || __sparc__)
+			ret = r_debug_ptrace (dbg, PTRACE_GETREGS, pid, &regs, NULL);
+#elif __riscv
+			// theres no PTRACE_GETREGS implemented for rv64
+			struct iovec iov;
+			iov.iov_base = &regs;
+			iov.iov_len = sizeof (regs);
+			ret = ptrace (PTRACE_GETREGSET, pid, NT_PRSTATUS, &iov);
+#else
+			/* linux -{arm/mips/x86/x86_64} */
+			ret = r_debug_ptrace (dbg, PTRACE_GETREGS, pid, NULL, &regs);
+#endif
+			/*
+			 * if perror here says 'no such process' and the
+			 * process exists still.. is because there's a missing call
+			 * to 'wait'. and the process is not yet available to accept
+			 * more ptrace queries.
+			 */
+			if (ret != 0) {
+				r_sys_perror ("PTRACE_GETREGS");
+				return false;
+			}
+			size = R_MIN (sizeof (regs), size);
+			memcpy (buf, &regs, size);
+			// r_print_hexdump (NULL, 0, buf, size, 16, 16, 0);
+			return size;
+		}
+		break;
 	}
 	return false;
 }
 
-int linux_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
+bool linux_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
 	int pid = dbg->tid;
 
 	if (type == R_REG_TYPE_DRX) {
@@ -1259,7 +1325,7 @@ int linux_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
 			}
 			if (r_debug_ptrace (dbg, PTRACE_POKEUSER, pid,
 					(void *)r_offsetof (struct user, u_debugreg[i]), (r_ptrace_data_t)val[i])) {
-				eprintf ("ptrace error for dr %d\n", i);
+				R_LOG_ERROR ("ptrace failed for dr %d", i);
 				r_sys_perror ("ptrace POKEUSER");
 			}
 		}
@@ -1269,7 +1335,7 @@ int linux_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
 #endif
 	}
 	if (type == R_REG_TYPE_GPR) {
-#if __arm64__ || __aarch64__ || __s390x__
+#if __arm64__ || __aarch64__ || __s390x__ || __riscv
 		struct iovec io = {
 			.iov_base = (void*)buf,
 			.iov_len = sizeof (R_DEBUG_REG_T)
@@ -1291,22 +1357,31 @@ int linux_reg_write(RDebug *dbg, int type, const ut8 *buf, int size) {
 		}
 		return true;
 	}
-	if (type == R_REG_TYPE_FPU) {
+	if (type == R_REG_TYPE_FPU || type == R_REG_TYPE_VEC64 || type == R_REG_TYPE_VEC128) {
 #if __i386__ || __x86_64__
 		int ret = r_debug_ptrace (dbg, PTRACE_SETFPREGS, pid, 0, (void*)buf);
 		return (ret != 0) ? false : true;
+#elif (__arm64__ || __aarch64__) && defined(PTRACE_SETREGSET)
+		// ARM64 FPU/VEC register writing using ptrace SETREGSET
+		struct iovec iov;
+		iov.iov_base = (void*)buf;
+		iov.iov_len = size;
+
+		int ret = r_debug_ptrace (dbg, PTRACE_SETREGSET, pid, (void*)(size_t)NT_PRFPREG, &iov);
+		if (ret != 0) {
+			r_sys_perror ("PTRACE_SETREGSET NT_PRFPREG");
+			return false;
+		}
+		return true;
 #endif
 	}
 	return false;
 }
 
-RList *linux_desc_list (int pid) {
-	RList *ret = NULL;
-	char path[512], file[512], buf[512];
+RList *linux_desc_list(int pid) {
+	char path[512], buf[512];
 	struct dirent *de;
-	RDebugDesc *desc;
 	int type, perm;
-	int len, len2;
 	struct stat st;
 	DIR *dd = NULL;
 
@@ -1315,40 +1390,51 @@ RList *linux_desc_list (int pid) {
 		r_sys_perror ("opendir /proc/x/fd");
 		return NULL;
 	}
-	ret = r_list_new ();
+	RList *ret = r_list_new ();
 	if (!ret) {
 		closedir (dd);
 		return NULL;
 	}
 	ret->free = (RListFree)r_debug_desc_free;
-	while ((de = (struct dirent *)readdir(dd))) {
+	while ((de = (struct dirent *)readdir (dd))) {
 		if (de->d_name[0] == '.') {
 			continue;
 		}
-		len = strlen (path);
-		len2 = strlen (de->d_name);
-		if (len + len2 + 1 >= sizeof(file)) {
+		char *fn = r_str_newf ("/proc/%d/fd/%s", pid, de->d_name);
+		memset (buf, 0, sizeof (buf));
+		if (readlink (fn, buf, sizeof (buf) - 1) == -1) {
 			r_list_free (ret);
 			closedir (dd);
-			eprintf ("Filename is too long");
-			return NULL;
-		}
-		memcpy (file, path, len);
-		memcpy (file + len, de->d_name, len2 + 1);
-		buf[0] = 0;
-		if (readlink (file, buf, sizeof (buf) - 1) == -1) {
+			free (fn);
+			r_sys_perror ("readlink failure");
 			return NULL;
 		}
 		buf[sizeof (buf) - 1] = 0;
 		type = perm = 0;
-		if (stat (file, &st) != -1) {
-			type  = st.st_mode & S_IFIFO  ? 'P':
+
+		// Read file type
+		if (stat (fn, &st) != -1) {
+			bool isfifo = st.st_mode & S_IFIFO;
 #ifdef S_IFSOCK
-				st.st_mode & S_IFSOCK ? 'S':
+			/* Do *not* remove the == here. S_IFSOCK can be multiple
+			 * bits, and we must match all of them. */
+			bool issock = (st.st_mode & S_IFSOCK) == S_IFSOCK;
 #endif
-				st.st_mode & S_IFCHR  ? 'C':'-';
+			bool ischr = st.st_mode & S_IFCHR;
+			if (isfifo) {
+				type = 'P';
+#ifdef S_IFSOCK
+			} else if (issock) {
+				type = 'S';
+#endif
+			} else if (ischr) {
+				type = 'C';
+			} else {
+				type = '-';
+			}
 		}
-		if (lstat(path, &st) != -1) {
+		// Read permissions // TOCTOU
+		if (lstat (fn, &st) != -1) {
 			if (st.st_mode & S_IRUSR) {
 				perm |= R_PERM_R;
 			}
@@ -1356,8 +1442,28 @@ RList *linux_desc_list (int pid) {
 				perm |= R_PERM_W;
 			}
 		}
-		//TODO: Offset
-		desc = r_debug_desc_new (atoi (de->d_name), buf, perm, type, 0);
+		free (fn);
+		// Get offset
+		fn = r_str_newf ("/proc/%d/fdinfo/%s", pid, de->d_name);
+		int f = open (fn, O_RDONLY);
+		char fdinfo[512];
+		fdinfo[0] = 0;
+		if (f >= 0) {
+			if (read (f, fdinfo, sizeof (fdinfo) - 1) < 0) {
+				R_LOG_WARN ("failed to read %s", fn);
+				close (f);
+				r_list_free (ret);
+				closedir (dd);
+				free (fn);
+				return NULL;
+			}
+			fdinfo[sizeof (fdinfo) - 1] = '\0';
+			close (f);
+		}
+		free (fn);
+		/* First line of fdinfo is "pos: [offset]" */
+		ut64 offset = (int) r_num_math (NULL, r_str_trim_head_ro (fdinfo + 4));
+		RDebugDesc *desc = r_debug_desc_new (atoi (de->d_name), buf, perm, type, offset);
 		if (!desc) {
 			break;
 		}
