@@ -16,21 +16,21 @@
 // no attsyntax for arm
 static char *regs[] = R_GP;
 
-static void emit_init (REgg *egg) {
+static void emit_init(REgg *egg) {
 	/* TODO */
 }
 
-static char *emit_syscall (REgg *egg, int num) {
+static char *emit_syscall(REgg *egg, int num) {
 	char buf[32];
 	snprintf (buf, sizeof (buf), "syscall (%d)\n", num);
 	return strdup (buf);
 }
 
-static void emit_frame (REgg *egg, int sz) {
+static void emit_frame(REgg *egg, int sz) {
 	r_egg_printf (egg, "frame (%d)\n", sz);
 }
 
-static void emit_frame_end (REgg *egg, int sz, int ctx) {
+static void emit_frame_end(REgg *egg, int sz, int ctx) {
 	r_egg_printf (egg, "frame_end (%d, %d)\n", sz, ctx);
 }
 
@@ -43,7 +43,7 @@ static void emit_comment(REgg *egg, const char *fmt, ...) {
 	va_end (ap);
 }
 
-static void emit_equ (REgg *egg, const char *key, const char *value) {
+static void emit_equ(REgg *egg, const char *key, const char *value) {
 	r_egg_printf (egg, "equ (%s, %s)\n", key, value);
 }
 
@@ -57,17 +57,23 @@ static void emit_set_string(REgg *egg, const char *dstvar, const char *str, int
 }
 
 static void emit_call(REgg *egg, const char *str, int atr) {
-	if (atr) r_egg_printf (egg, "call ([%s])\n", str);
-	else r_egg_printf (egg, "call (%s)\n", str);
+	if (atr) {
+		r_egg_printf (egg, "call ([%s])\n", str);
+	} else {
+		r_egg_printf (egg, "call (%s)\n", str);
+	}
 }
 
 static void emit_jmp(REgg *egg, const char *str, int atr) {
-	if (atr) r_egg_printf (egg, "goto ([%s])\n", str);
-	else r_egg_printf (egg, "goto (%s)\n", str);
+	if (atr) {
+		r_egg_printf (egg, "goto ([%s])\n", str);
+	} else {
+		r_egg_printf (egg, "goto (%s)\n", str);
+	}
 }
 
-static void emit_arg (REgg *egg, int xs, int num, const char *str) {
-	// TODO: enhace output here
+static void emit_arg(REgg *egg, int xs, int num, const char *str) {
+	// TODO: enhance output here
 	r_egg_printf (egg, "arg.%d.%d=%s\n", xs, num, str);
 }
 
@@ -75,28 +81,28 @@ static void emit_get_result(REgg *egg, const char *ocn) {
 	r_egg_printf (egg, "get_result (%s)\n", ocn);
 }
 
-static void emit_restore_stack (REgg *egg, int size) {
+static void emit_restore_stack(REgg *egg, int size) {
 	r_egg_printf (egg, "restore_stack (%d)\n", size);
 	// XXX: must die.. or add emit_store_stack. not needed by ARM
 	// r_egg_printf (egg, "  add sp, %d\n", size);
 }
 
-static void emit_get_while_end (REgg *egg, char *str, const char *ctxpush, const char *label) {
+static void emit_get_while_end(REgg *egg, char *str, const char *ctxpush, const char *label) {
 	r_egg_printf (egg, "get_while_end (%s, %s, %s)\n", str, ctxpush, label);
 }
 
-static void emit_while_end (REgg *egg, const char *labelback) {
+static void emit_while_end(REgg *egg, const char *labelback) {
 	r_egg_printf (egg, "while_end (%s)\n", labelback);
 }
 
-static void emit_get_var (REgg *egg, int type, char *out, int idx) {
+static void emit_get_var(REgg *egg, int type, char *out, int idx) {
 	switch (type) {
 	case 0: sprintf (out, "fp,$%d", -idx); break; /* variable */
 	case 1: sprintf (out, "sp,$%d", idx); break; /* argument */ // XXX: MUST BE r0, r1, r2, ..
 	}
 }
 
-static void emit_trap (REgg *egg) {
+static void emit_trap(REgg *egg) {
 	r_egg_printf (egg, "trap\n");
 }
 
@@ -113,25 +119,26 @@ static void emit_branch(REgg *egg, char *b, char *g, char *e, char *n, int sz, c
 	/* NOTE that jb/ja are inverted to fit cmp opcode */
 	if (b) {
 		*b = '\0';
-		op = e?"bge":"bgt";
-		arg = b+1;
-	} else
-	if (g) {
+		op = e? "bge": "bgt";
+		arg = b + 1;
+	} else if (g) {
 		*g = '\0';
-		op = e?"ble":"blt";
-		arg = g+1;
+		op = e? "ble": "blt";
+		arg = g + 1;
 	}
 	if (!arg) {
 		if (e) {
-			arg = e+1;
+			arg = e + 1;
 			op = "bne";
 		} else {
 			arg = "0";
-			op = n?"bne":"beq";
+			op = n? "bne": "beq";
 		}
 	}
 
-	if (*arg=='=') arg++; /* for <=, >=, ... */
+	if (*arg == '=') {
+		arg++; /* for <=, >=, ... */
+	}
 	p = r_egg_mkvar (egg, str, arg, 0);
 	r_egg_printf (egg, "%s (%s) => (%s)\n", op, p, dst);
 	free (p);
@@ -154,17 +161,23 @@ static void emit_mathop(REgg *egg, int ch, int vs, int type, const char *eq, con
 	case '/': op = "div"; break;
 	default:  op = "mov"; break;
 	}
-	if (!eq) eq = R_AX;
-	if (!p) p = R_AX;
+	if (!eq) {
+		eq = R_AX;
+	}
+	if (!p) {
+		p = R_AX;
+	}
 #if 0
 	// TODO:
 	eprintf ("TYPE = %c\n", type);
 	eprintf ("  %s%c %c%s, %s\n", op, vs, type, eq, p);
 	eprintf ("  %s %s, [%s]\n", op, p, eq);
 #endif
-	if (type == '*')
+	if (type == '*') {
 		r_egg_printf (egg, "%s (%s, [%s])\n", op, p, eq);
-	else r_egg_printf (egg, "%s (%s, %s)\n", op, p, eq);
+	} else {
+		r_egg_printf (egg, "%s (%s, %s)\n", op, p, eq);
+	}
 }
 
 static const char* emit_regs(REgg *egg, int idx) {
