@@ -1,21 +1,15 @@
-/* radare2 - LGPL - Copyright 2009-2024 - nibble, pancake, xvilka */
+/* radare2 - LGPL - Copyright 2009-2025 - nibble, pancake, xvilka */
 
 #ifndef R2_ANAL_H
 #define R2_ANAL_H
 
-/* use old refs and function storage */
-// still required by core in lot of places
-#define USE_VARSUBS 0
-
 #include <r_cons.h>
-#include <r_types.h>
 #include <r_io.h>
 #include <r_esil.h>
 #include <r_arch.h>
 #include <r_list.h>
 #include <r_util/r_print.h>
 #include <r_search.h>
-#include <r_util.h>
 #include <r_bind.h>
 #include <r_syscall.h>
 #include <r_flag.h>
@@ -63,6 +57,15 @@ typedef struct r_anal_range_t {
 	RBNode rb;
 } RAnalRange;
 
+enum {
+	R_ANAL_REFLINE_TYPE_UTF8 = 1,
+	R_ANAL_REFLINE_TYPE_WIDE = 2,  /* reflines have a space between them */
+	R_ANAL_REFLINE_TYPE_MIDDLE_BEFORE = 4, /* do not consider starts/ends of
+	                                        * reflines (used for comment lines before disasm) */
+	R_ANAL_REFLINE_TYPE_MIDDLE_AFTER = 8, /* as above but for lines after disasm */
+	R_ANAL_REFLINE_TYPE_SPLIT = 16 /* use reflines2 for upward lines */
+};
+
 enum {
 	R_ANAL_DATA_TYPE_NULL = 0,
 	R_ANAL_DATA_TYPE_UNKNOWN = 1,
@@ -273,7 +276,7 @@ typedef struct r_anal_function_t {
 	char *realname; // R2_590: add realname for the mangled one
 	int bits; // ((> bits 0) (set-bits bits))
 	int type;
-	const char *cc; // calling convention, should come from RAnal.constpool
+	const char *callconv; // calling convention, should come from RAnal.constpool
 	ut64 addr;
 	HtUP/*<ut64, char *>*/ *labels;
 	HtPP/*<char *, ut64 *>*/ *label_addrs;
@@ -297,6 +300,7 @@ typedef struct r_anal_function_t {
 	RAnalFcnMeta meta;
 	RList *imports; // maybe bound to class?
 	struct r_anal_t *anal; // this function is associated with this instance
+	ut64 ts; // timestamp when the function was registered. useful to sort them by order or "incremental projects"). afla
 } RAnalFunction;
 
 typedef struct r_anal_func_arg_t {
@@ -333,9 +337,8 @@ typedef enum {
 
 /* meta */
 typedef struct r_anal_meta_item_t {
-	// XXX missing size
-	RAnalMetaType type;
-	int subtype;
+	uint8_t type; // RAnalMetaType type;
+	uint8_t subtype;
 	char *str;
 	const RSpace *space;
 } RAnalMetaItem;
@@ -385,11 +388,14 @@ typedef struct r_anal_options_t {
 	bool propagate_noreturn;
 	bool recursive_noreturn; // anal.rnr
 	bool slow;
-#if R2_USE_NEW_ABI
 	bool flagends;
-	bool zigndups; // R2_600 -  not used yet..
-	// R2_600 - add zign_dups field for "zign.dups" config
-#endif
+	bool zigndups;
+	bool icods; // R2_600 -- add anal.icods or anal.xrefs.indirect references. needed for stm8 at least
+	char *tparser;
+	bool dynprefix;
+	const char *defprefix;
+	const char *prefix_marker;
+	ut64 prefix_radius;
 } RAnalOptions;
 
 // XXX we have cc / calling conventions / abi settings already no need for a custom enum here
@@ -431,7 +437,7 @@ typedef struct r_anal_t {
 	ut8 *last_disasm_reg;
 	int last_disasm_reg_size;
 	RSyscall *syscall;
-	int diff_ops;
+	bool diff_ops;
 	double diff_thbb;
 	double diff_thfcn;
 	RIOBind iob;
@@ -471,6 +477,7 @@ typedef struct r_anal_t {
 	RAnalCallbacks cb;
 	RAnalOptions opt;
 	RList *reflines;
+	RList *reflines2;
 	RListComparator columnSort;
 	int stackptr;
 	bool (*log)(struct r_anal_t *anal, const char *msg);
@@ -525,7 +532,7 @@ typedef struct r_anal_var_access_t {
 } RAnalVarAccess;
 
 typedef struct r_anal_var_constraint_t {
-	_RAnalCond cond;
+	RAnalCondType cond;
 	ut64 val;
 } RAnalVarConstraint;
 
@@ -592,11 +599,12 @@ typedef struct r_anal_bind_t {
 	RAnalUse use;
 } RAnalBind;
 
-#define R_ANAL_COND_SINGLE(x) (!x->arg[1] || x->arg[0]==x->arg[1])
+#define R_ANAL_CONDTYPE_SINGLE(x) (!x->right || x->left==x->right)
 
 typedef struct r_anal_cond_t {
 	int type; // filled by CJMP opcode
-	RArchValue *arg[2]; // filled by CMP opcode
+	RArchValue *left; // filled by CMP left opcode
+	RArchValue *right; // filled by CMP right opcode
 } RAnalCond;
 
 typedef struct r_anal_bb_t {
@@ -719,6 +727,26 @@ typedef struct r_anal_esil_cfg_t {
 	RGraph *g;
 } RAnalEsilCFG;
 
+// this is 80-bit offsets so we can address every piece of esil in an instruction
+typedef struct r_anal_esil_expr_offset_t {
+	ut64 off;
+	ut16 idx;
+} RAnalEsilEOffset;
+
+typedef enum {
+	R_ANAL_ESIL_BLOCK_ENTER_NORMAL = 0,
+	R_ANAL_ESIL_BLOCK_ENTER_TRUE,
+	R_ANAL_ESIL_BLOCK_ENTER_FALSE,
+	R_ANAL_ESIL_BLOCK_ENTER_GLUE,
+} RAnalEsilBlockEnterType;
+
+typedef struct r_anal_esil_basic_block_t {
+	RAnalEsilEOffset first;
+	RAnalEsilEOffset last;
+	char *expr;	//synthesized esil-expression for this block
+	RAnalEsilBlockEnterType enter;	//maybe more type is needed here
+} RAnalEsilBB;
+
 enum {
 	R_ANAL_ESIL_DFG_TAG_CONST = 1,
 	R_ANAL_ESIL_DFG_TAG_VAR = 2,
@@ -761,6 +789,8 @@ typedef bool (*RAnalCmdCallback)(/* Rcore */RAnal *anal, const char* input);
 typedef int (*RAnalOpCallback)(RAnal *a, RAnalOp *op, ut64 addr, const ut8 *data, int len, RAnalOpMask mask);
 typedef int (*RAnalOpAsmCallback)(RAnal *a, ut64 addr, const char *str, ut8 *outbuf, int outlen);
 
+typedef bool (*RAnalPluginEligible)(RAnal *a);
+
 typedef bool (*RAnalRegProfCallback)(RAnal *a);
 typedef char*(*RAnalRegProfGetCallback)(RAnal *a);
 typedef int (*RAnalFPBBCallback)(RAnal *a, RAnalBlock *bb);
@@ -773,6 +803,10 @@ typedef int (*REsilCB)(REsil *esil);
 typedef int (*REsilLoopCB)(REsil *esil, RAnalOp *op);
 typedef int (*REsilTrapCB)(REsil *esil, int trap_type, int trap_code);
 
+typedef char *(*RAnalTypesParser)(RAnal *a, const char *s);
+typedef char *(*RAnalTypesParserFile)(RAnal *a, const char *s, const char *dir);
+typedef char *(*RAnalTypesParserText)(RAnal *a, const char *s);
+
 typedef struct r_anal_plugin_t {
 	RPluginMeta meta;
 
@@ -780,10 +814,16 @@ typedef struct r_anal_plugin_t {
 
 	bool (*init)(RAnal *a);
 	bool (*fini)(RAnal *a);
+	RAnalPluginEligible eligible;
 
 	// legacy r_anal_functions
 	RAnalOpCallback op;
 	RAnalCmdCallback cmd;
+
+	// implement custom types parser and dumper
+	RAnalTypesParserText tparse_text;
+	RAnalTypesParserFile tparse_file;
+	RAnalTypesParser tdump;
 #if 1
 	/// XXX unused but referenced, maybe worth checking in case we want them for anal
 	RAnalFPBBCallback fingerprint_bb;
@@ -894,7 +934,7 @@ R_API bool r_anal_block_recurse_depth_first(RAnalBlock *block, RAnalBlockCb cb,
 R_API RList *r_anal_block_recurse_list(RAnalBlock *block);
 
 // return one shortest path from block to dst or NULL if none exists.
-R_API R_NULLABLE RList/*<RAnalBlock *>*/ *r_anal_block_shortest_path(RAnalBlock *block, ut64 dst);
+R_API RList/*<RAnalBlock *>*/ * R_NULLABLE r_anal_block_shortest_path(RAnalBlock *block, ut64 dst);
 
 // Add a case to the block's switch_op.
 // If block->switch_op is NULL, it will be created with the given switch_addr.
@@ -940,7 +980,7 @@ R_API RList *r_anal_get_functions_in(RAnal *anal, ut64 addr);
 // returns the function that has its entrypoint at addr or NULL
 R_API RAnalFunction *r_anal_get_function_at(RAnal *anal, ut64 addr);
 
-R_API bool r_anal_function_delete(RAnalFunction *fcn);
+R_API bool r_anal_function_delete(RAnal *anal, RAnalFunction *fcn);
 
 // rhange the entrypoint of fcn
 // This can fail (and return false) if there is already another function at the new address
@@ -953,7 +993,6 @@ R_API bool r_anal_function_rename(RAnalFunction *fcn, const char *name);
 R_API void r_anal_function_add_block(RAnalFunction *fcn, RAnalBlock *bb);
 R_API void r_anal_function_remove_block(RAnalFunction *fcn, RAnalBlock *bb);
 
-
 // size of the entire range that the function spans, including holes.
 // this is exactly r_anal_function_max_addr() - r_anal_function_min_addr()
 R_API ut64 r_anal_function_linear_size(RAnalFunction *fcn);
@@ -1039,11 +1078,11 @@ R_API void r_anal_pin_unset(RAnal *a, ut64 addr);
 
 /* fcn.c */
 R_API ut32 r_anal_function_cost(RAnalFunction *fcn);
-R_API int r_anal_function_count_edges(const RAnalFunction *fcn, R_NULLABLE int *ebbs);
+R_API int r_anal_function_count_edges(const RAnalFunction *fcn, int * R_NULLABLE ebbs);
 
 R_API RAnalFunction *r_anal_get_function_byname(RAnal *anal, const char *name);
 
-R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int reftype);
+R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, int reftype);
 R_API int r_anal_function_del(RAnal *anal, ut64 addr);
 R_API bool r_anal_function_add_bb(RAnal *anal, RAnalFunction *fcn,
 		ut64 addr, ut64 size,
@@ -1056,7 +1095,6 @@ R_API void r_anal_function_update_analysis(RAnalFunction *fcn);
 
 #define R_ANAL_FCN_VARKIND_LOCAL 'v'
 
-
 R_API int r_anal_function_var_del_byindex(RAnal *a, ut64 fna, const char kind, int scope, ut32 idx);
 /* args */
 R_API int r_anal_var_count(RAnal *a, RAnalFunction *fcn, int kind, int type);
@@ -1071,7 +1109,6 @@ R_API int r_anal_function_complexity(RAnalFunction *fcn);
 R_API int r_anal_function_loops(RAnalFunction *fcn);
 R_API void r_anal_trim_jmprefs(RAnal *anal, RAnalFunction *fcn);
 R_API void r_anal_del_jmprefs(RAnal *anal, RAnalFunction *fcn);
-R_API char *r_anal_function_get_json(RAnalFunction *function);
 R_API RAnalFunction *r_anal_function_next(RAnal *anal, ut64 addr);
 R_API char *r_anal_function_get_signature(RAnalFunction *function);
 R_API bool r_anal_function_del_signature(RAnal *a, const char *name);
@@ -1086,6 +1123,7 @@ R_API int r_anal_function_instrcount(RAnalFunction *fcn);
 R_API bool r_anal_function_islineal(RAnalFunction *fcn);
 R_API const char *r_anal_pin_get(RAnal *a, const char *name);
 R_API const char *r_anal_pin_at(RAnal *a, ut64 addr);
+R_API const char * R_NONNULL r_anal_fcn_prefix_at(RAnal *anal, ut64 addr);
 R_API bool r_anal_pin_set(RAnal *a, const char *name, const char *cmd);
 
 typedef bool (* RAnalRefCmp)(RAnalRef *ref, void *data);
@@ -1106,7 +1144,7 @@ R_API RVecAnalRef *r_anal_xrefs_get(RAnal *anal, ut64 to);
 R_API RVecAnalRef *r_anal_refs_get(RAnal *anal, ut64 from);
 R_API bool r_anal_xrefs_has_xrefs_at(RAnal *anal, ut64 at);
 R_API RVecAnalRef *r_anal_xrefs_get_from(RAnal *anal, ut64 to);
-R_API void r_anal_xrefs_list(RAnal *anal, int rad, const char *arg);
+R_API void r_anal_xrefs_list(RAnal *anal, int rad, const char *arg, RTable *t);
 R_API ut64 r_anal_xrefs_count(RAnal *anal);
 R_API ut64 r_anal_xrefs_count_at(RAnal *anal, ut64 to);
 R_API RVecAnalRef *r_anal_function_get_refs(RAnalFunction *fcn);
@@ -1123,7 +1161,7 @@ R_API void r_anal_save_parsed_type(RAnal *anal, const char *parsed);
 
 /* var.c */
 R_API R_OWN char *r_anal_function_autoname_var(RAnalFunction *fcn, char kind, const char *pfx, int ptr);
-R_API R_BORROW RAnalVar *r_anal_function_set_var(RAnalFunction *fcn, int delta, char kind, R_NULLABLE const char *type, int size, bool isarg, R_NONNULL const char *name);
+R_API R_BORROW RAnalVar *r_anal_function_set_var(RAnalFunction *fcn, int delta, char kind, const char * R_NULLABLE type, int size, bool isarg, const char * R_NONNULL name);
 R_API bool r_anal_function_set_var_prot(RAnalFunction *fcn, RList /*<RAnalVarProt>*/ *l);
 R_API R_BORROW RAnalVar *r_anal_function_get_var(RAnalFunction *fcn, char kind, int delta);
 R_API RList *r_anal_var_deserialize(const char *ser);
@@ -1139,11 +1177,11 @@ R_API st64 r_anal_function_get_var_stackptr_at(RAnalFunction *fcn, st64 delta, u
 R_API const char *r_anal_function_get_var_reg_at(RAnalFunction *fcn, st64 delta, ut64 addr);
 R_API R_BORROW RPVector *r_anal_function_get_vars_used_at(RAnalFunction *fcn, ut64 op_addr);
 
-R_API bool r_anal_var_rename(RAnalVar *var, const char *new_name, bool verbose);
-R_API void r_anal_var_set_type(RAnalVar *var, const char *type);
-R_API void r_anal_var_delete(RAnalVar *var);
+R_API bool r_anal_var_rename(RAnal *anal, RAnalVar *var, const char *new_name);
+R_API void r_anal_var_set_type(RAnal *anal, RAnalVar *var, const char *type);
+R_API bool r_anal_var_delete(RAnal *anal, RAnalVar *var);
 R_API ut64 r_anal_var_addr(RAnalVar *var);
-R_API void r_anal_var_set_access(RAnalVar *var, const char *reg, ut64 access_addr, int access_type, st64 stackptr);
+R_API bool r_anal_var_set_access(RAnal *anal, RAnalVar *var, const char *reg, ut64 access_addr, int access_type, st64 stackptr);
 R_API void r_anal_var_remove_access_at(RAnalVar *var, ut64 address);
 R_API void r_anal_var_clear_accesses(RAnalVar *var);
 R_API void r_anal_var_add_constraint(RAnalVar *var, R_BORROW RAnalVarConstraint *constraint);
@@ -1167,12 +1205,12 @@ typedef struct r_anal_function_vars_cache {
 	RList *rvars;
 	RList *svars;
 } RAnalFcnVarsCache;
+
 R_API void r_anal_function_vars_cache_init(RAnal *anal, RAnalFcnVarsCache *cache, RAnalFunction *fcn);
 R_API void r_anal_function_vars_cache_fini(RAnalFcnVarsCache *cache);
 
-R_API char *r_anal_function_format_sig(R_NONNULL RAnal *anal, R_NONNULL RAnalFunction *fcn, R_NULLABLE char *fcn_name,
-		R_NULLABLE RAnalFcnVarsCache *reuse_cache, R_NULLABLE const char *fcn_name_pre, R_NULLABLE const char *fcn_name_post);
-
+R_API char *r_anal_function_format_sig(RAnal *anal, RAnalFunction *fcn, char * R_NULLABLE fcn_name,
+		RAnalFcnVarsCache * R_NULLABLE reuse_cache, const char * R_NULLABLE fcn_name_pre, const char * R_NULLABLE fcn_name_post);
 
 /* project */
 #define R_ANAL_THRESHOLDFCN 0.7F
@@ -1180,14 +1218,13 @@ R_API char *r_anal_function_format_sig(R_NONNULL RAnal *anal, R_NONNULL RAnalFun
 
 /* diff.c */
 R_API RAnalDiff *r_anal_diff_new(void);
-R_API void r_anal_diff_setup(RAnal *anal, int doops, double thbb, double thfcn);
-R_API void r_anal_diff_setup_i(RAnal *anal, int doops, int thbb, int thfcn);
+R_API void r_anal_diff_setup(RAnal *anal, bool doops, double thbb, double thfcn);
+R_API void r_anal_diff_setup_i(RAnal *anal, bool doops, int thbb, int thfcn);
 R_API void r_anal_diff_free(RAnalDiff *diff);
 R_API int r_anal_diff_fingerprint_bb(RAnal *anal, RAnalBlock *bb);
 R_API size_t r_anal_diff_fingerprint_fcn(RAnal *anal, RAnalFunction *fcn);
 R_API bool r_anal_diff_bb(RAnal *anal, RAnalFunction *fcn, RAnalFunction *fcn2);
 R_API int r_anal_diff_fcn(RAnal *anal, RList *fcns, RList *fcns2);
-R_API int r_anal_diff_eval(RAnal *anal);
 
 R_API RAnalCond *r_anal_cond_new(void);
 R_API RAnalCond *r_anal_cond_new_from_op(RAnalOp *op);
@@ -1197,20 +1234,21 @@ R_API char *r_anal_cond_tostring(RAnalCond *cond);
 R_API int r_anal_cond_eval(RAnal *anal, RAnalCond *cond);
 R_API RAnalCond *r_anal_cond_new_from_string(const char *str);
 R_API const char *r_anal_cond_type_tostring(int cc);
+R_API const char *r_anal_cond_typeexpr_tostring(int cc);
 
 /* jmptbl */
 R_API bool r_anal_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *block, ut64 jmpaddr, ut64 table, ut64 tablesize, ut64 default_addr);
 
 // TODO: should be renamed
 R_API bool try_get_delta_jmptbl_info(RAnal *a, RAnalFunction *fcn, ut64 jmp_addr, ut64 lea_addr, ut64 *table_size, ut64 *default_case, st64 *start_casenum_shift);
-R_API bool try_walkthrough_jmptbl(RAnal *analysis, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, st64 start_casenum_shift, ut64 jmptbl_loc, ut64 jmptbl_off, ut64 sz, ut64 jmptbl_size, ut64 default_case, bool ret0);
+R_API bool r_anal_jmptbl_walk(RAnal *analysis, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, st64 start_casenum_shift, ut64 jmptbl_loc, ut64 jmptbl_off, ut64 sz, ut64 jmptbl_size, ut64 default_case, bool ret0);
 R_API bool try_walkthrough_casetbl(RAnal *analysis, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, st64 start_casenum_shift, ut64 jmptbl_loc, ut64 casetbl_loc, ut64 jmptbl_off, ut64 sz, ut64 jmptbl_size, ut64 default_case, bool ret0);
 R_API bool try_get_jmptbl_info(RAnal *analysis, RAnalFunction *fcn, ut64 addr, RAnalBlock *my_bb, ut64 *table_size, ut64 *default_case, st64 *start_casenum_shift);
 R_API int walkthrough_arm_jmptbl_style(RAnal *anal, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, ut64 jmptbl_loc, ut64 sz, ut64 jmptbl_size, ut64 default_case, int ret0);
 
 /* reflines.c */
 R_API RList* /*<RAnalRefline>*/ r_anal_reflines_get(RAnal *anal,
-		ut64 addr, const ut8 *buf, ut64 len, int nlines, int linesout, int linescall);
+		ut64 addr, const ut8 *buf, ut64 len, int nlines, int linesout, int linescall, int splitmode);
 R_API int r_anal_reflines_middle(RAnal *anal, RList *list, ut64 addr, int len);
 R_API RAnalRefStr *r_anal_reflines_str(void *core, ut64 addr, int opts);
 R_API void r_anal_reflines_str_free(RAnalRefStr *refstr);
@@ -1297,7 +1335,7 @@ R_API const char *r_meta_get_string(RAnal *a, RAnalMetaType type, ut64 addr);
 R_API void r_meta_set_data_at(RAnal *a, ut64 addr, ut64 wordsz);
 
 // Returns the item with given type that starts at addr in the current space or NULL. The size of this item  optionally returned through size.
-R_API RAnalMetaItem *r_meta_get_at(RAnal *a, ut64 addr, RAnalMetaType type, R_OUT R_NULLABLE ut64 *size);
+R_API RAnalMetaItem *r_meta_get_at(RAnal *a, ut64 addr, RAnalMetaType type, R_OUT ut64 * R_NULLABLE size);
 
 // Returns the node for one meta item with the given type that contains addr in the current space or NULL.
 // To get all the nodes, use r_meta_get_all_in().
@@ -1325,10 +1363,10 @@ R_API void r_meta_rebase(RAnal *anal, ut64 diff);
 R_API ut64 r_meta_get_size(RAnal *a, RAnalMetaType type);
 
 R_API const char *r_meta_type_tostring(int type);
-R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int rad, PJ *pj, bool show_full);
-R_API void r_meta_print_list_all(RAnal *a, int type, int rad, const char *tq);
-R_API void r_meta_print_list_at(RAnal *a, ut64 addr, int rad, const char *tq);
-R_API void r_meta_print_list_in_function(RAnal *a, int type, int rad, ut64 addr, const char *tq);
+R_API void r_meta_print(RAnal *a, RAnalMetaItem *d, ut64 start, ut64 size, int rad, PJ *pj, RTable *t, bool show_full);
+R_API void r_meta_print_list_all(RAnal *a, int type, int rad, const char *tq, RTable *t);
+R_API void r_meta_print_list_at(RAnal *a, ut64 addr, int rad, const char *tq, RTable *t);
+R_API void r_meta_print_list_in_function(RAnal *a, int type, int rad, ut64 addr, const char *tq, RTable *t);
 
 /* hints */
 
@@ -1351,7 +1389,7 @@ R_API void r_anal_hint_set_ret(RAnal *a, ut64 addr, ut64 val);
 R_API void r_anal_hint_set_high(RAnal *a, ut64 addr);
 R_API void r_anal_hint_set_stackframe(RAnal *a, ut64 addr, ut64 size);
 R_API void r_anal_hint_set_val(RAnal *a, ut64 addr, ut64 v);
-R_API void r_anal_hint_set_arch(RAnal *a, ut64 addr, R_NULLABLE const char *arch); // arch == NULL => use global default
+R_API void r_anal_hint_set_arch(RAnal *a, ut64 addr, const char * R_NULLABLE arch); // arch == NULL => use global default
 R_API void r_anal_hint_set_bits(RAnal *a, ut64 addr, int bits); // bits == NULL => use global default
 R_API void r_anal_hint_unset_val(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_high(RAnal *a, ut64 addr);
@@ -1371,10 +1409,10 @@ R_API void r_anal_hint_unset_newbits(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_stackframe(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_arch(RAnal *a, ut64 addr);
 R_API void r_anal_hint_unset_bits(RAnal *a, ut64 addr);
-R_API R_NULLABLE const RVector/*<const RAnalAddrHintRecord>*/ *r_anal_addr_hints_at(RAnal *anal, ut64 addr);
+R_API const RVector/*<const RAnalAddrHintRecord>*/ * R_NULLABLE r_anal_addr_hints_at(RAnal * R_NONNULL anal, ut64 addr);
 typedef bool (*RAnalAddrHintRecordsCb)(ut64 addr, const RVector/*<const RAnalAddrHintRecord>*/ *records, void *user);
 R_API void r_anal_addr_hints_foreach(RAnal *anal, RAnalAddrHintRecordsCb cb, void *user);
-typedef bool (*RAnalArchHintCb)(ut64 addr, R_NULLABLE const char *arch, void *user);
+typedef bool (*RAnalArchHintCb)(ut64 addr, const char * R_NULLABLE arch, void *user);
 R_API void r_anal_arch_hints_foreach(RAnal *anal, RAnalArchHintCb cb, void *user);
 typedef bool (*RAnalBitsHintCb)(ut64 addr, int bits, void *user);
 R_API void r_anal_bits_hints_foreach(RAnal *anal, RAnalBitsHintCb cb, void *user);
@@ -1382,12 +1420,12 @@ R_API void r_anal_bits_hints_foreach(RAnal *anal, RAnalBitsHintCb cb, void *user
 // get the hint-specified arch value to be considered at addr
 // hint_addr will optionally be set to the address where the hint that specifies this arch is placed or UT64_MAX
 // if there is no hint affecting addr.
-R_API R_NULLABLE R_BORROW const char *r_anal_hint_arch_at(RAnal *anal, ut64 addr, R_NULLABLE ut64 *hint_addr);
+R_API R_BORROW const char * R_NULLABLE  r_anal_hint_arch_at(RAnal *anal, ut64 addr, ut64 * R_NULLABLE hint_addr);
 
 // get the hint-specified bits value to be considered at addr
 // hint_addr will optionally be set to the address where the hint that specifies this arch is placed or UT64_MAX
 // if there is no hint affecting addr.
-R_API int r_anal_hint_bits_at(RAnal *anal, ut64 addr, R_NULLABLE ut64 *hint_addr);
+R_API int r_anal_hint_bits_at(RAnal *anal, ut64 addr, ut64 * R_NULLABLE hint_addr);
 
 R_API RAnalHint *r_anal_hint_get(RAnal *anal, ut64 addr); // accumulate all available hints affecting the given address
 
@@ -1444,7 +1482,7 @@ R_API ut64 r_anal_vtable_info_get_size(RVTableContext *context, RVTableInfo *vta
 R_API bool r_anal_vtable_begin(RAnal *anal, RVTableContext *context);
 R_API RVTableInfo *r_anal_vtable_parse_at(RVTableContext *context, ut64 addr);
 R_API RList *r_anal_vtable_search(RVTableContext *context);
-R_API void r_anal_list_vtables(RAnal *anal, int rad);
+R_API char *r_anal_vtables_list(RAnal *anal, int rad);
 
 /* rtti */
 R_API char *r_anal_rtti_msvc_demangle_class_name(RVTableContext *context, const char *name);
@@ -1528,12 +1566,12 @@ R_API RVector/*<RAnalVTable>*/ *r_anal_class_vtable_get_all(RAnal *anal, const c
 R_API RAnalClassErr r_anal_class_vtable_set(RAnal *anal, const char *class_name, RAnalVTable *vtable);
 R_API RAnalClassErr r_anal_class_vtable_delete(RAnal *anal, const char *class_name, const char *vtable_id);
 
-R_API void r_anal_class_print(RAnal *anal, const char *class_name, bool detailed);
+R_API char *r_anal_class_print(RAnal *anal, const char *class_name, bool detailed);
 R_API void r_anal_class_json(RAnal *anal, PJ *j, const char *class_name);
-R_API void r_anal_class_list(RAnal *anal, int mode);
-R_API void r_anal_class_list_bases(RAnal *anal, const char *class_name);
-R_API void r_anal_class_list_vtables(RAnal *anal, const char *class_name);
-R_API void r_anal_class_list_vtable_offset_functions(RAnal *anal, const char *class_name, ut64 offset);
+R_API char *r_anal_class_list(RAnal *anal, int mode);
+R_API char *r_anal_class_list_bases(RAnal *anal, const char *class_name);
+R_API char *r_anal_class_list_vtables(RAnal *anal, const char *class_name);
+R_API char *r_anal_class_list_vtable_offset_functions(RAnal *anal, const char *class_name, ut64 offset);
 R_API RGraph/*<RGraphNodeInfo>*/ *r_anal_class_get_inheritance_graph(RAnal *anal);
 
 R_API RAnalEsilCFG *r_anal_esil_cfg_new(void);
@@ -1588,7 +1626,10 @@ R_API void r_anal_backtrace_list(RAnal *a, ut64 addr, int opt);
 
 /* plugin pointers */
 extern RAnalPlugin r_anal_plugin_null;
+extern RAnalPlugin r_anal_plugin_sbpf;
+extern RAnalPlugin r_anal_plugin_tcc;
 extern RAnalPlugin r_anal_plugin_a2f;
+extern RAnalPlugin r_anal_plugin_path;
 
 #ifdef __cplusplus
 }
