@@ -9,9 +9,9 @@ extern "C" {
 #include "r_bind.h"
 #include "r_list.h"
 
-R_LIB_VERSION_HEADER (r_socket);
+R_LIB_VERSION_HEADER(r_socket);
 
-#if __UNIX__
+#if R2__UNIX__
 #include <netinet/in.h>
 #include <sys/un.h>
 #include <poll.h>
@@ -28,7 +28,7 @@ R_LIB_VERSION_HEADER (r_socket);
 #include <openssl/err.h>
 #endif
 
-#if __UNIX__
+#if R2__UNIX__
 #include <netinet/tcp.h>
 #endif
 
@@ -37,9 +37,9 @@ R_LIB_VERSION_HEADER (r_socket);
 #define MSG_DONTWAIT 0
 #endif
 #ifndef SD_BOTH
-#define SD_RECEIVE  0
-#define SD_SEND 1
-#define SD_BOTH 2
+#define SD_RECEIVE 0
+#define SD_SEND    1
+#define SD_BOTH    2
 #endif
 
 #if _MSC_VER
@@ -49,7 +49,7 @@ R_LIB_VERSION_HEADER (r_socket);
 #endif
 
 typedef struct {
-#if __WINDOWS__
+#if R2__WINDOWS__
 	HANDLE pipe;
 	HANDLE child;
 #else
@@ -58,6 +58,12 @@ typedef struct {
 	int output[2];
 #endif
 	RCoreBind coreb;
+	/* HTTP/r2web support: when is_http is true, http_url contains
+	 * the base endpoint where commands are POSTed using the
+	 * r_socket_http_post API and r2pipe_cmd will return the
+	 * HTTP response instead of using fork+pipe. */
+	char *http_url;
+	bool is_http;
 } R2Pipe;
 
 typedef struct r_socket_t {
@@ -68,7 +74,7 @@ typedef struct r_socket_t {
 #endif
 	bool is_ssl;
 	int proto;
-	int local;	// TODO: merge ssl with local -> flags/options
+	int local; // TODO: merge ssl with local -> flags/options
 	int port;
 	struct sockaddr_in sa;
 #if HAVE_LIB_SSL
@@ -85,37 +91,43 @@ typedef struct r_socket_http_options {
 	bool httpauth;
 } RSocketHTTPOptions;
 
-#define R_SOCKET_PROTO_TCP IPPROTO_TCP
-#define R_SOCKET_PROTO_UDP IPPROTO_UDP
-#define R_SOCKET_PROTO_CAN 0xc42b05
-#define R_SOCKET_PROTO_UNIX 0x1337
-#define R_SOCKET_PROTO_NONE 0
+#define R_SOCKET_PROTO_TCP     IPPROTO_TCP
+#define R_SOCKET_PROTO_UDP     IPPROTO_UDP
+#define R_SOCKET_PROTO_CAN     0xc42b05
+#define R_SOCKET_PROTO_SERIAL  0x534147
+#define R_SOCKET_PROTO_UNIX    0x1337
+#define R_SOCKET_PROTO_NONE    0
 #define R_SOCKET_PROTO_DEFAULT R_SOCKET_PROTO_TCP
 
+// backward compat for yara-r2
+#define r2p_cmd   r2pipe_cmd
+#define r2p_open  r2pipe_open
+#define r2p_close r2pipe_close
+
 #ifdef R_API
 R_API RSocket *r_socket_new_from_fd(int fd);
 R_API RSocket *r_socket_new(bool is_ssl);
 R_API bool r_socket_spawn(RSocket *s, const char *cmd, unsigned int timeout);
 R_API bool r_socket_connect(RSocket *s, const char *host, const char *port, int proto, unsigned int timeout);
 R_API int r_socket_connect_serial(RSocket *sock, const char *path, int speed, int parity);
-#define r_socket_connect_tcp(a, b, c, d) r_socket_connect (a, b, c, R_SOCKET_PROTO_TCP, d)
-#define r_socket_connect_udp(a, b, c, d) r_socket_connect (a, b, c, R_SOCKET_PROTO_UDP, d)
-#if __UNIX__
-#define r_socket_connect_unix(a, b) r_socket_connect (a, b, b, R_SOCKET_PROTO_UNIX, 0)
+#define r_socket_connect_tcp(a, b, c, d) r_socket_connect(a, b, c, R_SOCKET_PROTO_TCP, d)
+#define r_socket_connect_udp(a, b, c, d) r_socket_connect(a, b, c, R_SOCKET_PROTO_UDP, d)
+#if R2__UNIX__
+#define r_socket_connect_unix(a, b) r_socket_connect(a, b, b, R_SOCKET_PROTO_UNIX, 0)
 #else
 #define r_socket_connect_unix(a, b) (false)
 #endif
 R_API bool r_socket_listen(RSocket *s, const char *port, const char *certfile);
 R_API int r_socket_port_by_name(const char *name);
-R_API int r_socket_close_fd(RSocket *s);
-R_API int r_socket_close(RSocket *s);
-R_API int r_socket_free(RSocket *s);
+R_API bool r_socket_close_fd(RSocket *s);
+R_API bool r_socket_close(RSocket *s);
+R_API void r_socket_free(RSocket *s);
 R_API RSocket *r_socket_accept(RSocket *s);
 R_API RSocket *r_socket_accept_timeout(RSocket *s, unsigned int timeout);
 R_API bool r_socket_block_time(RSocket *s, bool block, int sec, int usec);
 R_API int r_socket_flush(RSocket *s);
 R_API int r_socket_ready(RSocket *s, int secs, int usecs);
-R_API char *r_socket_to_string(RSocket *s);
+R_API char *r_socket_tostring(RSocket *s);
 R_API int r_socket_write(RSocket *s, const void *buf, int len);
 R_API int r_socket_puts(RSocket *s, char *buf);
 R_API void r_socket_printf(RSocket *s, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
@@ -141,8 +153,8 @@ R_API void r_socket_proc_printf(RSocketProc *sp, const char *fmt, ...) R_PRINTF_
 R_API int r_socket_proc_ready(RSocketProc *sp, int secs, int usecs);
 
 /* HTTP */
-R_API char *r_socket_http_get(const char *url, int *code, int *rlen);
-R_API char *r_socket_http_post(const char *url, const char *data, int *code, int *rlen);
+R_API char *r_socket_http_get(const char *url, const char **headers, int *code, int *rlen);
+R_API char *r_socket_http_post(const char *url, const char **headers, const char *data, int *code, int *rlen);
 R_API void r_socket_http_server_set_breaked(bool *b);
 
 typedef struct r_socket_http_request {
@@ -185,7 +197,7 @@ enum {
 typedef struct r_socket_rap_server_t {
 	RSocket *fd;
 	char *port;
-	ut8 buf[RAP_PACKET_MAX + 32];	// This should be used as a static buffer for everything done by the server
+	ut8 buf[RAP_PACKET_MAX + 32]; // This should be used as a static buffer for everything done by the server
 	rap_server_open open;
 	rap_server_seek seek;
 	rap_server_read read;
@@ -193,7 +205,7 @@ typedef struct r_socket_rap_server_t {
 	rap_server_cmd system;
 	rap_server_cmd cmd;
 	rap_server_close close;
-	void *user;	// Always first arg for callbacks
+	void *user; // Always first arg for callbacks
 } RSocketRapServer;
 
 R_API RSocketRapServer *r_socket_rap_server_new(bool is_ssl, const char *port);
@@ -208,7 +220,7 @@ R_API int r_socket_rap_client_open(RSocket *s, const char *file, int rw);
 R_API char *r_socket_rap_client_command(RSocket *s, const char *cmd, RCoreBind *c);
 R_API int r_socket_rap_client_write(RSocket *s, const ut8 *buf, int count);
 R_API int r_socket_rap_client_read(RSocket *s, ut8 *buf, int count);
-R_API int r_socket_rap_client_seek(RSocket *s, ut64 offset, int whence);
+R_API ut64 r_socket_rap_client_seek(RSocket *s, ut64 offset, int whence);
 
 /* run.c */
 #define R_RUN_PROFILE_NARGS 512
@@ -227,14 +239,14 @@ typedef struct r_run_profile_t {
 	char *_chgdir;
 	char *_chroot;
 	char *_libpath;
-	char *_preload;
+	RList *_preload;
 	int _bits;
+	bool _time;
 	int _pid;
 	char *_pidfile;
-	int _r2preload;
-	int _docore;
-	int _dofork;
-	int _dodebug;
+	bool _r2preload;
+	bool _docore;
+	bool _dofork;
 	int _aslr;
 	int _maxstack;
 	int _maxproc;
@@ -252,15 +264,17 @@ typedef struct r_run_profile_t {
 	int _timeout;
 	int _timeout_sig;
 	int _nice;
+	bool _stderrout;
+	bool _noprogram;
 } RRunProfile;
 
-R_API RRunProfile *r_run_new(const char *str);
+R_API RRunProfile *r_run_new(const char *R_NULLABLE str);
 R_API bool r_run_parse(RRunProfile *pf, const char *profile);
 R_API void r_run_free(RRunProfile *r);
 R_API bool r_run_parseline(RRunProfile *p, const char *b);
 R_API const char *r_run_help(void);
-R_API int r_run_config_env(RRunProfile *p);
-R_API int r_run_start(RRunProfile *p);
+R_API bool r_run_config_env(RRunProfile *p);
+R_API bool r_run_start(RRunProfile *p);
 R_API void r_run_reset(RRunProfile *p);
 R_API bool r_run_parsefile(RRunProfile *p, const char *b);
 R_API char *r_run_get_environ_profile(char **env);
