@@ -24,9 +24,9 @@ extern int ptrace(int _request, pid_t _pid, caddr_t _addr, int _data);
 #endif
 #ifdef _MSC_VER
 #include <windows.h>
-int gettimeofday (struct timeval* p, void* tz);
+int gettimeofday(struct timeval *p, void *tz);
 #endif
-#include "r_util/r_event.h"
+#include "r_event.h"
 #include "r_util/r_assert.h"
 #include "r_util/r_itv.h"
 #include "r_util/r_signal.h"
@@ -44,7 +44,7 @@ int gettimeofday (struct timeval* p, void* tz);
 #include "r_util/r_bitmap.h"
 #include "r_util/r_time.h"
 #include "r_util/r_debruijn.h"
-#include "r_util/r_ctypes.h"
+#include "r_util/r_type.h"
 #include "r_util/r_file.h"
 #include "r_util/r_hex.h"
 #include "r_util/r_log.h"
@@ -65,6 +65,7 @@ int gettimeofday (struct timeval* p, void* tz);
 #include "r_util/r_stack.h"
 #include "r_util/r_str.h"
 #include "r_util/r_strbuf.h"
+#include "r_util/r_bloom.h"
 #include "r_util/r_strpool.h"
 #include "r_util/r_str_constpool.h"
 #include "r_util/r_sys.h"
@@ -86,18 +87,18 @@ int gettimeofday (struct timeval* p, void* tz);
 #include "r_util/r_w32.h"
 #include "r_util/r_token.h"
 #include "r_util/r_axml.h"
+#include "r_util/libc.h"
 // requires io, core, ... #include "r_util/r_print.h"
 
-#if R2_USE_NEW_ABI
 R_API int r_lz4_compress(ut8 *obuf, ut8 *buf, size_t buf_size, const int max_chain);
-R_API ut8 *r_lz4_decompress(const ut8* input, size_t input_size, size_t *output_size);
-#endif
+R_API ut8 *r_lz4_decompress(const ut8 *input, size_t input_size, size_t *output_size);
+R_API int r_lz4_decompress_block(ut8 *g_buf, const int comp_len, int *pp, ut8 *obuf, int osz);
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-R_LIB_VERSION_HEADER (r_util);
+R_LIB_VERSION_HEADER(r_util);
 
 #ifdef __cplusplus
 }
