@@ -5,19 +5,6 @@
 extern "C" {
 #endif
 
-typedef struct r_range_tiny_t {
-	int pairs;
-	int count;
-	ut64 *ranges;
-} RRangeTiny;
-
-R_API RRangeTiny *r_tinyrange_new(void);
-R_API void r_tinyrange_init(RRangeTiny *bbr);
-R_API void r_tinyrange_fini(RRangeTiny *bbr);
-R_API bool r_tinyrange_in(RRangeTiny *bbr, ut64 at);
-R_API bool r_tinyrange_add(RRangeTiny *bbr, ut64 from, ut64 to);
-R_API void r_tinyrange_free(RRangeTiny *bbr);
-
 /* range.c */
 
 typedef struct r_range_item_t {
@@ -47,6 +34,7 @@ R_API int r_range_sort(RRange *rgs);
 R_API void r_range_percent(RRange *rgs);
 R_API int r_range_list(RRange *rgs, int rad);
 R_API int r_range_get_n(RRange *rgs, int n, ut64 *from, ut64 *to);
+R_API char *r_range_tostring(RRange *rgs);
 R_API RRange *r_range_inverse(RRange *rgs, ut64 from, ut64 to, int flags);
 R_API int r_range_overlap(ut64 a0, ut64 a1, ut64 b0, ut64 b1, int *d);
 
