@@ -1,10 +1,12 @@
 #ifndef R_STR_H
 #define R_STR_H
 
-#include <wchar.h>
-#include <stdarg.h>
 #include "r_str_util.h"
-#include "r_list.h"
+#include <r_list.h>
+#include <r_vec.h>
+#include <r_types_base.h>
+#include <stdarg.h>
+#include <wchar.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -20,8 +22,31 @@ typedef enum {
 	R_STRING_ENC_GUESS = 'g',
 } RStrEnc;
 
+typedef struct {
+	char *str;
+	char buf[64]; // 20s vs 11s - 2x faster if string fits in buffer
+	int len;
+	char *ptr;
+	bool weak;
+} RString;
+
+// string split using rvec
+typedef struct {
+	ut16 from;
+	ut16 to;
+} RStringSlice;
+
+R_VEC_TYPE (RVecStringSlice, RStringSlice);
+
+
 typedef int (*RStrRangeCallback) (void *, int);
 
+// can be replaced with RString
+#define r_strf_buffer(s) char strbuf[s]
+#define r_strf_var(n,s,f, ...) char n[s]; snprintf (n, s, f, __VA_ARGS__);
+#define r_strf(s,...) (snprintf (strbuf, sizeof (strbuf), s, __VA_ARGS__)? strbuf: strbuf)
+#define r_str_var(n,s,f) char n[s]; r_str_ncpy (n, f, s);
+
 typedef struct r_charset_rune_t {
 	ut8 *ch;
 	ut8 *hx;
@@ -40,30 +65,34 @@ typedef struct r_charset_t {
 
 #define R_STR_ISEMPTY(x) (!(x) || !*(x))
 #define R_STR_ISNOTEMPTY(x) ((x) && *(x))
-#define R_STR_DUP(x) ((x) ? strdup ((x)) : NULL)
-#define r_str_array(x,y) ((y>=0 && y<(sizeof(x)/sizeof(*x)))?x[y]:"")
+#define r_str_array(x,y) ((y >= 0 && y < (sizeof (x) / sizeof (*(x))))?(x)[(y)]: "")
 R_API RCharset *r_charset_new(void);
 R_API void r_charset_free(RCharset *charset);
 R_API RCharsetRune *r_charset_rune_new(const ut8 *ch, const ut8 *hx);
 R_API void r_charset_rune_free(RCharsetRune *rcr);
-R_API size_t r_charset_encode_str(RCharset *rc, ut8 *out, size_t out_len, const ut8 *in, size_t in_len);
+R_API size_t r_charset_encode_str(RCharset *rc, ut8 *out, size_t out_len, const ut8 *in, size_t in_len, bool early_exit);
 R_API size_t r_charset_decode_str(RCharset *rc, ut8 *out, size_t out_len, const ut8 *in, size_t in_len);
 R_API bool r_charset_open(RCharset *c, const char *cs);
 R_API bool r_charset_use(RCharset *c, const char *cf);
 R_API RList *r_charset_list(RCharset *c);
 R_API void r_charset_close(RCharset *c);
-R_API RCharsetRune * add_rune(RCharsetRune *rcsr, const ut8 *ch, const ut8 *hx);
+R_API RCharsetRune *add_rune(RCharsetRune *rcsr, const ut8 *ch, const ut8 *hx);
 R_API RCharsetRune *search_from_hex(RCharsetRune *rcsr, const ut8 *hx);
 R_API RCharsetRune *search_from_char(RCharsetRune *rcsr, const ut8 *ch);
 
 // str
 R_API char *r_str_repeat(const char *ch, int sz);
 R_API const char *r_str_pad(const char ch, int len);
+R_API char *r_str_pad2(char *pad, size_t padsz, const char ch, int sz);
+
 R_API const char *r_str_rstr(const char *base, const char *p);
-R_API const char *r_strstr_ansi (const char *a, const char *b);
+R_API const char *r_strstr_ansi(const char *a, const char *b);
 R_API const char *r_str_rchr(const char *base, const char *p, int ch);
+R_API const char *r_str_nchr(const char *str, char chr, int maxlen);
 R_API const char *r_str_closer_chr(const char *b, const char *s);
 R_API int r_str_bounds(const char *str, int *h);
+R_API bool r_str_eq(const char *s1, const char *s2);
+R_API bool r_str_eqi(const char *s1, const char *s2);
 R_API char *r_str_crop(const char *str, unsigned int x, unsigned int y, unsigned int x2, unsigned int y2);
 R_API char *r_str_scale(const char *r, int w, int h);
 R_API bool r_str_range_in(const char *r, ut64 addr);
@@ -78,7 +107,6 @@ R_API void r_str_filter_zeroline(char *str, int len);
 R_API size_t r_str_utf8_codepoint(const char *s, size_t left);
 R_API bool r_str_char_fullwidth(const char *s, size_t left);
 R_API int r_str_write(int fd, const char *b);
-R_API size_t r_str_ncpy(char *dst, const char *src, size_t n);
 R_API void r_str_sanitize(char *c);
 R_API char *r_str_sanitize_sdb_key(const char *s);
 R_API const char *r_str_casestr(const char *a, const char *b);
@@ -86,12 +114,19 @@ R_API const char *r_str_firstbut(const char *s, char ch, const char *but);
 R_API const char *r_str_firstbut_escape(const char *s, char ch, const char *but);
 R_API const char *r_str_lastbut(const char *s, char ch, const char *but);
 R_API int r_str_split(char *str, char ch);
+R_API char *r_str_slice(const char *str, RStringSlice s);
+R_API const char *r_str_asciitable(void);
+R_API const char *r_str_chartable(int c);
+
+R_API RVecStringSlice *r_str_split_vec(const char *str, const char *c, int n);
 R_API RList *r_str_split_list(char *str, const char *c, int n);
 R_API RList *r_str_split_duplist(const char *str, const char *c, bool trim);
 R_API size_t *r_str_split_lines(char *str, size_t *count);
-R_API char* r_str_replace(char *str, const char *key, const char *val, int g);
-R_API char *r_str_replace_icase(char *str, const char *key, const char *val, int g, int keep_case);
-R_API char *r_str_replace_in(char *str, ut32 sz, const char *key, const char *val, int g);
+R_API R_MUSTUSE char* r_str_replace(char *str, const char *key, const char *val, int g);
+R_API R_MUSTUSE char* r_str_replace_all(char *str, const char *key, const char *val);
+R_API R_MUSTUSE char *r_str_replace_icase(char *str, const char *key, const char *val, int g, int keep_case);
+R_API void r_str_replace_in(char *str, ut32 sz, const char *key, const char *val, int g);
+R_API R_MUSTUSE char* r_str_replace_thunked(char *str, char *clean, int *thunk, int clen, const char *key, const char *val, int g);
 #define r_str_cpy(x,y) memmove ((x), (y), strlen (y) + 1);
 #define r_str_cat(x,y) memmove ((x) + strlen (x), (y), strlen (y) + 1);
 R_API int r_str_bits(char *strout, const ut8 *buf, int len, const char *bitz);
@@ -109,7 +144,8 @@ R_API int r_str_arg_unescape(char *arg);
 R_API char **r_str_argv(const char *str, int *_argc);
 R_API void r_str_argv_free(char **argv);
 R_API char *r_str_new(const char *str);
-R_API int r_snprintf (char *string, int len, const char *fmt, ...) R_PRINTF_CHECK(3, 4);
+R_API char *r_str_fixspaces(char *str);
+R_API int r_snprintf(char *string, int len, const char *fmt, ...) R_PRINTF_CHECK(3, 4);
 R_API bool r_str_is_ascii(const char *str);
 R_API char *r_str_nextword(char *s, char ch);
 R_API bool r_str_is_printable(const char *str);
@@ -120,28 +156,37 @@ R_API char *r_str_newf(const char *fmt, ...) R_PRINTF_CHECK(1, 2);
 R_API char *r_str_newvf(const char *fmt, va_list ap);
 R_API int r_str_distance(const char *a, const char *b);
 R_API char *r_str_newlen(const char *str, int len);
-R_API const char *r_str_sysbits(const int v);
+R_API const char *r_str_sysbits(const RSysBits v);
 R_API char *r_str_trunc_ellipsis(const char *str, int len);
-R_API const char *r_str_bool(int b);
+R_API const char *r_str_bool(bool b);
 R_API bool r_str_is_true(const char *s);
 R_API bool r_str_is_false(const char *s);
 R_API bool r_str_is_bool(const char *val);
 R_API const char *r_str_ansi_chrn(const char *str, size_t n);
+R_API size_t r_str_ansi_strip(char *str);
+R_API char *r_str_html_strip(const char *ptr, int *newlen);
+R_API bool r_str_html_rgbparse(const char *p, ut8 *r, ut8 *g, ut8 *b, ut8 *a);
 R_API size_t r_str_ansi_len(const char *str);
 R_API size_t r_str_ansi_nlen(const char *str, size_t len);
 R_API int r_str_ansi_trim(char *str, int str_len, int n);
 R_API int r_str_ansi_filter(char *str, char **out, int **cposs, int len);
+R_API char *r_str_ansi_resetbg(const char *str, const char *bgcolor);
 R_API char *r_str_ansi_crop(const char *str, unsigned int x, unsigned int y, unsigned int x2, unsigned int y2);
 R_API int r_str_word_count(const char *string);
-R_API int r_str_char_count(const char *string, char ch);
+R_API size_t r_str_char_count(const char *string, char ch);
 R_API char *r_str_word_get0set(char *stra, int stralen, int idx, const char *newstr, int *newlen);
+// R_UNUSED R_API char *r_str_insert(char *src, int pos, const char *str);
 R_API int r_str_word_set0(char *str);
 R_API int r_str_word_set0_stack(char *str);
 R_API const char *r_str_word_get0(const char *str, int idx);
 R_API char *r_str_word_get_first(const char *string);
 R_API void r_str_trim(char *str);
+R_API void r_str_trim_emptylines(char *str);
+R_API int r_str_ntrim(char *str, int n);
+R_API char *r_str_md2txt(const char *page, bool usecolor);
 R_API char *r_str_wrap(const char *str, int w);
 R_API char *r_str_trim_dup(const char *str);
+R_API char *r_str_trim_ndup(const char *str, size_t n);
 R_API char *r_str_trim_lines(char *str);
 R_API void r_str_trim_head(char *str);
 R_API const char *r_str_trim_head_ro(const char *str);
@@ -153,24 +198,27 @@ R_API ut64 r_str_hash64(const char *str);
 R_API char *r_str_trim_nc(char *str);
 R_API const char *r_str_nstr(const char *from, const char *to, int size);
 R_API const char *r_str_lchr(const char *str, char chr);
+#if R2_600
+R_API char *r_str_lstr(const char *s, const char *sub);
+#endif
 R_API const char *r_sub_str_lchr(const char *str, int start, int end, char chr);
 R_API const char *r_sub_str_rchr(const char *str, int start, int end, char chr);
 R_API char *r_str_ichr(char *str, char chr);
 R_API bool r_str_ccmp(const char *dst, const char *orig, int ch);
 R_API bool r_str_cmp_list(const char *list, const char *item, char sep);
-R_API int r_str_cmp(const char *dst, const char *orig, int len);
 R_API int r_str_casecmp(const char *dst, const char *orig);
 R_API int r_str_ncasecmp(const char *dst, const char *orig, size_t n);
-R_API int r_str_ccpy(char *dst, char *orig, int ch);
 R_API const char *r_str_get(const char *str);
 R_API const char *r_str_get_fail(const char *str, const char *failstr);
 R_API const char *r_str_getf(const char *str);
 R_API char *r_str_ndup(const char *ptr, int len);
-R_API char *r_str_dup(char *ptr, const char *string);
 R_API int r_str_inject(char *begin, char *end, char *str, int maxlen);
 R_API int r_str_delta(char *p, char a, char b);
 R_API void r_str_filter(char *str, int len);
-R_API const char * r_str_tok(const char *str1, const char b, size_t len);
+R_API const char *r_str_tok(const char *str1, const char b, size_t len);
+R_API char *r_str_tok_next(char *s);
+
+R_API char *r_str_tok_r(char *str, const char *delim, char **save_ptr);
 R_API wchar_t *r_str_mb_to_wc(const char *buf);
 R_API char *r_str_wc_to_mb(const wchar_t *buf);
 R_API wchar_t *r_str_mb_to_wc_l(const char *buf, int len);
@@ -210,8 +258,7 @@ R_API char *r_str_utf16_encode(const char *s, int len);
 R_API char *r_str_escape_utf8_for_json(const char *s, int len);
 R_API char *r_str_escape_utf8_for_json_strip(const char *buf, int buf_size);
 R_API char *r_str_encoded_json(const char *buf, int buf_size, int encoding);
-R_API char *r_str_home(const char *str);
-R_API char *r_str_r2_prefix(const char *str);
+R_API R_MUSTUSE char *r_str_r2_prefix(const char *str);
 R_API size_t r_str_nlen(const char *s, int n);
 R_API size_t r_str_nlen_w(const char *s, int n);
 R_API size_t r_wstr_clen(const char *s);
@@ -225,15 +272,44 @@ R_API void r_str_case(char *str, bool up);
 R_API void r_str_trim_path(char *s);
 R_API ut8 r_str_contains_macro(const char *input_value);
 R_API void r_str_truncate_cmd(char *string);
-R_API char* r_str_replace_thunked(char *str, char *clean, int *thunk, int clen,
-				  const char *key, const char *val, int g);
 R_API bool r_str_glob(const char *str, const char *glob);
-R_API int r_str_binstr2bin(const char *str, ut8 *out, int outlen);
 R_API char *r_str_between(const char *str, const char *prefix, const char *suffix);
+#undef r_str_startswith
+static inline size_t r_str_ncpy(char *dst, const char *src, size_t n) {
+	size_t i;
+
+	// do not do anything if n is 0
+	if (n == 0) {
+		return 0;
+	}
+
+	n--;
+	for (i = 0; src[i] && n > 0; i++, n--) {
+		dst[i] = src[i];
+	}
+	dst[i] = 0;
+	return i;
+}
 R_API bool r_str_startswith(const char *str, const char *needle);
+R_UNUSED static inline bool r_str_startswith_inline(const char *str, const char *needle) {
+	if (!str || !needle) {
+		return false;
+	}
+	return !strncmp (str, needle, strlen (needle));
+}
+#define r_str_startswith r_str_startswith_inline
+R_UNUSED static const char *r_str_skip_prefix(const char *str, const char *prefix) {
+	if (r_str_startswith (str, prefix)) {
+		str += strlen (prefix);
+	}
+	return str;
+}
+static inline char *R_STR_NDUP(const char * R_NULLABLE x, int len) {
+	int _len = len; return (_len > 0) ? r_str_ndup (x, _len) : NULL;
+}
 R_API bool r_str_endswith(const char *str, const char *needle);
-R_API bool r_str_isnumber (const char *str);
-R_API const char *r_str_last (const char *in, const char *ch);
+R_API bool r_str_isnumber(const char *str);
+R_API const char *r_str_last(const char *in, const char *ch);
 R_API char* r_str_highlight(char *str, const char *word, const char *color, const char *color_reset);
 R_API char *r_qrcode_gen(const ut8 *text, int len, bool utf8, bool inverted);
 R_API char *r_str_from_ut64(ut64 val);
@@ -247,6 +323,28 @@ R_API char *r_str_donut(int size);
 R_API char *r_str_version(const char *program);
 R_API char *r_str_ss(const char* msg, const char *nl, int cs);
 
+R_API char *r_str_after(char *s, char c);
+R_API int r_str_printf(char * R_NONNULL buffer, size_t buffer_size, const char * R_NONNULL format, ...);
+  /// __attribute__ ((format (printf, 3, 4)));
+R_API int r_str_scanf(const char * R_NONNULL buffer, const char * R_NONNULL format, ...);
+  /// __attribute__ ((format (scanf, 2, 3)));
+
+// rstr
+
+R_API char *r_string_get(const RString *s, int *len);
+R_API RString r_string_new(const char *is, int len);
+R_API void r_string_free(RString *s);
+R_API RString r_string_from(const char *is, int len);
+R_API void r_string_unweak(RString *a);
+R_API void r_string_trim(RString *s);
+R_API R_WIP RString r_string_newf(const char *fmt, ...);
+R_API bool r_string_append(RString *a, const char * R_NONNULL s);
+R_API void r_string_appendf(RString *a, const char * R_NONNULL fmt, ...);
+
+R_API char *r_str_pseudo_transform(const char **rules, const char *asm_str);
+R_API char *r_str_pseudo_subvar(char *pseudo, void *varmap);
+
+
 #ifdef __cplusplus
 }
 #endif
