@@ -14,7 +14,10 @@ extern "C" {
 #define R_NSEC_PER_USEC 1000ULL
 #define R_USEC_PER_MSEC 1000ULL
 
-#define ASCTIME_BUF_MINLEN (26)
+#define ASCTIME_BUF_MAXLEN (26)
+
+// current calendar time, alias for time(0). affected by calendar, timezone changes, localtime, etc
+R_API ut64 r_time_today(void);
 
 // wall clock time in microseconds
 R_API ut64 r_time_now(void);
@@ -22,15 +25,28 @@ R_API ut64 r_time_now(void);
 // monotonic time in microseconds
 R_API ut64 r_time_now_mono(void);
 
-R_API char *r_time_stamp_to_str(ut32 timeStamp);
-R_API ut32 r_time_dos_time_stamp_to_posix(ut32 timeStamp);
-R_API bool r_time_stamp_is_dos_format(const ut32 certainPosixTimeStamp, const ut32 possiblePosixOrDosTimeStamp);
-R_API const char *r_time_to_string(ut64 ts);
+R_API R_MUSTUSE char *r_time_secs_tostring(time_t timeStamp);
+R_API R_MUSTUSE char *r_time_usecs_tostring(ut64 ts);
+
+R_API int r_time_beats(ut64 ts, int *sub);
+
+R_API ut64 r_time_dos_today(ut32 timeStamp, int tz);
+R_API ut64 r_time_hfs_today(ut32 hfsts, int tz);
+R_API ut64 r_time_unix_today(ut32 unxts, int tz);
+R_API ut64 r_time_w32_today(ut64 ts, int tz);
 
 // Cross platform thread-safe time functions
 R_API char *r_asctime_r(const struct tm *tm, char *buf);
 R_API char *r_ctime_r(const time_t *timer, char *buf);
 
+/** profiling */
+typedef struct r_prof_t {
+	struct timeval when;
+	double result;
+} RProfile;
+R_API void r_prof_start(RProfile *p);
+R_API double r_prof_end(RProfile *p);
+
 #define R_TIME_PROFILE_ENABLED 0
 
 #if R_TIME_PROFILE_ENABLED
