@@ -1,13 +1,10 @@
-/* radare2 - LGPL - Copyright 2021-2024 - condret */
+/* radare2 - LGPL - Copyright 2021-2025 - condret */
 
 #include <r_io.h>
 
 R_API RIOBank *r_io_bank_new(const char *name) {
 	R_RETURN_VAL_IF_FAIL (name, NULL);
 	RIOBank *bank = R_NEW0 (RIOBank);
-	if (!bank) {
-		return NULL;
-	}
 	bank->name = strdup (name);
 	bank->submaps = r_crbtree_new (free);
 	if (!bank->submaps) {
@@ -53,7 +50,7 @@ R_API void r_io_bank_free(RIOBank *bank) {
 R_API void r_io_bank_init(RIO *io) {
 	R_RETURN_IF_FAIL (io);
 	r_io_bank_fini (io);
-	io->banks = r_id_storage_new (0, UT32_MAX);
+	r_id_storage_init (&io->banks, 0, UT32_MAX);
 }
 
 static bool _bank_free_cb(void *user, void *data, ut32 id) {
@@ -63,16 +60,14 @@ static bool _bank_free_cb(void *user, void *data, ut32 id) {
 
 R_API void r_io_bank_fini(RIO *io) {
 	R_RETURN_IF_FAIL (io);
-	if (io->banks) {
-		r_id_storage_foreach (io->banks, _bank_free_cb, NULL);
-		r_id_storage_free (io->banks);
-		io->banks = NULL;
-	}
+	r_id_storage_foreach (&io->banks, _bank_free_cb, NULL);
+	r_id_storage_fini (&io->banks);
+	io->banks = (const RIDStorage){0};
 }
 
 R_API RIOBank *r_io_bank_get(RIO *io, const ut32 bankid) {
-	R_RETURN_VAL_IF_FAIL (io && io->banks, NULL);
-	return (RIOBank *)r_id_storage_get (io->banks, bankid);
+	R_RETURN_VAL_IF_FAIL (io, NULL);
+	return (RIOBank *)r_id_storage_get (&io->banks, bankid);
 }
 
 typedef struct {
@@ -92,17 +87,17 @@ static bool find_bank(void *data, void *user, ut32 id) {
 }
 
 R_API RIOBank *r_io_bank_get_byname(RIO *io, const char *bankname) {
-	R_RETURN_VAL_IF_FAIL (io && io->banks && bankname, NULL);
+	R_RETURN_VAL_IF_FAIL (io && bankname, NULL);
 	Boring boo = { .io = io, .name = bankname, .bank = NULL };
 	eprintf ("ooME (%s)\n", boo.name);
-	r_id_storage_foreach (io->banks, &find_bank, &boo);
+	r_id_storage_foreach (&io->banks, &find_bank, &boo);
 	return boo.bank;
 }
 
 R_API ut32 r_io_bank_first(RIO *io) {
 	R_RETURN_VAL_IF_FAIL (io, UT32_MAX);
-	ut32 bankid = -1;
-	r_id_storage_get_lowest (io->banks, &bankid);
+	ut32 bankid = UT32_MAX;
+	r_id_storage_get_lowest (&io->banks, &bankid);
 	return bankid;
 }
 
@@ -116,9 +111,34 @@ R_API bool r_io_bank_use(RIO *io, ut32 bankid) {
 	return false;
 }
 
+typedef struct {
+	RIO *io;
+	bool dup;
+	RIOBank *bank;
+} NameBankDup;
+
+static bool namedupchk(void *user, void *data, ut32 id) {
+	NameBankDup *nbd = (NameBankDup*)user;
+	if (nbd->dup) {
+		return false;
+	}
+	RIOBank *bank = (RIOBank *)data;
+	if (!strcmp (nbd->bank->name, bank->name)) {
+		nbd->dup = true;
+	}
+	return true;
+}
+
 R_API bool r_io_bank_add(RIO *io, RIOBank *bank) {
-	R_RETURN_VAL_IF_FAIL (io && io->banks && bank, false);
-	return r_id_storage_add (io->banks, bank, &bank->id);
+	R_RETURN_VAL_IF_FAIL (io && bank, false);
+	NameBankDup nbd = { io, false, bank };
+	r_id_storage_foreach (&io->banks, namedupchk, &nbd);
+	if (nbd.dup) {
+		R_LOG_ERROR ("Cannot have two banks with the same name");
+		return false;
+	}
+	// TODO: we can have two banks with the same name.. which maybe its not
+	return r_id_storage_add (&io->banks, bank, &bank->id);
 }
 
 static RIOMapRef *_mapref_from_map(RIOMap *map) {
@@ -155,7 +175,7 @@ static int _find_sm_by_vaddr_cb(void *incoming, void *in, void *user) {
 	return 1;
 }
 
-static int _find_lowest_intersection_sm_cb(void *incoming, void *in, void *user) {
+static int _find_intersection_sm_cb(void *incoming, void *in, void *user) {
 	RIOSubMap *bd = (RIOSubMap *)incoming, *sm = (RIOSubMap *)in;
 	if (r_io_submap_overlap (bd, sm)) {
 		return 0;
@@ -168,7 +188,7 @@ static int _find_lowest_intersection_sm_cb(void *incoming, void *in, void *user)
 
 // returns the node containing the submap with lowest itv.addr, that intersects with sm
 static RRBNode *_find_entry_submap_node(RIOBank *bank, RIOSubMap *sm) {
-	RRBNode *node = r_crbtree_find_node (bank->submaps, sm, _find_lowest_intersection_sm_cb, NULL);
+	RRBNode *node = r_crbtree_find_node (bank->submaps, sm, _find_intersection_sm_cb, NULL);
 	if (!node) {
 		return NULL;
 	}
@@ -743,34 +763,48 @@ R_API bool r_io_bank_locate(RIO *io, const ut32 bankid, ut64 *addr, const ut64 s
 	fake_sm.itv.addr = *addr + (load_align - *addr % load_align) % load_align;
 	fake_sm.itv.size = size;
 	fake_sm.mapref = (const RIOMapRef) {0};
-	RRBNode *entry = _find_entry_submap_node (bank, &fake_sm);
+	RRBNode *entry = r_crbtree_find_node (bank->submaps, &fake_sm, _find_intersection_sm_cb, NULL);
 	if (!entry) {
 		// no submaps in this bank
 		*addr = fake_sm.itv.addr;
 		return true;
 	}
-	// this is a bit meh: first iteration can never be successful,
-	// bc entry->sm will always intersect with fake_sm, if
-	// _find_entry_submap_node suceeded previously
-	ut64 next_location = fake_sm.itv.addr;
-	while (entry) {
-		RIOSubMap *sm = (RIOSubMap *)entry->data;
-		if (size <= r_io_submap_from (sm) - next_location) {
-			*addr = next_location;
-			return true;
-		}
-		next_location = (r_io_submap_to (sm) + 1) +
-			(load_align - ((r_io_submap_to (sm) + 1) % load_align)) % load_align;
+	ut64 res = 0;
+	if (load_align == 1) {
+		RIOSubMap *sm = entry->data;
+		ut64 next_location = r_io_submap_to (sm) + 1;
 		entry = r_rbnode_next (entry);
+		while (entry) {
+			sm = entry->data;
+			if (size <= r_io_submap_from (sm) - next_location) {
+				*addr = next_location;
+				return true;
+			}
+			next_location = r_io_submap_to (sm) + 1;
+			entry = r_rbnode_next (entry);
+		}
+		res = next_location;
+	} else {
+		do {
+			RIOSubMap *sm = (RIOSubMap *)entry->data;
+			fake_sm.itv.addr = (r_io_submap_to (sm) + 1) +
+				(load_align - ((r_io_submap_to (sm) + 1) % load_align)) % load_align;
+			entry = r_crbtree_find_node (bank->submaps, &fake_sm, _find_intersection_sm_cb, NULL);
+		} while (entry);
+		res = fake_sm.itv.addr;
 	}
-	if (next_location == 0LL) {
+	if (res == 0LL) {
 		// overflow from last submap in the tree => no location
 		return false;
 	}
-	if (UT64_MAX - size + 1 < next_location) {
+	if ((res + size) < *addr) {
+		//probably not needed
 		return false;
 	}
-	*addr = next_location;
+	if ((UT64_MAX - size + 1) < res) {
+		return false;
+	}
+	*addr = res;
 	return true;
 }
 
@@ -922,11 +956,83 @@ R_API bool r_io_bank_write_to_overlay_at(RIO *io, const ut32 bankid, ut64 addr,
 					     r_io_submap_to (sm)) - (addr + buf_off) + 1;
 		ret &= r_io_map_write_to_overlay (map, addr + buf_off, &buf[buf_off], write_len);
 		node = r_rbnode_next (node);
-		sm = node ? (RIOSubMap *)node->data : NULL;
+		sm = node? (RIOSubMap *)node->data: NULL;
 	}
 	return ret;
 }
 
+typedef struct bank_overlay_foreach_user_t {
+	RIO *io;
+	RIOMap *map;
+	RIOSubMap *sm;
+	RIOOverlayForeach cb;
+	void *user;
+} BOFUser;
+
+static void bof_cb(RInterval itv, const ut8 *data, void *user) {
+	BOFUser *bof = user;
+	if (!r_itv_overlap (itv, bof->sm->itv)) {
+		return;
+	}
+	RInterval ov_itv = r_itv_intersect (itv, bof->sm->itv);
+	union {
+		ut8 data[sizeof (ut8 *)];
+		ut8 *ptr;
+	} m;
+	if (R_UNLIKELY (r_itv_size (ov_itv) > sizeof (ut8 *))) {
+		m.ptr = R_NEWS (ut8, r_itv_size (ov_itv));
+		if (!m.ptr) {
+			return;
+		}
+	}
+	const ut8 *o_data = &data[
+		(r_itv_begin (itv) < r_itv_begin (ov_itv))?
+		(r_itv_begin (ov_itv) - r_itv_begin (itv)): 0];
+	const ut64 pa = r_itv_begin (ov_itv) - r_io_map_from (bof->map) + bof->map->delta;
+	if (R_UNLIKELY (r_itv_size (ov_itv) > sizeof (ut8 *))) {
+		if (r_io_fd_read_at (bof->io, bof->map->fd, pa, m.ptr,
+			r_itv_size (ov_itv)) != r_itv_size (ov_itv)) {
+			R_LOG_WARN ("r_io_fd_read_at failed");
+			free (m.ptr);
+			return;
+		}
+		bof->cb (ov_itv, m.ptr, o_data, bof->user);
+		free (m.ptr);
+		return;
+	}
+	if (r_io_fd_read_at (bof->io, bof->map->fd, pa, m.data, r_itv_size (ov_itv)) != r_itv_size (ov_itv)) {
+		R_LOG_WARN ("r_io_fd_read_at failed");
+		return;
+	}
+	bof->cb (ov_itv, m.data, o_data, bof->user);
+}
+
+R_API void r_io_bank_overlay_foreach(RIO *io, const ut32 bankid, RIOOverlayForeach cb, void *user) {
+	R_RETURN_IF_FAIL (io && cb);
+	RIOBank *bank = r_io_bank_get (io, bankid);
+	if (!io->overlay || !bank || !bank->submaps || !bank->submaps->size) {
+		return;
+	}
+	RRBNode *node = r_crbtree_first_node (bank->submaps);
+	if (!node) {
+		return;
+	}
+	do {
+		RIOSubMap *sm = node->data;
+		RIOMap *map = r_io_map_get_by_ref (io, &sm->mapref);
+		if (R_UNLIKELY (!map)) {
+			R_LOG_WARN ("RIOBank %u got corrupted", bankid);
+		}
+		BOFUser bof = { io,
+			map,
+			sm,
+			cb,
+			user,
+		};
+		r_io_map_overlay_foreach (map, bof_cb, (void *)&bof);
+	} while ((node = r_rbnode_next (node)), node);
+}
+
 // reads only from single submap at addr and returns amount of bytes read.
 // if no submap is mapped at addr, fcn returns 0. returns -1 on error
 R_API int r_io_bank_read_from_submap_at(RIO *io, const ut32 bankid, ut64 addr, ut8 *buf, int len) {
@@ -1079,7 +1185,7 @@ R_API void r_io_bank_del_map(RIO *io, const ut32 bankid, const ut32 mapid) {
 
 R_API void r_io_bank_del(RIO *io, const ut32 bankid) {
 	R_RETURN_IF_FAIL (io);
-	r_id_storage_delete (io->banks, bankid);
+	r_id_storage_delete (&io->banks, bankid);
 	if (io->bank == bankid) {
 		io->bank = r_io_bank_first (io);
 	}
