@@ -1,7 +1,6 @@
 /* radare - LGPL - Copyright 2008-2024 - pancake */
 
 #include <r_io.h>
-#include <r_lib.h>
 #include "../io_memory.h"
 
 static bool __check(RIO *io, const char *pathname, bool many) {
@@ -16,7 +15,7 @@ static RIODesc *__open(RIO *io, const char *pathname, int rw, int mode) {
 			return NULL;
 		}
 		mal->offset = 0;
-		mal->buf = (ut8*)r_socket_http_get (pathname, &code, &rlen);
+		mal->buf = (ut8*)r_socket_http_get (pathname, NULL, &code, &rlen);
 		if (mal->buf && rlen > 0) {
 			mal->size = rlen;
 			return r_io_desc_new (io, &r_io_plugin_malloc, pathname,
