@@ -5,6 +5,7 @@
 #include <r_io.h>
 #include <r_lib.h>
 #include <r_cons.h>
+#include <r_core.h>
 
 #if __APPLE__ && DEBUGGER
 
@@ -232,7 +233,7 @@ static int __read(RIO *io, RIODesc *desc, ut8 *buf, int len) {
 	if (dd ->magic != r_str_hash ("mach")) {
 		return -1;
 	}
-	memset (buf, 0xff, len);
+	memset (buf, io->Oxff, len);
 	int pid = __get_pid (desc);
 	task_t task = pid_to_task (desc, pid);
 	if (task_is_dead (desc, pid)) {
@@ -259,12 +260,12 @@ static int __read(RIO *io, RIODesc *desc, ut8 *buf, int len) {
 			break;
 		case KERN_INVALID_ADDRESS:
 			if (blocksize == 1) {
-				memset (buf+copied, 0xff, len-copied);
+				memset (buf+copied, io->Oxff, len-copied);
 				return size+copied;
 			}
 			blocksize = 1;
 			blen = 1;
-			buf[copied] = 0xff;
+			buf[copied] = io->Oxff;
 			break;
 		}
 		if (err == -1 || size < 1) {
@@ -272,12 +273,12 @@ static int __read(RIO *io, RIODesc *desc, ut8 *buf, int len) {
 		}
 		if (size == 0) {
 			if (blocksize == 1) {
-				memset (buf + copied, 0xff, len - copied);
+				memset (buf + copied, io->Oxff, len - copied);
 				return len;
 			}
 			blocksize = 1;
 			blen = 1;
-			buf[copied] = 0xff;
+			buf[copied] = io->Oxff;
 		}
 		copied += blen;
 	}
@@ -511,6 +512,14 @@ static char *__system(RIO *io, RIODesc *fd, const char *cmd) {
 		}
 		return NULL;
 	}
+	if (r_str_startswith (cmd, "tls")) {
+#if __arm64__
+		RCore *core = io->coreb.core;
+		io->coreb.cmd (core, "dxr 60d03bd5");
+#else
+		R_LOG_TODO ("Not implemented for this architecture");
+#endif
+	}
 	if (r_str_startswith (cmd, "pid")) {
 		RIOMachData *iodd = fd->data;
 		RIOMach *riom = iodd->data;
