@@ -38,7 +38,6 @@ static int __read(RIO *io, RIODesc *desc, ut8 *buf, int count) {
 	RIOMalloc *mal = (RIOMalloc*)desc->data;
 	if (mal) {
 		ut64 addr = mal->offset;
-		r_cons_break_push (NULL, NULL);
 		RIOSocketData *sdat = mal->data;
 		RSocket *s = sdat->sc;
 		ut8 *mem = malloc (4096);
@@ -56,7 +55,6 @@ static int __read(RIO *io, RIODesc *desc, ut8 *buf, int count) {
 			}
 			free (mem);
 		}
-		r_cons_break_pop ();
 		mal->offset = addr;
 		if (sdat->ios && sdat->ios->buf) {
 			return r_buf_read_at (sdat->ios->buf, mal->offset, buf, count);
