@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2020-2024 - GustavoLCR */
+/* radare - LGPL - Copyright 2020-2025 - GustavoLCR */
 
 #define INITGUID
 #include <r_core.h>
@@ -157,8 +157,9 @@ static STDMETHODIMP __input_cb(PDEBUG_INPUT_CALLBACKS This, ULONG BufferSize) {
 	PDEBUG_INPUT_CALLBACKS_IMPL impl = (PDEBUG_INPUT_CALLBACKS_IMPL)This;
 	DbgEngContext *idbg = impl->m_idbg;
 	ITHISCALL (dbgCtrl, GetPromptText, prompt, sizeof (prompt), NULL);
-	r_line_set_prompt (prompt);
-	const char *str = r_line_readline ();
+	RCons *cons = r_cons_singleton ();
+	r_line_set_prompt (cons->line, prompt);
+	const char *str = r_line_readline (cons);
 	char *ret = r_str_ndup (str, R_MIN (strlen (str), BufferSize));
 	ITHISCALL (dbgCtrl, ReturnInput, ret);
 	return S_OK;
@@ -342,7 +343,7 @@ fail:
 	return NULL;
 }
 
-static bool windbg_init(void) {
+static bool windbg_init(RIODesc *desc) {
 	if (w32_DebugCreate && w32_DebugConnectWide) {
 		return true;
 	}
@@ -398,7 +399,7 @@ static RIODesc *windbg_open(RIO *io, const char *uri, int perm, int mode) {
 	if (!windbg_check (io, uri, 0)) {
 		return NULL;
 	}
-	if (!windbg_init ()) {
+	if (!windbg_init (NULL)) {
 		return NULL;
 	}
 	HRESULT hr = E_FAIL;
@@ -680,7 +681,6 @@ RIOPlugin r_io_plugin_windbg = {
 	},
 	.uris = WINDBGURI,
 	.isdbg = true,
-	.init = windbg_init,
 	.open = windbg_open,
 	.seek = windbg_lseek,
 	.read = windbg_read,
