@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2009-2023 - pancake */
+/* radare2 - LGPL - Copyright 2009-2025 - pancake */
 
 #include <r_lang.h>
 #include <r_util.h>
@@ -10,7 +10,7 @@ R_LIB_VERSION (r_lang);
 #include "p/pipe.c"
 #include "p/c.c"
 #include "p/s.c"
-#include "p/v.c"
+// #include "p/v.c"
 #include "p/vala.c"
 #include "p/rust.c"
 #include "p/zig.c"
@@ -27,46 +27,47 @@ R_LIB_VERSION (r_lang);
 #include "p/nim.c"
 #include "p/dart.c"
 
+static void r_lang_session_free(void *p) {
+	RLangSession *s = (RLangSession*)p;
+	if (s && s->plugin && s->plugin->fini) {
+		s->plugin->fini (s);
+	}
+	free (s);
+}
+
 R_API RLang *r_lang_new(void) {
 	RLang *lang = R_NEW0 (RLang);
-	if (!lang) {
-		return NULL;
-	}
 	lang->user = NULL;
 	lang->langs = r_list_new ();
-	if (!lang->langs) {
-		r_lang_free (lang);
-		return NULL;
-	}
 	lang->defs = r_list_new ();
-	if (!lang->defs) {
-		r_lang_free (lang);
-		return NULL;
-	}
-	lang->sessions = r_list_newf (free);
+	lang->sessions = r_list_newf (r_lang_session_free);
 	lang->defs->free = (RListFree)r_lang_def_free;
-	lang->cb_printf = (PrintfCallback)printf;
+	const bool load_plugins = !r_sys_getenv_asbool ("R2_DEBUG_NOLANG");
+	if (load_plugins) {
 #if HAVE_SYSTEM
 #if R2__UNIX__
-	r_lang_plugin_add (lang, &r_lang_plugin_s);
-	r_lang_plugin_add (lang, &r_lang_plugin_c);
-	r_lang_plugin_add (lang, &r_lang_plugin_cpipe);
+		r_lang_plugin_add (lang, &r_lang_plugin_s);
+		r_lang_plugin_add (lang, &r_lang_plugin_c);
+		r_lang_plugin_add (lang, &r_lang_plugin_cpipe);
 #endif
-	r_lang_plugin_add (lang, &r_lang_plugin_v);
-	r_lang_plugin_add (lang, &r_lang_plugin_vala);
-	r_lang_plugin_add (lang, &r_lang_plugin_rust);
-	r_lang_plugin_add (lang, &r_lang_plugin_zig);
-	r_lang_plugin_add (lang, &r_lang_plugin_pipe);
+	//	r_lang_plugin_add (lang, &r_lang_plugin_v);
+		r_lang_plugin_add (lang, &r_lang_plugin_vala);
+		r_lang_plugin_add (lang, &r_lang_plugin_rust);
+		r_lang_plugin_add (lang, &r_lang_plugin_zig);
+		r_lang_plugin_add (lang, &r_lang_plugin_pipe);
 #endif
-	r_lang_plugin_add (lang, &r_lang_plugin_go);
-	r_lang_plugin_add (lang, &r_lang_plugin_poke);
-	r_lang_plugin_add (lang, &r_lang_plugin_spp);
-	r_lang_plugin_add (lang, &r_lang_plugin_lib);
-	r_lang_plugin_add (lang, &r_lang_plugin_asm);
+		r_lang_plugin_add (lang, &r_lang_plugin_go);
+		r_lang_plugin_add (lang, &r_lang_plugin_poke);
+		r_lang_plugin_add (lang, &r_lang_plugin_spp);
+		r_lang_plugin_add (lang, &r_lang_plugin_lib);
+		r_lang_plugin_add (lang, &r_lang_plugin_asm);
+		r_lang_plugin_add (lang, &r_lang_plugin_tsc);
+		r_lang_plugin_add (lang, &r_lang_plugin_nim);
+		r_lang_plugin_add (lang, &r_lang_plugin_dart);
+	}
+#if WANT_QJS
 	r_lang_plugin_add (lang, &r_lang_plugin_qjs);
-	r_lang_plugin_add (lang, &r_lang_plugin_tsc);
-	r_lang_plugin_add (lang, &r_lang_plugin_nim);
-	r_lang_plugin_add (lang, &r_lang_plugin_dart);
+#endif
 
 	return lang;
 }
@@ -172,59 +173,6 @@ R_API bool r_lang_plugin_remove(RLang *lang, RLangPlugin *plugin) {
 	return true;
 }
 
-/* TODO: deprecate all list methods */
-R_API void r_lang_list(RLang *lang, int mode) {
-	RListIter *iter;
-	RLangPlugin *h;
-	if (!lang) {
-		return;
-	}
-	PJ *pj = NULL;
-	RTable *table = NULL;
-	if (mode == 'j') {
-		pj = pj_new ();
-		pj_a (pj);
-	} else if (mode == ',') {
-		table = r_table_new ("langs");
-		RTableColumnType *typeString = r_table_type ("string");
-		r_table_add_column (table, typeString, "name", 0);
-		r_table_add_column (table, typeString, "license", 0);
-		r_table_add_column (table, typeString, "desc", 0);
-	}
-	r_list_foreach (lang->langs, iter, h) {
-		const char *license = h->meta.license
-			? h->meta.license : "???";
-		if (mode == 'j') {
-			pj_o (pj);
-			pj_ks (pj, "name", r_str_get (h->meta.name));
-			pj_ks (pj, "license", license);
-			pj_ks (pj, "description", r_str_get (h->meta.desc));
-			pj_end (pj);
-		} else if (mode == 'q') {
-			lang->cb_printf ("%s\n", h->meta.name);
-		} else if (mode == ',') {
-			r_table_add_row (table,
-				r_str_get (h->meta.name),
-				r_str_get (h->meta.license),
-				r_str_get (h->meta.desc), 0);
-		} else {
-			lang->cb_printf ("%-8s %6s  %s\n",
-				h->meta.name, license, h->meta.desc);
-		}
-	}
-	if (pj) {
-		pj_end (pj);
-		char *s = pj_drain (pj);
-		lang->cb_printf ("%s\n", s);
-		free (s);
-	} else if (table) {
-		char *s = r_table_tostring (table);
-		lang->cb_printf ("%s\n", s);
-		free (s);
-		r_table_free (table);
-	}
-}
-
 R_API RLangPlugin *r_lang_get_by_extension(RLang *lang, const char *ext) {
 	RListIter *iter;
 	RLangPlugin *h;
@@ -366,7 +314,7 @@ R_API bool r_lang_prompt(RLang *lang) {
 	char buf[1024];
 	const char *p;
 
-	if (!lang || !lang->session) {
+	if (!lang->session) {
 		return false;
 	}
 
@@ -376,7 +324,7 @@ R_API bool r_lang_prompt(RLang *lang) {
 		return true;
 	}
 	/* init line */
-	RLine *line = r_line_singleton ();
+	RLine *line = lang->cons->line;
 	RLineHistory hist = line->history;
 	RLineHistory histnull = {0};
 	RLineCompletion oc = line->completion;
@@ -387,9 +335,9 @@ R_API bool r_lang_prompt(RLang *lang) {
 
 	/* foo */
 	for (;;) {
-		r_cons_flush ();
-		snprintf (buf, sizeof (buf)-1, "%s> ", plugin->meta.name);
-		r_line_set_prompt (buf);
+		r_cons_flush (lang->cons);
+		snprintf (buf, sizeof (buf) - 1, "%s> ", plugin->meta.name);
+		r_line_set_prompt (lang->cons->line, buf);
 #if 0
 		printf ("%s> ", lang->cur->name);
 		fflush (stdout);
@@ -397,11 +345,11 @@ R_API bool r_lang_prompt(RLang *lang) {
 		if (feof (stdin)) break;
 		r_str_trim_tail (buf);
 #endif
-		p = r_line_readline ();
+		p = r_line_readline (lang->cons);
 		if (!p) {
 			break;
 		}
-		r_line_hist_add (p);
+		r_line_hist_add (lang->cons->line, p);
 		strncpy (buf, p, sizeof (buf) - 1);
 		if (*buf == '!') {
 			if (buf[1]) {
@@ -409,11 +357,11 @@ R_API bool r_lang_prompt(RLang *lang) {
 			} else {
 				char *foo, *code = NULL;
 				do {
-					foo = r_cons_editor (NULL, code);
+					foo = r_cons_editor (lang->cons, NULL, code);
 					r_lang_run (lang, foo, 0);
 					free (code);
 					code = foo;
-				} while (r_cons_yesno ('y', "Edit again? (Y/n)"));
+				} while (r_cons_yesno (lang->cons, 'y', "Edit again? (Y/n)"));
 				free (foo);
 			}
 			continue;
@@ -453,7 +401,7 @@ R_API bool r_lang_prompt(RLang *lang) {
 		}
 	}
 	// XXX: leaking history
-	r_line_set_prompt (prompt);
+	r_line_set_prompt (line->cons->line, prompt);
 	line->completion = oc;
 	line->history = hist;
 	clearerr (stdin);
