@@ -1,9 +1,9 @@
-/* radare - LPGL - Copyright 2017-2022 condret */
+/* radare - LPGL - Copyright 2017-2025 - condret */
 
 #include <r_lang.h>
 
 static bool lang_lib_file_run(RLangSession *user, const char *file) {
-	char *libpath = R_STR_DUP (file);
+	char *libpath = strdup (file);
 	void *lib;
 	if (!libpath) {
 		return false;
@@ -21,14 +21,20 @@ static bool lang_lib_file_run(RLangSession *user, const char *file) {
 		return false;
 	}
 
-	lib = r_lib_dl_open (libpath);
+	lib = r_lib_dl_open (libpath, false);
 	if (lib) {
 		void (*fcn)(RCore *);
 		fcn = r_lib_dl_sym (lib, "entry");
 		if (fcn) {
 			fcn (user->user_data);
 		} else {
-			R_LOG_ERROR ("Cannot find 'entry' symbol in library");
+			void *rp = r_lib_dl_sym (lib, "radare_plugin");
+			if (rp) {
+				RCore *core = user->lang->user;
+				user->lang->cmdf (core, "'L %s", file);
+			} else {
+				R_LOG_ERROR ("Cannot find 'entry' symbol in library");
+			}
 		}
 		r_lib_dl_close (lib);
 	}
