@@ -3,7 +3,7 @@
  * Copyright (c) Ian F. Darwin 1986-1995.
  * Software written by Ian F. Darwin and others;
  * maintained 1995-present by Christos Zoulas and others.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -13,7 +13,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *  
+ *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -54,33 +54,29 @@ typedef unsigned int ssize_t;
 typedef unsigned long unichar;
 
 struct stat;
-const char *file_fmttime(unsigned int, int, char *);
-int file_buffer(struct r_magic_set *, int, const char *, const void *, size_t);
-int file_fsmagic(struct r_magic_set *, const char *, struct stat *);
-int file_pipe2file(struct r_magic_set *, int, const void *, size_t);
-int file_printf(struct r_magic_set *, const char *, ...);
-int file_reset(struct r_magic_set *);
-int file_tryelf(struct r_magic_set *, int, const unsigned char *, size_t);
-int file_zmagic(struct r_magic_set *, int, const char *, const ut8*, size_t);
-int file_ascmagic(struct r_magic_set *, const unsigned char *, size_t);
-int file_is_tar(struct r_magic_set *, const unsigned char *, size_t);
-int file_softmagic(struct r_magic_set *, const unsigned char *, size_t, int);
-struct mlist *file_apprentice(struct r_magic_set *, const char *, size_t, int);
-ut64 file_signextend(RMagic *, struct r_magic *, ut64);
-void file_delmagic(struct r_magic *, int type, size_t entries);
-void file_badread(struct r_magic_set *);
-void file_badseek(struct r_magic_set *);
-void file_oomem(struct r_magic_set *, size_t);
-void file_error(struct r_magic_set *, int, const char *, ...);
-void file_magerror(struct r_magic_set *, const char *, ...);
-void file_magwarn(struct r_magic_set *, const char *, ...);
-void file_mdump(struct r_magic *);
-void file_showstr(FILE *, const char *, size_t);
-size_t file_mbswidth(const char *);
-const char *file_getbuffer(struct r_magic_set *);
-ssize_t sread(int, void *, size_t, int);
-int file_check_mem(struct r_magic_set *, unsigned int);
-int file_looks_utf8(const unsigned char *, size_t, unichar *, size_t *);
+R_IPI const char *__magic_file_fmttime(unsigned int, int, char *);
+R_IPI int __magic_file_buffer(RMagic *, int, const char *, const void *, size_t);
+R_IPI int __magic_file_fsmagic(RMagic *, const char *, struct stat *);
+R_IPI int __magic_file_printf(RMagic *, const char *, ...);
+R_IPI int __magic_file_reset(RMagic *);
+R_IPI int __magic_file_zmagic(RMagic *, int, const char *, const ut8*, size_t);
+R_IPI int __magic_file_ascmagic(RMagic *, const unsigned char *, size_t);
+R_IPI int __magic_file_is_tar(RMagic *, const unsigned char *, size_t);
+R_IPI int __magic_file_softmagic(RMagic *, const unsigned char *, size_t, int);
+R_IPI struct mlist *__magic_file_apprentice(RMagic *, const char *, size_t, int);
+R_IPI ut64 __magic_file_signextend(RMagic *, struct r_magic *, ut64);
+R_IPI void __magic_file_delmagic(struct r_magic *, int type, size_t entries);
+R_IPI void __magic_file_badread(RMagic *);
+R_IPI void __magic_file_badseek(RMagic *);
+R_IPI void __magic_file_oomem(RMagic *, size_t);
+R_IPI void __magic_file_error(RMagic *, int, const char *, ...);
+R_IPI void __magic_file_magerror(RMagic *, const char *, ...);
+R_IPI void __magic_file_magwarn(RMagic *, const char *, ...);
+R_IPI void __magic_file_mdump(RMagic *, struct r_magic *);
+R_IPI void __magic_file_showstr(FILE *, const char *, size_t);
+R_IPI const char *__magic_file_getbuffer(RMagic *);
+R_IPI int __magic_file_check_mem(RMagic *, unsigned int);
+R_IPI int __magic_file_looks_utf8(const unsigned char *, size_t, unichar *, size_t *);
 
 #ifndef HAVE_VASPRINTF
 int vasprintf(char **ptr, const char *format_string, va_list vargs);
