@@ -19,8 +19,18 @@ typedef struct {
 	// bool quiet;
 } REggState;
 
-/* egg callback */
-static int __lib_egg_cb(RLibPlugin *pl, void *user, void *data) {
+typedef struct {
+	const char *name;
+	const char *desc;
+} REggEnv;
+
+static REggEnv env[] = {
+	{ "R2_NOPLUGINS", "do not load any plugin" }
+};
+
+static void ragg_show_env(bool show_desc);
+
+static bool __lib_egg_cb(RLibPlugin *pl, void *user, void *data) {
 	REggPlugin *hand = (REggPlugin *)data;
 	REggState *es = (REggState *)user;
 	r_egg_plugin_add (es->e, hand);
@@ -56,14 +66,12 @@ static void __load_plugins(REggState *es) {
 
 static REggState *__es_new(bool load_plugins) {
 	REggState *es = R_NEW0 (REggState);
-	if (es) {
-		es->l = r_lib_new (NULL, NULL);
-		es->e = r_egg_new ();
-		es->a = r_anal_new ();
-		r_anal_bind (es->a, &es->e->rasm->analb);
-		if (load_plugins) {
-			__load_plugins (es);
-		}
+	es->l = r_lib_new (NULL, NULL);
+	es->e = r_egg_new ();
+	es->a = r_anal_new ();
+	r_anal_bind (es->a, &es->e->rasm->analb);
+	if (load_plugins) {
+		__load_plugins (es);
 	}
 	return es;
 }
@@ -94,6 +102,7 @@ static int usage(int v) {
 			" -f [format]     output format (raw, c, pe, elf, mach0, python, javascript)\n"
 			" -F              output native format (osx=mach0, linux=elf, ..)\n"
 			" -h              show this help\n"
+			" -H ([var])      display variable\n"
 			" -i [shellcode]  include shellcode plugin, uses options. see -L\n"
 			" -I [path]       add include path\n"
 			" -k [os]         operating system's kernel (linux,bsd,osx,w32)\n"
@@ -114,8 +123,8 @@ static int usage(int v) {
 			" -w [off:hex]    patch hexpairs at given offset\n"
 			" -x              execute\n"
 			" -X [hexpairs]   execute rop chain, using the stack provided\n"
-			" -z              output in C string syntax\n"
-			"R2_NOPLUGINS=1   do not load any plugin\n");
+			" -z              output in C string syntax\n");
+			ragg_show_env (true);
 	}
 	return 1;
 }
@@ -185,7 +194,25 @@ static int openfile(const char *f, int x) {
 #endif
 	return fd;
 }
-#define ISEXEC (fmt!='r')
+#define ISEXEC (fmt != 'r')
+
+static void ragg_env_print(const char *name) {
+	char *value = r_sys_getenv (name);
+	printf ("%s\n", R_STR_ISNOTEMPTY (value) ? value : "");
+	free (value);
+}
+
+static void ragg_show_env(bool show_desc) {
+	int id = 0;
+	for (id = 0; id < (sizeof (env) / sizeof (env[0])); id++) {
+		if (show_desc) {
+			printf ("%s\t%s\n", env[id].name, env[id].desc);
+		} else {
+			printf ("%s=", env[id].name);
+			ragg_env_print(env[id].name);
+		}
+	}
+}
 
 R_API int r_main_ragg2(int argc, const char **argv) {
 	const char *file = NULL;
@@ -209,7 +236,7 @@ R_API int r_main_ragg2(int argc, const char **argv) {
 	const char *encoder = NULL;
 	const char *eggprg = NULL;
 	char *sequence = NULL;
-	int bits = (R_SYS_BITS & R_SYS_BITS_64)? 64: 32;
+	int bits = R_SYS_BITS_CHECK (R_SYS_BITS, 64)? 64: 32;
 	int fmt = 0;
 	const char *ofile = NULL;
 	int ofileauto = 0;
@@ -224,7 +251,13 @@ R_API int r_main_ragg2(int argc, const char **argv) {
 	REggState *es = __es_new (load_plugins);
 
 	RGetopt opt;
-	r_getopt_init (&opt, argc, argv, "a:b:B:c:C:d:D:e:E:f:Fhi:I:k:Ln:N:o:Op:P:q:rsS:vw:xX:z");
+	r_getopt_init (&opt, argc, argv, "a:b:B:c:C:d:D:e:E:f:FhH:i:I:k:Ln:N:o:Op:P:q:rsS:vw:xX:z");
+	if (argc == 2 && !strcmp (argv[1], "-H")) {
+		ragg_show_env (false);
+		__es_free (es);
+		free (sequence);
+		return 0;
+	}
 	while ((c = r_getopt_next (&opt)) != -1) {
 		switch (c) {
 		case 'a':
@@ -405,6 +438,11 @@ R_API int r_main_ragg2(int argc, const char **argv) {
 			__es_free (es);
 			free (sequence);
 			return usage (1);
+		case 'H':
+			ragg_env_print (opt.arg);
+			__es_free (es);
+			free (sequence);
+			return 0;
 		case 'v':
 			free (sequence);
 			__es_free (es);
