@@ -32,9 +32,6 @@ R_API RSearchKeyword* r_search_keyword_new(const ut8 *kwbuf, int kwlen, const ut
 		return NULL;
 	}
 	kw = R_NEW0 (RSearchKeyword);
-	if (!kw) {
-		return NULL;
-	}
 	kw->type = R_SEARCH_KEYWORD_TYPE_BINARY;
 	kw->data = (void *) data;
 	kw->keyword_length = kwlen;
@@ -91,7 +88,7 @@ R_API RSearchKeyword* r_search_keyword_new_str(const char *kwbuf, const char *bm
 	return kw;
 }
 
-R_API RSearchKeyword* r_search_keyword_new_wide(const char *kwbuf, const char *bmstr, const char *data, bool ignore_case) {
+R_API RSearchKeyword* r_search_keyword_new_wide(const char *kwbuf, const char *bmstr, const char *data, bool ignore_case, bool be) {
 	RSearchKeyword *kw;
 	int len;
 	const char *p2;
@@ -100,7 +97,7 @@ R_API RSearchKeyword* r_search_keyword_new_wide(const char *kwbuf, const char *b
 	int bmlen = 0;
 
 	if (bmstr) {
-		bmbuf = malloc (strlen (bmstr)+1);
+		bmbuf = malloc (strlen (bmstr) + 1);
 		if (!bmbuf) {
 			return NULL;
 		}
@@ -114,7 +111,7 @@ R_API RSearchKeyword* r_search_keyword_new_wide(const char *kwbuf, const char *b
 	str = malloc ((len + 1) * 2);
 	for (p2 = kwbuf, p = str; *p2; ) {
 		RRune ch;
-		int num_utf8_bytes = r_utf8_decode ((const ut8 *)p2, kwbuf + len - p2, &ch);
+		const int num_utf8_bytes = r_utf8_decode ((const ut8 *)p2, kwbuf + len - p2, &ch);
 		if (num_utf8_bytes < 1) {
 			R_LOG_WARN ("Malformed UTF8 at pos %d", (int)(p2 - kwbuf));
 			p[0] = *p2;
@@ -126,8 +123,10 @@ R_API RSearchKeyword* r_search_keyword_new_wide(const char *kwbuf, const char *b
 		if (ignore_case && ch <= 0xff) {
 			ch = tolower (ch);
 		}
-		int num_wide_bytes = r_utf16le_encode ((ut8 *)p, ch);
-		r_warn_if_fail (num_wide_bytes != 0);
+		const int num_wide_bytes = be
+			? r_utf16be_encode ((ut8 *)p, ch)
+			: r_utf16le_encode ((ut8 *)p, ch);
+		R_WARN_IF_FAIL (num_wide_bytes != 0);
 		p2 += num_utf8_bytes;
 		p += num_wide_bytes;
 	}
@@ -222,7 +221,7 @@ R_API RSearchKeyword *r_search_keyword_new_regexp(const char *str, const char *d
 	for (start = i; str[i]; i++) {
 		if (str[i] == '/' && str[i - 1] != '\\') {
 			break;
-		} else if (str[i - 1] == '\\' && isalpha ((unsigned char)str[i])) {
+		} else if (str[i - 1] == '\\' && isalpha (str[i] & 0xff)) {
 			specials++;
 		}
 	}
