@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2006-2024 - pancake */
+/* radare - LGPL - Copyright 2006-2025 - pancake */
 
 /* must be included first because of winsock2.h and windows.h */
 #include <r_socket.h>
@@ -171,9 +171,6 @@ static bool __listen_unix(RSocket *s, const char *file) {
 
 R_API RSocket *r_socket_new(bool is_ssl) {
 	RSocket *s = R_NEW0 (RSocket);
-	if (!s) {
-		return NULL;
-	}
 	s->is_ssl = is_ssl;
 	s->port = 0;
 #if __UNIX_
@@ -788,6 +785,7 @@ R_API char *r_socket_tostring(RSocket *s) {
 R_API int r_socket_write(RSocket *s, const void *buf, int len) {
 	int ret, delta = 0;
 #if R2__UNIX__
+	// this is on non-linux only in theory..
 	r_sys_signal (SIGPIPE, SIG_IGN);
 #endif
 	for (;;) {
@@ -806,9 +804,12 @@ R_API int r_socket_write(RSocket *s, const void *buf, int len) {
 #endif
 		if (s->proto == R_SOCKET_PROTO_SERIAL) {
 			ret = write (s->fd, (char *)buf + delta, b);
-			eprintf ("SERIAL WRITE %d\n", ret);
 		} else {
+#if __linux__
+			ret = send (s->fd, (char *)buf + delta, b, MSG_NOSIGNAL);
+#else
 			ret = send (s->fd, (char *)buf + delta, b, 0);
+#endif
 		}
 		//if (ret == 0) return -1;
 		if (ret < 1) {
