@@ -1,14 +1,9 @@
-/* radare2 - LGPL - Copyright 2017-2022 - wargio */
+/* radare2 - LGPL - Copyright 2017-2025 - wargio, pancake */
 
 #include <r_util.h>
 #include "x509.h"
 
-extern void r_x509_name_json (PJ *pj, RX509Name *name);
-extern void r_x509_free_crl (RX509CertificateRevocationList *crl);
-extern void r_x509_crlentry_dump (RX509CRLEntry *crle, const char *pad, RStrBuf *sb);
-static bool r_pkcs7_parse_attributes(RPKCS7Attributes *attribute, RASN1Object *object);
-
-static bool r_pkcs7_parse_contentinfo(RPKCS7ContentInfo *ci, RASN1Object *object) {
+static bool r_pkcs7_contentinfo_parse(RPKCS7ContentInfo *ci, RASN1Object *object) {
 	if (!ci || !object || object->list.length < 1 || !object->list.objects[0]) {
 		return false;
 	}
@@ -16,7 +11,7 @@ static bool r_pkcs7_parse_contentinfo(RPKCS7ContentInfo *ci, RASN1Object *object
 	if (object->list.length > 1) {
 		RASN1Object *obj1 = object->list.objects[1];
 		if (obj1) {
-			ci->content = r_asn1_create_binary (obj1->sector, obj1->length);
+			ci->content = r_asn1_binary_new (obj1->sector, obj1->length);
 		}
 	}
 	return true;
@@ -34,25 +29,21 @@ static bool r_pkcs7_parse_certificaterevocationlists(RPKCS7CertificateRevocation
 		}
 		crls->length = object->list.length;
 		for (i = 0; i < crls->length; i++) {
-			crls->elements[i] = r_x509_parse_crl (object->list.objects[i]);
+			crls->elements[i] = r_x509_crl_parse (object->list.objects[i]);
 		}
 	}
 	return true;
 }
 
-static void r_pkcs7_free_certificaterevocationlists(RPKCS7CertificateRevocationLists *crls) {
+static void r_pkcs7_certificaterevocationlists_fini(RPKCS7CertificateRevocationLists *crls) {
 	ut32 i;
-	if (crls) {
-		for (i = 0; i < crls->length; i++) {
-			r_x509_free_crl (crls->elements[i]);
-			crls->elements[i] = NULL;
-		}
-		R_FREE (crls->elements);
-		// Used internally pkcs #7, so it shouldn't free crls.
+	for (i = 0; i < crls->length; i++) {
+		r_x509_crl_free (crls->elements[i]);
 	}
+	R_FREE (crls->elements);
 }
 
-static bool r_pkcs7_parse_extendedcertificatesandcertificates(RPKCS7ExtendedCertificatesAndCertificates *ecac, RASN1Object *object) {
+static bool r_pkcs7_extendedcertificatesandcertificates_parse(RPKCS7ExtendedCertificatesAndCertificates *ecac, RASN1Object *object) {
 	ut32 i;
 	if (!ecac || !object) {
 		return false;
@@ -64,26 +55,24 @@ static bool r_pkcs7_parse_extendedcertificatesandcertificates(RPKCS7ExtendedCert
 		}
 		ecac->length = object->list.length;
 		for (i = 0; i < ecac->length; i++) {
-			ecac->elements[i] = r_x509_parse_certificate (object->list.objects[i]);
+			ecac->elements[i] = r_x509_certificate_parse (object->list.objects[i]);
 			object->list.objects[i] = NULL;
 		}
 	}
 	return true;
 }
 
-static void r_pkcs7_free_extendedcertificatesandcertificates(RPKCS7ExtendedCertificatesAndCertificates *ecac) {
+static void r_pkcs7_extendedcertificatesandcertificates_fini(RPKCS7ExtendedCertificatesAndCertificates *ecac) {
 	ut32 i;
-	if (ecac) {
-		for (i = 0; i < ecac->length; i++) {
-			r_x509_free_certificate (ecac->elements[i]);
-			ecac->elements[i] = NULL;
-		}
-		R_FREE (ecac->elements);
-		// Used internally pkcs #7, so it shouldn't free ecac.
+	for (i = 0; i < ecac->length; i++) {
+		r_x509_certificate_free (ecac->elements[i]);
+		ecac->elements[i] = NULL;
 	}
+	R_FREE (ecac->elements);
+	// Used internally pkcs #7, so it shouldn't free ecac.
 }
 
-static bool r_pkcs7_parse_digestalgorithmidentifier(RPKCS7DigestAlgorithmIdentifiers *dai, RASN1Object *object) {
+static bool r_pkcs7_digestalgorithmidentifier_parse(RPKCS7DigestAlgorithmIdentifiers *dai, RASN1Object *object) {
 	ut32 i;
 	if (!dai || !object) {
 		return false;
@@ -95,101 +84,128 @@ static bool r_pkcs7_parse_digestalgorithmidentifier(RPKCS7DigestAlgorithmIdentif
 		}
 		dai->length = object->list.length;
 		for (i = 0; i < dai->length; i++) {
-			// r_x509_parse_algorithmidentifier returns bool,
-			// so i have to allocate before calling the function
-			dai->elements[i] = (RX509AlgorithmIdentifier *)malloc (sizeof (RX509AlgorithmIdentifier));
-			//should i handle invalid memory? the function checks the pointer
-			//or it should return if dai->elements[i] == NULL ?
+			dai->elements[i] = R_NEW (RX509AlgorithmIdentifier);
+			// should i handle invalid memory? the function checks the pointer
+			// or it should return if dai->elements[i] == NULL ?
 			if (dai->elements[i]) {
 				//Memset is needed to initialize to 0 the structure and avoid garbage.
 				memset (dai->elements[i], 0, sizeof (RX509AlgorithmIdentifier));
-				r_x509_parse_algorithmidentifier (dai->elements[i], object->list.objects[i]);
+				r_x509_algorithmidentifier_parse (dai->elements[i], object->list.objects[i]);
 			}
 		}
 	}
 	return true;
 }
 
-static void r_pkcs7_free_digestalgorithmidentifier(RPKCS7DigestAlgorithmIdentifiers *dai) {
+static void r_pkcs7_digestalgorithmidentifier_fini(RPKCS7DigestAlgorithmIdentifiers *dai) {
 	ut32 i;
-	if (dai) {
-		for (i = 0; i < dai->length; i++) {
-			if (dai->elements[i]) {
-				r_x509_free_algorithmidentifier (dai->elements[i]);
-				// r_x509_free_algorithmidentifier doesn't free the pointer
-				// because on x509 the original use was internal.
-				R_FREE (dai->elements[i]);
-			}
+	for (i = 0; i < dai->length; i++) {
+		if (dai->elements[i]) {
+			r_x509_algorithmidentifier_fini (dai->elements[i]);
+			R_FREE (dai->elements[i]);
 		}
-		R_FREE (dai->elements);
-		// Used internally pkcs #7, so it shouldn't free dai.
 	}
+	R_FREE (dai->elements);
 }
 
-static void r_pkcs7_free_contentinfo(RPKCS7ContentInfo *ci) {
-	if (ci) {
-		r_asn1_binary_free (ci->content);
-		r_asn1_string_free (ci->contentType);
-		// Used internally pkcs #7, so it shouldn't free ci.
-	}
+static void r_pkcs7_contentinfo_fini(RPKCS7ContentInfo *ci) {
+	r_asn1_binary_free (ci->content);
+	r_asn1_string_free (ci->contentType);
 }
 
-static bool r_pkcs7_parse_issuerandserialnumber(RPKCS7IssuerAndSerialNumber *iasu, RASN1Object *object) {
+static bool r_pkcs7_issuerandserialnumber_parse(RPKCS7IssuerAndSerialNumber *iasu, RASN1Object *object) {
 	if (!iasu || !object || object->list.length < 2) {
 		return false;
 	}
-	r_x509_parse_name (&iasu->issuer, object->list.objects[0]);
+	r_x509_name_parse (&iasu->issuer, object->list.objects[0]);
 	RASN1Object *obj1 = object->list.objects[1];
 	if (obj1) {
-		iasu->serialNumber = r_asn1_create_binary (obj1->sector, obj1->length);
+		iasu->serialNumber = r_asn1_binary_new (obj1->sector, obj1->length);
 	}
 	return true;
 }
 
-static void r_pkcs7_free_issuerandserialnumber(RPKCS7IssuerAndSerialNumber *iasu) {
-	if (iasu) {
-		r_x509_free_name (&iasu->issuer);
-		r_asn1_binary_free (iasu->serialNumber);
-		// Used internally pkcs #7, so it shouldn't free iasu.
-	}
+static void r_pkcs7_issuerandserialnumber_fini(RPKCS7IssuerAndSerialNumber *iasu) {
+	r_x509_name_fini (&iasu->issuer);
+	r_asn1_binary_free (iasu->serialNumber);
 }
 
-static bool r_pkcs7_parse_signerinfo(RPKCS7SignerInfo *si, RASN1Object *object) {
-	RASN1Object **elems;
+static RPKCS7Attribute *r_pkcs7_attribute_parse(RASN1Object *object) {
+	RPKCS7Attribute *attribute;
+	if (!object || object->list.length < 1) {
+		return NULL;
+	}
+	attribute = R_NEW0 (RPKCS7Attribute);
+	if (!attribute) {
+		return NULL;
+	}
+	if (object->list.objects[0]) {
+		attribute->oid = r_asn1_stringify_oid (object->list.objects[0]->sector, object->list.objects[0]->length);
+	}
+	if (object->list.length == 2) {
+		RASN1Object *obj1 = object->list.objects[1];
+		if (obj1) {
+			attribute->data = r_asn1_binary_new (obj1->sector, obj1->length);
+		}
+	}
+	return attribute;
+}
+
+static bool r_pkcs7_attributes_parse(RPKCS7Attributes *attributes, RASN1Object *object) {
+	ut32 i;
+	if (!attributes || !object || !object->list.length) {
+		return false;
+	}
+
+	attributes->length = object->list.length;
+	if (attributes->length > 0) {
+		attributes->elements = R_NEWS0 (RPKCS7Attribute *, attributes->length);
+		if (!attributes->elements) {
+			attributes->length = 0;
+			return false;
+		}
+		for (i = 0; i < object->list.length; i++) {
+			attributes->elements[i] = r_pkcs7_attribute_parse (object->list.objects[i]);
+		}
+	}
+	return true;
+}
+
+static bool r_pkcs7_signerinfo_parse(RPKCS7SignerInfo *si, RASN1Object *object) {
 	ut32 shift = 3;
 	if (!si || !object || object->list.length < 5) {
 		return false;
 	}
-	elems = object->list.objects;
-	//Following RFC
+	RASN1Object **elems = object->list.objects;
+	// Following RFC
 	si->version = (ut32)elems[0]->sector[0];
-	r_pkcs7_parse_issuerandserialnumber (&si->issuerAndSerialNumber, elems[1]);
-	r_x509_parse_algorithmidentifier (&si->digestAlgorithm, elems[2]);
+	r_pkcs7_issuerandserialnumber_parse (&si->issuerAndSerialNumber, elems[1]);
+	r_x509_algorithmidentifier_parse (&si->digestAlgorithm, elems[2]);
 	if (shift < object->list.length && elems[shift]->klass == CLASS_CONTEXT && elems[shift]->tag == 0) {
-		r_pkcs7_parse_attributes (&si->authenticatedAttributes, elems[shift]);
+		r_pkcs7_attributes_parse (&si->authenticatedAttributes, elems[shift]);
 		shift++;
 	}
 	if (shift < object->list.length) {
-		r_x509_parse_algorithmidentifier (&si->digestEncryptionAlgorithm, elems[shift]);
+		r_x509_algorithmidentifier_parse (&si->digestEncryptionAlgorithm, elems[shift]);
 		shift++;
 	}
 	if (shift < object->list.length) {
 		RASN1Object *obj1 = object->list.objects[shift];
 		if (obj1) {
-			si->encryptedDigest = r_asn1_create_binary (obj1->sector, obj1->length);
+			si->encryptedDigest = r_asn1_binary_new (obj1->sector, obj1->length);
 			shift++;
 		}
 	}
 	if (shift < object->list.length) {
 		RASN1Object *elem = elems[shift];
 		if (elem && elem->klass == CLASS_CONTEXT && elem->tag == 1) {
-			r_pkcs7_parse_attributes (&si->unauthenticatedAttributes, elems[shift]);
+			r_pkcs7_attributes_parse (&si->unauthenticatedAttributes, elems[shift]);
 		}
 	}
 	return true;
 }
 
-static void r_pkcs7_free_attribute(RPKCS7Attribute *attribute) {
+static void r_pkcs7_attribute_free(RPKCS7Attribute *attribute) {
 	if (attribute) {
 		r_asn1_binary_free (attribute->data);
 		r_asn1_string_free (attribute->oid);
@@ -197,30 +213,30 @@ static void r_pkcs7_free_attribute(RPKCS7Attribute *attribute) {
 	}
 }
 
-static void r_pkcs7_free_attributes(RPKCS7Attributes *attributes) {
+static void r_pkcs7_attributes_fini(RPKCS7Attributes *attributes) {
 	ut32 i;
 	if (attributes) {
 		for (i = 0; i < attributes->length; i++) {
-			r_pkcs7_free_attribute (attributes->elements[i]);
+			r_pkcs7_attribute_free (attributes->elements[i]);
 		}
 		R_FREE (attributes->elements);
 		// Used internally pkcs #7, so it shouldn't free attributes.
 	}
 }
 
-static void r_pkcs7_free_signerinfo(RPKCS7SignerInfo *si) {
+static void r_pkcs7_signerinfo_free(RPKCS7SignerInfo *si) {
 	if (si) {
-		r_pkcs7_free_issuerandserialnumber (&si->issuerAndSerialNumber);
-		r_x509_free_algorithmidentifier (&si->digestAlgorithm);
-		r_pkcs7_free_attributes (&si->authenticatedAttributes);
-		r_x509_free_algorithmidentifier (&si->digestEncryptionAlgorithm);
+		r_pkcs7_issuerandserialnumber_fini (&si->issuerAndSerialNumber);
+		r_x509_algorithmidentifier_fini (&si->digestAlgorithm);
+		r_pkcs7_attributes_fini (&si->authenticatedAttributes);
+		r_x509_algorithmidentifier_fini (&si->digestEncryptionAlgorithm);
 		r_asn1_binary_free (si->encryptedDigest);
-		r_pkcs7_free_attributes (&si->unauthenticatedAttributes);
+		r_pkcs7_attributes_fini (&si->unauthenticatedAttributes);
 		free (si);
 	}
 }
 
-static bool r_pkcs7_parse_signerinfos(RPKCS7SignerInfos *ss, RASN1Object *object) {
+static bool r_pkcs7_signerinfos_parse(RPKCS7SignerInfos *ss, RASN1Object *object) {
 	ut32 i;
 	if (!ss || !object) {
 		return false;
@@ -232,30 +248,28 @@ static bool r_pkcs7_parse_signerinfos(RPKCS7SignerInfos *ss, RASN1Object *object
 		}
 		ss->length = object->list.length;
 		for (i = 0; i < ss->length; i++) {
-			// r_pkcs7_parse_signerinfo returns bool,
+			// r_pkcs7_signerinfo_parse returns bool,
 			// so i have to allocate before calling the function
 			ss->elements[i] = R_NEW0 (RPKCS7SignerInfo);
 			//should i handle invalid memory? the function checks the pointer
 			//or it should return if si->elements[i] == NULL ?
-			r_pkcs7_parse_signerinfo (ss->elements[i], object->list.objects[i]);
+			r_pkcs7_signerinfo_parse (ss->elements[i], object->list.objects[i]);
 		}
 	}
 	return true;
 }
 
-static void r_pkcs7_free_signerinfos(RPKCS7SignerInfos *ss) {
-	ut32 i;
+static void r_pkcs7_signerinfos_fini(RPKCS7SignerInfos *ss) {
 	if (ss) {
+		ut32 i;
 		for (i = 0; i < ss->length; i++) {
-			r_pkcs7_free_signerinfo (ss->elements[i]);
-			ss->elements[i] = NULL;
+			r_pkcs7_signerinfo_free (ss->elements[i]);
 		}
 		R_FREE (ss->elements);
-		// Used internally pkcs #7, so it shouldn't free ss.
 	}
 }
 
-static bool r_pkcs7_parse_signeddata(RPKCS7SignedData *sd, RASN1Object *object) {
+static bool r_pkcs7_signeddata_parse(RPKCS7SignedData *sd, RASN1Object *object) {
 	ut32 shift = 3;
 	if (!sd || !object || object->list.length < 4) {
 		return false;
@@ -264,12 +278,12 @@ static bool r_pkcs7_parse_signeddata(RPKCS7SignedData *sd, RASN1Object *object)
 	RASN1Object **elems = object->list.objects;
 	// Following RFC
 	sd->version = (ut32)elems[0]->sector[0];
-	r_pkcs7_parse_digestalgorithmidentifier (&sd->digestAlgorithms, elems[1]);
-	r_pkcs7_parse_contentinfo (&sd->contentInfo, elems[2]);
+	r_pkcs7_digestalgorithmidentifier_parse (&sd->digestAlgorithms, elems[1]);
+	r_pkcs7_contentinfo_parse (&sd->contentInfo, elems[2]);
 	// Optional
 	if (object->list.length > 3 && shift < object->list.length && elems[shift] &&
 		elems[shift]->klass == CLASS_CONTEXT && elems[shift]->tag == 0) {
-		r_pkcs7_parse_extendedcertificatesandcertificates (&sd->certificates, elems[shift]);
+		r_pkcs7_extendedcertificatesandcertificates_parse (&sd->certificates, elems[shift]);
 		shift++;
 	}
 	// Optional
@@ -279,26 +293,21 @@ static bool r_pkcs7_parse_signeddata(RPKCS7SignedData *sd, RASN1Object *object)
 		shift++;
 	}
 	if (shift < object->list.length) {
-		r_pkcs7_parse_signerinfos (&sd->signerinfos, elems[shift]);
+		r_pkcs7_signerinfos_parse (&sd->signerinfos, elems[shift]);
 	}
 	return true;
 }
 
-static void r_pkcs7_free_signeddata(RPKCS7SignedData *sd) {
-	if (sd) {
-		r_pkcs7_free_digestalgorithmidentifier (&sd->digestAlgorithms);
-		r_pkcs7_free_contentinfo (&sd->contentInfo);
-		r_pkcs7_free_extendedcertificatesandcertificates (&sd->certificates);
-		r_pkcs7_free_certificaterevocationlists (&sd->crls);
-		r_pkcs7_free_signerinfos (&sd->signerinfos);
-		// Used internally pkcs #7, so it shouldn't free sd.
-	}
+static void r_pkcs7_signeddata_fini(RPKCS7SignedData *sd) {
+	r_pkcs7_digestalgorithmidentifier_fini (&sd->digestAlgorithms);
+	r_pkcs7_contentinfo_fini (&sd->contentInfo);
+	r_pkcs7_extendedcertificatesandcertificates_fini (&sd->certificates);
+	r_pkcs7_certificaterevocationlists_fini (&sd->crls);
+	r_pkcs7_signerinfos_fini (&sd->signerinfos);
 }
 
-R_API RCMS *r_pkcs7_parse_cms(const ut8 *buffer, ut32 length) {
-	if (!buffer || !length) {
-		return NULL;
-	}
+R_API RCMS *r_pkcs7_cms_parse(const ut8 *buffer, ut32 length) {
+	R_RETURN_VAL_IF_FAIL (buffer && length, NULL);
 	RCMS *container = R_NEW0 (RCMS);
 	if (!container) {
 		return NULL;
@@ -320,61 +329,20 @@ R_API RCMS *r_pkcs7_parse_cms(const ut8 *buffer, ut32 length) {
 		}
 	}
 	if (object->list.objects[1]) {
-		r_pkcs7_parse_signeddata (&container->signedData, object->list.objects[1]->list.objects[0]);
+		r_pkcs7_signeddata_parse (&container->signedData, object->list.objects[1]->list.objects[0]);
 	}
 	r_asn1_object_free (object);
 	return container;
 }
 
-R_API void r_pkcs7_free_cms(RCMS *container) {
+R_API void r_pkcs7_cms_free(RCMS *container) {
 	if (container) {
 		r_asn1_string_free (container->contentType);
-		r_pkcs7_free_signeddata (&container->signedData);
+		r_pkcs7_signeddata_fini (&container->signedData);
 		free (container);
 	}
 }
 
-static RPKCS7Attribute *r_pkcs7_parse_attribute(RASN1Object *object) {
-	RPKCS7Attribute *attribute;
-	if (!object || object->list.length < 1) {
-		return NULL;
-	}
-	attribute = R_NEW0 (RPKCS7Attribute);
-	if (!attribute) {
-		return NULL;
-	}
-	if (object->list.objects[0]) {
-		attribute->oid = r_asn1_stringify_oid (object->list.objects[0]->sector, object->list.objects[0]->length);
-	}
-	if (object->list.length == 2) {
-		RASN1Object *obj1 = object->list.objects[1];
-		if (obj1) {
-			attribute->data = r_asn1_create_binary (obj1->sector, obj1->length);
-		}
-	}
-	return attribute;
-}
-
-static bool r_pkcs7_parse_attributes(RPKCS7Attributes *attributes, RASN1Object *object) {
-	ut32 i;
-	if (!attributes || !object || !object->list.length) {
-		return false;
-	}
-
-	attributes->length = object->list.length;
-	if (attributes->length > 0) {
-		attributes->elements = R_NEWS0 (RPKCS7Attribute *, attributes->length);
-		if (!attributes->elements) {
-			attributes->length = 0;
-			return false;
-		}
-		for (i = 0; i < object->list.length; i++) {
-			attributes->elements[i] = r_pkcs7_parse_attribute (object->list.objects[i]);
-		}
-	}
-	return true;
-}
-
 #if 0
 // XXX: unused
 static void r_pkcs7_signerinfos_dump(RX509CertificateRevocationList *crl, const char* pad, RStrBuf *sb) {
@@ -445,10 +413,10 @@ static void r_x509_signedinfo_dump(RPKCS7SignerInfo *si, const char *pad, RStrBu
 	s = si->digestEncryptionAlgorithm.algorithm;
 	r_strbuf_appendf (sb, "%sDigest Encryption Algorithm\n%s%s\n", pad2, pad3, s ? s->string : "Missing");
 
-	//	if ((o = si->encryptedDigest)) { s = r_asn1_stringify_bytes (o->binary, o->length);
-	//	} else { s = NULL; }
-	//	eprintf ("%sEncrypted Digest: %u bytes\n%s\n", pad2, o ? o->length : 0, s ? s->string : "Missing");
-	//	r_asn1_string_free (s);
+	// if ((o = si->encryptedDigest)) { s = r_asn1_stringify_bytes (o->binary, o->length);
+	// } else { s = NULL; }
+	// eprintf ("%sEncrypted Digest: %u bytes\n%s\n", pad2, o ? o->length : 0, s ? s->string : "Missing");
+	// r_asn1_string_free (s);
 	r_strbuf_appendf (sb, "%sEncrypted Digest: %u bytes\n", pad2, o ? o->length : 0);
 	r_strbuf_appendf (sb, "%sUnauthenticated Attributes:\n", pad2);
 	for (i = 0; i < si->unauthenticatedAttributes.length; i++) {
@@ -640,7 +608,7 @@ static bool r_pkcs7_parse_spcdata(SpcAttributeTypeAndOptionalValue *data, RASN1O
 	RASN1Object *obj1 = object->list.objects[1];
 	if (object->list.length > 1) {
 		if (obj1) {
-			data->data = r_asn1_create_binary (obj1->sector, obj1->length);
+			data->data = r_asn1_binary_new (obj1->sector, obj1->length);
 		}
 	}
 	return true;
@@ -651,11 +619,11 @@ static bool r_pkcs7_parse_spcmessagedigest(SpcDigestInfo *messageDigest, RASN1Ob
 		!object->list.objects[0] || !object->list.objects[1]) {
 		return false;
 	}
-	if (!r_x509_parse_algorithmidentifier (&messageDigest->digestAlgorithm, object->list.objects[0])) {
+	if (!r_x509_algorithmidentifier_parse (&messageDigest->digestAlgorithm, object->list.objects[0])) {
 		return false;
 	}
 	RASN1Object *obj1 = object->list.objects[1];
-	messageDigest->digest = r_asn1_create_binary (obj1->sector, obj1->length);
+	messageDigest->digest = r_asn1_binary_new (obj1->sector, obj1->length);
 	return true;
 }
 
@@ -700,23 +668,61 @@ beach:
 	return spcinfo;
 }
 
-static void r_pkcs7_free_spcdata(SpcAttributeTypeAndOptionalValue *data) {
-	if (data) {
-		r_asn1_string_free (data->type);
-		r_asn1_binary_free (data->data);
-	}
+static void r_pkcs7_spcdata_fini(SpcAttributeTypeAndOptionalValue *data) {
+	r_asn1_string_free (data->type);
+	r_asn1_binary_free (data->data);
 }
 
-static void r_pkcs7_free_spcmessagedigest(SpcDigestInfo *messageDigest) {
-	if (messageDigest) {
-		r_asn1_binary_free (messageDigest->digest);
-		r_x509_free_algorithmidentifier (&messageDigest->digestAlgorithm);
-	}
+static void r_pkcs7_spcmessagedigest_fini(SpcDigestInfo *messageDigest) {
+	r_asn1_binary_free (messageDigest->digest);
+	r_x509_algorithmidentifier_fini (&messageDigest->digestAlgorithm);
 }
 
-R_API void r_pkcs7_free_spcinfo(SpcIndirectDataContent *spcinfo) {
+R_API void r_pkcs7_spcinfo_free(SpcIndirectDataContent *spcinfo) {
 	if (spcinfo) {
-		r_pkcs7_free_spcdata (&spcinfo->data);
-		r_pkcs7_free_spcmessagedigest (&spcinfo->messageDigest);
+		r_pkcs7_spcdata_fini (&spcinfo->data);
+		r_pkcs7_spcmessagedigest_fini (&spcinfo->messageDigest);
+		free (spcinfo);
 	}
 }
+
+R_API SpcIndirectDataContent *r_pkcs7_spcinfo_parse(RCMS *cms) {
+	R_RETURN_VAL_IF_FAIL (cms, NULL);
+
+	RASN1String *type = cms->signedData.contentInfo.contentType;
+	if (type && strcmp (type->string, "spcIndirectDataContext")) {
+		return NULL;
+	}
+
+	SpcIndirectDataContent *spcinfo = R_NEW0 (SpcIndirectDataContent);
+	if (!spcinfo) {
+		return NULL;
+	}
+
+	RASN1Binary *content = cms->signedData.contentInfo.content;
+	if (!content) {
+		free (spcinfo);
+		return NULL;
+	}
+	RASN1Object *object = r_asn1_object_parse (content->binary, content->binary, content->length, 0);
+	if (!object || object->list.length < 2 || !object->list.objects ||
+		!object->list.objects[0] || !object->list.objects[1]) {
+		R_FREE (spcinfo);
+		goto beach;
+	}
+	if (object->list.objects[0]) {
+		if (!r_pkcs7_parse_spcdata (&spcinfo->data, object->list.objects[0])) {
+			R_FREE (spcinfo);
+			goto beach;
+		}
+	}
+	if (object->list.objects[1]) {
+		if (!r_pkcs7_parse_spcmessagedigest (&spcinfo->messageDigest, object->list.objects[1])) {
+			R_FREE (spcinfo);
+			goto beach;
+		}
+	}
+beach:
+	r_asn1_object_free (object);
+	return spcinfo;
+}
