@@ -135,7 +135,7 @@ static int matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch p
 	}
 
 	/* prescreening; this does wonders for this rather slow code */
-	if (g->must != NULL) {
+	if (g->must) {
 		for (dp = start; dp < stop; dp++) {
 			if (*dp == g->must[0] && stop - dp >= g->mlen && !memcmp (dp, g->must, (size_t)g->mlen)) {
 				break;
@@ -215,8 +215,7 @@ static int matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch p
 					STATETEARDOWN (m);
 					return R_REGEX_ESPACE;
 				}
-				m->lastpos = (char **)malloc ((g->nplus+1) *
-							sizeof(char *));
+				m->lastpos = (char **)calloc (g->nplus + 1, sizeof (char *));
 			}
 			if (g->nplus > 0 && !m->lastpos) {
 				free (m->pmatch);
@@ -236,7 +235,7 @@ static int matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch p
 			break;
 		}
 		for (;;) {
-			if (dp != NULL || endp <= m->coldp) {
+			if (dp || endp <= m->coldp) {
 				break;		/* defeat */
 			}
 			endp = slow (m, m->coldp, endp - 1, gf, gl);
@@ -256,7 +255,7 @@ static int matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch p
 #endif
 			dp = backref (m, m->coldp, endp, gf, gl, (sopno)0, 0);
 		}
-		if (dp != NULL || dp != endp) {	/* found a shorter one */
+		if (dp || dp != endp) {	/* found a shorter one */
 			break;
 		}
 
@@ -287,8 +286,8 @@ static int matcher(struct re_guts *g, char *string, size_t nmatch, RRegexMatch p
 		}
 	}
 
-	free ((char *)m->pmatch);
-	free ((char *)m->lastpos);
+	free (m->pmatch);
+	free (m->lastpos);
 	STATETEARDOWN (m);
 	return 0;
 }
@@ -367,7 +366,7 @@ dissect(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 			ssub = ss + 1;
 			esub = es - 1;
 			/* did innards match? */
-			if (slow(m, sp, rest, ssub, esub) != NULL) {
+			if (slow(m, sp, rest, ssub, esub)) {
 				dp = dissect(m, sp, rest, ssub, esub);
 				if (dp != rest) return NULL;
 			} else if (sp != rest) return NULL;
@@ -378,7 +377,7 @@ dissect(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 			for (;;) {
 				/* how long could this one be? */
 				rest = slow(m, sp, stp, ss, es);
-				if (rest != NULL) {	/* it did match */
+				if (rest) {	/* it did match */
 					/* could the rest match the rest? */
 					tail = slow(m, rest, stop, es, stopst);
 					if (tail == stop)
@@ -483,11 +482,11 @@ dissect(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 
 /*
  - backref - figure out what matched what, figuring in back references
- */
-static char *			/* == stop (success) or NULL (failure) */
-backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
-    sopno lev, int rec)			/* PLUS nesting level */
-{
+== stop (success) or NULL (failure)
+*/
+
+static char *backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst, sopno lev, int rec) {
+	/* PLUS nesting level */
 	int i;
 	sopno ss;	/* start sop of current subRE */
 	char *sp;	/* start of string matched by it */
@@ -523,20 +522,16 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 				return NULL;
 			break;
 		case OBOL:
-			if ( (sp == m->beginp && !(m->eflags&R_REGEX_NOTBOL)) ||
+			if ((sp == m->beginp && !(m->eflags&R_REGEX_NOTBOL)) ||
 					(sp < m->endp && *(sp-1) == '\n' &&
 						(m->g->cflags&R_REGEX_NEWLINE)) )
-				{ /* yes */ }
-			else
-				return NULL;
+			{ /* yes */ } else { return NULL; }
 			break;
 		case OEOL:
-			if ( (sp == m->endp && !(m->eflags&R_REGEX_NOTEOL)) ||
+			if ((sp == m->endp && !(m->eflags&R_REGEX_NOTEOL)) ||
 					(sp < m->endp && *sp == '\n' &&
 						(m->g->cflags&R_REGEX_NEWLINE)) )
-				{ /* yes */ }
-			else
-				return NULL;
+				{ /* yes */ } else { return NULL; }
 			break;
 		case OBOW:
 			if (( (sp == m->beginp && !(m->eflags&R_REGEX_NOTBOL)) ||
@@ -544,20 +539,14 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 						(m->g->cflags&R_REGEX_NEWLINE)) ||
 					(sp > m->beginp && !ISWORD ((unsigned char)*(sp-1))) ) &&
 					(sp < m->endp && ISWORD ((unsigned char)*sp)) )
-				{ /* yes */ }
-			else
-				return NULL;
+				{ /* yes */ } else { return NULL; }
 			break;
 		case OEOW:
-			if (( (sp == m->endp && !(m->eflags&R_REGEX_NOTEOL)) ||
-					(sp < m->endp && *sp == '\n' &&
-						(m->g->cflags&R_REGEX_NEWLINE)) ||
-					(sp < m->endp && !ISWORD ((unsigned char)*sp)) ) &&
-					(sp > m->beginp && ISWORD ((unsigned char)*(sp-1))) )
-				{ /* yes */ }
-			else
-				return NULL;
-			break;
+			if (( (sp == m->endp && !(m->eflags&R_REGEX_NOTEOL))
+				|| (sp < m->endp && *sp == '\n' && (m->g->cflags&R_REGEX_NEWLINE))
+				|| (sp < m->endp && !ISWORD ((unsigned char)*sp)) ) && (sp > m->beginp && ISWORD ((unsigned char)*(sp-1))) )
+				{ /* yes */ } else { return NULL; }
+		       	break;
 		case O_QUEST:
 			break;
 		case OOR1:	/* matches null but needs to skip */
@@ -634,8 +623,9 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 		}
 		for (;;) {	/* find first matching branch */
 			dp = backref(m, sp, stop, ssub, esub, lev, rec);
-			if (dp != NULL)
+			if (dp) {
 				return dp;
+			}
 			/* that one missed, try next one */
 			if (OP (m->g->strip[esub]) == O_CH)
 				return NULL;	/* there is none */
@@ -645,12 +635,13 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 			}
 			ssub = esub + 1;
 			esub += OPND (m->g->strip[esub]);
-			if (OP (m->g->strip[esub]) == OOR2)
+			if (OP (m->g->strip[esub]) == OOR2) {
 				esub--;
-			else
+			} else {
 				if (OP (m->g->strip[esub]) != O_CH) {
 					break;
 				}
+			}
 		}
 		break;
 	case OLPAREN:		/* must undo assignment if rest fails */
@@ -659,10 +650,11 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 			offsave = m->pmatch[i].rm_so;
 			m->pmatch[i].rm_so = sp - m->offp;
 			dp = backref(m, sp, stop, ss+1, stopst, lev, rec);
-			if (dp != NULL)
-				return(dp);
+			if (dp) {
+				return dp;
+			}
 			m->pmatch[i].rm_so = offsave;
-			return(NULL);
+			return NULL;
 		}
 		break;
 	case ORPAREN:		/* must undo assignment if rest fails */
@@ -687,11 +679,9 @@ backref(struct match *m, char *start, char *stop, sopno startst, sopno stopst,
 }
 
 /*
- - fast - step through the string at top speed
+ - fast - step through the string at top speed where tentative match ended, or NULL
  */
-static char *			/* where tentative match ended, or NULL */
-fast(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
-{
+static char * fast(struct match *m, char *start, char *stop, sopno startst, sopno stopst) {
 	states st = m->st;
 	states fresh = m->fresh;
 	states tmp = m->tmp;
@@ -718,12 +708,12 @@ fast(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 		/* is there an EOL and/or BOL between lastc and c? */
 		flagch = '\0';
 		i = 0;
-		if ( (lastc == '\n' && m->g->cflags&R_REGEX_NEWLINE) ||
+		if ((lastc == '\n' && m->g->cflags&R_REGEX_NEWLINE) ||
 				(lastc == OUT && !(m->eflags&R_REGEX_NOTBOL)) ) {
 			flagch = BOL;
 			i = m->g->nbol;
 		}
-		if ( (c == '\n' && m->g->cflags&R_REGEX_NEWLINE) ||
+		if ((c == '\n' && m->g->cflags&R_REGEX_NEWLINE) ||
 				(c == OUT && !(m->eflags&R_REGEX_NOTEOL)) ) {
 			flagch = (flagch == BOL) ? BOLEOL : EOL;
 			i += m->g->neol;
@@ -736,11 +726,11 @@ fast(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 		}
 
 		/* how about a word boundary? */
-		if ( (flagch == BOL || (lastc != OUT && !ISWORD (lastc))) &&
+		if ((flagch == BOL || (lastc != OUT && !ISWORD (lastc))) &&
 					(c != OUT && ISWORD(c)) ) {
 			flagch = BOW;
 		}
-		if ( (lastc != OUT && ISWORD (lastc)) &&
+		if ((lastc != OUT && ISWORD (lastc)) &&
 				(flagch == EOL || (c != OUT && !ISWORD (c))) ) {
 			flagch = EOW;
 		}
@@ -777,11 +767,9 @@ fast(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 }
 
 /*
- - slow - step through the string more deliberately
+ - slow - step through the string more deliberately where it ended
  */
-static char *			/* where it ended */
-slow(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
-{
+static char * slow(struct match *m, char *start, char *stop, sopno startst, sopno stopst) {
 	states st = m->st;
 	states empty = m->empty;
 	states tmp = m->tmp;
@@ -806,12 +794,12 @@ slow(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 		/* is there an EOL and/or BOL between lastc and c? */
 		flagch = '\0';
 		i = 0;
-		if ( (lastc == '\n' && m->g->cflags&R_REGEX_NEWLINE) ||
+		if ((lastc == '\n' && m->g->cflags&R_REGEX_NEWLINE) ||
 				(lastc == OUT && !(m->eflags&R_REGEX_NOTBOL)) ) {
 			flagch = BOL;
 			i = m->g->nbol;
 		}
-		if ( (c == '\n' && m->g->cflags&R_REGEX_NEWLINE) ||
+		if ((c == '\n' && m->g->cflags&R_REGEX_NEWLINE) ||
 				(c == OUT && !(m->eflags&R_REGEX_NOTEOL)) ) {
 			flagch = (flagch == BOL) ? BOLEOL : EOL;
 			i += m->g->neol;
@@ -824,11 +812,11 @@ slow(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 		}
 
 		/* how about a word boundary? */
-		if ( (flagch == BOL || (lastc != OUT && !ISWORD (lastc))) &&
+		if ((flagch == BOL || (lastc != OUT && !ISWORD (lastc))) &&
 					(c != OUT && ISWORD (c)) ) {
 			flagch = BOW;
 		}
-		if ( (lastc != OUT && ISWORD (lastc)) &&
+		if ((lastc != OUT && ISWORD (lastc)) &&
 				(flagch == EOL || (c != OUT && !ISWORD (c))) ) {
 			flagch = EOW;
 		}
@@ -866,13 +854,12 @@ slow(struct match *m, char *start, char *stop, sopno startst, sopno stopst)
 /*
  - step - map set of states reachable before char to set reachable after
  */
-static states
-step(struct re_guts *g,
-    sopno start,		/* start state within strip */
-    sopno stop,			/* state after stop state within strip */
-    states bef,			/* states reachable before */
-    int ch,			/* character or NONCHAR code */
-    states aft)			/* states already known reachable after */
+static states step(struct re_guts *g,
+		sopno start,		/* start state within strip */
+		sopno stop,			/* state after stop state within strip */
+		states bef,			/* states reachable before */
+		int ch,			/* character or NONCHAR code */
+		states aft)			/* states already known reachable after */
 {
 	cset *cs;
 	sop s;
