@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2024 - pancake */
+/* radare - LGPL - Copyright 2009-2025 - pancake */
 
 #include <r_userconf.h>
 #include <stdlib.h>
@@ -32,9 +32,10 @@ extern int backtrace_symbols_fd(void**, size_t, int);
 #include <r_util.h>
 #include <r_lib.h>
 
-static R_TH_LOCAL char** env = NULL;
-static R_TH_LOCAL char *prefix = NULL;
-static R_TH_LOCAL bool unsignable = false;
+static R_TH_LOCAL char** Genv = NULL;
+static R_TH_LOCAL char *Gprefix = NULL;
+static R_TH_LOCAL char *Gr2prefix = NULL;
+static R_TH_LOCAL bool Gunsignable = false; // OK
 
 #if (__linux__ && __GNU_LIBRARY__) || defined(NETBSD_WITH_BACKTRACE) || \
   defined(FREEBSD_WITH_BACKTRACE) || __DragonFly__ || __sun
@@ -42,18 +43,32 @@ static R_TH_LOCAL bool unsignable = false;
 #endif
 #if __APPLE__
 #include <errno.h>
-#ifdef __MAC_10_8
+#include <TargetConditionals.h>
+// iOS don't have this
+#if !TARGET_OS_IPHONE
 #define HAVE_ENVIRON 1
 #else
 #define HAVE_ENVIRON 0
 #endif
 
-#if HAVE_ENVIRON
-#include <execinfo.h>
+// _NSGetEnviron is used on both macOS and iOS paths
+#if defined(__has_include)
+# if __has_include(<crt_externs.h>)
+#  include <crt_externs.h>
+# endif
+#else
+# include <crt_externs.h>
 #endif
-// iOS don't have this we can't hardcode
-// #include <crt_externs.h>
+// Fallback declaration if header is unavailable
+#ifndef _NSGetEnviron
 extern char ***_NSGetEnviron(void);
+#endif
+
+// Provide environ via _NSGetEnviron when available
+#if HAVE_ENVIRON
+#define environ (*_NSGetEnviron())
+#endif
+
 # ifndef PROC_PIDPATHINFO_MAXSIZE
 #  define PROC_PIDPATHINFO_MAXSIZE 1024
 int proc_pidpath(int pid, void * buffer, ut32 buffersize);
@@ -69,7 +84,9 @@ int proc_pidpath(int pid, void * buffer, ut32 buffersize);
 # include <sys/wait.h>
 #endif
 # include <signal.h>
+#ifndef __APPLE__
 extern char **environ;
+#endif
 
 #ifdef __HAIKU__
 # define Sleep sleep
@@ -141,11 +158,12 @@ static const struct {const char* name; ut64 bit;} arch_bit_array[] = {
 	{ "lm32", R_SYS_ARCH_LM32},
 	{ "v850", R_SYS_ARCH_V850},
 	{ "bpf", R_SYS_ARCH_BPF},
+	{ "sbpf", R_SYS_ARCH_SBPF},
 	{NULL, 0}
 };
 
 R_API void r_sys_signable(bool v) {
-	unsignable = !v;
+	Gunsignable = !v;
 }
 
 R_API int r_sys_fork(void) {
@@ -176,7 +194,7 @@ R_API int r_sys_sigaction(int *sig, void(*handler)(int)) {
 #if WANT_DEBUGSTUFF
 	struct sigaction sigact = { };
 	int ret, i;
-	if (unsignable) {
+	if (Gunsignable) {
 		return -1;
 	}
 
@@ -203,7 +221,7 @@ R_API int r_sys_sigaction(int *sig, void(*handler)(int)) {
 }
 #else
 R_API int r_sys_sigaction(int *sig, void(*handler)(int)) {
-	if (unsignable) {
+	if (Gunsignable) {
 		return -1;
 	}
 	if (!sig) {
@@ -333,6 +351,17 @@ R_API ut8 *r_sys_unxz(const ut8 *buf, size_t len, size_t *olen) {
 #define HAVE_BACKTRACE 1
 #endif
 
+// Ensure backtrace() declarations are visible on Apple when supported
+#if defined(__APPLE__) && defined(APPLE_WITH_BACKTRACE)
+# if defined(__has_include)
+#  if __has_include(<execinfo.h>)
+#   include <execinfo.h>
+#  endif
+# else
+// Older SDKs may not have execinfo.h; include only when available above
+# endif
+#endif
+
 R_API void r_sys_backtrace(void) {
 #if WANT_DEBUGSTUFF
 #ifdef HAVE_BACKTRACE
@@ -399,34 +428,42 @@ R_API int r_sys_usleep(int usecs) {
 #endif
 }
 
-R_API int r_sys_clearenv(void) {
+R_API bool r_sys_clearenv(void) {
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_ENVIRON)) {
+		return false;
+	}
 #if R2__UNIX__
 #if __APPLE__ && !HAVE_ENVIRON
 	/* do nothing */
-	if (!env) {
+	if (!Genv) {
 		r_sys_env_init ();
-		return 0;
+		return true;
 	}
-	char **e = env;
-	while (*e) {
-		*e++ = NULL;
+	char **e = Genv;
+	if (e) {
+		while (*e) {
+			*e++ = NULL;
+		}
 	}
 #else
 	if (!environ) {
-		return 0;
+		return false;
 	}
 	while (*environ) {
 		*environ++ = NULL;
 	}
 #endif
-	return 0;
+	return true;
 #else
 #pragma message ("r_sys_clearenv : unimplemented for this platform")
-	return 0;
+	return false;
 #endif
 }
 
 R_API int r_sys_setenv(const char *key, const char *value) {
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_ENVIRON)) {
+		return -1;
+	}
 	if (!key) {
 		return 0;
 	}
@@ -437,6 +474,10 @@ R_API int r_sys_setenv(const char *key, const char *value) {
 	}
 	return setenv (key, value, 1);
 #elif R2__WINDOWS__
+	if (!value) {
+		SetEnvironmentVariableA(key, NULL);
+		return 0;
+	}
 	LPTSTR key_ = r_sys_conv_utf8_to_win (key);
 	LPTSTR value_ = r_sys_conv_utf8_to_win (value);
 	int ret = SetEnvironmentVariable (key_, value_);
@@ -452,6 +493,68 @@ R_API int r_sys_setenv(const char *key, const char *value) {
 #endif
 }
 
+R_API int r_sys_setenv2(const char *key, const ut8 *value, size_t len) {
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_ENVIRON)) {
+		return -1;
+	}
+	if (!key) {
+		return 0;
+	}
+	if (!value) {
+		r_sys_setenv (key, NULL);
+		return 0;
+	}
+	ut8 *buf = malloc (len + 1);
+	ut8 *zeroes = calloc (1, len);
+	memcpy (buf, value, len);
+	size_t i = 0;
+	bool nullbytes = false;
+	for (i = 0; i < len; i++) {
+		if (!buf[i]) {
+			buf[i] = 'X';
+			zeroes[i] = 1;
+			nullbytes = true;
+		}
+	}
+	buf[len] = 0;
+	if (r_sys_setenv (key, (char *)buf) != 0) {
+		free (zeroes);
+		free (buf);
+		return -1;
+	}
+	if (nullbytes) {
+		R_LOG_WARN ("Environment corrupted after null bytes injected via r_sys_setenv2");
+	}
+	char *p = getenv (key);
+	for (i = 0; i < len; i++) {
+		if (zeroes[i]) {
+			p[i] = 0;
+		}
+	}
+	free (zeroes);
+	free (buf);
+	return 0;
+}
+
+R_API int r_sys_setenv_sep(const char *key, const char *value, bool prefix) {
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_ENVIRON)) {
+		return false;
+	}
+	char *o = r_sys_getenv (key);
+	if (R_STR_ISEMPTY (o)) {
+		int res = r_sys_setenv (key, value);
+		free (o);
+		return res;
+	}
+	char *v = prefix
+		? r_str_newf ("%s" R_SYS_ENVSEP "%s", o, value)
+		: r_str_newf ("%s" R_SYS_ENVSEP "%s", value, o);
+	int res = r_sys_setenv (key, v);
+	free (v);
+	free (o);
+	return res;
+}
+
 #if WANT_DEBUGSTUFF
 #if R2__UNIX__
 static char *crash_handler_cmd = NULL;
@@ -514,6 +617,9 @@ R_API bool r_sys_crash_handler(const char *cmd) {
 #endif
 
 R_API char *r_sys_getenv(const char *key) {
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_ENVIRON)) {
+		return false;
+	}
 #if R2__WINDOWS__
 	DWORD dwRet;
 	LPTSTR envbuf = NULL, key_ = NULL, tmp_ptr;
@@ -558,6 +664,18 @@ err_r_sys_get_env:
 #endif
 }
 
+R_API void r_sys_setenv_asbool(const char *key, bool v) {
+	R_RETURN_IF_FAIL (key);
+	r_sys_setenv (key, v? "1": "0");
+}
+
+R_API void r_sys_setenv_asut64(const char *key, ut64 n) {
+	R_RETURN_IF_FAIL (key);
+	char *env = r_str_newf ("%"PFMT64d, n);
+	r_sys_setenv (key, env);
+	free (env);
+}
+
 R_API bool r_sys_getenv_asbool(const char *key) {
 	R_RETURN_VAL_IF_FAIL (key, false);
 	char *env = r_sys_getenv (key);
@@ -724,7 +842,7 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, int ilen, char
 		}
 		close (sh_err[1]);
 		close (sh_in[0]);
-		if (!inputptr || !*inputptr) {
+		if (R_STR_ISEMPTY (inputptr)) {
 			close (sh_in[1]);
 		}
 		// we should handle broken pipes somehow better
@@ -746,10 +864,12 @@ R_API int r_sys_cmd_str_full(const char *cmd, const char *input, int ilen, char
 				FD_SET (sh_in[1], &wfds);
 			}
 			memset (buffer, 0, sizeof (buffer));
+
 			nfd = select (sh_err[0] + 1, &rfds, &wfds, NULL, NULL);
-			if (nfd < 0) {
-				// eprintf ("nfd %d 2%c", nfd, 10);
+			if (nfd < 0 && errno == EINTR) {
+				continue;
 			}
+
 			if (output && FD_ISSET (sh_out[0], &rfds)) {
 				if ((bytes = read (sh_out[0], buffer, sizeof (buffer))) < 1) {
 					break;
@@ -1024,6 +1144,9 @@ R_API const char *r_sys_arch_str(int arch) {
 
 #define USE_FORK 0
 R_API int r_sys_run(const ut8 *buf, int len) {
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_EXEC)) {
+		return -1;
+	}
 	const int sz = 4096;
 	int pdelta, ret, (*cb)();
 // TODO: define R_SYS_ALIGN_FORWARD in r_util.h
@@ -1067,7 +1190,11 @@ R_API int r_sys_run(const ut8 *buf, int len) {
 	return ret;
 }
 
+// TODO. maybe this should be moved into socket/run?
 R_API int r_sys_run_rop(const ut8 *buf, int len) {
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_EXEC)) {
+		return -1;
+	}
 #if USE_FORK
 	int st;
 #endif
@@ -1260,6 +1387,9 @@ R_API char *r_sys_pid_to_path(int pid) {
 }
 
 R_API void r_sys_env_init(void) {
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_ENVIRON)) {
+		return;
+	}
 	char **envp = r_sys_get_environ ();
 	if (envp) {
 		r_sys_set_environ (envp);
@@ -1267,20 +1397,26 @@ R_API void r_sys_env_init(void) {
 }
 
 R_API char **r_sys_get_environ(void) {
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_ENVIRON)) {
+		return NULL;
+	}
 #if __APPLE__ && !HAVE_ENVIRON
-	env = *_NSGetEnviron();
+	Genv = *_NSGetEnviron();
 #else
-	env = environ;
+	Genv = environ;
 #endif
 	// return environ if available??
-	if (!env) {
-		env = r_lib_dl_sym (NULL, "environ");
+	if (!Genv) {
+		Genv = r_lib_dl_sym (NULL, "environ");
 	}
-	return env;
+	return Genv;
 }
 
 R_API void r_sys_set_environ(char **e) {
-	env = e;
+	if (!r_sandbox_check (R_SANDBOX_GRAIN_ENVIRON)) {
+		return;
+	}
+	Genv = e;
 }
 
 R_API char *r_sys_whoami(void) {
@@ -1358,32 +1494,29 @@ R_API bool r_sys_tts(const char *txt, bool bg) {
 	return false;
 }
 
-// leaks and globs
-static R_TH_LOCAL char *r2_prefix = NULL;
-
 R_API const char *r_sys_prefix(const char *pfx) {
-	if (!r2_prefix) {
-		r2_prefix = r_sys_getenv ("R2_PREFIX");
-		if (R_STR_ISEMPTY (r2_prefix)) {
-			free (r2_prefix);
-			r2_prefix = strdup (R2_PREFIX);
+	if (!Gr2prefix) {
+		Gr2prefix = r_sys_getenv ("R2_PREFIX");
+		if (R_STR_ISEMPTY (Gr2prefix)) {
+			free (Gr2prefix);
+			Gr2prefix = strdup (R2_PREFIX);
 		}
 	}
-	if (!prefix) {
+	if (!Gprefix) {
 #if R2__WINDOWS__
-		prefix = r_sys_get_src_dir_w32 ();
-		if (!prefix) {
-			prefix = strdup (r2_prefix);
+		Gprefix = r_sys_get_src_dir_w32 ();
+		if (!Gprefix) {
+			Gprefix = strdup (Gr2prefix);
 		}
 #else
-		prefix = strdup (r2_prefix);
+		Gprefix = strdup (Gr2prefix);
 #endif
 	}
 	if (pfx) {
-		free (prefix);
-		prefix = strdup (pfx);
+		free (Gprefix);
+		Gprefix = strdup (pfx);
 	}
-	return prefix;
+	return Gprefix;
 }
 
 R_API RSysInfo *r_sys_info(void) {
@@ -1391,14 +1524,12 @@ R_API RSysInfo *r_sys_info(void) {
 	struct utsname un = {{0}};
 	if (uname (&un) != -1) {
 		RSysInfo *si = R_NEW0 (RSysInfo);
-		if (si) {
-			si->sysname  = strdup (un.sysname);
-			si->nodename = strdup (un.nodename);
-			si->release  = strdup (un.release);
-			si->version  = strdup (un.version);
-			si->machine  = strdup (un.machine);
-			return si;
-		}
+		si->sysname  = strdup (un.sysname);
+		si->nodename = strdup (un.nodename);
+		si->release  = strdup (un.release);
+		si->version  = strdup (un.version);
+		si->machine  = strdup (un.machine);
+		return si;
 	}
 #elif R2__WINDOWS__
 	HKEY key;
@@ -1408,10 +1539,6 @@ R_API RSysInfo *r_sys_info(void) {
 	DWORD minor;
 	char tmp[256] = {0};
 	RSysInfo *si = R_NEW0 (RSysInfo);
-	if (!si) {
-		return NULL;
-	}
-
 	if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", 0,
 		KEY_QUERY_VALUE, &key) != ERROR_SUCCESS) {
 		r_sys_perror ("r_sys_info/RegOpenKeyExA");
