@@ -1,8 +1,8 @@
-/* radare - LGPL - Copyright 2019-2024 - pancake */
+/* radare - LGPL - Copyright 2019-2025 - pancake */
 
 #include <r_util/r_table.h>
 #include <r_core.h>
-#include "r_cons.h"
+#include <r_cons.h>
 
 #define READ_SHOW_FLAG(t, bitflag) (((t)->showMode & bitflag) != 0)
 #define WRITE_SHOW_FLAG(t, bitflag, condition) \
@@ -40,13 +40,11 @@ static R_TH_LOCAL RListComparator Gcmp = NULL;
 
 R_API RListInfo *r_listinfo_new(const char *name, RInterval pitv, RInterval vitv, int perm, const char *extra) {
 	RListInfo *info = R_NEW (RListInfo);
-	if (info) {
-		info->name = name ? strdup (name) : NULL;
-		info->pitv = pitv;
-		info->vitv = vitv;
-		info->perm = perm;
-		info->extra = extra ? strdup (extra) : NULL;
-	}
+	info->name = name ? strdup (name) : NULL;
+	info->pitv = pitv;
+	info->vitv = vitv;
+	info->perm = perm;
+	info->extra = extra ? strdup (extra) : NULL;
 	return info;
 }
 
@@ -56,8 +54,10 @@ static void r_listinfo_fini(RListInfo *info) {
 }
 
 R_API void r_listinfo_free(RListInfo *info) {
-	r_listinfo_fini (info);
-	free (info);
+	if (info) {
+		r_listinfo_fini (info);
+		free (info);
+	}
 }
 static int sortString(const void *a, const void *b) {
 	return strcmp (a, b);
@@ -70,7 +70,7 @@ static int sortNumber(const void *a, const void *b) {
 static int sortFloat(const void *a, const void *b) {
 	double fa = strtod ((const char *) a, NULL);
 	double fb = strtod ((const char *) b, NULL);
-	return (fa * 100) - (fb * 100);
+	return (int)((fa * 100) - (fb * 100));
 }
 
 // maybe just index by name instead of exposing those symbols as global
@@ -95,7 +95,6 @@ R_API RTableColumnType *r_table_type(const char *name) {
 	return NULL;
 }
 
-// TODO: unused for now, maybe good to call after filter :?
 static void __table_adjust(RTable *t) {
 	RListIter *iter, *iter2;
 	RTableColumn *col;
@@ -112,6 +111,9 @@ static void __table_adjust(RTable *t) {
 			RTableColumn *c = r_list_get_n (t->cols, ncol);
 			if (c) {
 				c->width = R_MAX (c->width, itemLength);
+				if (t->maxColumnWidth > 0) {
+					c->width = R_MIN (t->maxColumnWidth, c->width);
+				}
 			}
 			ncol ++;
 		}
@@ -125,9 +127,11 @@ R_API void r_table_row_free(void *_row) {
 }
 
 R_API void r_table_column_free(void *_col) {
-	RTableColumn *col = _col;
-	free (col->name);
-	free (col);
+	if (_col) {
+		RTableColumn *col = _col;
+		free (col->name);
+		free (col);
+	}
 }
 
 R_API RTableRow *r_table_row_clone(RTableRow *row) {
@@ -141,10 +145,8 @@ R_API RTableRow *r_table_row_clone(RTableRow *row) {
 }
 
 R_API RTableColumn *r_table_column_clone(RTableColumn *col) {
+	R_RETURN_VAL_IF_FAIL (col, NULL);
 	RTableColumn *c = R_NEW0 (RTableColumn);
-	if (!c) {
-		return NULL;
-	}
 	memcpy (c, col, sizeof (*c));
 	c->name = strdup (c->name);
 	return c;
@@ -152,40 +154,44 @@ R_API RTableColumn *r_table_column_clone(RTableColumn *col) {
 
 R_API RTable *r_table_new(const char *name) {
 	RTable *t = R_NEW0 (RTable);
-	if (t) {
-		t->name = strdup (name);
-		t->cols = r_list_newf (r_table_column_free);
-		t->rows = r_list_newf (r_table_row_free);
-		SET_SHOW_HEADER (t, true);
-		SET_SHOW_SUM (t, false);
-	}
+	t->name = strdup (name);
+	t->cols = r_list_newf (r_table_column_free);
+	t->rows = r_list_newf (r_table_row_free);
+	t->maxColumnWidth = 32;
+	t->wrapColumns = false;
+	SET_SHOW_HEADER (t, true);
+	SET_SHOW_SUM (t, false);
 	return t;
 }
 
+R_API void r_table_set_width(RTable *t, int maxColumnWidth, bool wrap) {
+	R_RETURN_IF_FAIL (t);
+	t->maxColumnWidth = maxColumnWidth;
+	t->wrapColumns = wrap;
+}
+
 R_API void r_table_free(RTable *t) {
-	if (!t) {
-		return;
+	if (R_LIKELY (t)) {
+		r_list_free (t->cols);
+		r_list_free (t->rows);
+		free (t->name);
+		free (t);
 	}
-	r_list_free (t->cols);
-	r_list_free (t->rows);
-	free (t->name);
-	free (t);
 }
 
 R_API void r_table_add_column(RTable *t, RTableColumnType *type, const char *name, int maxWidth) {
 	RTableColumn *c = R_NEW0 (RTableColumn);
-	if (c) {
-		c->name = strdup (name);
-		c->maxWidth = maxWidth;
-		c->type = type;
-		int itemLength = r_str_len_utf8_ansi (name) + 1;
-		c->width = itemLength;
-		r_list_append (t->cols, c);
-		c->total = -1;
-	}
+	c->name = strdup (name);
+	c->maxWidth = maxWidth;
+	c->type = type;
+	int itemLength = r_str_len_utf8_ansi (name) + 1;
+	c->width = itemLength;
+	r_list_append (t->cols, c);
+	c->total = -1;
 }
 
 R_API RTableRow *r_table_row_new(RList *items) {
+	R_RETURN_VAL_IF_FAIL (items, NULL);
 	RTableRow *row = R_NEW (RTableRow);
 	row->items = items;
 	return row;
@@ -202,15 +208,31 @@ static bool __addRow(RTable *t, RList *items, const char *arg, int col) {
 	return false;
 }
 
+static void wrap_items(RTable *t, RList *items) {
+	if (t->wrapColumns && t->maxColumnWidth > 0) {
+		char *item;
+		RListIter *iter;
+		r_list_foreach (items, iter, item) {
+			int itemLength = r_str_len_utf8_ansi (item);
+			if (itemLength + 3 > t->maxColumnWidth) {
+				char *p = (char *)r_str_ansi_chrn (item, t->maxColumnWidth - 3);
+				strcpy (p, "..");
+			}
+		}
+	}
+}
+
 R_API void r_table_add_row_list(RTable *t, RList *items) {
 	R_RETURN_IF_FAIL (t && items);
 	RTableRow *row = r_table_row_new (items);
+	wrap_items (t, items);
 	r_list_append (t->rows, row);
 	// throw warning if not enough columns defined in header
 	t->totalCols = R_MAX (t->totalCols, r_list_length (items));
 }
 
 R_API void r_table_set_columnsf(RTable *t, const char *fmt, ...) {
+	R_RETURN_IF_FAIL (t && fmt);
 	va_list ap;
 	va_start (ap, fmt);
 	RTableColumnType *typeString = r_table_type ("string");
@@ -323,6 +345,7 @@ R_API void r_table_add_row(RTable *t, const char *name, ...) {
 		col++;
 	}
 	va_end (ap);
+	wrap_items (t, items);
 	RTableRow *row = r_table_row_new (items);
 	r_list_append (t->rows, row);
 	// throw warning if not enough columns defined in header
@@ -475,9 +498,7 @@ static int __strbuf_append_col_aligned(RStrBuf *sb, RTableColumn *col, const cha
 }
 
 R_API char *r_table_tostring(RTable *t) {
-	if (!t) { // guard
-		return strdup ("");
-	}
+	R_RETURN_VAL_IF_FAIL (t, NULL);
 	if (SHOULD_SHOW_R2 (t)) {
 		return r_table_tor2cmds (t);
 	}
@@ -563,7 +584,8 @@ R_API char *r_table_tosimplestring(RTable *t) {
 		}
 		r_list_foreach (t->cols, iter, col) {
 			bool nopad = !iter->n;
-			(void)__strbuf_append_col_aligned (sb, col, sdb_itoa (col->total, 10, tmp, sizeof (tmp)), nopad);
+			char *num = col->total == -1 ? "" : sdb_itoa (col->total, 10, tmp, sizeof (tmp));
+			(void)__strbuf_append_col_aligned (sb, col, num, nopad);
 		}
 	}
 	return r_strbuf_drain (sb);
@@ -877,7 +899,7 @@ R_API void r_table_sort(RTable *t, int nth, bool dec) {
 		Gnth = nth;
 		if (col->type && col->type->cmp) {
 			Gcmp = col->type->cmp;
-			t->rows->sorted = false; //force sorting
+			t->rows->sorted = false; // force sorting
 			r_list_sort (t->rows, cmp);
 			if (dec) {
 				r_list_reverse (t->rows);
@@ -1219,7 +1241,7 @@ R_API bool r_table_query(RTable *t, const char *q) {
 	RList *queries = r_str_split_list (qq, ",",  0);
 	char *query;
 	r_list_foreach (queries, iter, query) {
-		RList *q = r_str_split_list (query, "/", 2);
+		RList *q = r_str_split_list (query, "/", 3);
 		const char *columnName = r_list_get_n (q, 0);
 		const char *operation = r_list_get_n (q, 1);
 		const char *operand = r_list_get_n (q, 2);
