@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2024 - pancake */
+/* radare - LGPL - Copyright 2007-2025 - pancake */
 
 #define R_LOG_ORIGIN "util.num"
 
@@ -69,7 +69,7 @@ R_API void r_num_irand(void) {
 }
 
 R_API int r_num_rand(int max) {
-	static R_TH_LOCAL bool rand_initialized = false;
+	static R_TH_LOCAL bool rand_initialized = false; // OK
 	if (!rand_initialized) {
 		r_num_irand ();
 		rand_initialized = true;
@@ -98,12 +98,10 @@ R_API void r_num_minmax_swap_i(int *a, int *b) {
 
 R_API RNum *r_num_new(RNumCallback cb, RNumCallback2 cb2, void *ptr) {
 	RNum *num = R_NEW0 (RNum);
-	if (num) {
-		num->value = 0LL;
-		num->callback = cb;
-		num->cb_from_value = cb2;
-		num->userptr = ptr;
-	}
+	num->value = 0LL;
+	num->callback = cb;
+	num->cb_from_value = cb2;
+	num->userptr = ptr;
 	return num;
 }
 
@@ -129,7 +127,11 @@ R_API void r_num_free(RNum *num) {
  * insufficient memory was available.
  */
 R_API char *r_num_units(char *buf, size_t len, ut64 num) {
+#if R2_NO_LONG_DOUBLE
+	double fnum;
+#else
 	long double fnum;
+#endif
 	char unit;
 	const char *fmt_str;
 	if (!buf) {
@@ -139,7 +141,11 @@ R_API char *r_num_units(char *buf, size_t len, ut64 num) {
 			return NULL;
 		}
 	}
+#if R2_NO_LONG_DOUBLE
+	fnum = (double)num;
+#else
 	fnum = (long double)num;
+#endif
 	if (num >= EB) {
 		unit = 'E'; fnum /= EB;
 	} else if (num >= PB) {
@@ -155,9 +161,15 @@ R_API char *r_num_units(char *buf, size_t len, ut64 num) {
 	} else {
 		unit = '\0';
 	}
+#if R2_NO_LONG_DOUBLE
+	fmt_str = (ceil (fnum) == fnum)
+		? "%.0" LDBLFMT "%c"
+		: "%.1" LDBLFMT "%c";
+#else
 	fmt_str = ((double)ceill (fnum) == (double)fnum)
 		? "%.0" LDBLFMT "%c"
 		: "%.1" LDBLFMT "%c";
+#endif
 	snprintf (buf, len, fmt_str, fnum, unit);
 	return buf;
 }
@@ -165,9 +177,9 @@ R_API char *r_num_units(char *buf, size_t len, ut64 num) {
 R_API const char *r_num_get_name(RNum *num, ut64 n) {
 	R_RETURN_VAL_IF_FAIL (num, NULL);
 	if (num->cb_from_value) {
-		int ok = 0;
+		bool ok = false;
 		const char *msg = num->cb_from_value (num, n, &ok);
-		if (msg && *msg) {
+		if (R_STR_ISNOTEMPTY (msg)) {
 			return msg;
 		}
 		if (ok) {
@@ -177,7 +189,7 @@ R_API const char *r_num_get_name(RNum *num, ut64 n) {
 	return NULL;
 }
 
-static void error(R_NULLABLE RNum *num, const char *err_str) {
+static void error(RNum * R_NULLABLE num, const char *err_str) {
 	if (num) {
 		if (err_str) {
 			num->nc.errors++;
@@ -234,8 +246,8 @@ R_API ut64 r_num_from_ternary(const char *inp) {
 
 // TODO: try to avoid the use of sscanf
 /* old get_offset */
-R_API ut64 r_num_get(R_NULLABLE RNum *num, const char *str) {
-	int i, j, ok;
+R_API ut64 r_num_get(RNum * R_NULLABLE num, const char *str) {
+	int i, j;
 	char lch;
 	ut64 ret = 0LL;
 	ut32 s, a;
@@ -260,7 +272,7 @@ R_API ut64 r_num_get(R_NULLABLE RNum *num, const char *str) {
 	}
 	/* resolve string with an external callback */
 	if (num && num->callback) {
-		ok = 0;
+		bool ok = false;
 		ret = num->callback (num->userptr, str, &ok);
 		if (ok) {
 			return ret;
@@ -353,7 +365,7 @@ R_API ut64 r_num_get(R_NULLABLE RNum *num, const char *str) {
 			break;
 		case 'b': // binary
 			ret = 0;
-			ok = true;
+			bool ok = true;
 			if (strlen (str) <= 65) { // 64 bit + the 'b' suffix
 				for (j = 0, i = strlen (str) - 2; i >= 0; i--, j++) {
 					if (str[i] == '1') {
@@ -445,14 +457,29 @@ R_API ut64 r_num_get(R_NULLABLE RNum *num, const char *str) {
 				error (num, "invalid gigabyte number");
 			}
 			break;
+#if 0
+		case 's':
+			// support 's' for miliseconds to seconds
+			if (isdigit (*str)) {
+				if (sscanf (str, "%"PFMT64d"%n", &ret, &chars_read)) {
+					ret *= 1000;
+				} else {
+					error (num, "invalid seconds");
+				}
+			}
+			break;
+#endif
 		default:
 			errno = 0;
-			ret = strtoull (str, &endptr, 10);
-			if (errno == ERANGE) {
-				error (num, "number won't fit into 64 bits");
-			}
-			if (!isdigit (*str)) {
+			if (*str != '-' && !isdigit (*str)) {
 				error (num, "unknown symbol");
+			} else {
+				ret = strtoull (str, &endptr, 10);
+				if (errno == EINVAL) {
+					error (num, "invalid symbol");
+				} else if (errno == ERANGE) {
+					error (num, "number won't fit into 64 bits");
+				}
 			}
 			break;
 		}
@@ -463,18 +490,6 @@ R_API ut64 r_num_get(R_NULLABLE RNum *num, const char *str) {
 	return ret;
 }
 
-R_API ut64 r_num_math(RNum *num, const char *str) {
-	const char *err = NULL;
-	if (R_STR_ISEMPTY (str)) {
-		return 0LL;
-	}
-	ut64 ret = r_num_calc (num, str, &err); // TODO: rename r_num_calc to r_num_math_err()
-	if (err) {
-		R_LOG_DEBUG ("(%s) in (%s)", err, str);
-	}
-	return ret;
-}
-
 R_API int r_num_is_float(RNum *num, const char *str) {
 	return (isdigit (*str) && (strchr (str, '.') || str[strlen (str) - 1] == 'f'));
 }
@@ -546,7 +561,7 @@ R_API ut64 r_num_chs(int cylinder, int head, int sector, int sectorsize) {
 	return (ut64)cylinder * (ut64)head * (ut64)sector * (ut64)sectorsize;
 }
 
-R_API int r_num_conditional(R_NULLABLE RNum *num, const char *str) {
+R_API int r_num_conditional(RNum * R_NULLABLE num, const char *str) {
 	char *lgt, *t, *p, *s = strdup (str);
 	int res = 0;
 	ut64 n, a, b;
@@ -624,7 +639,7 @@ R_API int r_num_is_valid_input(RNum *num, const char *input_value) {
 	return !(value == 0 && input_value && *input_value != '0') || !(value == 0 && input_value && *input_value != '@');
 }
 
-R_API ut64 r_num_get_input_value(R_NULLABLE RNum *num, const char *input_value) {
+R_API ut64 r_num_get_input_value(RNum * R_NULLABLE num, const char *input_value) {
 	ut64 value = input_value ? r_num_math (num, input_value) : 0;
 	return value;
 }
@@ -679,7 +694,7 @@ R_API char* r_num_as_string(RNum *___, ut64 n, bool printable_only) {
 	return NULL;
 }
 
-R_API bool r_is_valid_input_num_value(R_NULLABLE RNum *num, const char *input_value) {
+R_API bool r_is_valid_input_num_value(RNum * R_NULLABLE num, const char *input_value) {
 	if (!input_value) {
 		return false;
 	}
@@ -911,3 +926,16 @@ R_API char *r_num_list_join(RList *str, const char *sep) {
 	}
 	return r_strbuf_drain (sb);
 }
+
+/* Returns the number that has bits + 1 least significant bits set. */
+R_API ut64 r_num_genmask(int bits) {
+	ut64 m = UT64_MAX;
+	if (bits > 0 && bits < 64) {
+		m = (ut64)(((ut64)(2) << bits) - 1);
+		if (!m) {
+			m = UT64_MAX;
+		}
+	}
+	return m;
+}
+
