@@ -289,7 +289,8 @@ add_data(zip_t *za, zip_source_t *src, zip_dirent_t *de, zip_uint32_t changed) {
     zip_int64_t offstart, offdata, offend, data_length;
     zip_stat_t st;
     zip_file_attributes_t attributes;
-    zip_source_t *src_final, *src_tmp;
+    zip_source_t *src_final = NULL;
+    zip_source_t *src_tmp = NULL;
     int ret;
     int is_zip64;
     zip_flags_t flags;
@@ -435,6 +436,7 @@ add_data(zip_t *za, zip_source_t *src, zip_dirent_t *de, zip_uint32_t changed) {
 
 
     if (needs_encrypt) {
+#if 0
         zip_encryption_implementation impl = NULL;
         const char *password = NULL;
 
@@ -444,8 +446,6 @@ add_data(zip_t *za, zip_source_t *src, zip_dirent_t *de, zip_uint32_t changed) {
         else if (za->default_password) {
             password = za->default_password;
         }
-
-#if 0
         if ((impl = _zip_get_encryption_implementation(de->encryption_method, ZIP_CODEC_ENCODE)) == NULL) {
             zip_error_set(&za->error, ZIP_ER_ENCRNOTSUPP, 0);
             zip_source_free(src_final);
@@ -473,11 +473,13 @@ add_data(zip_t *za, zip_source_t *src, zip_dirent_t *de, zip_uint32_t changed) {
 #endif
         }
 
-        if ((src_tmp = impl(za, src_final, de->encryption_method, ZIP_CODEC_ENCODE, password)) == NULL) {
+#if 0
+        if (impl && (src_tmp = impl(za, src_final, de->encryption_method, ZIP_CODEC_ENCODE, password)) == NULL) {
             /* error set by impl */
             zip_source_free(src_final);
             return -1;
         }
+#endif
 
         zip_source_free(src_final);
         src_final = src_tmp;
