@@ -2,6 +2,6 @@
 
 rm -rf shlr/capstone
 make mrproper
-cp -f plugins.static.nogpl.cfg plugins.cfg
+cp -f dist/plugins-cfg/plugins.static.nogpl.cfg plugins.cfg
 ./configure --prefix=/usr --with-libr
 make -j4
