@@ -7,9 +7,11 @@ if [ -z "$a" ]; then
 fi
 #[ -z "$a" ] && a="r_str_chop "
 #[ -z "$b" ] && b="r_str_trim "
-git grep "$a" libr binr | less -R
+WHERE="libr binr shlr mk sys"
+# WHERE="mk sys"
+git grep "$a" ${WHERE} | less -R
 if [ -n "$b" ]; then
-	FILES=`git grep "$a" libr binr | cut -d : -f 1 | uniq `
+	FILES=`git grep "$a" ${WHERE} | cut -d : -f 1 | uniq `
 	for f in $FILES ; do
 		echo "Sedded $a"
 		sed -e "s/$a/$b/g" < $f > $f.sub
