@@ -5,12 +5,15 @@
 import sys
 
 full_version = False
+no_newline = False
 
 for arg in sys.argv[1:]:
     if arg == '-h' or arg == '--help':
         print('Only flag accepted here is --full-version')
         sys.exit(0)
-    if arg == '--full-version':
+    elif arg == '-n':
+        no_newline = True
+    elif arg == '--full-version':
         full_version = True
     else:
         print('Option %s not supported' % (arg,))
@@ -20,7 +23,10 @@ r2root=os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
 with open(os.path.join(r2root, 'configure.acr'), 'r') as f:
     f.readline()
     version = f.readline().split()[1]
-    sys.stdout.write(version + '\n')
+    if no_newline:
+        sys.stdout.write(version)
+    else:
+        sys.stdout.write(version + '\n')
     if full_version:
         versions = version.split('.')
         version_major = versions[0] if len(versions) > 0 else 0
