@@ -33,7 +33,7 @@ bool test_graph_to_agraph(void) {
 		.get_title = _graph_node_info_get_title,
 		.get_body = _graph_node_info_get_body
 	};
-	RAGraph *agraph = r_agraph_new_from_graph (graph, &cbs, NULL);
+	RAGraph *agraph = r_agraph_new_from_graph (core, graph, &cbs, NULL);
 	mu_assert_notnull (agraph, "Couldn't create the graph");
 	mu_assert_eq (agraph->graph->nodes->length, 4, "Wrong node count");
 
