@@ -84,14 +84,16 @@ bool test_r_str_rwx(void) {
 	int rw =  r_str_rwx ("rw-");
 	int rx = r_str_rwx ("rx");
 	int none = r_str_rwx ("---");
+	int invalid = r_str_rwx ("invalid");
 	int number = r_str_rwx ("999");
 	int rx_number = r_str_rwx ("5");
 	int rwx_number = r_str_rwx ("7");
 	mu_assert_eq (rwx, 7, "rwx");
 	mu_assert_eq (rw, 6, "rw");
 	mu_assert_eq (rx, 5, "rx");
-	mu_assert_eq (none, -1, "no permissions");
-	mu_assert_eq (number, 0, "large input number string");
+	mu_assert_eq (none, 0, "no permissions");
+	mu_assert_eq (invalid, -1, "invalid permissions string");
+	mu_assert_eq (number, -1, "large input number string");
 	mu_assert_eq (rx_number, 5, "rx number");
 	mu_assert_eq (rwx_number, 7, "rwx number");
 	mu_end;
@@ -193,6 +195,13 @@ bool test_r_str_split_list(void) {
 		mu_assert_eq (r_list_length (r), 2, "split newline");
 		free (hi);
 	}
+	{
+		char* hi = strdup ("hello,world,comma,world");
+		RList *r = r_str_split_list (hi, ",", 3);
+		mu_assert_eq (r_list_length (r), 3, "split commas");
+		mu_assert_streq (r_list_get_n (r, 2), "comma,world", "split commas");
+		free (hi);
+	}
 	mu_end;
 }
 
